package hu.bme.mit.viatra2.genmodelexport;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.codegen.ecore.genmodel.GenAnnotation;
import org.eclipse.emf.codegen.ecore.genmodel.GenClass;
import org.eclipse.emf.codegen.ecore.genmodel.GenDelegationKind;
import org.eclipse.emf.codegen.ecore.genmodel.GenEnum;
import org.eclipse.emf.codegen.ecore.genmodel.GenEnumLiteral;
import org.eclipse.emf.codegen.ecore.genmodel.GenFeature;
import org.eclipse.emf.codegen.ecore.genmodel.GenJDKLevel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenModelFactory;
import org.eclipse.emf.codegen.ecore.genmodel.GenOperation;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.codegen.ecore.genmodel.GenParameter;
import org.eclipse.emf.codegen.ecore.genmodel.GenPropertyKind;
import org.eclipse.emf.codegen.ecore.genmodel.GenProviderKind;
import org.eclipse.emf.codegen.ecore.genmodel.GenResourceKind;
import org.eclipse.emf.codegen.ecore.genmodel.GenRuntimeVersion;
import org.eclipse.emf.codegen.ecore.genmodel.GenTypeParameter;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.ETypeParameter;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;

/**
 * @author Zoltan Ujhelyi
 */
public class ExportGenModel {

	GenModelFactory factory = GenModelFactory.eINSTANCE;
	private Map<String, EObject> viatra2Emf = new HashMap<String, EObject>();

	/**
	 * @param viatra2Emf
	 *            the viatra2Emf to set
	 */
	public void setViatra2Emf(Map<String, EObject> viatra2Emf) {
		this.viatra2Emf = viatra2Emf;
	}

	/**
	 * Follows a relation and calculates the value element of the target. If the
	 * value element or the relation does not exists, it returns null.
	 * @param source
	 * @param relationType
	 * @param manager
	 * @return
	 */
	private String relationValue(IEntity source, String relationType,
			IModelManager manager) {
		IEntity target = relationTarget(source, relationType, manager);
		if (target == null) return null;
		return target.getValue();
	}

	private IEntity relationTarget(IEntity source, String relationType,
			IModelManager manager) {
		IModelElement elementByName = manager.getElementByName(relationType);
		IRelation relation = source.getRelationByType(elementByName);
		if (relation == null) return null;
		return (IEntity) relation.getTo();
	}

	private Collection<IEntity> relationTargets(IEntity source,
			String relationType, IModelManager manager) {
		IRelation elementByName = manager.getRelationByName(relationType);
		Collection<IRelation> relations = source
				.getAllRelationFromByType(elementByName);
		ArrayList<IEntity> targets = new ArrayList<IEntity>();
		for (IRelation relation : relations) {
			targets.add((IEntity) relation.getTo());
		}
		return targets;
	}

	/**
	 * @param entity
	 * @param manager
	 */
	public GenModel handleGenModel(IEntity entity, IModelManager manager) {
		GenModel genModel = factory.createGenModel();
		String copyrightText = relationValue(entity,
				"nemf.packages.genmodel.GenModel.copyrightText", manager);
		if (copyrightText != null) genModel.setCopyrightText(copyrightText);

		String modelDirectory = relationValue(entity,
				"nemf.packages.genmodel.GenModel.modelDirectory", manager);
		if (modelDirectory != null) genModel.setModelDirectory(modelDirectory);

		String creationCommands = relationValue(entity,
				"nemf.packages.genmodel.GenModel.creationCommands", manager);
		if (creationCommands != null)
			genModel
					.setCreationCommands(Boolean.parseBoolean(creationCommands));

		String creationIcons = relationValue(entity,
				"nemf.packages.genmodel.GenModel.creationCommands", manager);
		if (creationIcons != null)
			genModel.setCreationIcons(Boolean.parseBoolean(creationIcons));

		String creationSubmenus = relationValue(entity,
				"nemf.packages.genmodel.GenModel.creationSubmenus", manager);
		if (creationSubmenus != null)
			genModel
					.setCreationSubmenus(Boolean.parseBoolean(creationSubmenus));

		String editDirectory = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editDirectory", manager);
		if (editDirectory != null) genModel.setEditDirectory(editDirectory);

		String editorDirectory = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editorDirectory", manager);
		if (editorDirectory != null)
			genModel.setEditorDirectory(editorDirectory);

		String modelPluginID = relationValue(entity,
				"nemf.packages.genmodel.GenModel.modelPluginID", manager);
		if (modelPluginID != null) genModel.setModelPluginID(modelPluginID);

		EList<String> foreignModels = genModel.getForeignModel();
		for (IEntity foreignModel : relationTargets(entity,
				"nemf.packages.genmodel.GenModel.foreignModel", manager)) {
			foreignModels.add(foreignModel.getValue());
		}

		String templateDirectory = relationValue(entity,
				"nemf.packages.genmodel.GenModel.templateDirectory", manager);
		if (templateDirectory != null)
			genModel.setTemplateDirectory(templateDirectory);

		String runtimeJar = relationValue(entity,
				"nemf.packages.genmodel.GenModel.runtimeJar", manager);
		if (runtimeJar != null)
			genModel.setRuntimeJar(Boolean.parseBoolean(runtimeJar));

		String modelName = relationValue(entity,
				"nemf.packages.genmodel.GenModel.modelName", manager);
		if (modelName != null) genModel.setModelName(modelName);

		String dynamicTemplates = relationValue(entity,
				"nemf.packages.genmodel.GenModel.dynamicTemplates", manager);
		if (dynamicTemplates != null)
			genModel
					.setDynamicTemplates(Boolean.parseBoolean(dynamicTemplates));

		String redirection = relationValue(entity,
				"nemf.packages.genmodel.GenModel.redirection", manager);
		if (redirection != null) genModel.setRedirection(redirection);

		String forceOverwrite = relationValue(entity,
				"nemf.packages.genmodel.GenModel.forceOverwrite", manager);
		if (forceOverwrite != null)
			genModel.setForceOverwrite(Boolean.parseBoolean(forceOverwrite));

		String nonExternalizedStringTag = relationValue(entity,
				"nemf.packages.genmodel.GenModel.nonExternalizedStringTag",
				manager);
		if (nonExternalizedStringTag != null)
			genModel.setNonExternalizedStringTag(nonExternalizedStringTag);

		String modelPluginClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.modelPluginClass", manager);
		if (modelPluginClass != null)
			genModel.setModelPluginClass(modelPluginClass);

		String editPluginClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editPluginClass", manager);
		if (editPluginClass != null)
			genModel.setEditorPluginClass(editPluginClass);

		String editorPluginClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editorPluginClass", manager);
		if (editorPluginClass != null)
			genModel.setEditorPluginClass(editorPluginClass);

		String updateClasspath = relationValue(entity,
				"nemf.packages.genmodel.GenModel.updateClasspath", manager);
		if (updateClasspath != null)
			genModel.setUpdateClasspath(Boolean.parseBoolean(updateClasspath));

		String generateSchema = relationValue(entity,
				"nemf.packages.genmodel.GenModel.generateSchema", manager);
		if (generateSchema != null)
			genModel.setGenerateSchema(Boolean.parseBoolean(generateSchema));

		String nonNLSMarkers = relationValue(entity,
				"nemf.packages.genmodel.GenModel.nonNLSMarkers", manager);
		if (nonNLSMarkers != null)
			genModel.setNonNLSMarkers(Boolean.parseBoolean(nonNLSMarkers));

		Collection<IEntity> staticPackageEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenModel.staticPackages", manager);
		EList<String> staticPackages = genModel.getStaticPackages();
		for (IEntity staticPackageEntity : staticPackageEntities) {
			String staticPackage = staticPackageEntity.getValue();
			staticPackages.add(staticPackage);
		}

		Collection<IEntity> modelPluginVariableEntities = relationTargets(
				entity, "nemf.packages.genmodel.GenModel.modelPluginVariables",
				manager);
		EList<String> modelPluginVariables = genModel.getModelPluginVariables();
		for (IEntity modelPluginVariableEntity : modelPluginVariableEntities) {
			String modelPluginVariable = modelPluginVariableEntity.getValue();
			modelPluginVariables.add(modelPluginVariable);
		}

		String rootExtendsInterface = relationValue(entity,
				"nemf.packages.genmodel.GenModel.rootExtendsInterface", manager);
		if (rootExtendsInterface != null)
			genModel.setRootExtendsInterface(rootExtendsInterface);

		String rootExtendsClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.rootExtendsClass", manager);
		if (rootExtendsClass != null)
			genModel.setRootExtendsClass(rootExtendsClass);

		String rootImplementsInterface = relationValue(entity,
				"nemf.packages.genmodel.GenModel.rootImplementsInterface",
				manager);
		if (rootImplementsInterface != null)
			genModel.setRootImplementsInterface(rootImplementsInterface);

		String suppressEMFTypes = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressEMFTypes", manager);
		if (suppressEMFTypes != null)
			genModel
					.setSuppressEMFTypes(Boolean.parseBoolean(suppressEMFTypes));

		String suppressEMFMetaData = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressEMFMetaData", manager);
		if (suppressEMFMetaData != null)
			genModel.setSuppressEMFMetaData(Boolean
					.parseBoolean(suppressEMFMetaData));

		String suppressEMFModelTags = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressEMFModelTags", manager);
		if (suppressEMFModelTags != null)
			genModel.setSuppressEMFModelTags(Boolean
					.parseBoolean(suppressEMFModelTags));

		String suppressInterfaces = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressInterfaces", manager);
		if (suppressInterfaces != null)
			genModel.setSuppressInterfaces(Boolean
					.parseBoolean(suppressInterfaces));

		String featureMapWrapperInterface = relationValue(entity,
				"nemf.packages.genmodel.GenModel.featureMapWrapperInterface",
				manager);
		if (featureMapWrapperInterface != null)
			genModel.setFeatureMapWrapperInterface(featureMapWrapperInterface);

		String featureMapWrapperInternalInterface = relationValue(
				entity,
				"nemf.packages.genmodel.GenModel.featureMapWrapperInternalInterface",
				manager);
		if (featureMapWrapperInternalInterface != null)
			genModel
					.setFeatureMapWrapperInterface(featureMapWrapperInternalInterface);

		String featureMapWrapperClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.featureMapWrapperClass",
				manager);
		if (featureMapWrapperClass != null)
			genModel.setFeatureMapWrapperClass(featureMapWrapperClass);

		String runtimeCompatibility = relationValue(entity,
				"nemf.packages.genmodel.GenModel.runtimeCompatibility", manager);
		if (runtimeCompatibility != null)
			genModel.setRuntimeCompatibility(Boolean
					.parseBoolean(runtimeCompatibility));

		String richClientPlatform = relationValue(entity,
				"nemf.packages.genmodel.GenModel.richClientPlatform", manager);
		if (richClientPlatform != null)
			genModel.setRichClientPlatform(Boolean
					.parseBoolean(richClientPlatform));

		String reflectiveDelegation = relationValue(entity,
				"nemf.packages.genmodel.GenModel.reflectiveDelegation", manager);
		if (reflectiveDelegation != null)
			genModel.setReflectiveDelegation(Boolean
					.parseBoolean(reflectiveDelegation));

		String codeFormatting = relationValue(entity,
				"nemf.packages.genmodel.GenModel.codeFormatting", manager);
		if (codeFormatting != null)
			genModel.setCodeFormatting(Boolean.parseBoolean(codeFormatting));

		String testsDirectory = relationValue(entity,
				"nemf.packages.genmodel.GenModel.testsDirectory", manager);
		if (testsDirectory != null) genModel.setTestsDirectory(testsDirectory);

		String testSuiteClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.testSuiteClass", manager);
		if (testSuiteClass != null) genModel.setTestSuiteClass(testSuiteClass);

		String booleanFlagsField = relationValue(entity,
				"nemf.packages.genmodel.GenModel.booleanFlagsField", manager);
		if (booleanFlagsField != null)
			genModel.setBooleanFlagsField(booleanFlagsField);

		String booleanFlagsReservedBits = relationValue(entity,
				"nemf.packages.genmodel.GenModel.booleanFlagsReservedBits",
				manager);
		if (booleanFlagsReservedBits != null)
			genModel.setBooleanFlagsReservedBits(Integer
					.parseInt(booleanFlagsReservedBits));

		String importerID = relationValue(entity,
				"nemf.packages.genmodel.GenModel.importerID", manager);
		if (importerID != null) genModel.setImporterID(importerID);

		String bundleManifest = relationValue(entity,
				"nemf.packages.genmodel.GenModel.bundleManifest", manager);
		if (bundleManifest != null)
			genModel.setBundleManifest(Boolean.parseBoolean(bundleManifest));

		String featureDelegation = relationValue(entity,
				"nemf.packages.genmodel.GenModel.featureDelegation", manager);
		if (featureDelegation != null)
			genModel.setFeatureDelegation(GenDelegationKind
					.get(featureDelegation));

		String containmentProxies = relationValue(entity,
				"nemf.packages.genmodel.GenModel.containmentProxies", manager);
		if (containmentProxies != null)
			genModel.setContainmentProxies(Boolean
					.parseBoolean(containmentProxies));

		String minimalReflectiveMethods = relationValue(entity,
				"nemf.packages.genmodel.GenModel.minimalReflectiveMethods",
				manager);
		if (minimalReflectiveMethods != null)
			genModel.setMinimalReflectiveMethods(Boolean
					.parseBoolean(minimalReflectiveMethods));

		String suppressContainment = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressContainment", manager);
		if (suppressContainment != null)
			genModel.setSuppressContainment(Boolean
					.parseBoolean(suppressContainment));

		String suppressNotification = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressNotification", manager);
		if (suppressNotification != null)
			genModel.setSuppressNotification(Boolean
					.parseBoolean(suppressNotification));

		String arrayAccessors = relationValue(entity,
				"nemf.packages.genmodel.GenModel.arrayAccessors", manager);
		if (arrayAccessors != null)
			genModel.setArrayAccessors(Boolean.parseBoolean(arrayAccessors));

		String suppressUnsettable = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressUnsettable", manager);
		if (suppressUnsettable != null)
			genModel.setSuppressUnsettable(Boolean
					.parseBoolean(suppressUnsettable));

		String facadeHelperClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.facadeHelperClass", manager);
		if (facadeHelperClass != null)
			genModel.setFacadeHelperClass(facadeHelperClass);

		String complianceLevel = relationValue(entity,
				"nemf.packages.genmodel.GenModel.complianceLevel", manager);
		if (complianceLevel != null)
			genModel.setComplianceLevel(GenJDKLevel.get(complianceLevel));

		String suppressGenModelAnnotations = relationValue(entity,
				"nemf.packages.genmodel.GenModel.suppressGenModelAnnotations",
				manager);
		if (suppressGenModelAnnotations != null)
			genModel.setSuppressGenModelAnnotations(Boolean
					.parseBoolean(suppressGenModelAnnotations));

		String binaryCompatibleReflectiveMethods = relationValue(
				entity,
				"nemf.packages.genmodel.GenModel.binaryCompatibleReflectiveMethods",
				manager);
		if (binaryCompatibleReflectiveMethods != null)
			genModel.setBinaryCompatibleReflectiveMethods(Boolean
					.parseBoolean(binaryCompatibleReflectiveMethods));

		String publicConstructors = relationValue(entity,
				"nemf.packages.genmodel.GenModel.publicConstructors", manager);
		if (publicConstructors != null)
			genModel.setPublicConstructors(Boolean
					.parseBoolean(publicConstructors));

		Collection<IEntity> templatePluginVariableEntities = relationTargets(
				entity,
				"nemf.packages.genmodel.GenModel.templatePluginVariables",
				manager);
		EList<String> templatePluginVariables = genModel
				.getTemplatePluginVariables();
		for (IEntity templatePluginVariableEntity : templatePluginVariableEntities) {
			String templatePluginVariable = templatePluginVariableEntity
					.getValue();
			templatePluginVariables.add(templatePluginVariable);
		}

		String providerRootExtendsClass = relationValue(entity,
				"nemf.packages.genmodel.GenModel.providerRootExtendsClass",
				manager);
		if (providerRootExtendsClass != null)
			genModel.setProviderRootExtendsClass(providerRootExtendsClass);

		String editPluginID = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editPluginID", manager);
		if (editPluginID != null) genModel.setEditPluginID(editPluginID);

		Collection<IEntity> editPluginVariableEntities = relationTargets(
				entity, "nemf.packages.genmodel.GenModel.editPluginVariables",
				manager);
		EList<String> editPluginVariables = genModel.getEditPluginVariables();
		for (IEntity editPluginVariableEntity : editPluginVariableEntities) {
			String editPluginVariable = editPluginVariableEntity.getValue();
			editPluginVariables.add(editPluginVariable);
		}

		String editorPluginID = relationValue(entity,
				"nemf.packages.genmodel.GenModel.editorPluginID", manager);
		if (editorPluginID != null) genModel.setEditPluginID(editorPluginID);

		Collection<IEntity> editorPluginVariableEntities = relationTargets(
				entity,
				"nemf.packages.genmodel.GenModel.editorPluginVariables",
				manager);
		EList<String> editorPluginVariables = genModel.getEditPluginVariables();
		for (IEntity editorPluginVariableEntity : editorPluginVariableEntities) {
			String editorPluginVariable = editorPluginVariableEntity.getValue();
			editorPluginVariables.add(editorPluginVariable);
		}

		String testsPluginID = relationValue(entity,
				"nemf.packages.genmodel.GenModel.testsPluginID", manager);
		if (testsPluginID != null) genModel.setTestsPluginID(testsPluginID);

		Collection<IEntity> testsPluginVariableEntities = relationTargets(
				entity, "nemf.packages.genmodel.GenModel.testsPluginVariables",
				manager);
		EList<String> testsPluginVariables = genModel.getTestsPluginVariables();
		for (IEntity testsPluginVariableEntity : testsPluginVariableEntities) {
			String testsPluginVariable = testsPluginVariableEntity.getValue();
			testsPluginVariables.add(testsPluginVariable);
		}

		Collection<IEntity> packages = relationTargets(entity,
				"nemf.packages.genmodel.GenModel.genPackages", manager);
		EList<GenPackage> genPackages = genModel.getGenPackages();
		for (IEntity packageEntity : packages) {
			GenPackage genPackage = handlePackage(packageEntity, manager);
			genPackages.add(genPackage);

		}

		String optimizedHasChildren = relationValue(entity,
				"nemf.packages.genmodel.GenModel.optimizedHasChildren", manager);
		if (optimizedHasChildren != null)
			genModel.setOptimizedHasChildren(Boolean
					.parseBoolean(optimizedHasChildren));

		String tableProviders = relationValue(entity,
				"nemf.packages.genmodel.GenModel.tableProviders", manager);
		if (tableProviders != null)
			genModel.setTableProviders(Boolean.parseBoolean(tableProviders));

		String colorProviders = relationValue(entity,
				"nemf.packages.genmodel.GenModel.colorProviders", manager);
		if (colorProviders != null)
			genModel.setColorProviders(Boolean.parseBoolean(colorProviders));

		String fontProviders = relationValue(entity,
				"nemf.packages.genmodel.GenModel.fontProviders", manager);
		if (fontProviders != null)
			genModel.setFontProviders(Boolean.parseBoolean(fontProviders));

		String runtimeVersion = relationValue(entity,
				"nemf.packages.genmodel.GenModel.runtimeVersion", manager);
		if (runtimeVersion != null)
			genModel.setRuntimeVersion(GenRuntimeVersion.get(runtimeVersion));

		String language = relationValue(entity,
				"nemf.packages.genmodel.GenModel.language", manager);
		if (language != null) genModel.setLanguage(language);

		String packedEnums = relationValue(entity,
				"nemf.packages.genmodel.GenModel.packedEnums", manager);
		if (packedEnums != null)
			genModel.setPackedEnums(Boolean.parseBoolean(packedEnums));

		generateAnnotations(entity, genModel.getGenAnnotations(), manager);
		return genModel;
	}

	private GenPackage handlePackage(IEntity entity, IModelManager manager) {
		GenPackage genPackage = factory.createGenPackage();
		String prefix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.prefix", manager);
		if (prefix != null) genPackage.setPrefix(prefix);

		String basePackage = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.basePackage", manager);
		if (basePackage != null) genPackage.setBasePackage(basePackage);

		String resource = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.resource", manager);
		if (resource != null)
			genPackage.setResource(GenResourceKind.get(resource));

		String disposableProviderFactory = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.disposableProviderFactory",
				manager);
		if (disposableProviderFactory != null)
			genPackage.setDisposableProviderFactory((Boolean
					.parseBoolean(disposableProviderFactory)));

		String adapterFactory = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.adapterFactory", manager);
		if (adapterFactory != null)
			genPackage.setAdapterFactory(Boolean.parseBoolean(adapterFactory));

		String loadInitialization = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.loadInitialization", manager);
		if (loadInitialization != null)
			genPackage.setLoadInitialization(Boolean
					.parseBoolean(loadInitialization));

		String interfacePackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.interfacePackageSuffix",
				manager);
		if (interfacePackageSuffix != null)
			genPackage.setInterfacePackageSuffix(interfacePackageSuffix);

		String metaDataPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.metaDataPackageSuffix",
				manager);
		if (metaDataPackageSuffix != null)
			genPackage.setMetaDataPackageSuffix(metaDataPackageSuffix);

		String classPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.classPackageSuffix", manager);
		if (classPackageSuffix != null)
			genPackage.setClassPackageSuffix(classPackageSuffix);

		String utilityPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.utilityPackageSuffix",
				manager);
		if (utilityPackageSuffix != null)
			genPackage.setUtilityPackageSuffix(utilityPackageSuffix);

		String providerPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.providerPackageSuffix",
				manager);
		if (providerPackageSuffix != null)
			genPackage.setProviderPackageSuffix(providerPackageSuffix);

		String presentationPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.presentationPackageSuffix",
				manager);
		if (presentationPackageSuffix != null)
			genPackage.setPresentationPackageSuffix(presentationPackageSuffix);

		String testPackageSuffix = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.testsPackageSuffix", manager);
		if (testPackageSuffix != null)
			genPackage.setTestsPackageSuffix(testPackageSuffix);

		String generateExampleClass = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.generateExampleClass",
				manager);
		if (generateExampleClass != null)
			genPackage.setGenerateExampleClass(Boolean
					.parseBoolean(generateExampleClass));

		String literalsInInterface = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.literalsInterface", manager);
		if (literalsInInterface != null)
			genPackage.setLiteralsInterface(Boolean
					.parseBoolean(literalsInInterface));

		String dataTypeConverters = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.dataTypeConverters", manager);
		if (dataTypeConverters != null)
			genPackage.setDataTypeConverters(Boolean
					.parseBoolean(dataTypeConverters));

		String multipleEditorPages = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.multipleEditorPages",
				manager);
		if (multipleEditorPages != null)
			genPackage.setMultipleEditorPages(Boolean
					.parseBoolean(multipleEditorPages));

		String generateModelWizard = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.generateModelWizard",
				manager);
		if (generateModelWizard != null)
			genPackage.setGenerateModelWizard(Boolean
					.parseBoolean(generateModelWizard));

		String extensibleProviderFactory = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.extensibleProviderFactory",
				manager);
		if (extensibleProviderFactory != null)
			genPackage.setExtensibleProviderFactory(Boolean
					.parseBoolean(extensibleProviderFactory));

		String childCreationExtenders = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.childCreationExtenders",
				manager);
		if (childCreationExtenders != null)
			genPackage.setChildCreationExtenders(Boolean
					.parseBoolean(childCreationExtenders));

		String contentTypeIdentifier = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.childCreationExtenders",
				manager);
		if (contentTypeIdentifier != null)
			genPackage.setContentTypeIdentifier(contentTypeIdentifier);

		String fileExtensions = relationValue(entity,
				"nemf.packages.genmodel.GenPackage.childCreationExtenders",
				manager);
		if (fileExtensions != null)
			genPackage.setFileExtensions(fileExtensions);

		Collection<IEntity> genEnumEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenPackage.genEnums", manager);
		EList<GenEnum> genEnums = genPackage.getGenEnums();
		for (IEntity genEnumEntity : genEnumEntities) {
			GenEnum genEnum = handleEnum(genEnumEntity, manager);
			genEnums.add(genEnum);
		}

		Collection<IEntity> classes = relationTargets(entity,
				"nemf.packages.genmodel.GenPackage.genClasses", manager);
		EList<GenClass> genClasses = genPackage.getGenClasses();
		for (IEntity classEntity : classes) {
			GenClass genClass = handleClass(classEntity, manager);
			genClasses.add(genClass);
		}

		Collection<IEntity> nestedGenPackageEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenPackage.nestedGenPackages", manager);
		EList<GenPackage> nestedGenPackages = genPackage.getNestedGenPackages();
		for (IEntity nestedPackageEntity : nestedGenPackageEntities) {
			GenPackage nestedGenPackage = handlePackage(nestedPackageEntity,
					manager);
			nestedGenPackages.add(nestedGenPackage);
		}

		generateAnnotations(entity, genPackage.getGenAnnotations(), manager);

		String ecorePackage = relationTarget(entity,
				"nemf.packages.genmodel.GenPackage.ecorePackage", manager)
				.getFullyQualifiedName();
		genPackage.setEcorePackage((EPackage) viatra2Emf.get(ecorePackage));
		return genPackage;
	}

	private GenClass handleClass(IEntity entity, IModelManager manager) {
		GenClass genClass = factory.createGenClass();

		String provider = relationValue(entity,
				"nemf.packages.genmodel.GenClass.provider", manager);
		if (provider != null)
			genClass.setProvider(GenProviderKind.get(provider));

		String image = relationValue(entity,
				"nemf.packages.genmodel.GenClass.image", manager);
		if (image != null) genClass.setImage(Boolean.parseBoolean(image));

		String dynamic = relationValue(entity,
				"nemf.packages.genmodel.GenClass.dynamic", manager);
		if (dynamic != null)
			genClass.setDynamic(Boolean.parseBoolean(dynamic));

		Collection<IEntity> genFeatureEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenClass.genFeatures", manager);
		EList<GenFeature> genFeatures = genClass.getGenFeatures();
		for (IEntity genFeatureEntity : genFeatureEntities) {
			GenFeature genFeature = handleFeature(genFeatureEntity, manager);
			genFeatures.add(genFeature);
		}

		// Resolving label feature
		IEntity labelFeatureEntity = relationTarget(entity,
				"nemf.packages.genmodel.GenClass.labelFeature", manager);
		if (labelFeatureEntity != null) {
			String ecoreOperation = relationTarget(labelFeatureEntity,
					"nemf.packages.genmodel.GenOperation.ecoreOperation",
					manager).getFullyQualifiedName();
			genClass.setLabelFeature(featureAssignment.get(viatra2Emf
					.get(ecoreOperation)));
		}

		Collection<IEntity> genOperationEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenClass.genOperations", manager);
		EList<GenOperation> genOperations = genClass.getGenOperations();
		for (IEntity genOperationEntity : genOperationEntities) {
			GenOperation genOperation = handleOperation(genOperationEntity,
					manager);
			genOperations.add(genOperation);
		}

		Collection<IEntity> genTypeParameterEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenClassifier.genTypeParameters",
				manager);
		EList<GenTypeParameter> genTypeParameters = genClass
				.getGenTypeParameters();
		for (IEntity genTypeParameterEntity : genTypeParameterEntities) {
			GenTypeParameter genTypeParameter = handleTypeParameter(
					genTypeParameterEntity, manager);
			genTypeParameters.add(genTypeParameter);
		}

		generateAnnotations(entity, genClass.getGenAnnotations(), manager);

		String ecoreClass = relationTarget(entity,
				"nemf.packages.genmodel.GenClass.ecoreClass", manager)
				.getFullyQualifiedName();
		genClass.setEcoreClass((EClass) viatra2Emf.get(ecoreClass));
		return genClass;
	}

	private HashMap<EStructuralFeature, GenFeature> featureAssignment = new HashMap<EStructuralFeature, GenFeature>();

	private GenFeature handleFeature(IEntity entity, IModelManager manager) {
		GenFeature genFeature = factory.createGenFeature();

		String property = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.children", manager);
		if (property != null)
			genFeature.setProperty(GenPropertyKind.get(property));

		String notify = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.notify", manager);
		if (notify != null) genFeature.setNotify(Boolean.parseBoolean(notify));

		String children = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.children", manager);
		if (children != null)
			genFeature.setChildren(Boolean.parseBoolean(children));

		String createChild = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.createChild", manager);
		if (createChild != null)
			genFeature.setCreateChild(Boolean.parseBoolean(createChild));

		String propertyCategory = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.propertyCategory", manager);
		if (propertyCategory != null)
			genFeature.setPropertyCategory(propertyCategory);

		Collection<IEntity> propertyFilterFlagEntities = relationTargets(
				entity,
				"nemf.packages.genmodel.GenFeature.propertyFilterFlags",
				manager);
		EList<String> propertyFilterFlags = genFeature.getPropertyFilterFlags();
		for (IEntity propertyFilterFlagEntity : propertyFilterFlagEntities) {
			String value = propertyFilterFlagEntity.getValue();
			propertyFilterFlags.add(value);
		}

		String propertyDescription = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.propertyDescription",
				manager);
		if (propertyDescription != null)
			genFeature.setPropertyDescription(propertyDescription);

		String propertyMultiLine = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.propertyMultiLine", manager);
		if (propertyMultiLine != null)
			genFeature.setPropertyMultiLine(Boolean
					.parseBoolean(propertyMultiLine));

		String propertySortChoices = relationValue(entity,
				"nemf.packages.genmodel.GenFeature.propertySortChoices",
				manager);
		if (propertySortChoices != null)
			genFeature.setPropertySortChoices(Boolean
					.parseBoolean(propertySortChoices));

		generateAnnotations(entity, genFeature.getGenAnnotations(), manager);

		String ecoreFeature = relationTarget(entity,
				"nemf.packages.genmodel.GenFeature.ecoreFeature", manager)
				.getFullyQualifiedName();
		EStructuralFeature ecoreFeatureValue = (EStructuralFeature) viatra2Emf
				.get(ecoreFeature);
		featureAssignment.put(ecoreFeatureValue, genFeature);
		genFeature.setEcoreFeature(ecoreFeatureValue);

		return genFeature;
	}

	private GenOperation handleOperation(IEntity entity, IModelManager manager) {
		GenOperation operation = factory.createGenOperation();

		Collection<IEntity> genParameterEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenOperation.genParameters", manager);
		EList<GenParameter> genParameters = operation.getGenParameters();
		for (IEntity genParameterEntity : genParameterEntities) {
			GenParameter genParameter = handleParameter(genParameterEntity,
					manager);
			genParameters.add(genParameter);
		}

		Collection<IEntity> genTypeParameterEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenOperation.genTypeParameters",
				manager);
		EList<GenTypeParameter> genTypeParameters = operation
				.getGenTypeParameters();
		for (IEntity genTypeParameterEntity : genTypeParameterEntities) {
			GenTypeParameter genTypeParameter = handleTypeParameter(
					genTypeParameterEntity, manager);
			genTypeParameters.add(genTypeParameter);
		}

		generateAnnotations(entity, operation.getGenAnnotations(), manager);

		String ecoreOperation = relationTarget(entity,
				"nemf.packages.genmodel.GenOperation.ecoreOperation", manager)
				.getFullyQualifiedName();
		operation
				.setEcoreOperation((EOperation) viatra2Emf.get(ecoreOperation));
		return operation;

	}

	private GenParameter handleParameter(IEntity entity, IModelManager manager) {
		GenParameter parameter = factory.createGenParameter();

		String ecoreParameter = relationTarget(entity,
				"nemf.packages.genmodel.GenParameter.ecoreParameter", manager)
				.getFullyQualifiedName();
		parameter
				.setEcoreParameter((EParameter) viatra2Emf.get(ecoreParameter));

		generateAnnotations(entity, parameter.getGenAnnotations(), manager);

		return parameter;
	}

	private GenTypeParameter handleTypeParameter(IEntity entity,
			IModelManager manager) {
		GenTypeParameter parameter = factory.createGenTypeParameter();

		String ecoreParameter = relationTarget(entity,
				"nemf.packages.genmodel.GenTypeParameter.ecoreTypeParameter",
				manager).getFullyQualifiedName();
		parameter.setEcoreTypeParameter((ETypeParameter) viatra2Emf
				.get(ecoreParameter));

		generateAnnotations(entity, parameter.getGenAnnotations(), manager);

		return parameter;
	}

	private GenEnum handleEnum(IEntity entity, IModelManager manager) {
		GenEnum genEnum = factory.createGenEnum();

		String typeSafeEnumCompatible = relationValue(entity,
				"nemf.packages.genmodel.GenEnum.typeSafeEnumCompatible",
				manager);
		genEnum.setTypeSafeEnumCompatible(Boolean
				.parseBoolean(typeSafeEnumCompatible));

		Collection<IEntity> genEnumLiteralEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenEnum.genEnumLiterals", manager);
		EList<GenEnumLiteral> genEnumLiterals = genEnum.getGenEnumLiterals();
		for (IEntity genEnumLiteralEntity : genEnumLiteralEntities) {
			GenEnumLiteral enumLiteral = handleEnumLiteral(
					genEnumLiteralEntity, manager);
			genEnumLiterals.add(enumLiteral);
		}

		Collection<IEntity> genTypeParameterEntities = relationTargets(entity,
				"nemf.packages.genmodel.GenClassifier.genTypeParameters",
				manager);
		EList<GenTypeParameter> genTypeParameters = genEnum
				.getGenTypeParameters();
		for (IEntity genTypeParameterEntity : genTypeParameterEntities) {
			GenTypeParameter genTypeParameter = handleTypeParameter(
					genTypeParameterEntity, manager);
			genTypeParameters.add(genTypeParameter);
		}

		generateAnnotations(entity, genEnum.getGenAnnotations(), manager);

		String ecoreEnum = relationTarget(entity,
				"nemf.packages.genmodel.GenEnum.ecoreEnum", manager)
				.getFullyQualifiedName();
		genEnum.setEcoreEnum((EEnum) viatra2Emf.get(ecoreEnum));
		return genEnum;
	}

	private GenEnumLiteral handleEnumLiteral(IEntity entity,
			IModelManager manager) {
		GenEnumLiteral genEnumLiteral = factory.createGenEnumLiteral();

		generateAnnotations(entity, genEnumLiteral.getGenAnnotations(), manager);

		return genEnumLiteral;
	}

	private void generateAnnotations(IEntity base,
			EList<GenAnnotation> genAnnotations, IModelManager manager) {
		Collection<IEntity> genAnnotationEntities = relationTargets(base,
				"nemf.packages.genmodel.GenBase.genAnnotations", manager);
		for (IEntity genAnnotationEntity : genAnnotationEntities) {
			GenAnnotation genAnnotation = handleAnnotation(genAnnotationEntity,
					manager);
			genAnnotations.add(genAnnotation);
		}
	}

	private GenAnnotation handleAnnotation(IEntity entity, IModelManager manager) {
		GenAnnotation genAnnotation = factory.createGenAnnotation();

		String source = relationValue(entity,
				"nemf.packages.genmodel.GenAnnotation.source", manager);
		if (source != null) genAnnotation.setSource(source);

		return genAnnotation;
	}
}
