package hu.bme.mit.viatra2.genmodelexport;

import hu.optxware.emf2viatra.core.importer.EcoreInstanceImporter_OLD;
import hu.optxware.emf2viatra.utils.NameUtility;

import java.io.File;
import java.io.FileInputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.FrameworkManagerException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;
import org.eclipse.viatra2.imports.NativeImporterFactory;

/**
 * @author Zoltan Ujhelyi
 */
public class GenmodelTransformationHandler extends AbstractHandler {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.
	 * ExecutionEvent)
	 */
	//@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		ISelection currentSelection = HandlerUtil.getCurrentSelection(event);
		if (currentSelection instanceof IStructuredSelection) {

			Object _element = ((IStructuredSelection) currentSelection)
					.getFirstElement();
			if (_element instanceof IFile) {
				TransformationJob job = new TransformationJob("Genmodel Generation");
				job.ecore = (IFile) _element;
				job.setUser(true);
				job.schedule();
			}
		}
		return null;
	}

	private class TransformationJob extends Job{
		
		public IFile ecore;

		public TransformationJob(String name) {
			super(name);
		}

		@Override
		protected IStatus run(IProgressMonitor monitor) {
			long startTime = System.currentTimeMillis();
			//IFile newVtcl = null;
			FrameworkManager fwManager = FrameworkManager.getInstance();
			IFramework framework = null;
			try {
				IContainer container = ecore.getParent();
				// Loading framework
				monitor.subTask("Loading VIATRA2 framework");
				framework = fwManager.createFramework();
				File vpml = FrameworkManager.getFileFromBundle(
						"hu.bme.mit.viatra2.genmodelexport",
						"models/nemf-meta.vpml");
				framework.mergeStream(new FileInputStream(vpml));

				IModelSpace modelspace = framework.getTopmodel();
				IModelManager modelManager = modelspace.getModelManager();
				// Importing ecore model(s)
				monitor.subTask("Importing ecore models");
				framework
						.nativeImport(ecore.getLocation().toOSString(),
								"hu.optxware.viatra.emf.live.core.importer.EcoreInstance");
				NativeImporterFactory importerFactory = framework
						.getNativeImporters()
						.get(
								"hu.optxware.viatra.emf.live.core.importer.EcoreInstance");
				EcoreInstanceImporter_OLD importer = (EcoreInstanceImporter_OLD) importerFactory
						.getImporterInstance();
				Map<String, EObject> emfMapping = importer.getMapping();

				ResourceSet set = importer.getResourceSet();
				Resource ecoreRes = set.getResource(URI
						.createPlatformResourceURI(ecore.getFullPath()
								.toString(), true), true);
				EObject root = ecoreRes.getContents().get(0);
				String name = NameUtility.getNameOfEObject(root);
				// Transformation
				monitor.subTask("Executing transformation");
				//newVtcl = container.getFile(new Path("transformation.vtcl"));
				File vtcl = FrameworkManager.getFileFromBundle(
						"hu.bme.mit.viatra2.genmodelexport",
						"models/ecore2genmodel_cropped.vtcl");
				//newVtcl.create(new FileInputStream(vtcl), false, null);
				//framework.loadMachine(newVtcl.getLocation().toOSString(),
				framework.loadMachine(vtcl.getPath(),
						framework.getLoadersForExtension("vtcl").iterator()
								.next());
				Machine machine = (Machine) framework
						.getMachineByFQN("ecore2genmodel");
				HashMap<String, Object> machineParameters = new HashMap<String, Object>();
				machineParameters.put("EcoreModel", String.format(
						"nemf.resources.%s", name));
				machineParameters.put("GenModelName", name);
				machineParameters.put("PluginID", ecore.getProject()
						.getName());
				long nowTime = System.currentTimeMillis();
				framework.runEntrypoint(machine, machineParameters, null);
				// Exporting genmodel
				long endTrTime = System.currentTimeMillis();
				System.out.println(String.format("Elapsed transformation time: %d ms", endTrTime-nowTime));
				monitor.subTask("Genmodel export");
				IEntity entity = modelManager.getEntityByName(String
						.format("nemf.resources.%s_genmodel", name));
				ExportGenModel export = new ExportGenModel();
				export.setViatra2Emf(emfMapping);
				GenModel genModel = export.handleGenModel(entity,
						modelManager);
				Resource genModelRes = set
						.createResource(URI.createPlatformResourceURI(
								container.getFullPath().toFile() + "/"
										+ ecore.getName() + ".genmodel",
								true));
				genModelRes.getContents().add(genModel);

				genModelRes.save(Collections.emptyMap());
				
			} catch (Exception e) {
				e.printStackTrace();
				//throw new ExecutionException("Error during Genmodel creation", e);
				return new Status(Status.ERROR, "hu.bme.mit.viatra2.genmodelexport", "Error during Genmodel creation.", e);
			} finally {
				monitor.done();
				//if (newVtcl != null) try {
				//	newVtcl.delete(true, null);
				//} catch (CoreException e) {
				//	e.printStackTrace();
				//}
				if (framework != null) try {
					fwManager.disposeFramework(framework.getId());
				} catch (FrameworkManagerException e) {
					e.printStackTrace();
				}
			}
			long endTime = System.currentTimeMillis();
			System.out.println(String.format("Elapsed time: %d ms", endTime-startTime));
			return Status.OK_STATUS;
		}
	};
	
}
