package hu.optxware.emf2viatra.core.eresource.impl;

import hu.optxware.emf2viatra.core.eresource.InternalViatraObject;
import hu.optxware.emf2viatra.core.eresource.ViatraResource;
import hu.optxware.emf2viatra.core.listener.ViatraListener;
import hu.optxware.emf2viatra.core.managers.EMF2ViatraSynchronizer;
import hu.optxware.emf2viatra.core.managers.EMFCoreEntConst;
import hu.optxware.emf2viatra.core.managers.EMFCoreMetamodelManager;
import hu.optxware.emf2viatra.core.managers.Viatra2EMFSynchronizer;
import hu.optxware.emf2viatra.core.managers.ViatraEMFFrameworkManager;
import hu.optxware.emf2viatra.core.managers.ViatraEMFManagersFactory;
import hu.optxware.emf2viatra.utils.AdaptingAdapter;
import hu.optxware.emf2viatra.utils.TestAdapter;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.NotificationChain;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.common.notify.impl.NotificationChainImpl;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.emf.common.notify.impl.NotifierImpl;
import org.eclipse.emf.common.notify.impl.NotifyingListImpl;
import org.eclipse.emf.common.util.AbstractTreeIterator;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.InternalEObject.EStore;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.URIConverter;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.util.EcoreUtil.ContentTreeIterator;
import org.eclipse.emf.ecore.util.EcoreUtil.ProperContentIterator;
import org.eclipse.emf.ecore.util.InternalEList;
import org.eclipse.emf.ecore.util.NotifyingInternalEListImpl;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.framework.FrameworkException;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.FrameworkManagerException;
import org.eclipse.viatra2.framework.IFramework;

public class ViatraResourceImpl extends NotifierImpl implements ViatraResource, Resource.Internal {

	private final boolean printDebug = true;

	private void log(String x) {
		if (printDebug) {
			System.out.println(x);
		}
	}

	private IFramework fw;

	private EMFCoreMetamodelManager emfCoreManager;

	private Viatra2EMFSynchronizer viatra2EMFSync;

	private EMF2ViatraSynchronizer emf2ViatraSync;

	private ViatraEMFFrameworkManager viatraEMFManager;

	private ViatraListener viatraListener;

	// Copied from ResourceImpl.
	private URI uri;

	// Copied from ResourceImpl.
	private ResourceSet resourceSet;

	// Copied from ResourceImpl.
	private EList<Diagnostic> errors;

	// Copied from ResourceImpl.
	private EList<Diagnostic> warnings;

	// Copied from ResourceImpl.
	private boolean isModified;

	// Copied from ResourceImpl.
	private boolean isLoaded;

	// Copied from ResourceImpl.
	private boolean isLoading;

	// Copied from ResourceImpl.
	private long timeStamp;

	// Copied from ResourceImpl.
	private Adapter modificationTrackingAdapter;

	// Copied from ResourceImpl.
	private ContentsEList<EObject> contents;

	// Copied from ResourceImpl.
	public ViatraResourceImpl() {
		super();
	}

	// Copied from ResourceImpl.
	public ViatraResourceImpl(URI uri) {
		this();
		this.uri = uri;
	}

	public ViatraListener getViatraListener() {
		return viatraListener;
	}

	// Copied from ResourceImpl.
	public EList<EObject> getContents() {
		if (contents == null) {
			contents = new ContentsEList<EObject>();
		}
		return contents;
	}

	// Unused, untested TODO
	// Copied from ResourceImpl.
	public TreeIterator<EObject> getAllContents() {
		return new AbstractTreeIterator<EObject>(this, false) {
			private static final long serialVersionUID = 1L;

			@Override
			public Iterator<EObject> getChildren(Object object) {
				return object == ViatraResourceImpl.this ? ViatraResourceImpl.this.getContents().iterator()
						: ((EObject) object).eContents().iterator();
			}
		};
	}

	public void load(Map<?, ?> options) throws IOException {
		if (!isLoaded) {
			Notification notification = setLoaded(true);
			isLoading = true;
			if (errors != null) {
				errors.clear();
			}
			if (warnings != null) {
				warnings.clear();
			}

			final FrameworkManager iFrameWorkManager = FrameworkManager.getInstance();
			final String iFileName = uri.devicePath();

			final IRunnableWithProgress runnable = new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					if (fw == null && iFileName != null) {
						try {
							monitor.beginTask("Loading VPM modelspace", 3);
							monitor.worked(2);
							fw = iFrameWorkManager.createFramework(iFileName);
							monitor.done();
						} catch (FrameworkManagerException e) {
							e.printStackTrace();
						}
					}
				}
			};

			if (fw == null && iFileName != null) {
				String[] hacktomb = iFrameWorkManager.getAllFrameWorks();
				for (String fwid : hacktomb) {
					IFramework tempfw = iFrameWorkManager.getFramework(fwid);
					if (tempfw.getCurrentFilename().replaceAll("\\u005c", "/").equals(iFileName)) {
						fw = tempfw;
					}
				}
				if (fw == null) {
					runLongTransaction(runnable);
				}
			}

			// set-up phase
			if (fw != null) {
				emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(fw);
				viatra2EMFSync = ViatraEMFManagersFactory.getInstance().getViatra2EMFSynchronizer(fw);
				emf2ViatraSync = ViatraEMFManagersFactory.getInstance().getEMF2ViatraSynchronizer(fw);
				viatraEMFManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(fw);

				IEntity resourcesRoot = emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.RESOURCES);
				for (IModelElement viatraElement : resourcesRoot.getContents()) {
					if (viatraElement instanceof IEntity) {
						IEntity viatraInstanceEntity = (IEntity) viatraElement;
						if (uri.fragment() == null || uri.fragment().equals(viatraInstanceEntity.getName())) {
							EObject eObject = (EObject) viatra2EMFSync.getObjectForIEntity(viatraInstanceEntity);
							if (eObject != null) {
								getContents().add(eObject);
							}
						}
					}
				}

				viatraListener = new ViatraListener(fw, this);
				// setTrackingModification(true);

				// Testing adapters:
				// TODO
				TreeIterator<EObject> iter = getAllContents();
				while (iter.hasNext()) {
					EObject object = iter.next();
					object.eAdapters().add(new AdaptingAdapter());
					object.eAdapters().add(new TestAdapter());
				}

			}

			isLoading = false;
			if (notification != null) {
				eNotify(notification);
			}
			if (isTrackingModification()) {
				setModified(false);
			}
		}
	}

	private void runLongTransaction(final IRunnableWithProgress runnable) {
		final Display display = Display.getDefault();

		display.syncExec(new Runnable() {
			public void run() {
				ProgressMonitorDialog d = new ProgressMonitorDialog(display.getActiveShell());
				try {
					d.run(true, false, runnable);
				} catch (Exception e) {
					e.printStackTrace();
					fw.getLogger().fatal(
							"Error executing operation (" + e.getClass().getCanonicalName() + ") [" + e.getMessage()
									+ "]");
				}
			}
		});

	}

	public void load(InputStream inputStream, Map<?, ?> options) throws IOException {
		throw new UnsupportedOperationException();
	}

	// Unused, untested TODO
	public void unload() {
		if (isLoaded) {
			// unloadingContents = new
			// BasicEList.FastCompare<EObject>(getContents());
			Notification notification = setLoaded(false);
			try {
				log("Unload happened.");
				// doUnload();
			} finally {
				// unloadingContents = null;
				if (notification != null) {
					eNotify(notification);
				}
				setTimeStamp(URIConverter.NULL_TIME_STAMP);
			}
		}
	}

	public void save(Map<?, ?> options) throws IOException {
		try {
			fw.saveFile(uri.toFileString());
			if (isTrackingModification()) {
				setModified(false);
			}
		} catch (FrameworkException e) {
			e.printStackTrace();
		}
	}

	public void save(OutputStream outputStream, Map<?, ?> options) throws IOException {
		throw new UnsupportedOperationException();
	}

	public void delete(Map<?, ?> options) throws IOException {
		throw new UnsupportedOperationException();
	}

	private void changeStateFromDynamic(InternalViatraObject object) {
		if (object instanceof ViatraObjectImpl) {
			ViatraObjectImpl viatraObjectImpl = (ViatraObjectImpl) object;
			if (viatraObjectImpl.getRelatedViatraEntity() != null && !viatraObjectImpl.isPersisted()) {
				for (EStructuralFeature feature : viatraObjectImpl.eClass().getEAllStructuralFeatures()) {
					// // TODO : Beletenni egy ellen�rz�st, hogy nem-e
					// perziszt�lt
					// // m�r az refer�lt elem
					// if (feature instanceof EReference && !((EReference)
					// feature).isContainment()) {
					// // TODO: Ha olyan elemre refer�l "nem tartalmaz�ssal",
					// // ami nem perziszt�lt, akkor TODO
					// // TODO: Ez ideiglenes (u.a. mint lentebb), �t kell m�g
					// // gondolni
					// Object nonPersistedValue =
					// viatraObjectImpl.eGet(feature);
					// if (nonPersistedValue instanceof List) {
					// List nonPersistedValueList = (List) nonPersistedValue;
					// for (int i = 0; i < nonPersistedValueList.size(); i++) {
					// viatraObjectImpl.eStore().add(viatraObjectImpl, feature,
					// i,
					// nonPersistedValueList.get(i));
					// }
					// } else if (nonPersistedValue != null) {
					// viatraObjectImpl.eStore()
					// .set(viatraObjectImpl, feature, EStore.NO_INDEX,
					// nonPersistedValue);
					// }
					// } else {
					Object nonPersistedValue = viatraObjectImpl.eGet(feature);
					if (nonPersistedValue instanceof List) {
						List nonPersistedValueList = (List) nonPersistedValue;
						for (int i = 0; i < nonPersistedValueList.size(); i++) {
							Object value = nonPersistedValueList.get(i);
							if (value instanceof EObject && ((EObject) value).eIsProxy()) {
								EObject resolved = EcoreUtil.resolve((EObject) value, object);
								if (resolved.eIsProxy()) {
									// TODO the proxy cannot be resolved!
									System.out.println("The proxy: " + value + " cannot be resolved!");
								} else {
									value = resolved;
								}
							}
							viatraObjectImpl.eStore().add(viatraObjectImpl, feature, i, value);
						}
					} else if (nonPersistedValue != null) {
						if (nonPersistedValue instanceof EObject && ((EObject) nonPersistedValue).eIsProxy()) {
							EObject resolved = EcoreUtil.resolve((EObject) nonPersistedValue, object);
							if (resolved.eIsProxy()) {
								// TODO the proxy cannot be resolved!
								System.out.println("!!!!!!The proxy: " + nonPersistedValue + " cannot be resolved!");
							} else {
								nonPersistedValue = resolved;
							}
						}
						viatraObjectImpl.eStore().set(viatraObjectImpl, feature, EStore.NO_INDEX, nonPersistedValue);
						// }
					}
				}
			}
		}
	}

	// TODO TODO TODO
	private void changeStateToDynamic(EObject object) {
		if (((ViatraObjectImpl) object).getRelatedViatraEntity() != null) {

		}
		((InternalViatraObject) object).setPersisted(false);
		((InternalViatraObject) object).setRelatedViatraEntity(null);
	}

	public void attached(EObject object) {
		if (object instanceof InternalViatraObject) {
			// Collects the elements that need to be set to persisted
			List<InternalViatraObject> persistedElements = new ArrayList<InternalViatraObject>();

			InternalViatraObject internalViatraObj = (InternalViatraObject) object;
			log("!Attached root: " + internalViatraObj);
			changeStateFromDynamic(internalViatraObj);
			persistedElements.add(internalViatraObj);

			// TODO: It has to be tested whether it can be skipped or not
//			for (TreeIterator<EObject> tree = getAllProperContents(internalViatraObj); tree.hasNext();) {
//				EObject next = tree.next();
//				if (next instanceof InternalViatraObject) {
//					InternalViatraObject nextInternalViatraObject = (InternalViatraObject) next;
//					log("!Attached next: " + next);
//					changeStateFromDynamic(nextInternalViatraObject);
//					persistedElements.add(nextInternalViatraObject);
//				}
//			}

			// At the end set all the elements to persisted
			for (InternalViatraObject persisted : persistedElements) {
				persisted.setPersisted(true);
			}
		}

	}

	// TODO TODO TODO
	public void detached(EObject object) {
		if (object instanceof InternalViatraObject) {
			// Collects the elements that need to be set to unpersisted
			List<InternalViatraObject> unPersistedElements = new ArrayList<InternalViatraObject>();

			InternalViatraObject internalViatraObj = (InternalViatraObject) object;
			log("!Detached root: " + internalViatraObj);
			// changeStateToDynamic(internalViatraObj);
			unPersistedElements.add(internalViatraObj);

			for (TreeIterator<EObject> tree = getAllProperContents(internalViatraObj); tree.hasNext();) {
				EObject next = tree.next();
				if (next instanceof InternalViatraObject) {
					InternalViatraObject nextInternalViatraObject = (InternalViatraObject) next;
					log("!Detached next: " + nextInternalViatraObject);
					// changeStateToDynamic(nextInternalViatraObject);
					unPersistedElements.add(nextInternalViatraObject);
				}
			}

			// At the end set all the elements to unpersisted
			for (InternalViatraObject unPersisted : unPersistedElements) {
				unPersisted.setPersisted(false);
				// TODO: Ez a helye?
				unPersisted.setRelatedViatraEntity(null);
			}

		}
	}

	// Copied from ResourceImpl.
	public URI getURI() {
		return uri;
	}

	// Copied from ResourceImpl.
	public void setURI(URI uri) {
		URI oldURI = this.uri;
		this.uri = uri;
		if (eNotificationRequired()) {
			Notification notification = new NotificationImpl(Notification.SET, oldURI, uri) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__URI;
				}
			};
			eNotify(notification);
		}
	}

	// Copied from ResourceImpl.
	public ResourceSet getResourceSet() {
		return resourceSet;
	}

	// Copied from ResourceImpl.
	public NotificationChain basicSetResourceSet(ResourceSet resourceSet, NotificationChain notifications) {
		ResourceSet oldResourceSet = this.resourceSet;
		if (oldResourceSet != null) {
			notifications = ((InternalEList<Resource>) oldResourceSet.getResources()).basicRemove(this, notifications);
		}

		this.resourceSet = resourceSet;

		if (eNotificationRequired()) {
			if (notifications == null) {
				notifications = new NotificationChainImpl(2);
			}
			notifications.add(new NotificationImpl(Notification.SET, oldResourceSet, resourceSet) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__RESOURCE_SET;
				}
			});
		}

		return notifications;
	}

	// Copied from ResourceImpl.
	public EList<Diagnostic> getErrors() {
		if (errors == null) {
			errors = new NotifyingListImpl<Diagnostic>() {
				private static final long serialVersionUID = 1L;

				@Override
				protected boolean isNotificationRequired() {
					return ViatraResourceImpl.this.eNotificationRequired();
				}

				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID() {
					return RESOURCE__ERRORS;
				}
			};
		}
		return errors;
	}

	// Copied from ResourceImpl.
	public EList<Diagnostic> getWarnings() {
		if (warnings == null) {
			warnings = new NotifyingListImpl<Diagnostic>() {
				private static final long serialVersionUID = 1L;

				@Override
				protected boolean isNotificationRequired() {
					return ViatraResourceImpl.this.eNotificationRequired();
				}

				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID() {
					return RESOURCE__WARNINGS;
				}
			};
		}
		return warnings;
	}

	// Copied from ResourceImpl.
	public long getTimeStamp() {
		return timeStamp;
	}

	// Copied from ResourceImpl.
	public void setTimeStamp(long timeStamp) {
		long oldTimeStamp = this.timeStamp;
		this.timeStamp = timeStamp;
		if (eNotificationRequired()) {
			Notification notification = new NotificationImpl(Notification.SET, oldTimeStamp, timeStamp) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__TIME_STAMP;
				}
			};
			eNotify(notification);
		}
	}

	// Copied from ResourceImpl.
	public boolean isLoaded() {
		return isLoaded;
	}

	// Copied from ResourceImpl.
	private Notification setLoaded(boolean isLoaded) {
		boolean oldIsLoaded = this.isLoaded;
		this.isLoaded = isLoaded;

		if (eNotificationRequired()) {
			Notification notification = new NotificationImpl(Notification.SET, oldIsLoaded, isLoaded) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__IS_LOADED;
				}
			};
			return notification;
		} else {
			return null;
		}
	}

	// Copied from ResourceImpl.
	public boolean isLoading() {
		return isLoading;
	}

	// Copied from ResourceImpl.
	public boolean isModified() {
		return isModified;
	}

	// Copied from ResourceImpl.
	public void setModified(boolean isModified) {
		boolean oldIsModified = this.isModified;
		this.isModified = isModified;
		if (eNotificationRequired()) {
			Notification notification = new NotificationImpl(Notification.SET, oldIsModified, isModified) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__IS_MODIFIED;
				}
			};
			eNotify(notification);
		}
	}

	// Copied from ResourceImpl.
	public boolean isTrackingModification() {
		return modificationTrackingAdapter != null;
	}

	// Unused, untested TODO
	// Copied from ResourceImpl.
	public void setTrackingModification(boolean isTrackingModification) {
		boolean oldIsTrackingModification = modificationTrackingAdapter != null;

		if (oldIsTrackingModification != isTrackingModification) {
			if (isTrackingModification) {
				modificationTrackingAdapter = new ModificationTrackingAdapter();
				for (TreeIterator<EObject> i = getAllProperContents(getContents()); i.hasNext();) {
					EObject eObject = i.next();
					eObject.eAdapters().add(modificationTrackingAdapter);
				}
			} else {
				Adapter oldModificationTrackingAdapter = modificationTrackingAdapter;
				modificationTrackingAdapter = null;
				for (TreeIterator<EObject> i = getAllProperContents(getContents()); i.hasNext();) {
					EObject eObject = i.next();
					eObject.eAdapters().remove(oldModificationTrackingAdapter);
				}
			}
		}

		if (eNotificationRequired()) {
			Notification notification = new NotificationImpl(Notification.SET, oldIsTrackingModification,
					isTrackingModification) {
				@Override
				public Object getNotifier() {
					return ViatraResourceImpl.this;
				}

				@Override
				public int getFeatureID(Class<?> expectedClass) {
					return RESOURCE__IS_TRACKING_MODIFICATION;
				}
			};
			eNotify(notification);
		}
	}

	public EObject getEObject(String uriFragment) {
		EObject result = null;
		for (TreeIterator<EObject> i = getAllProperContents(getContents()); i.hasNext();) {
			EObject eObject = i.next();
			if (uriFragment.equals(((InternalViatraObject) eObject).getRelatedViatraEntity().getFullyQualifiedName())) {
				result = eObject;
			}
		}
		return result;
	}

	public String getURIFragment(EObject object) {
		// TODO: �t kell gondolni, hogy mi van, ha nincs relatedViatraEntity
		if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
			return ((InternalViatraObject) object).getRelatedViatraEntity().getFullyQualifiedName();
		} else {
			// TODO TODO TODO
			return "/null";
		}
	}

	// Copied from ResourceImpl.
	private TreeIterator<EObject> getAllProperContents(EObject eObject) {
		return EcoreUtil.getAllProperContents(eObject, false);
	}

	// Copied from ResourceImpl.
	private TreeIterator<EObject> getAllProperContents(List<EObject> contents) {
		return new ContentTreeIterator<EObject>(contents, false) {
			private static final long serialVersionUID = 1L;

			@SuppressWarnings("unchecked")
			@Override
			public Iterator<EObject> getChildren(Object object) {
				return object == this.object ? ((List<EObject>) object).iterator()
						: new ProperContentIterator<EObject>(((EObject) object));
			}
		};
	}

	// Copied from ResourceImpl.
	private class ModificationTrackingAdapter extends AdapterImpl {
		@Override
		public void notifyChanged(Notification notification) {
			switch (notification.getEventType()) {
			case Notification.SET:
			case Notification.UNSET:
			case Notification.MOVE: {
				if (!notification.isTouch()) {
					setModified(true);
				}
				break;
			}
			case Notification.ADD:
			case Notification.REMOVE:
			case Notification.ADD_MANY:
			case Notification.REMOVE_MANY: {
				setModified(true);
				break;
			}
			}
		}
	}

	// Copied from ResourceImpl.
	private class ContentsEList<E extends Object & EObject> extends NotifyingInternalEListImpl<E> {
		private static final long serialVersionUID = 1L;

		@Override
		public Object getNotifier() {
			return ViatraResourceImpl.this;
		}

		@Override
		public int getFeatureID() {
			return RESOURCE__CONTENTS;
		}

		@Override
		protected boolean isNotificationRequired() {
			return ViatraResourceImpl.this.eNotificationRequired();
		}

		// Note: changed from false. What the heck..
		@Override
		protected boolean useEquals() {
			return true;
		}

		@Override
		protected boolean hasInverse() {
			return true;
		}

		@Override
		protected boolean isUnique() {
			return true;
		}

		@Override
		public NotificationChain inverseAdd(E object, NotificationChain notifications) {
			InternalEObject eObject = (InternalEObject) object;
			before();
			IEntity vEntity = emf2ViatraSync.getIEntityForObject(null, object);
			try {
				viatraEMFManager.vAddToResource(vEntity);
			} catch (VPMCoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			after();
			notifications = eObject.eSetResource(ViatraResourceImpl.this, notifications);
			ViatraResourceImpl.this.attached(eObject);
			return notifications;
		}

		@Override
		public NotificationChain inverseRemove(E object, NotificationChain notifications) {
			InternalEObject eObject = (InternalEObject) object;
			if (ViatraResourceImpl.this.isLoaded) {
				before();
				IEntity vEntity = emf2ViatraSync.getIEntityForObject(null, object);
				try {
					viatraEMFManager.vRemoveFromResource(vEntity);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				ViatraResourceImpl.this.detached(eObject);
				after();
			}
			return eObject.eSetResource(null, notifications);
		}

		@Override
		protected Object[] newData(int capacity) {
			return new EObject[capacity];
		}

		@Override
		protected void didAdd(int index, E object) {
			super.didAdd(index, object);
			if (index == size - 1) {
				loaded();
			}
			modified();
		}

		@Override
		protected void didRemove(int index, E object) {
			super.didRemove(index, object);
			modified();
		}

		@Override
		protected void didSet(int index, E newObject, E oldObject) {
			super.didSet(index, newObject, oldObject);
			modified();
		}

		@Override
		protected void didClear(int oldSize, Object[] oldData) {
			if (oldSize == 0) {
				loaded();
			} else {
				super.didClear(oldSize, oldData);
			}
		}

		protected void loaded() {
			if (!ViatraResourceImpl.this.isLoaded()) {
				Notification notification = ViatraResourceImpl.this.setLoaded(true);
				if (notification != null) {
					ViatraResourceImpl.this.eNotify(notification);
				}
			}
		}

		protected void modified() {
			if (isTrackingModification()) {
				setModified(true);
			}
		}
	}

	private void before() {
		ViatraListener listener = getViatraListener();
		if (listener != null) {
			listener.setLocalChange(true);
		}
	}

	private void after() {
		ViatraListener listener = getViatraListener();
		if (listener != null) {
			listener.setLocalChange(false);
		}
	}

}
