package hu.optxware.emf2viatra.core.eresource.impl;

import hu.optxware.emf2viatra.core.eresource.InternalViatraObject;
import hu.optxware.emf2viatra.core.eresource.ViatraResource;
import hu.optxware.emf2viatra.core.listener.ViatraListener;
import hu.optxware.emf2viatra.core.managers.EMF2ViatraSynchronizer;
import hu.optxware.emf2viatra.core.managers.Viatra2EMFSynchronizer;
import hu.optxware.emf2viatra.core.managers.ViatraEMFFrameworkManager;
import hu.optxware.emf2viatra.core.managers.ViatraEMFManagersFactory;

import java.util.ArrayList;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.InternalEObject.EStore;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.framework.IFramework;

public class ViatraStoreImpl implements EStore {

	// TODO: Even though ptintDebug is set to false, it can be a little runtime
	// overhead because of call of the log method (call a toString() method of
	// some objects). So we need to eliminate all the log at the end
	private final boolean printDebug = false;

	// private EMFCoreMetamodelManager emfCoreManager;
	private Viatra2EMFSynchronizer viatra2EMFSync;
	private EMF2ViatraSynchronizer emf2ViatraSync;
	private ViatraEMFFrameworkManager viatraEMFManager;

	// private IFramework fw;

	public ViatraStoreImpl() {
	}

	public void initManager(IFramework fw) {
		// this.fw = fw;
		viatra2EMFSync = ViatraEMFManagersFactory.getInstance().getViatra2EMFSynchronizer(fw);
		emf2ViatraSync = ViatraEMFManagersFactory.getInstance().getEMF2ViatraSynchronizer(fw);
		viatraEMFManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(fw);
	}

	private void before(InternalEObject object) {
		Resource resource = object.eResource();
		if (resource instanceof ViatraResource) {
			if (resource != null) {
				ViatraListener listener = ((ViatraResource) resource).getViatraListener();
				if (listener != null) {
					listener.setLocalChange(true);
				}
			}
		}
	}

	private void after(InternalEObject object) {
		Resource resource = object.eResource();
		if (resource instanceof ViatraResource) {
			if (resource != null) {
				ViatraListener listener = ((ViatraResource) resource).getViatraListener();
				if (listener != null) {
					listener.setLocalChange(false);
				}
			}
		}
	}

	private void log(String x) {
		if (printDebug) {
			System.out.println(x);
		}
	}

	public void add(InternalEObject object, EStructuralFeature feature, int index, Object value) {
		log("add");
		log("	Object: " + object + ", feature: " + feature + ", index: " + index + ", value: " + value);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity entityValue = emf2ViatraSync.getIEntityForObject(feature, value);

				try {
					viatraEMFManager.vAdd(viatraInstance, viatraRelationType, index, entityValue);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				after(object);
				return;
			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return;
			}
		}
	}

	public void clear(InternalEObject object, EStructuralFeature feature) {
		log("Clear: ");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				try {
					viatraEMFManager.vClear(viatraInstance, viatraRelationType);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				after(object);
				return;
			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return;
			}
		}
	}

	public boolean contains(InternalEObject object, EStructuralFeature feature, Object value) {
		log("Contains");
		log("	Object: " + object + ", feature: " + feature + ", value: " + value);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity entityValue = null;
				if (value instanceof InternalViatraObject) {
					entityValue = ((InternalViatraObject) value).getRelatedViatraEntity();
				}

				if (entityValue != null) {
					after(object);
					return viatraEMFManager.vContains(viatraInstance, viatraRelationType, entityValue);
				}

			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return false;
			}
		}
		after(object);
		return false;
	}

	public EObject create(EClass class1) {
		log("create");
		log("	EClass: " + class1);

		return null;
	}

	public Object get(InternalEObject object, EStructuralFeature feature, int index) {
		log("get");
		log("	Object: " + object + ", feature: " + feature + ", index: " + index);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity relatedViatraEntity = viatraEMFManager.vGet(viatraInstance, viatraRelationType, index);

				Object result = viatra2EMFSync.getObjectForIEntity(relatedViatraEntity);
				log("	get: " + result);
				after(object);
				return result;
			} else {
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}
	}

	public InternalEObject getContainer(InternalEObject object) {
		log("getContainer");
		log("	Object: " + object);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IEntity relatedViatraEntity = viatraEMFManager.vGetContainer(viatraInstance);

				InternalViatraObject container = (InternalViatraObject) viatra2EMFSync
						.getObjectForIEntity(relatedViatraEntity);

				log("	Container: " + container);
				after(object);
				return container;

			} else {
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}
	}

	public EStructuralFeature getContainingFeature(InternalEObject object) {
		log("getContainingFeature");
		log("	Object: " + object);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation containingRelationType = viatraEMFManager.vGetContainingRelationType(viatraInstance);

				EStructuralFeature containingFeature = viatra2EMFSync
						.getEStructuralFeatureForIRelation(containingRelationType);

				log("	Container: " + containingFeature);
				after(object);
				return containingFeature;

			} else {
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}

	}

	public int hashCode(InternalEObject object, EStructuralFeature feature) {
		log("hashCode");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		after(object);
		return 0;
	}

	public int indexOf(InternalEObject object, EStructuralFeature feature, Object value) {
		log("indexOf");
		log("	Object: " + object + ", feature: " + feature + ", value: " + value);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity entityValue = null;
				if (value instanceof InternalViatraObject) {
					entityValue = ((InternalViatraObject) value).getRelatedViatraEntity();
				} else {
					// TODO TODO TODO TODO
					// Mi van, ha egy elemi java object-tel van dolgunk?
				}

				int result = 0;
				if (entityValue != null) {
					result = viatraEMFManager.vIndexOf(viatraInstance, viatraRelationType, entityValue);
				}
				log("	IndexOf: " + result);
				after(object);
				return result;

			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return 0;
			}
		}
		after(object);
		return 0;
	}

	public boolean isEmpty(InternalEObject object, EStructuralFeature feature) {
		log("isEmpty");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				boolean result = viatraEMFManager.vIsEmpty(viatraInstance, viatraRelationType);

				log("	isEmpty: " + result);
				after(object);
				return result;

			} else {
				after(object);
				return true;
			}
		} else {
			after(object);
			return true;
		}
	}

	public boolean isSet(InternalEObject object, EStructuralFeature feature) {
		log("isSet");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				boolean result = viatraEMFManager.vIsSet(viatraInstance, viatraRelationType);

				log("	isSet: " + result);
				after(object);
				return result;

			} else {
				after(object);
				return false;
			}
		} else {
			after(object);
			return false;
		}
	}

	public int lastIndexOf(InternalEObject object, EStructuralFeature feature, Object value) {
		log("LastIndexOf");
		log("	Object: " + object + ", feature: " + feature + ", value: " + value);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity entityValue = null;
				if (value instanceof InternalViatraObject) {
					entityValue = ((InternalViatraObject) value).getRelatedViatraEntity();
				} else {
					// TODO TODO TODO TODO
					// Mi van, ha egy elemi java object-tel van dolgunk?
				}

				int result = 0;
				if (entityValue != null) {
					result = viatraEMFManager.vLastIndexOf(viatraInstance, viatraRelationType, entityValue);
				}

				log("	LastIndexOf: " + result);
				after(object);
				return result;

			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return 0;
			}
		}
		after(object);
		return 0;
	}

	public Object move(InternalEObject object, EStructuralFeature feature, int targetIndex, int sourceIndex) {
		log("move");
		log("	Object: " + object + ", feature: " + feature + ", targetIndex: " + targetIndex + ", sourceIndex: "
				+ sourceIndex);
		before(object);

		after(object);
		return null;
	}

	public Object remove(InternalEObject object, EStructuralFeature feature, int index) {
		log("remove");
		log("	Object: " + object + ", feature: " + feature + ", index: " + index);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity relatedViatraEntity = viatraEMFManager.vGet(viatraInstance, viatraRelationType, index);
				Object deletedObj = viatra2EMFSync.getObjectForIEntity(relatedViatraEntity);

				try {
					viatraEMFManager.vRemove(viatraInstance, viatraRelationType, index);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				// if (deletedObj instanceof InternalViatraObject) {
				// ((InternalViatraObject)
				// deletedObj).setRelatedViatraEntity(null);
				// }

				after(object);
				return deletedObj;
			} else {
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}

	}

	public Object set(InternalEObject object, EStructuralFeature feature, int index, Object value) {
		log("Set");
		log("	Object: " + object + ", feature: " + feature + ", index: " + index + ", value: " + value);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				IEntity relatedViatraEntity = viatraEMFManager.vGet(viatraInstance, viatraRelationType, index);

				Object oldValue = viatra2EMFSync.getObjectForIEntity(relatedViatraEntity);

				IEntity entityValue = emf2ViatraSync.getIEntityForObject(feature, value);

				try {
					viatraEMFManager.vSet(viatraInstance, viatraRelationType, index, entityValue);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				after(object);
				return oldValue;
			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}
	}

	public int size(InternalEObject object, EStructuralFeature feature) {
		log("size");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				int result = viatraEMFManager.vGetSize(viatraInstance, viatraRelationType);
				log("	Size: " + result);

				after(object);
				return result;

			} else {
				after(object);
				return 0;
			}
		} else {
			after(object);
			return 0;
		}

	}

	public Object[] toArray(InternalEObject object, EStructuralFeature feature) {
		log("toArray01");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);

				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				ArrayList<IEntity> relatedViatraEntites = viatraEMFManager.vGetAll(viatraInstance, viatraRelationType);

				ArrayList<Object> eObjects = new ArrayList<Object>();

				for (IEntity o : relatedViatraEntites) {
					eObjects.add(viatra2EMFSync.getObjectForIEntity(o));
				}

				after(object);
				return eObjects.toArray();

			} else {
				after(object);
				return null;
			}
		} else {
			after(object);
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T[] toArray(InternalEObject object, EStructuralFeature feature, T[] a) {
		log("toArray T[");
		log("	Object: " + object + ", feature: " + feature + ", array: " + a);
		before(object);

		Object[] array = toArray(object, feature);
		int size = array.length;

		if (a.length < size) {
			a = (T[]) java.lang.reflect.Array.newInstance(a.getClass().getComponentType(), size);
		}

		System.arraycopy(array, 0, a, 0, size);
		if (a.length > size) {
			a[size] = null;
		}

		after(object);
		return a;
	}

	public void unset(InternalEObject object, EStructuralFeature feature) {
		log("unset");
		log("	Object: " + object + ", feature: " + feature);
		before(object);

		if (object instanceof InternalViatraObject) {
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				IEntity viatraInstance = ((InternalViatraObject) object).getRelatedViatraEntity();
				initManager(viatraInstance.getModelSpace().getFramework());

				IRelation viatraRelationType = emf2ViatraSync.getIRelationForEStructuralFeature(feature);
				if (viatraRelationType == null) {
					// TODO: Handle null for viatraRelationType!!!
				}

				try {
					viatraEMFManager.vUnset(viatraInstance, viatraRelationType);
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				after(object);
				return;
			} else {
				// TODO: Ha nincs Viatr�ba perziszt�lva, de �llitani akarjuk
				after(object);
				return;
			}
		}
	}

}
