package hu.optxware.emf2viatra.core.importer;

import hu.optxware.emf2viatra.core.managers.EMF2ViatraSynchronizer;
import hu.optxware.emf2viatra.core.managers.EMFCoreEntConst;
import hu.optxware.emf2viatra.core.managers.EMFCoreMetamodelManager;
import hu.optxware.emf2viatra.core.managers.EMFCoreRelConst;
import hu.optxware.emf2viatra.core.managers.ViatraEMFFrameworkManager;
import hu.optxware.emf2viatra.core.managers.ViatraEMFManagersFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;

public class EcoreImporter implements NativeImporter {

	private EMFCoreMetamodelManager emfCoreManager;
	private ViatraEMFFrameworkManager viatraEMFFrameworkManager = null;
	private EMF2ViatraSynchronizer emf2ViatraSync;

	private List<EReference> eReferenceList = new ArrayList<EReference>();
	private List<EAttribute> eAttributeList = new ArrayList<EAttribute>();

	List<EClass> allClasses;
	ResourceSet resSet;

	public void process(InputStream f, IFramework fw) throws VPMRuntimeException {
		System.out.println("InputStream not supported.");
	}

	private List<Resource> processed = new ArrayList<Resource>();

	public void processFile(String f, IFramework fw) throws VPMRuntimeException {
		emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(fw);
		emf2ViatraSync = ViatraEMFManagersFactory.getInstance().getEMF2ViatraSynchronizer(fw);
		viatraEMFFrameworkManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(fw);
		;

		resSet = new ResourceSetImpl();
		Resource res = resSet.createResource(URI.createFileURI(f));
		try {
			res.load(null);
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			boolean processing = true;
			allClasses = new ArrayList<EClass>();

			while (processing) {
				processing = false;
				List<Resource> tmp = new ArrayList<Resource>();
				tmp.addAll(resSet.getResources());
				for (Resource r : tmp) {
					if (!processed.contains(r)) {
						processPackage((EPackage) r.getContents().get(0), emfCoreManager
								.getIEntityForEcoreElement(EMFCoreEntConst.PACKAGES));
						processing = true;
						processed.add(r);
					}
				}
			}

			for (EClass cls : allClasses) {
				processSuperTypes(cls);
			}

			oppositePairs.clear();
			for (EAttribute eAttribute : eAttributeList) {
				processAttribute(eAttribute);
			}

			for (EReference eReference : eReferenceList) {
				processReference(eReference);
			}
			processOppositePairs();

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private void processPackage(EPackage ePackage) throws VPMCoreException {
		processPackage(ePackage, emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.PACKAGES));

		oppositePairs.clear();
		for (EAttribute eAttribute : eAttributeList) {
			processAttribute(eAttribute);
		}

		for (EReference eReference : eReferenceList) {
			processReference(eReference);
		}
		processOppositePairs();
	}

	private void processPackage(EPackage ePackage, IEntity parent) throws VPMCoreException {
		IEntity viatraInstanceEPackage = viatraEMFFrameworkManager.newEntity(parent, ePackage.getName());
		viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
				.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EPACKAGE), viatraInstanceEPackage);
		setStringAttribute(viatraInstanceEPackage, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME), ePackage.getName());
		setStringAttribute(viatraInstanceEPackage, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EPACKAGE_NSURI), ePackage.getNsURI());
		setStringAttribute(viatraInstanceEPackage, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EPACKAGE_NSPREFIX), ePackage.getNsPrefix());

		for (EClassifier cls : ePackage.getEClassifiers()) {
			if (cls instanceof EDataType && !(cls instanceof EEnum)) {
				processEDataType(viatraInstanceEPackage, (EDataType) cls);
			}
		}
		for (EClassifier cls : ePackage.getEClassifiers()) {
			if (cls instanceof EEnum) {
				processEEnum(viatraInstanceEPackage, (EEnum) cls);
			}
		}
		for (EClassifier cls : ePackage.getEClassifiers()) {
			if (cls instanceof EClass) {
				processClass(viatraInstanceEPackage, (EClass) cls);
				allClasses.add((EClass) cls);
			}
		}

		/*
		 * for (EClassifier cls : ePackage.getEClassifiers()) { if (cls
		 * instanceof EClass) { processSuperTypes(viatraInstanceEPackage,
		 * (EClass) cls); } }
		 */

		for (EPackage pack : ePackage.getESubpackages()) {
			processPackage(pack, viatraInstanceEPackage);
		}

	}

	private void processSuperTypes(EClass eClass) {
		if (eClass != null) {

			IEntity viatraInstanceSourceEClass = emf2ViatraSync.getIEntityForEClass(eClass);
			for (EClass supClass : eClass.getESuperTypes()) {
				IEntity viatraInstanceSuperEClass = emf2ViatraSync.getIEntityForEClass(supClass);
				viatraEMFFrameworkManager.newSupertypeOf(viatraInstanceSuperEClass, viatraInstanceSourceEClass);
			}
		}
	}

	private void processClass(IEntity viatraInstancePackage, EClass eClass) throws VPMCoreException {
		if (viatraInstancePackage != null && eClass != null) {
			IEntity viatraInstanceEClass = viatraEMFFrameworkManager.newEntity(viatraInstancePackage, eClass.getName());
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASS), viatraInstanceEClass);

			viatraEMFFrameworkManager.newSupertypeOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EOBJECT), viatraInstanceEClass);

			setStringAttribute(viatraInstanceEClass, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME), eClass.getName());
			for (EAttribute eAttribute : eClass.getEAttributes()) {
				processAttribute(eAttribute, viatraInstanceEClass);
			}
			for (EReference eReference : eClass.getEReferences()) {
				EClassifier cl = eReference.getEType(); // Needed to resolve
				// eProxies
				eReferenceList.add(eReference);
			}
		}
	}

	private void processAttribute(EAttribute eAttribute, IEntity container) throws VPMCoreException {
		if (eAttribute != null && container != null) {
			if (emf2ViatraSync.getIEntityForEDataType(eAttribute.getEAttributeType()) != null) {
				IRelation viatraInstanceEAttribute = viatraEMFFrameworkManager.newRelation(container, emf2ViatraSync
						.getIEntityForEDataType(eAttribute.getEAttributeType()), eAttribute.getName());
				viatraEMFFrameworkManager
						.newInstanceOf(emfCoreManager
								.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE),
								viatraInstanceEAttribute);

				if (eAttribute.isMany()) {
					viatraEMFFrameworkManager.newSupertypeOf(emfCoreManager
							.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION),
							viatraInstanceEAttribute);
				}

				setStringAttribute(container, viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_NAME), eAttribute
						.getName());
				setBooleanAttribute(viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_CHANGEABLE),
						eAttribute.isChangeable());
				setBooleanAttribute(viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_TRANSIENT),
						eAttribute.isTransient());
				setBooleanAttribute(viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_UNSETTABLE),
						eAttribute.isUnsettable());
				setBooleanAttribute(viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_VOLATILE),
						eAttribute.isVolatile());
				setBooleanAttribute(viatraInstanceEAttribute, emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_MANY), eAttribute
						.isMany());

			} else {
				eAttributeList.add(eAttribute);
			}
		}
	}

	private void processReference(EReference eReference) throws VPMCoreException {
		if (eReference != null) {
			IEntity viatraInstanceSourceEClass = emf2ViatraSync.getIEntityForEClass(eReference.getEContainingClass());
			IRelation viatraInstanceEReference = viatraEMFFrameworkManager.newRelation(viatraInstanceSourceEClass,
					emf2ViatraSync.getIEntityForEClass(eReference.getEReferenceType()), eReference.getName());
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE), viatraInstanceEReference);

			if (eReference.isMany()) {
				viatraEMFFrameworkManager.newSupertypeOf(emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION),
						viatraInstanceEReference);
			}

			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_CONTAINER), eReference
					.isContainer());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_CONTAINMENT), eReference
					.isContainment());
			setStringAttribute(viatraInstanceSourceEClass, viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_NAME), eReference
					.getName());

			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_CHANGEABLE),
					eReference.isChangeable());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_TRANSIENT), eReference
					.isTransient());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_UNSETTABLE),
					eReference.isUnsettable());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_VOLATILE), eReference
					.isVolatile());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_MANY), eReference
					.isMany());

			// Manage the hashmap used for setting the opposite
			EReference eoppositeRefType = eReference.getEOpposite();
			if (eoppositeRefType != null && !oppositePairs.containsKey(eoppositeRefType)) {
				oppositePairs.put(eReference, eoppositeRefType);
			}
		}
	}

	private void processAttribute(EAttribute eAttribute) throws VPMCoreException {
		if (eAttribute != null) {
			IEntity viatraInstanceSourceEClass = emf2ViatraSync.getIEntityForEClass(eAttribute.getEContainingClass());
			IRelation viatraInstanceEReference = viatraEMFFrameworkManager.newRelation(viatraInstanceSourceEClass,
					emf2ViatraSync.getIEntityForEDataType(eAttribute.getEAttributeType()), eAttribute.getName());
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE), viatraInstanceEReference);

			if (eAttribute.isMany()) {
				viatraEMFFrameworkManager.newSupertypeOf(emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION),
						viatraInstanceEReference);
			}

			setStringAttribute(viatraInstanceSourceEClass, viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_NAME), eAttribute
					.getName());

			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_CHANGEABLE),
					eAttribute.isChangeable());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_TRANSIENT), eAttribute
					.isTransient());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_UNSETTABLE),
					eAttribute.isUnsettable());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_VOLATILE), eAttribute
					.isVolatile());
			setBooleanAttribute(viatraInstanceEReference, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_MANY), eAttribute
					.isMany());

		}
	}

	private void processOppositePairs() throws VPMCoreException {
		for (EReference ref1 : oppositePairs.keySet()) {
			EReference ref2 = ref1.getEOpposite();
			IRelation viatraInstanceRef1 = emf2ViatraSync.getIRelationForEReferenceElement(ref1);
			IRelation viatraInstanceRef2 = emf2ViatraSync.getIRelationForEReferenceElement(ref2);
			IRelation viatraInstanceNew1To2 = viatraEMFFrameworkManager.newRelation(viatraInstanceRef1,
					viatraInstanceRef2);
			IRelation viatraInstanceNew2To1 = viatraEMFFrameworkManager.newRelation(viatraInstanceRef2,
					viatraInstanceRef1);
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_EOPPOSITE),
					viatraInstanceNew1To2);
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_EOPPOSITE),
					viatraInstanceNew2To1);
		}
	}

	private HashMap<EReference, EReference> oppositePairs = new HashMap<EReference, EReference>();

	private void processEEnum(IEntity viatraInstanceEPackage, EEnum eEnum) throws VPMCoreException {
		if (eEnum != null) {
			IEntity viatraInstanceEEnum = viatraEMFFrameworkManager.newEntity(viatraInstanceEPackage, eEnum.getName());
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM), viatraInstanceEEnum);
			setStringAttribute(viatraInstanceEEnum, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME), eEnum.getName());
			for (EEnumLiteral eEnumLiteral : eEnum.getELiterals()) {
				IEntity viatraInstanceEEnumLiteral = viatraEMFFrameworkManager.newEntity(viatraInstanceEEnum,
						eEnumLiteral.getName(), eEnumLiteral.getName());
				viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
						.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL),
						viatraInstanceEEnumLiteral);
				viatraEMFFrameworkManager.newInstanceOf(viatraInstanceEEnum, viatraInstanceEEnumLiteral);
			}
		}
	}

	private void processEDataType(IEntity viatraInstanceEPackage, EDataType eData) throws VPMCoreException {
		if (eData != null) {
			IEntity viatraInstanceEDataType = viatraEMFFrameworkManager.newEntity(viatraInstanceEPackage, eData.getName());
			viatraEMFFrameworkManager.newInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE), viatraInstanceEDataType);
			setStringAttribute(viatraInstanceEDataType, emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME), eData.getName());
		}
	}

	private void setStringAttribute(IEntity sourceContainer, IRelation relationType, String s) throws VPMCoreException {
		setStringAttribute(sourceContainer, sourceContainer, relationType, s);
	}

	private void setStringAttribute(IEntity container, IModelElement source, IRelation relationType, String s)
			throws VPMCoreException {
		if (container != null && source != null && relationType != null && s != null) {
			IEntity attribute = viatraEMFFrameworkManager.newEntity(container, null, s);
			viatraEMFFrameworkManager.newInstanceOf(viatraEMFFrameworkManager.getEntityByName("datatypes.String"),
					attribute);
			IRelation atrel = viatraEMFFrameworkManager.newRelation(source, attribute);
			viatraEMFFrameworkManager.newInstanceOf(relationType, atrel);
		}
	}

	private void setBooleanAttribute(IModelElement sourceContainer, IRelation relationType, Boolean b)
			throws VPMCoreException {
		if (sourceContainer != null && relationType != null && b != null) {
			IRelation atrel;
			if (b)
				atrel = viatraEMFFrameworkManager.newRelation(sourceContainer, viatraEMFFrameworkManager
						.getEntityByName("datatypes.Boolean.true"));
			else
				atrel = viatraEMFFrameworkManager.newRelation(sourceContainer, viatraEMFFrameworkManager
						.getEntityByName("datatypes.Boolean.false"));
			viatraEMFFrameworkManager.newInstanceOf(relationType, atrel);
		}
	}

}
