package hu.optxware.emf2viatra.core.listener;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;

public class ManyStructuralFeatureSet {

	private IEntity mainEntityInstance;

	private IRelation relationType;

	/**
	 * List to store the order of the relations (i.e. in the form of
	 * {@link ManyStructuralFeature}).
	 */
	private ArrayList<ManyStructuralFeature> manyStructuralFeatures = null;

	private HashMap<IRelation, ManyStructuralFeature> relationInstance2ManyStrFeatures = null;

	/**
	 * During the creation of this {@link ManyStructuralFeatureSet} these
	 * elements will be explicitly added to the feature list. Moreover they will
	 * be exactly the same as they added. Note only the index attribute can be
	 * changed
	 */
	private HashMap<IRelation, ManyStructuralFeature> explicitlyAddedStrFeatures = null;

	public ManyStructuralFeatureSet(IEntity mainEntityInstance, IRelation relationType, IRelation excludingRelInstance) {
		this(mainEntityInstance, relationType, excludingRelInstance, null);
	}

	public ManyStructuralFeatureSet(IEntity mainEntityInstance, IRelation relationType,
			HashMap<IRelation, ManyStructuralFeature> explicitlyAddedStrFeatures) {
		this(mainEntityInstance, relationType, null, explicitlyAddedStrFeatures);
	}

	public ManyStructuralFeatureSet(IEntity mainEntityInstance, IRelation relationType) {
		this(mainEntityInstance, relationType, null, null);
	}

	private ManyStructuralFeatureSet(IEntity mainEntityInstance, IRelation relationType,
			IRelation excludingRelInstance, HashMap<IRelation, ManyStructuralFeature> explicitlyAddedStrFeatures) {
		this.mainEntityInstance = mainEntityInstance;
		this.relationType = relationType;

		this.explicitlyAddedStrFeatures = explicitlyAddedStrFeatures;

		if (getMainEntityInstance() != null && getRelationType() != null) {

			manyStructuralFeatures = new ArrayList<ManyStructuralFeature>();
			relationInstance2ManyStrFeatures = new HashMap<IRelation, ManyStructuralFeature>();

			LinkedList<IRelation> orderedRelationList = new LinkedList<IRelation>();

			Collection<IRelation> typeConformRelations2 = mainEntityInstance.getAllRelationFromByType(relationType);
			if (excludingRelInstance != null && typeConformRelations2.contains(excludingRelInstance)) {
				typeConformRelations2.remove(excludingRelInstance);
			}
			if (this.explicitlyAddedStrFeatures != null) {
				for (IRelation rel : this.explicitlyAddedStrFeatures.keySet()) {
					if (!typeConformRelations2.contains(rel)) {
						typeConformRelations2.add(rel);
					}
				}
			}

			for (IModelElement element : typeConformRelations2) {
				IRelation rel = (IRelation) element;

				ManyStructuralFeature feature = null;
				if (this.explicitlyAddedStrFeatures != null && this.explicitlyAddedStrFeatures.keySet().contains(rel)) {
					feature = this.explicitlyAddedStrFeatures.get(rel);
				} else {
					feature = new ManyStructuralFeature(rel);
				}

				if (feature != null && feature.isValid()) {
					relationInstance2ManyStrFeatures.put(rel, feature);
				}
			}

			int allRelSize = relationInstance2ManyStrFeatures.size();
			// Needs to prevent "next circle"
			int i = 0;
			// Get an arbitrary element from the list
			ManyStructuralFeature chosenElement = null;
			if (allRelSize > 0) {
				chosenElement = relationInstance2ManyStrFeatures.get(relationInstance2ManyStrFeatures.keySet()
						.toArray()[0]);
			}

			if (chosenElement != null) {
				orderedRelationList.clear();
				orderedRelationList.add(chosenElement.getRelationInstance());
				i++;

				ManyStructuralFeature nextElement = relationInstance2ManyStrFeatures.get(chosenElement
						.getNextRelationInstance());
				while (nextElement != null && i <= allRelSize) {
					orderedRelationList.addLast(nextElement.getRelationInstance());
					i++;
					nextElement = relationInstance2ManyStrFeatures.get(nextElement.getNextRelationInstance());
				}

				ManyStructuralFeature prevElement = relationInstance2ManyStrFeatures.get(chosenElement
						.getPrevRelationInstance());
				while (prevElement != null && i <= allRelSize) {
					orderedRelationList.addFirst(prevElement.getRelationInstance());
					i++;
					prevElement = relationInstance2ManyStrFeatures.get(prevElement.getPrevRelationInstance());
				}
			}

			if (orderedRelationList.size() == relationInstance2ManyStrFeatures.size()) {
				for (int index = 0; index < orderedRelationList.size(); index++) {
					IRelation relation = orderedRelationList.get(index);
					ManyStructuralFeature feature = relationInstance2ManyStrFeatures.get(relation);
					// Only here we know the index, so we have to set it
					feature.setIndex(index);
					manyStructuralFeatures.add(relationInstance2ManyStrFeatures.get(relation));
				}
			} else {
				manyStructuralFeatures.clear();
				relationInstance2ManyStrFeatures.clear();
			}
		}
	}

	public IEntity getMainEntityInstance() {
		return mainEntityInstance;
	}

	public IRelation getRelationType() {
		return relationType;
	}

	public List<ManyStructuralFeature> getStructuralFeatures() {
		return manyStructuralFeatures;
	}

	public ManyStructuralFeature getStructuralFeature(IRelation relationInstance) {
		if (relationInstance2ManyStrFeatures != null) {
			return relationInstance2ManyStrFeatures.get(relationInstance);
		}
		return null;
	}

	public Set<IRelation> getRelationInstances() {
		if (relationInstance2ManyStrFeatures != null) {
			return relationInstance2ManyStrFeatures.keySet();
		}
		return null;
	}

}
