package hu.optxware.emf2viatra.core.listener;

import java.util.ArrayList;
import java.util.Collection;

import hu.optxware.emf2viatra.core.managers.EMFCoreEntConst;
import hu.optxware.emf2viatra.core.managers.EMFCoreMetamodelManager;
import hu.optxware.emf2viatra.core.managers.ViatraEMFManagersFactory;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;

public class SingleStructuralFeature {

	private IRelation relationInstance;

	private IRelation relationType;

	private IEntity mainType;

	private IEntity mainInstance;

	private IEntity valueType;

	private IEntity valueInstance;

	public SingleStructuralFeature(IRelation relationInstance) {
		this.relationInstance = relationInstance;
		initRelationType();

		if (relationInstance.getFrom() instanceof IEntity) {
			mainInstance = (IEntity) relationInstance.getFrom();
			initMainType();
		}

		if (relationInstance.getTo() instanceof IEntity) {
			valueInstance = (IEntity) relationInstance.getTo();
			initValueType();
		}
	}

	public boolean isValid() {
		if (mainInstance == null || relationInstance == null || valueInstance == null || mainType == null
				|| relationType == null || valueType == null) {
			return false;
		}

		boolean fromTypeIsOK = relationType.getFrom().equals(mainType)
				|| (relationType.getFrom()).isSubtypeOf(mainType);
		boolean toTypeIsOK = relationType.getTo().equals(valueType) || relationType.getTo().isSubtypeOf(valueType);

		if (fromTypeIsOK && toTypeIsOK) {
			return true;
		} else {
			return false;
		}
	}

	public IRelation getRelationInstance() {
		return relationInstance;
	}

	public IRelation getRelationType() {
		return relationType;
	}

	public IEntity getMainType() {
		return mainType;
	}

	public IEntity getMainInstance() {
		return mainInstance;
	}

	public IEntity getValueType() {
		return valueType;
	}

	public IEntity getValueInstance() {
		return valueInstance;
	}

	public void setRelationInstance(IRelation relationInstance) {
		this.relationInstance = relationInstance;
		initRelationType();
	}

	public void setRelationType(IRelation relationType) {
		this.relationType = relationType;
	}

	public void setMainInstance(IEntity mainInstance) {
		this.mainInstance = mainInstance;
		initMainType();
	}
	
	public void setMainType(IEntity mainType) {
		this.mainType = mainType;
	}

	public void setValueInstance(IEntity valueInstance) {
		this.valueInstance = valueInstance;
		initValueType();
	}
	
	public void setValueType(IEntity valueType) {
		this.valueType = valueType;
	}


	private void initValueType() {
		EMFCoreMetamodelManager emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(
				relationInstance.getModelSpace().getFramework());
		IEntity eEnumliteralEntity = emfCoreManager
				.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL);

		if (valueInstance.getTypes().size() == 1) {
			setValueType((IEntity) valueInstance.getTypes().toArray()[0]);
		}
		// If value is Enum, we have to deal with the two types
		else if (valueInstance.getTypes().size() == 2 && valueInstance.getTypes().contains(eEnumliteralEntity)) {
			Collection<IModelElement> types = new ArrayList<IModelElement>();
			types.addAll(valueInstance.getTypes());
			types.remove(eEnumliteralEntity);
			setValueType((IEntity) types.toArray()[0]);
		}

	}

	private void initRelationType() {
		if (relationInstance.getTypes().size() == 1) {
			setRelationType((IRelation) relationInstance.getTypes().toArray()[0]);
		}
	}

	private void initMainType() {
		if (mainInstance.getTypes().size() == 1) {
			setMainType((IEntity) mainInstance.getTypes().toArray()[0]);
		}
	}

}
