package hu.optxware.emf2viatra.core.listener;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;

public class SingleStructuralFeatureSet {

	private IEntity mainEntityInstance;

	private IRelation relationType;

	private List<SingleStructuralFeature> singleStructuralFeatures = null;

	public SingleStructuralFeatureSet(IEntity mainEntityInstance, IRelation relationType) {
		this.mainEntityInstance = mainEntityInstance;
		this.relationType = relationType;

		singleStructuralFeatures = new ArrayList<SingleStructuralFeature>();
		singleStructuralFeatures.clear();
		for (IModelElement relationInstance : mainEntityInstance.getAllRelationFromByType(relationType)) {
			singleStructuralFeatures.add(new SingleStructuralFeature((IRelation) relationInstance));
		}
	}

	public boolean isValid() {
		SingleStructuralFeature result = getValid();
		if (result != null) {
			return true;
		} else {
			return false;
		}
	}

	public SingleStructuralFeature getValid() {
		int correctStructuralFeatureNumber = 0;

		SingleStructuralFeature validResult = null;

		for (SingleStructuralFeature structuralFeature : singleStructuralFeatures) {
			if (structuralFeature.isValid()) {
				correctStructuralFeatureNumber++;
				validResult = structuralFeature;
			}
		}

		if (correctStructuralFeatureNumber == 1) {
			return validResult;
		} else {
			return null;
		}
	}

	public boolean isValidAllButOne(SingleStructuralFeature exceptional) {
		SingleStructuralFeature result = getValidAllButOne(exceptional);
		if (result != null) {
			return true;
		} else {
			return false;
		}
	}

	public SingleStructuralFeature getValidAllButOne(SingleStructuralFeature exceptional) {
		int correctStructuralFeatureNumber = 0;

		SingleStructuralFeature validResult = null;

		for (SingleStructuralFeature structuralFeature : singleStructuralFeatures) {
			if (!exceptional.getRelationInstance().equals(structuralFeature.getRelationInstance())
					&& structuralFeature.isValid()) {
				correctStructuralFeatureNumber++;
				validResult = structuralFeature;
			}
		}

		if (correctStructuralFeatureNumber == 1) {
			return validResult;
		} else {
			return null;
		}
	}

	public boolean isValidAllAndOne(SingleStructuralFeature additional) {
		SingleStructuralFeature result = getValidAllAndOne(additional);
		if (result != null) {
			return true;
		} else {
			return false;
		}
	}

	public SingleStructuralFeature getValidAllAndOne(SingleStructuralFeature additional) {
		int correctStructuralFeatureNumber = 0;

		SingleStructuralFeature validResult = null;

		for (SingleStructuralFeature structuralFeature : singleStructuralFeatures) {
			if (structuralFeature.isValid()) {
				correctStructuralFeatureNumber++;
				validResult = structuralFeature;
			}
		}

		if (additional.isValid()) {
			correctStructuralFeatureNumber++;
			validResult = additional;
		}

		if (correctStructuralFeatureNumber == 1) {
			return validResult;
		} else {
			return null;
		}
	}

	public boolean isValidAllButOneAndOne(SingleStructuralFeature exceptional, SingleStructuralFeature additional) {
		SingleStructuralFeature result = getValidAllButOneAndOne(exceptional, additional);
		if (result != null) {
			return true;
		} else {
			return false;
		}
	}

	public SingleStructuralFeature getValidAllButOneAndOne(SingleStructuralFeature exceptional,
			SingleStructuralFeature additional) {
		int correctStructuralFeatureNumber = 0;

		SingleStructuralFeature validResult = null;

		for (SingleStructuralFeature structuralFeature : singleStructuralFeatures) {
			if (!exceptional.getRelationInstance().equals(structuralFeature.getRelationInstance())
					&& structuralFeature.isValid()) {
				correctStructuralFeatureNumber++;
				validResult = structuralFeature;
			}
		}

		if (additional.isValid()) {
			correctStructuralFeatureNumber++;
			validResult = additional;
		}

		if (correctStructuralFeatureNumber == 1) {
			return validResult;
		} else {
			return null;
		}
	}

	public IEntity getMainEntityInstance() {
		return mainEntityInstance;
	}

	public IRelation getRelationType() {
		return relationType;
	}

	public List<SingleStructuralFeature> getStructuralFeatures() {
		return singleStructuralFeatures;
	}
}
