package hu.optxware.emf2viatra.core.listener;

import hu.optxware.emf2viatra.core.eresource.InternalViatraObject;
import hu.optxware.emf2viatra.core.eresource.ViatraResource;
import hu.optxware.emf2viatra.core.managers.EMFCoreEntConst;
import hu.optxware.emf2viatra.core.managers.EMFCoreMetamodelManager;
import hu.optxware.emf2viatra.core.managers.EMFCoreRelConst;
import hu.optxware.emf2viatra.core.managers.Viatra2EMFSynchronizer;
import hu.optxware.emf2viatra.core.managers.ViatraEMFFrameworkManager;
import hu.optxware.emf2viatra.core.managers.ViatraEMFManagersFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.transaction.RollbackException;
import org.eclipse.emf.transaction.Transaction;
import org.eclipse.emf.transaction.impl.InternalTransactionalEditingDomain;
import org.eclipse.viatra2.core.ICoreNotificationListener;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.core.notification.ICoreNotificationObject;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectCreateInstanceOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectDeleteInstanceOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetRelationFrom;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetRelationTo;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetValue;
import org.eclipse.viatra2.framework.IFramework;

public class ViatraListener implements ICoreNotificationListener {

	private final boolean printDebug = false;

	private ViatraResource resource;

	private Viatra2EMFSynchronizer viatra2EMFSync;
	private ViatraEMFFrameworkManager viatraEMFFrameworkManager;
	private EMFCoreMetamodelManager emfCoreMetamodelManager;

	private boolean isLocalChange;

	private void log(String x) {
		if (printDebug) {
			System.out.println(x);
		}
	}

	public ViatraListener(IFramework framework, ViatraResource inputResource) {
		resource = inputResource;
		framework.getTopmodel().getNotificationManager().addAllListener(this);
		viatra2EMFSync = ViatraEMFManagersFactory.getInstance().getViatra2EMFSynchronizer(framework);
		viatraEMFFrameworkManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(framework);
		emfCoreMetamodelManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(framework);
	}

	public boolean isLocalChange() {
		return isLocalChange;
	}

	public void setLocalChange(boolean isLocalChange) {
		this.isLocalChange = isLocalChange;
	}

	private boolean isNotificationForResource(ICoreNotificationObject notification) {
		boolean isForUs = false;

		Collection<IModelElement> notifiedObjects = notification.getNotifiedObjects();
		if (!notifiedObjects.isEmpty()) {
			for (EObject obj : resource.getContents()) {
				String nameOfResourceRoot = resource.getURIFragment(obj);
				for (IModelElement viatraElement : notifiedObjects) {
					if (viatraElement != null) {
						String nameOfViatraElement = viatraElement.getFullyQualifiedName();
						if (nameOfViatraElement != null) {
							if (nameOfViatraElement.startsWith(nameOfResourceRoot)) {
								isForUs = true;
								break;
							}
						}
					}
				}
			}
		} else {
			isForUs = true;
		}

		return isForUs;
	}

	public int getListenerCategory() {
		return 0;
	}

	public void actionPerformed(ICoreNotificationObject notification) {
		if (isNotificationForResource(notification) && !isLocalChange() && isEnabled()) {
			log("VIATRA Action Performed (in isNotificationForResource()): " + this);
			log("	type: " + notification.getActionType());
			log("	notified obj: " + notification.getNotifiedObjects());
			if (notification.getActionType() == ICoreNotificationObject.ACTION_DELETE_INSTANCEOF) {
				handleNotificationDeleteInstanceOf((ICoreNotificationObjectDeleteInstanceOf) notification);
			} else if (notification.getActionType() == ICoreNotificationObject.ACTION_CREATE_INSTANCEOF) {
				handleNotificationCreateInstanceOf((ICoreNotificationObjectCreateInstanceOf) notification);
			} else if (notification.getActionType() == ICoreNotificationObject.ACTION_SET_VALUE) {
				handleNotificationSetValue((ICoreNotificationObjectSetValue) notification);
			} else if (notification.getActionType() == ICoreNotificationObject.ACTION_SET_RELATION_TO) {
				handleNotificationSetRelationTo((ICoreNotificationObjectSetRelationTo) notification);
			} else if (notification.getActionType() == ICoreNotificationObject.ACTION_SET_RELATION_FROM) {
				handleNotificationSetRelationFrom((ICoreNotificationObjectSetRelationFrom) notification);
			}
		}
	}

	private boolean isEnabled() {
		return true;
	}

	private void handleNotificationSetValue(ICoreNotificationObjectSetValue notification) {
		IEntity viatraEntityChanged = notification.getEntity();
		String oldValue = notification.getOldValue();
		String newValue = notification.getNewValue();

		// EDataType value (Note: in case of EEnumLiteral there is 2 types, but
		// the changing of EEnumLiteral is prohibited)
		if (viatraEntityChanged.getTypes().size() == 1) {
			IEntity type = (IEntity) viatraEntityChanged.getTypes().toArray()[0];
			if (type.isInstanceOf(emfCoreMetamodelManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))) {
				for (IRelation relationInstance : viatraEntityChanged.getRelationsTo()) {
					if (relationInstance.getTypes().size() == 1) {
						IRelation relationType = (IRelation) relationInstance.getTypes().toArray()[0];

						// many multiplicity
						if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature(relationType)) {
							if (relationInstance.getFrom() instanceof IEntity) {
								IEntity mainInstance = (IEntity) relationInstance.getFrom();
								// Note: It need only because it is the easiest
								// way to reach the index of the realtion. It
								// will be set during the construction of the
								// featureSet
								ManyStructuralFeatureSet actualFeatureSet = new ManyStructuralFeatureSet(mainInstance,
										relationType);
								ManyStructuralFeature actualFeature = actualFeatureSet
										.getStructuralFeature(relationInstance);
								if (actualFeature != null && actualFeature.isValid()) {
									fireEMFNotificationSet(actualFeature, oldValue, newValue);
								}
							}

						}

						// one multiplicity
						else {
							SingleStructuralFeature actualFeature = new SingleStructuralFeature(relationInstance);
							if (actualFeature.isValid()) {
								fireEMFNotificationSet(actualFeature, oldValue, newValue);
							}
						}
					}
				}
			}
		}
	}

	private void handleNotificationCreateInstanceOf(ICoreNotificationObjectCreateInstanceOf notification) {
		IModelElement instance = notification.getInstance();
		IModelElement type = notification.getType();

		// EStructuralFeature
		if (type.isInstanceOf(emfCoreMetamodelManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE))) {

			// EStructuralFeature with many multiplicity
			if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature((IRelation) type)) {
				if (((IRelation) instance).getFrom() instanceof IEntity) {
					IEntity mainEntity = (IEntity) ((IRelation) instance).getFrom();
					ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(mainEntity, (IRelation) type,
							(IRelation) instance);
					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
				}
			}

			// EStructuralFeature with one multiplicity
			else {
				SingleStructuralFeature newFeature = new SingleStructuralFeature((IRelation) instance);
				checkAndFireEMFNotification_SingleCreate(newFeature);
			}
		}

		// Next relation
		else if (type.equals(emfCoreMetamodelManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT))) {
			if (((IRelation) instance).getFrom() instanceof IRelation
					&& ((IRelation) instance).getTo() instanceof IRelation) {
				IRelation from = (IRelation) ((IRelation) instance).getFrom();
				IRelation to = (IRelation) ((IRelation) instance).getTo();
				if (from.getFrom() instanceof IEntity && to.getFrom() instanceof IEntity
						&& from.getFrom().equals(to.getFrom())) {
					IRelation oldNextOfFrom = viatraEMFFrameworkManager.getNext(from, (IRelation) instance, null);
					IRelation oldPrevOfTo = viatraEMFFrameworkManager.getPrev(to, (IRelation) instance, null);

					ManyStructuralFeature oldFromFeature = new ManyStructuralFeature(from, viatraEMFFrameworkManager
							.getPrev(from), oldNextOfFrom);
					ManyStructuralFeature oldToFeature = new ManyStructuralFeature(to, oldPrevOfTo,
							viatraEMFFrameworkManager.getNext(to));

					HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();
					explicitlyAdded.put(from, oldFromFeature);
					explicitlyAdded.put(to, oldToFeature);

					ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(oldFromFeature
							.getMainInstance(), oldFromFeature.getRelationType(), explicitlyAdded);

					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
				}
			}
		}

		// EClassifier
		else if (type
				.isInstanceOf(emfCoreMetamodelManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASSIFIER))) {
			for (IRelation relationInstance : instance.getRelationsTo()) {
				if (relationInstance.getTypes().size() == 1) {
					IRelation relationType = (IRelation) relationInstance.getTypes().toArray()[0];

					// many multiplicity
					if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature(relationType)) {
						IEntity mainEntity = (IEntity) ((IRelation) instance).getFrom();
						ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(mainEntity,
								(IRelation) type, relationInstance);
						checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
					}

					// one multiplicity
					else {
						SingleStructuralFeature newFeature = new SingleStructuralFeature(relationInstance);
						checkAndFireEMFNotification_SingleCreate(newFeature);
					}
				}
			}
		}
	}

	private void handleNotificationDeleteInstanceOf(ICoreNotificationObjectDeleteInstanceOf notification) {

		IModelElement instance = notification.getInstance();
		IModelElement type = notification.getType();

		// EStructuralFeature
		if (type.isInstanceOf(emfCoreMetamodelManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE))) {

			// EStructuralFeature with many multiplicity
			if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature((IRelation) type)) {
				if (((IRelation) instance).getFrom() instanceof IEntity) {
					IEntity mainEntity = (IEntity) ((IRelation) instance).getFrom();
					ManyStructuralFeature deletedFeature = new ManyStructuralFeature((IRelation) instance);
					deletedFeature.setRelationType((IRelation) type);
					HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();
					explicitlyAdded.put((IRelation) instance, deletedFeature);
					ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(mainEntity, (IRelation) type,
							explicitlyAdded);
					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
				}
			}

			// EStructuralFeature with one multiplicity
			else {
				SingleStructuralFeature deletedFeature = new SingleStructuralFeature((IRelation) instance);
				deletedFeature.setRelationType((IRelation) type);
				checkAndFireEMFNotification_SingleDelete(deletedFeature);
			}
		}

		// Next relation
		else if (type.equals(emfCoreMetamodelManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT))) {
			if (((IRelation) instance).getFrom() instanceof IRelation
					&& ((IRelation) instance).getTo() instanceof IRelation) {
				IRelation from = (IRelation) ((IRelation) instance).getFrom();
				IRelation to = (IRelation) ((IRelation) instance).getTo();
				if (from.getFrom() instanceof IEntity && to.getFrom() instanceof IEntity
						&& from.getFrom().equals(to.getFrom())) {
					IRelation oldNextOfFrom = viatraEMFFrameworkManager.getNext(from, null, (IRelation) instance);
					IRelation oldPrevOfTo = viatraEMFFrameworkManager.getPrev(to, null, (IRelation) instance);

					ManyStructuralFeature oldFromFeature = new ManyStructuralFeature(from, viatraEMFFrameworkManager
							.getPrev(from), oldNextOfFrom);
					ManyStructuralFeature oldToFeature = new ManyStructuralFeature(to, oldPrevOfTo,
							viatraEMFFrameworkManager.getNext(to));

					HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();
					explicitlyAdded.put(from, oldFromFeature);
					explicitlyAdded.put(to, oldToFeature);

					ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(oldFromFeature
							.getMainInstance(), oldFromFeature.getRelationType(), explicitlyAdded);

					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
				}
			}
		}

		// EClassifier
		else if (type
				.isInstanceOf(emfCoreMetamodelManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASSIFIER))) {
			for (IRelation relationInstance : instance.getRelationsTo()) {
				if (relationInstance.getTypes().size() == 1) {
					IRelation relationType = (IRelation) relationInstance.getTypes().toArray()[0];

					// Many multiplicity
					if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature(relationType)) {
						ManyStructuralFeature deletedFeature = new ManyStructuralFeature(relationInstance);
						deletedFeature.setValueType((IEntity) type);
						IEntity mainEntity = (IEntity) relationInstance.getFrom();
						ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(mainEntity, relationType,
								relationInstance);
						checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
					}

					// One multiplicity
					else {
						SingleStructuralFeature deletedFeature = new SingleStructuralFeature(relationInstance);
						deletedFeature.setValueType((IEntity) type);
						checkAndFireEMFNotification_SingleDelete(deletedFeature);
					}
				}
			}
		}
	}

	private void handleNotificationSetRelationTo(ICoreNotificationObjectSetRelationTo notification) {
		IRelation relationInstance = notification.getRelation();
		IModelElement oldTo = notification.getOldTo();
		IModelElement newTo = notification.getNewTo();

		if (relationInstance.getTypes().size() == 1) {
			IRelation relationType = (IRelation) relationInstance.getTypes().toArray()[0];
			if (relationType.isInstanceOf(emfCoreMetamodelManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE))) {

				// Many multiplicity
				if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature(relationType)) {
					ManyStructuralFeature oldFeature = new ManyStructuralFeature(relationInstance);
					if (oldTo instanceof IEntity) {
						oldFeature.setValueInstance((IEntity) oldTo);
					} else {
						oldFeature.setValueInstance(null);
					}
					HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();

					explicitlyAdded.put(relationInstance, oldFeature);

					ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(oldFeature.getMainInstance(),
							relationType, explicitlyAdded);

					checkAndFireEMFNotification_ManySetRelationTo(oldFeatureSet, relationInstance);
				}

				// One multiplicity
				else {
					SingleStructuralFeature oldFeature = new SingleStructuralFeature(relationInstance);
					if (oldTo instanceof IEntity) {
						oldFeature.setValueInstance((IEntity) oldTo);
					} else {
						oldFeature.setValueInstance(null);
					}

					SingleStructuralFeature newFeature = new SingleStructuralFeature(relationInstance);
					if (newTo instanceof IEntity) {
						newFeature.setValueInstance((IEntity) newTo);
					} else {
						newFeature.setValueInstance(null);
					}

					checkAndFireEMFNotification_SingleSetRelationTo(oldFeature, newFeature);
				}
			} else if (relationType.equals(emfCoreMetamodelManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT))) {
				if (relationInstance.getFrom() instanceof IRelation && oldTo instanceof IRelation
						&& newTo instanceof IRelation) {
					IRelation fromRel = (IRelation) relationInstance.getFrom();
					IRelation oldToRel = (IRelation) oldTo;
					IRelation newToRel = (IRelation) newTo;

					if (fromRel.getFrom() instanceof IEntity && oldToRel.getFrom() instanceof IEntity
							&& newToRel.getFrom() instanceof IEntity && fromRel.getFrom().equals(oldToRel.getFrom())
							&& oldToRel.getFrom().equals(newToRel.getFrom())) {
						IRelation oldPrevOfOldToRel = viatraEMFFrameworkManager.getPrev(oldToRel, null,
								relationInstance);
						IRelation oldPrevOfNewToRel = viatraEMFFrameworkManager.getPrev(newToRel, relationInstance,
								null);
						IRelation oldNextOfFromRel = null;
						// If there is a next now(note, that this is surely
						// newToRel) then old Next is oldToRel
						if (viatraEMFFrameworkManager.getNext(fromRel) != null) {
							oldNextOfFromRel = oldToRel;
						}

						ManyStructuralFeature oldFeatureOfFrom = new ManyStructuralFeature(fromRel,
								viatraEMFFrameworkManager.getPrev(fromRel), oldNextOfFromRel);
						ManyStructuralFeature oldFeatureOfOldTo = new ManyStructuralFeature(oldToRel,
								oldPrevOfOldToRel, viatraEMFFrameworkManager.getNext(oldToRel));
						ManyStructuralFeature oldFeatureOfNewTo = new ManyStructuralFeature(newToRel,
								oldPrevOfNewToRel, viatraEMFFrameworkManager.getNext(newToRel));

						HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();
						explicitlyAdded.put(fromRel, oldFeatureOfFrom);
						explicitlyAdded.put(oldToRel, oldFeatureOfOldTo);
						explicitlyAdded.put(newToRel, oldFeatureOfNewTo);

						ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(oldFeatureOfFrom
								.getMainInstance(), oldFeatureOfFrom.getRelationType(), explicitlyAdded);

						checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
					}
				}
			}
		}

	}

	private void handleNotificationSetRelationFrom(ICoreNotificationObjectSetRelationFrom notification) {
		IRelation relationInstance = notification.getRelation();
		IModelElement oldFrom = notification.getOldFrom();
		IModelElement newFrom = notification.getNewFrom();

		if (relationInstance.getTypes().size() == 1) {
			IRelation relationType = (IRelation) relationInstance.getTypes().toArray()[0];
			if (relationType.isInstanceOf(emfCoreMetamodelManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE))) {

				// Many multiplicity
				if (viatraEMFFrameworkManager.isManyForViatraEStructuralFeature(relationType)) {
					// Do the notification check for old mainEntity (this is
					// oldFrom)
					ManyStructuralFeature oldFeatureForOldEntity = new ManyStructuralFeature(relationInstance);
					if (oldFrom instanceof IEntity) {
						oldFeatureForOldEntity.setMainInstance((IEntity) oldFrom);
					} else {
						oldFeatureForOldEntity.setMainInstance(null);
					}
					HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();

					explicitlyAdded.put(relationInstance, oldFeatureForOldEntity);

					ManyStructuralFeatureSet oldFeatureSetForOldEntity = new ManyStructuralFeatureSet(
							oldFeatureForOldEntity.getMainInstance(), relationType, explicitlyAdded);

					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSetForOldEntity);

					// Do the notification check for new mainEntity (this is
					// newFrom)
					ManyStructuralFeatureSet oldFeatureSetForNewEntity = new ManyStructuralFeatureSet(
							(IEntity) newFrom, relationType, relationInstance);

					checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSetForNewEntity);
				}

				// One multiplicity
				else {
					SingleStructuralFeature oldFeature = new SingleStructuralFeature(relationInstance);
					if (oldFrom instanceof IEntity) {
						oldFeature.setMainInstance((IEntity) oldFrom);
					} else {
						oldFeature.setMainInstance(null);
					}

					SingleStructuralFeature newFeature = new SingleStructuralFeature(relationInstance);
					if (newFrom instanceof IEntity) {
						newFeature.setMainInstance((IEntity) newFrom);
					} else {
						newFeature.setMainInstance(null);
					}

					checkAndFireEMFNotification_SingleSetRelationFrom(oldFeature, newFeature);
				}
			} else if (relationType.equals(emfCoreMetamodelManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT))) {
				if (relationInstance.getTo() instanceof IRelation && oldFrom instanceof IRelation
						&& newFrom instanceof IRelation) {
					IRelation toRel = (IRelation) relationInstance.getFrom();
					IRelation oldFromRel = (IRelation) oldFrom;
					IRelation newFromRel = (IRelation) newFrom;

					if (toRel.getFrom() instanceof IEntity && oldFromRel.getFrom() instanceof IEntity
							&& newFromRel.getFrom() instanceof IEntity && toRel.getFrom().equals(oldFromRel.getFrom())
							&& oldFromRel.getFrom().equals(newFromRel.getFrom())) {
						IRelation oldNextOfOldFromRel = viatraEMFFrameworkManager.getNext(oldFromRel, null,
								relationInstance);
						IRelation oldNextOfNewFromRel = viatraEMFFrameworkManager.getNext(newFromRel, relationInstance,
								null);
						IRelation oldPrevOfToRel = null;
						// If there is a next now(note, that this is surely
						// newToRel) then old Next is oldToRel
						if (viatraEMFFrameworkManager.getPrev(toRel) != null) {
							oldPrevOfToRel = oldFromRel;
						}

						ManyStructuralFeature oldFeatureOfTo = new ManyStructuralFeature(toRel, oldPrevOfToRel,
								viatraEMFFrameworkManager.getNext(toRel));
						ManyStructuralFeature oldFeatureOfOldFrom = new ManyStructuralFeature(oldFromRel,
								viatraEMFFrameworkManager.getPrev(oldFromRel), oldNextOfOldFromRel);
						ManyStructuralFeature oldFeatureOfNewFrom = new ManyStructuralFeature(newFromRel,
								viatraEMFFrameworkManager.getNext(newFromRel), oldNextOfNewFromRel);

						HashMap<IRelation, ManyStructuralFeature> explicitlyAdded = new HashMap<IRelation, ManyStructuralFeature>();
						explicitlyAdded.put(toRel, oldFeatureOfTo);
						explicitlyAdded.put(oldFromRel, oldFeatureOfOldFrom);
						explicitlyAdded.put(newFromRel, oldFeatureOfNewFrom);

						ManyStructuralFeatureSet oldFeatureSet = new ManyStructuralFeatureSet(oldFeatureOfTo
								.getMainInstance(), oldFeatureOfTo.getRelationType(), explicitlyAdded);

						checkAndFireEMFNotification_ManyGeneralAddRemoveCase(oldFeatureSet);
					}
				}
			}
		}

	}

	private void checkAndFireEMFNotification_SingleCreate(SingleStructuralFeature newFeature) {
		SingleStructuralFeatureSet actualFeatureSet = new SingleStructuralFeatureSet(newFeature.getMainInstance(),
				newFeature.getRelationType());

		boolean isValidBefore = actualFeatureSet.isValidAllButOne(newFeature);
		boolean isValidAfter = actualFeatureSet.isValid();

		if (!isValidBefore && isValidAfter) {
			fireEMFNotificationSet(newFeature);
		} else if (isValidBefore && !isValidAfter) {
			fireEMFNotificationUnset(actualFeatureSet.getValidAllButOne(newFeature));
		}
	}

	private void checkAndFireEMFNotification_SingleDelete(SingleStructuralFeature deletedFeature) {
		SingleStructuralFeatureSet actualFeatureSet = new SingleStructuralFeatureSet(deletedFeature.getMainInstance(),
				deletedFeature.getRelationType());

		boolean isValidBefore = actualFeatureSet.isValidAllAndOne(deletedFeature);
		boolean isValidAfter = actualFeatureSet.isValid();

		if (!isValidBefore && isValidAfter) {
			fireEMFNotificationSet(actualFeatureSet.getValid());
		} else if (isValidBefore && !isValidAfter) {
			fireEMFNotificationUnset(deletedFeature);
		}
	}

	private void checkAndFireEMFNotification_ManyGeneralAddRemoveCase(ManyStructuralFeatureSet oldFeatureSet) {
		ManyStructuralFeatureSet actualFeatureSet = new ManyStructuralFeatureSet(oldFeatureSet.getMainEntityInstance(),
				oldFeatureSet.getRelationType());

		List<IRelation> added = new ArrayList<IRelation>(actualFeatureSet.getRelationInstances());
		added.removeAll(oldFeatureSet.getRelationInstances());

		List<IRelation> removed = new ArrayList<IRelation>(oldFeatureSet.getRelationInstances());
		removed.removeAll(actualFeatureSet.getRelationInstances());

		for (IRelation rel : added) {
			fireEMFNotificationAdd(actualFeatureSet.getStructuralFeature(rel));
		}

		for (IRelation rel : removed) {
			fireEMFNotificationRemove(oldFeatureSet.getStructuralFeature(rel));
		}
	}

	private void checkAndFireEMFNotification_ManySetRelationTo(ManyStructuralFeatureSet oldFeatureSet,
			IRelation modifiedRelation) {
		ManyStructuralFeatureSet actualFeatureSet = new ManyStructuralFeatureSet(oldFeatureSet.getMainEntityInstance(),
				oldFeatureSet.getRelationType());

		List<IRelation> added = new ArrayList<IRelation>(actualFeatureSet.getRelationInstances());
		added.removeAll(oldFeatureSet.getRelationInstances());

		List<IRelation> removed = new ArrayList<IRelation>(oldFeatureSet.getRelationInstances());
		removed.removeAll(actualFeatureSet.getRelationInstances());

		for (IRelation rel : added) {
			fireEMFNotificationAdd(actualFeatureSet.getStructuralFeature(rel));
		}

		for (IRelation rel : removed) {
			fireEMFNotificationRemove(oldFeatureSet.getStructuralFeature(rel));
		}

		if (added.isEmpty() && removed.isEmpty()) {
			fireEMFNotificationSet(oldFeatureSet.getStructuralFeature(modifiedRelation), actualFeatureSet
					.getStructuralFeature(modifiedRelation));
		}
	}

	private void checkAndFireEMFNotification_SingleSetRelationTo(SingleStructuralFeature oldFeature,
			SingleStructuralFeature newFeature) {
		SingleStructuralFeatureSet actualFeatureSet = new SingleStructuralFeatureSet(newFeature.getMainInstance(),
				newFeature.getRelationType());

		boolean isValidBefore = actualFeatureSet.isValidAllButOneAndOne(newFeature, oldFeature);
		boolean isValidAfter = actualFeatureSet.isValid();

		if (isValidBefore && isValidAfter) {
			fireEMFNotificationSet(oldFeature, newFeature);
		} else if (!isValidBefore && isValidAfter) {
			fireEMFNotificationSet(newFeature);
		} else if (isValidBefore && !isValidAfter) {
			fireEMFNotificationUnset(oldFeature);
		}

	}

	private void checkAndFireEMFNotification_SingleSetRelationFrom(SingleStructuralFeature oldFeature,
			SingleStructuralFeature newFeature) {
		checkAndFireEMFNotification_SingleCreate(newFeature);
		checkAndFireEMFNotification_SingleDelete(oldFeature);
	}

	private void fireEMFNotificationAdd(ManyStructuralFeature feature) {
		EStructuralFeature strFeature = viatra2EMFSync.getEStructuralFeatureForIRelation(feature.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(feature.getMainInstance());
		Object valueObj = viatra2EMFSync.getObjectForIEntity(feature.getValueInstance());
		int index = feature.getIndex();

		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject mainViatraInternalObj = (InternalViatraObject) mainObj;
			fireNotificataion(mainViatraInternalObj, new ENotificationImpl(mainViatraInternalObj, Notification.ADD,
					strFeature, null, valueObj, index));
		}
	}

	private void fireEMFNotificationRemove(ManyStructuralFeature feature) {
		EStructuralFeature strFeature = viatra2EMFSync.getEStructuralFeatureForIRelation(feature.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(feature.getMainInstance());
		Object valueObj = viatra2EMFSync.getObjectForIEntity(feature.getValueInstance(), feature.getValueType());
		int index = feature.getIndex();

		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject mainViatraInternalObj = (InternalViatraObject) mainObj;
			fireNotificataion(mainViatraInternalObj, new ENotificationImpl(mainViatraInternalObj, Notification.REMOVE,
					strFeature, valueObj, null, index));
		}
	}

	private void fireEMFNotificationSet(SingleStructuralFeature singleStructuralFeature) {
		EStructuralFeature feature = viatra2EMFSync.getEStructuralFeatureForIRelation(singleStructuralFeature
				.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(singleStructuralFeature.getMainInstance());
		Object newObj = viatra2EMFSync.getObjectForIEntity(singleStructuralFeature.getValueInstance());
		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject mainViatraInternalObj = (InternalViatraObject) mainObj;
			fireNotificataion(mainViatraInternalObj, new ENotificationImpl(mainViatraInternalObj, Notification.SET,
					feature, null, newObj));
		}
	}

	private void fireEMFNotificationSet(SingleStructuralFeature oldSingleStructuralFeature,
			SingleStructuralFeature newSingleStructuralFeature) {
		EStructuralFeature feature = viatra2EMFSync.getEStructuralFeatureForIRelation(newSingleStructuralFeature
				.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(newSingleStructuralFeature.getMainInstance());
		Object newObj = viatra2EMFSync.getObjectForIEntity(newSingleStructuralFeature.getValueInstance());
		Object oldObj = viatra2EMFSync.getObjectForIEntity(oldSingleStructuralFeature.getValueInstance());
		int index = -1;
		if (oldSingleStructuralFeature instanceof ManyStructuralFeature
				&& newSingleStructuralFeature instanceof ManyStructuralFeature) {
			int oldIndex = ((ManyStructuralFeature) oldSingleStructuralFeature).getIndex();
			int newIndex = ((ManyStructuralFeature) newSingleStructuralFeature).getIndex();
			if (oldIndex == newIndex) {
				index = oldIndex;
			}
		}
		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject mainViatraInternalObj = (InternalViatraObject) mainObj;
			fireNotificataion(mainViatraInternalObj, new ENotificationImpl(mainViatraInternalObj, Notification.SET,
					feature, oldObj, newObj, index));
		}
	}

	private void fireEMFNotificationUnset(SingleStructuralFeature singleStructuralFeature) {
		EStructuralFeature feature = viatra2EMFSync.getEStructuralFeatureForIRelation(singleStructuralFeature
				.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(singleStructuralFeature.getMainInstance());
		Object oldObj = viatra2EMFSync.getObjectForIEntity(singleStructuralFeature.getValueInstance(),
				singleStructuralFeature.getValueType());
		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject mainViatraInternalObj = (InternalViatraObject) mainObj;
			fireNotificataion(mainViatraInternalObj, new ENotificationImpl(mainViatraInternalObj, Notification.UNSET,
					feature, oldObj, null));
		}
	}

	private void fireEMFNotificationSet(SingleStructuralFeature structuralFeature, String oldValue, String newValue) {
		EStructuralFeature feature = viatra2EMFSync.getEStructuralFeatureForIRelation(structuralFeature
				.getRelationType());
		Object mainObj = viatra2EMFSync.getObjectForIEntity(structuralFeature.getMainInstance());
		Object oldObj = viatra2EMFSync.createObjectForValueWithType(oldValue, structuralFeature.getValueType());
		Object newObj = viatra2EMFSync.createObjectForValueWithType(newValue, structuralFeature.getValueType());

		int index = -1;
		if (structuralFeature instanceof ManyStructuralFeature) {
			index = ((ManyStructuralFeature) structuralFeature).getIndex();
		}

		if (mainObj instanceof InternalViatraObject) {
			InternalViatraObject vObj = (InternalViatraObject) mainObj;
			ENotificationImpl notification = new ENotificationImpl(vObj, Notification.SET, feature, oldObj, newObj,
					index);
			fireNotificataion((EObject) mainObj, notification);
		}
	}

	/**
	 * fire the given notification on the given object. The responsibility of
	 * this method is to encapsulate the notification into a transaction in case
	 * of transactional editing domain.
	 * 
	 * @param mainObj
	 * @param notification
	 */
	private void fireNotificataion(EObject mainObj, ENotificationImpl notification) {
		Transaction transaction = null;
		EditingDomain domain = getEditingDomainFor(mainObj);
		// In case of transactional editing domain we have to start a
		// transaction
		if (domain instanceof InternalTransactionalEditingDomain) {
			InternalTransactionalEditingDomain transactionalEditingDomain = (InternalTransactionalEditingDomain) domain;
			try {
				transaction = transactionalEditingDomain.startTransaction(false, null);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		// fire the notification
		mainObj.eNotify(notification);
		if (transaction != null) {
			try {
				transaction.commit();
			} catch (RollbackException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private EditingDomain getEditingDomainFor(EObject object) {
		Resource resource = object.eResource();
		if (resource != null) {
			IEditingDomainProvider editingDomainProvider = (IEditingDomainProvider) EcoreUtil.getExistingAdapter(
					resource, IEditingDomainProvider.class);
			if (editingDomainProvider != null) {
				return editingDomainProvider.getEditingDomain();
			} else {
				ResourceSet resourceSet = resource.getResourceSet();
				if (resourceSet instanceof IEditingDomainProvider) {
					EditingDomain editingDomain = ((IEditingDomainProvider) resourceSet).getEditingDomain();
					return editingDomain;
				} else if (resourceSet != null) {
					editingDomainProvider = (IEditingDomainProvider) EcoreUtil.getExistingAdapter(resourceSet,
							IEditingDomainProvider.class);
					if (editingDomainProvider != null) {
						return editingDomainProvider.getEditingDomain();
					}
				}
			}
		}
		return null;
	}
}
