package hu.optxware.emf2viatra.core.managers;

import hu.optxware.emf2viatra.core.eresource.InternalViatraObject;

import java.util.Collection;
import java.util.HashMap;

import org.eclipse.emf.common.util.Enumerator;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.framework.IFramework;

public class EMF2ViatraSynchronizer {

	// private IFramework fw;

	private EMFCoreMetamodelManager emfCoreManager = null;

	private ViatraEMFFrameworkManager viatraEMFFrameworkManager = null;

	// TODO : Kell egy kln osztly, ami az sszerendelst vgzi az Object-ek
	// s a Entity-k kztt. Ez az EMF2ViatraSync ne tudjon a Viatra2EMF-rl!!!
	// TODO TODO TODO
	private Viatra2EMFSynchronizer viatra2EMFSynchronizer = null;

	public EMF2ViatraSynchronizer(IFramework fw) {
		super();
		// this.fw = fw;
		emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(fw);
		viatraEMFFrameworkManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(fw);
		viatra2EMFSynchronizer = ViatraEMFManagersFactory.getInstance().getViatra2EMFSynchronizer(fw);
	}

	public IEntity getIEntityForObject(EStructuralFeature feature, Object object) {
		String value = null;
		IEntity type = null;

		if (object instanceof InternalViatraObject) {
			InternalViatraObject internalVObj = (InternalViatraObject) object;
			if (((InternalViatraObject) object).getRelatedViatraEntity() != null) {
				return internalVObj.getRelatedViatraEntity();
			} else {
				// We have to make a new one
				EClass eClass = internalVObj.eClass();
				try {
					IEntity typeEnt = getIEntityForEClass(eClass);
					IEntity newEntity = viatraEMFFrameworkManager.createEObject(typeEnt);
					((InternalViatraObject) object).setRelatedViatraEntity(newEntity);

					// TODO Ez most azrt van benne, hogy az itt ltrehozott
					// elemek is bekerljenek a hashmap-be, amit a
					// viatra2EMFsynhronizer trol
					viatra2EMFSynchronizer.addIEntityInternalViatraPair(newEntity, (InternalViatraObject) object);

					return newEntity;
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

		} else if (feature != null && feature.getEType() instanceof EEnum) {
			Enumerator enumerator = (Enumerator) object;
			EEnum eEnum = (EEnum) feature.getEType();
			return getIEntityForEEnumLiteral(eEnum, enumerator);

		} else if (feature.getEType() instanceof EDataType) {
			EDataType dataType = (EDataType) feature.getEType();
			value = EcoreUtil.convertToString(dataType, object);
			type = getIEntityForEDataType(dataType);

			if (value != null && type != null) {
				try {
					IEntity newEntity = viatraEMFFrameworkManager.createData(type, value);
					return newEntity;
				} catch (VPMCoreException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		return null;
	}

	HashMap<EClass, IEntity> classMap = new HashMap<EClass, IEntity>();

	// TODO: Itt lehetne olyat, hogy betltnk egy metamodellt, amire szksg
	// lehet!!!
	public IEntity getIEntityForEClass(EClass eClass) {
		IEntity matchedClass = classMap.get(eClass);
		if (matchedClass != null)
			return matchedClass;

		String className = eClass.getName();

		IEntity matchedPackage = getIEntityForEPackage(eClass.getEPackage());

		if (matchedPackage != null) {
			for (IModelElement ent : matchedPackage.getContents()) {
				if (ent.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASS))
						&& viatraEMFFrameworkManager.getNameForViatraNamedElement((IEntity) ent).equals(className)) {
					matchedClass = (IEntity) ent;
					classMap.put(eClass, matchedClass);
					break;
				}
			}
		}

		// !!!!!!!!!!! Warning: Hack solution...
		// Cause: Ecore elements (nemf.ecore.*) could also be connected to
		// metamodels.
		if (matchedPackage == null && eClass.getEPackage().getNsURI().equals("http://www.eclipse.org/emf/2002/Ecore")) {
			IEntity viatraEcoreRoot = emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE);
			for (IModelElement viatraInstanceModelElement : viatraEcoreRoot.getElementsInNamespace()) {
				if (viatraInstanceModelElement instanceof IEntity
						&& ((IEntity) viatraInstanceModelElement).getName().equals(eClass.getName())) {
					matchedClass = (IEntity) viatraInstanceModelElement;
					classMap.put(eClass, matchedClass);
					break;
				}
			}
		}

		return matchedClass;
	}

	private HashMap<EPackage, IEntity> packageMap = new HashMap<EPackage, IEntity>();

	public IEntity getIEntityForEPackage(EPackage ePackage) {
		IEntity matchedPackage = packageMap.get(ePackage);
		if (matchedPackage != null)
			return matchedPackage;

		String packageUri = ePackage.getNsURI();

		for (IModelElement packageEntity : emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EPACKAGE)
				.getInstances()) {
			if (packageEntity instanceof IEntity) {

				Collection<IModelElement> nsUriEntities = packageEntity.getAllRelationTargetByType(emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EPACKAGE_NSURI));
				if (nsUriEntities.size() == 1) {
					IEntity nsUriEnt = (IEntity) nsUriEntities.toArray()[0];
					String nsUriValue = nsUriEnt.getValue();
					if (nsUriValue.equals(packageUri)) {
						matchedPackage = (IEntity) packageEntity;
						packageMap.put(ePackage, matchedPackage);
						break;
					}
				}
			}
		}

		return matchedPackage;

	}

	public IRelation getIRelationForEStructuralFeature(EStructuralFeature eStructuralFeature) {
		IEntity rootEntity = getIEntityForEClass(eStructuralFeature.getEContainingClass());
		if (eStructuralFeature instanceof EAttribute) {
			return getIRelationForEAttributeElement((EAttribute) eStructuralFeature, rootEntity);
		} else if (eStructuralFeature instanceof EReference) {
			return getIRelationForEReferenceElement((EReference) eStructuralFeature, rootEntity);
		} else {
			return null;
		}
	}

	private HashMap<EAttribute, IRelation> attrMap = new HashMap<EAttribute, IRelation>();

	private IRelation getIRelationForEAttributeElement(EAttribute eAttribute, IEntity containerEntity) {
		IRelation matchedRelation = attrMap.get(eAttribute);
		if (matchedRelation != null)
			return matchedRelation;
		String attrName = eAttribute.getName();

		for (IModelElement relEl : containerEntity.getAllRelationFromByType(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE))) {
			IRelation rel = (IRelation) relEl;
			if (viatraEMFFrameworkManager.getNameForViatraStructuralFeature(rel).equals(attrName)) {

				matchedRelation = rel;
				attrMap.put(eAttribute, matchedRelation);
				break;
			}
		}

		return matchedRelation;
	}

	public IRelation getIRelationForEAttributeElement(EAttribute eAttribute) {
		IEntity rootEntity = getIEntityForEClass(eAttribute.getEContainingClass());
		return getIRelationForEAttributeElement(eAttribute, rootEntity);
	}

	private HashMap<EReference, IRelation> relMap = new HashMap<EReference, IRelation>();

	private IRelation getIRelationForEReferenceElement(EReference eReference, IEntity containerEntity) {
		IRelation matchedRelation = relMap.get(eReference);
		if (matchedRelation != null)
			return matchedRelation;
		String refName = eReference.getName();

		for (IModelElement relEl : containerEntity.getAllRelationFromByType(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE))) {
			IRelation rel = (IRelation) relEl;
			if (viatraEMFFrameworkManager.getNameForViatraStructuralFeature(rel).equals(refName)) {
				matchedRelation = rel;
				relMap.put(eReference, matchedRelation);
				break;
			}
		}

		return matchedRelation;
	}

	public IRelation getIRelationForEReferenceElement(EReference eReference) {
		IEntity rootEntity = getIEntityForEClass(eReference.getEContainingClass());
		return getIRelationForEReferenceElement(eReference, rootEntity);
	}

	public IEntity getIEntityForEDataType(EDataType eDataType) {
		IEntity matchedEntity = null;

		String emfName = eDataType.getName();

		if (eDataType instanceof EEnum) {
			EPackage ePackage = eDataType.getEPackage();
			IEntity viatraPackageEntity = getIEntityForEPackage(ePackage);
			if (viatraPackageEntity != null) {
				for (IModelElement ent : viatraPackageEntity.getContents()) {
					if (ent.isInstanceOf(emfCoreManager
							.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM))
							&& viatraEMFFrameworkManager.getNameForViatraNamedElement((IEntity) ent).equals(emfName)) {
						matchedEntity = (IEntity) ent;
					}
				}
			}
		} else {
			// Search between the contents of the datatypes entity
			Collection<IModelElement> datatypes = emfCoreManager.getIEntityForEcoreElement(
					EMFCoreEntConst.ECORE_DATATYPES).getContents();
			for (IModelElement el : datatypes) {
				if (el.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))
						&& el.getName().equals(emfName)) {
					matchedEntity = (IEntity) el;
				}
			}
			// Search between the instances of the EDataType entity
			if (matchedEntity == null) {
				for (IModelElement el : emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE)
						.getInstances()) {
					if (el.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))
							&& el.getName().equals(emfName)) {
						matchedEntity = (IEntity) el;
					}
				}
			}

			// TODO: Warning, ha nem tallta meg a szksges data type-ot
		}
		return matchedEntity;
	}

	// !!!!!!!!!!! Warning: Hack solution...
	// Cause: Enumerator doesn't know anything about his EEnum!
	public IEntity getIEntityForEEnumLiteral(EEnum eEnum, Enumerator enumerator) {
		IEntity viatraEnumRoot = getIEntityForEDataType(eEnum);

		IEntity matchedEntity = null;

		String enumName = enumerator.getName();
		for (IModelElement ent : viatraEnumRoot.getContents()) {
			if (ent
					.isInstanceOf(emfCoreManager
							.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL))
					&& ent.isInstanceOf(viatraEnumRoot) && ((IEntity) ent).getValue().equals(enumName)) {
				matchedEntity = (IEntity) ent;
			}
		}
		return matchedEntity;
	}

}
