package hu.optxware.emf2viatra.core.managers;

import java.util.HashMap;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.framework.IFramework;

public class EMFCoreMetamodelManager {

	private IModelManager manager;

	private String NAMESPACE = "nemf";

	private HashMap<Integer, IEntity> entities = new HashMap<Integer, IEntity>();
	private HashMap<Integer, IRelation> relations = new HashMap<Integer, IRelation>();

	public EMFCoreMetamodelManager(IFramework framework) {
		manager = framework.getTopmodel().getModelManager();
	}

	public IEntity getIEntityForEcoreElement(int ecoreConstant) {
		if (entities.get(ecoreConstant) == null) {
			switch (ecoreConstant) {

			case EMFCoreEntConst.ECORE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore"));
				break;

			case EMFCoreEntConst.ECORE_ECLASS:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EClass"));
				break;

			case EMFCoreEntConst.ECORE_ECLASSIFIER:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EClassifier"));
				break;

			case EMFCoreEntConst.ECORE_EDATATYPE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EDataType"));
				break;

			case EMFCoreEntConst.ECORE_EMODELELEMENT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EModelElement"));
				break;

			case EMFCoreEntConst.ECORE_ENAMEDELEMENT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.ENamedElement"));
				break;

			case EMFCoreEntConst.ECORE_EPACKAGE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EPackage"));
				break;

			case EMFCoreEntConst.ECORE_ERESOURCE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EResource"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EBIGDECIMAL:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EBigDecimal"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EBIGINTEGER:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EBigInteger"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EBOOLEAN:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EBoolean"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EBYTE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EByte"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_ECHAR:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EChar"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EDATE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EDate"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EDOUBLE:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EDouble"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EENUM:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EEnum"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EEnumLiteral"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EFLOAT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EFloat"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_EINT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EInt"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_ELONG:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.ELong"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_ESHORT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EShort"));
				break;

			case EMFCoreEntConst.ECORE_DATATYPES_ESTRING:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.datatypes.EString"));
				break;

			case EMFCoreEntConst.PACKAGES:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".packages"));
				break;

			case EMFCoreEntConst.RESOURCES:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".resources"));
				break;

			case EMFCoreEntConst.TEMPINTANCEROOT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".tempInstanceRoot"));
				break;

			case EMFCoreEntConst.ECORE_EOBJECT:
				entities.put(ecoreConstant, manager.getEntityByName(NAMESPACE + ".ecore.EObject"));
				break;

			default:
				break;
			}
		}
		return entities.get(ecoreConstant);
	}

	public IRelation getIRelationForEcoreElement(int ecoreConstant) {
		if (relations.get(ecoreConstant) == null) {
			switch (ecoreConstant) {

			case EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EClass.EAttribute"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_EREFERENCE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EClass.EReference"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_CONTAINER:
				relations.put(ecoreConstant, manager
						.getRelationByName(NAMESPACE + ".ecore.EClass.EReference.container"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_CONTAINMENT:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EReference.containment"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_EOPPOSITE:
				relations.put(ecoreConstant, manager
						.getRelationByName(NAMESPACE + ".ecore.EClass.EReference.eOpposite"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EClass.EStructuralFeature"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_CHANGEABLE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.changeable"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_NAME:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.name"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_TRANSIENT:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.transient"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_UNSETTABLE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.unsettable"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_VOLATILE:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.volatile"));
				break;

			case EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.ENamedElement.name"));
				break;

			case EMFCoreRelConst.ECORE_EPACKAGE_NSPREFIX:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EPackage.nsPrefix"));
				break;

			case EMFCoreRelConst.ECORE_EPACKAGE_NSURI:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EPackage.nsUri"));
				break;

			case EMFCoreRelConst.ECORE_ERESOURCE_URI:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EResource.uri"));
				break;

			case EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_MANY:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClass.EStructuralFeature.many"));
				break;

			case EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EObject.orderedRelation.next"));
				break;

			case EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE + ".ecore.EObject.orderedRelation"));
				break;

			case EMFCoreRelConst.ECORE_ECLASSIFIER_INSTANCECLASSNAME:
				relations.put(ecoreConstant, manager.getRelationByName(NAMESPACE
						+ ".ecore.EClassifier.instanceClassName"));

			default:
				break;
			}
		}
		return relations.get(ecoreConstant);
	}
}
