package hu.optxware.emf2viatra.core.managers;

import hu.optxware.emf2viatra.core.eresource.InternalViatraObject;

import java.util.Collection;
import java.util.HashMap;

import org.eclipse.emf.common.util.Enumerator;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EFactory;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.framework.IFramework;

public class Viatra2EMFSynchronizer {

	// private IFramework fw;

	private EMFCoreMetamodelManager emfCoreManager = null;

	private ViatraEMFFrameworkManager viatraEMFFrameworkManager = null;

	private HashMap<IModelElement, InternalViatraObject> iEntity2Object = null;

	public Viatra2EMFSynchronizer(IFramework fw) {
		super();
		// this.fw = fw;
		emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(fw);
		viatraEMFFrameworkManager = ViatraEMFManagersFactory.getInstance().getViatraEMFFrameworkManager(fw);
		iEntity2Object = new HashMap<IModelElement, InternalViatraObject>();
		iEntity2Object.clear();
	}

	public Object getObjectForIEntity(IEntity viatraEntity) {
		if (viatraEntity == null) {
			return null;
		}

		Collection<IModelElement> types = viatraEntity.getTypes();
		IEntity typeOfEntity = null;
		if (!types.isEmpty()) {
			typeOfEntity = (IEntity) types.toArray()[0];
		}

		if (typeOfEntity != null) {
			return getObjectForIEntity(viatraEntity, typeOfEntity);
		} else {
			return null;
		}

	}

	public Object getObjectForIEntity(IEntity viatraEntity, IEntity typeOfEntity) {

		if (viatraEntity == null)
			return null;

		Object result = null;

		if (viatraEntity.equals(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.RESOURCES))) {
			// TODO : Esetleg lehet okoskodni, ha resource-hoz rtem
			// TODO
		} else {

			if (typeOfEntity.isInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM))) {
				result = createObjectForIEntityAsEENumLiteral(viatraEntity);
			} else if (typeOfEntity.isInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))) {
				result = createObjectForIEntityAsEDataType(viatraEntity, typeOfEntity);
			} else if (typeOfEntity
					.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASS))) {
				result = createObjectForIEntityAsEClass(viatraEntity);
			} else {
				// TODO valami gond van, vagy hack kell, mivel a type mr
				// meta-ban
				// van
			}
		}

		return result;
	}

	private Object createObjectForIEntityAsEENumLiteral(IEntity viatraEntity) {
		Enumerator result = null;
		Collection<IModelElement> types = viatraEntity.getTypes();
		String name = viatraEntity.getValue();

		if (types.size() == 2) {
			IModelElement type01 = (IModelElement) types.toArray()[0];
			IModelElement type02 = (IModelElement) types.toArray()[1];
			EEnum eenum = null;
			// type01 az EEnumLiteral
			if (type01.equals(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL))) {
				if (type02 instanceof IEntity) {
					eenum = getEEnumForIEntity((IEntity) type02);
				}
				// type02 az EEnumLiteral
			} else if (type02.equals(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUMLITERAL))) {
				if (type02 instanceof IEntity) {
					eenum = getEEnumForIEntity((IEntity) type01);
				}
			} else {
				// TODO Ez nem EEnumliteral!!! Kivtelkezels
			}

			if (eenum != null) {
				EEnumLiteral literal = eenum.getEEnumLiteral(name);
				result = literal.getInstance();
			}

		}
		return result;
	}

	private InternalViatraObject createObjectForIEntityAsEClass(IEntity viatraInstanceEntity) {
		EObject eObject = null;
		EClass eClass = null;

		if (iEntity2Object.get(viatraInstanceEntity) == null) {

			if (viatraInstanceEntity.getTypes().size() == 1) {
				IEntity viatraMetaEntity = (IEntity) viatraInstanceEntity.getTypes().toArray()[0];
				eClass = getEClassForIEntity(viatraMetaEntity);
			}

			if (eClass != null) {
				EPackage ePackage = eClass.getEPackage();
				EFactory eFactory = ePackage.getEFactoryInstance();
				eObject = eFactory.create(eClass);
			}

			if (eObject instanceof InternalViatraObject) {
				((InternalViatraObject) eObject).setRelatedViatraEntity(viatraInstanceEntity);
				((InternalViatraObject) eObject).setPersisted(true);
				iEntity2Object.put(viatraInstanceEntity, (InternalViatraObject) eObject);
			}
		}

		return iEntity2Object.get(viatraInstanceEntity);
	}

	private Object createObjectForIEntityAsEDataType(IEntity viatraEntity) {
		if (viatraEntity.getTypes().size() == 1) {
			return createObjectForIEntityAsEDataType(viatraEntity, (IEntity) viatraEntity.getTypes().toArray()[0]);
		} else {
			return null;
		}
	}

	private Object createObjectForIEntityAsEDataType(IEntity viatraEntity, IEntity viatraType) {
		String value = viatraEntity.getValue();
		return createObjectForValueWithType(value, viatraType);
	}

	public Object createObjectForValueWithType(String value, IEntity viatraType) {
		Object result = null;
		if (viatraType.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))) {
			String name = viatraEMFFrameworkManager.getNameForViatraNamedElement(viatraType);

			IEntity packageEntity = viatraType.getParent();

			if (packageEntity != null) {
				EPackage ePackage = getEPackageForIEntity(packageEntity);
				if (ePackage == null){
					ePackage = EPackage.Registry.INSTANCE.getEPackage("http://www.eclipse.org/emf/2002/Ecore");
				}
				
				EClassifier classifier = ePackage.getEClassifier(name);
				if (classifier instanceof EDataType) {
					result = EcoreUtil.createFromString((EDataType) classifier, value);
				}
			}
		}
		return result;
	}

	public EClass getEClassForIEntity(IEntity viatraEntity) {

		EClass eClass = null;
		if (viatraEntity.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_ECLASS))) {
			String name = viatraEMFFrameworkManager.getNameForViatraNamedElement(viatraEntity);

			IEntity packageEntity = viatraEntity.getParent();

			if (packageEntity != null) {
				EPackage ePackage = getEPackageForIEntity(packageEntity);
				EClassifier classifier = ePackage.getEClassifier(name);
				eClass = (EClass) classifier;
			}
		}
		return eClass;
	}

	public EStructuralFeature getEStructuralFeatureForIRelation(IRelation relation) {
		EStructuralFeature eStrFeature = null;
		if (relation != null && relation.isInstanceOf(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE))) {
			String name = viatraEMFFrameworkManager.getNameForViatraStructuralFeature(relation);
			IEntity parentEntity = (IEntity) relation.getFrom();
			EClass eClass = getEClassForIEntity(parentEntity);
			eStrFeature = eClass.getEStructuralFeature(name);
		}
		return eStrFeature;
	}

	public EPackage getEPackageForIEntity(IEntity viatraEntity) {
		try {

			EPackage ePackage = null;
			String nsUri = viatraEMFFrameworkManager.getNsUriForViatraPackageElement(viatraEntity);
			ePackage = EPackage.Registry.INSTANCE.getEPackage(nsUri);
			return ePackage;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public EEnum getEEnumForIEntity(IEntity viatraEntity) {
		EEnum result = null;
		if (viatraEntity.isInstanceOf(emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM))) {
			String name = viatraEMFFrameworkManager.getNameForViatraNamedElement(viatraEntity);

			IEntity packageEntity = viatraEntity.getParent();
			if (packageEntity != null) {
				EPackage ePackage = getEPackageForIEntity(packageEntity);
				EClassifier classifier = ePackage.getEClassifier(name);
				result = (EEnum) classifier;
			}

		}

		return result;
	}

	// TODO Nem szp, nem itt kell lennie, a hashMap-nek sem!
	public void addIEntityInternalViatraPair(IModelElement entity, InternalViatraObject object) {
		iEntity2Object.put(entity, object);
	}

}
