package hu.optxware.emf2viatra.core.managers;

import hu.optxware.emf2viatra.core.listener.ManyStructuralFeatureSet;
import hu.optxware.emf2viatra.core.listener.SingleStructuralFeature;
import hu.optxware.emf2viatra.core.listener.SingleStructuralFeatureSet;
import hu.optxware.emf2viatra.utils.NameUtility;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

import org.eclipse.emf.ecore.InternalEObject.EStore;
import org.eclipse.viatra2.core.EDeleteSemantics;
import org.eclipse.viatra2.core.ICoreNotificationListener;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.core.ITransactionManager;
import org.eclipse.viatra2.core.notification.ICoreNotificationObject;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.framework.IFramework;

public class ViatraEMFFrameworkManager {

	private static final boolean cacheForvGetAll = true;
	private static final boolean cacheForvGetAllRelations = true;
	private static final boolean cacheForvGetAllStructuralFeatures = false;

	private int NO_INDEX = EStore.NO_INDEX;

	// private IFramework fw;

	private EMFCoreMetamodelManager emfCoreManager;

	private IModelManager modelManager;

	// TODO �tn�zni!!!
	private ITransactionManager transactionalManager;

	public ViatraEMFFrameworkManager(IFramework fw) {
		super();
		// this.fw = fw;
		emfCoreManager = ViatraEMFManagersFactory.getInstance().getEMFcoreMetamodelManager(fw);
		modelManager = fw.getTopmodel().getModelManager();
		transactionalManager = fw.getTopmodel().getTransactionManager();

		// TODO: Remove Listener in a proper place
		fw.getTopmodel().getNotificationManager().addAllListener(new ICoreNotificationListener() {
			public void actionPerformed(ICoreNotificationObject notification) {
				initCache();
			}

			public int getListenerCategory() {
				// TODO Auto-generated method stub
				return 0;
			}
		});

		initCache();
	}

	private void initCache() {
		if (cache_structuralFeatureList == null) {
			cache_structuralFeatureList = new HashMap<IEntity, HashMap<IRelation, ArrayList<SingleStructuralFeature>>>();
		}
		if (cache_entityList == null) {
			cache_entityList = new HashMap<IEntity, HashMap<IRelation, ArrayList<IEntity>>>();
		}
		if (cache_relationList == null) {
			cache_relationList = new HashMap<IEntity, HashMap<IRelation, ArrayList<IRelation>>>();
		}
		cache_entityList.clear();
		cache_structuralFeatureList.clear();
		cache_relationList.clear();
	}

	// Map for performance tuning
	HashMap<IEntity, HashMap<IRelation, ArrayList<SingleStructuralFeature>>> cache_structuralFeatureList;

	private ArrayList<SingleStructuralFeature> vGetAllStructuralFeatures(IEntity viatraInstance,
			IRelation viatraRelationType) {
		ArrayList<SingleStructuralFeature> results = null;

		HashMap<IRelation, ArrayList<SingleStructuralFeature>> relToList = null;
		if (cacheForvGetAllStructuralFeatures) {
			relToList = cache_structuralFeatureList.get(viatraInstance);
			if (relToList == null) {
				relToList = new HashMap<IRelation, ArrayList<SingleStructuralFeature>>();
				cache_structuralFeatureList.put(viatraInstance, relToList);
			}
			results = relToList.get(viatraRelationType);
		}

		if (results == null) {
			results = new ArrayList<SingleStructuralFeature>();
			if (relToList != null) {
				relToList.put(viatraRelationType, results);
			}

			if (viatraRelationType.isSupertypeOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION))) {
				ManyStructuralFeatureSet featureSet = new ManyStructuralFeatureSet(viatraInstance, viatraRelationType);

				// TODO: VALIDATION!!!!
				results.addAll(featureSet.getStructuralFeatures());
			} else {
				// TODO : itt null kell-e, vagy ez, ami van???
				SingleStructuralFeatureSet featureSet = new SingleStructuralFeatureSet(viatraInstance,
						viatraRelationType);
				SingleStructuralFeature feature = featureSet.getValid();
				if (feature != null) {
					results.add(feature);
				}
			}
		}
		return results;
	}

	public IEntity vGet(IEntity viatraInstance, IRelation viatraRelationType, int index) {
		ArrayList<IEntity> entities = vGetAll(viatraInstance, viatraRelationType);
		if (index == NO_INDEX && entities.size() == 1) {
			return entities.get(0);
		} else if (index != NO_INDEX && entities.size() > index) {
			return entities.get(index);
		} else {
			return null;
		}
	}

	private IRelation vGetRelation(IEntity viatraInstance, IRelation viatraRelationType, int index) {
		ArrayList<IRelation> relations = vGetAllRelations(viatraInstance, viatraRelationType);
		if (index == NO_INDEX && relations.size() == 1) {
			return relations.get(0);
		} else if (index != NO_INDEX && relations.size() > index) {
			return relations.get(index);
		} else {
			return null;
		}
	}

	public void vSet(IEntity viatraInstance, IRelation viatraStructuralFeature, int index, IEntity value)
			throws VPMCoreException {
		try {
			transactionalManager.beginTransaction();

			IRelation relation = vGetRelation(viatraInstance, viatraStructuralFeature, index);

			if (relation == null && index != NO_INDEX) {
				// TODO: Check this
				throw new IndexOutOfBoundsException();
			}
			if (relation != null) {
				boolean isOldEntityHasToBeDeleted = false;
				IEntity oldEntity = vGet(viatraInstance, viatraStructuralFeature, index);
				Collection<IModelElement> types = oldEntity.getTypes();
				IEntity typeOfOldEntity = null;
				if (!types.isEmpty()) {
					typeOfOldEntity = (IEntity) types.toArray()[0];
				}
				if (typeOfOldEntity.isInstanceOf(emfCoreManager
						.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_EDATATYPE))
						&& !typeOfOldEntity.isInstanceOf(emfCoreManager
								.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM))) {
					// We need to delete the old element in case of simple data
					// type because we created a new one.
					isOldEntityHasToBeDeleted = true;
				}

				modelManager.setRelationTo(relation, value);

				if (isOldEntityHasToBeDeleted) {
					modelManager.deleteElement(oldEntity, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
				}
				// in this case relation = null, index = NO_INDEX
			} else {
				IRelation newRel = modelManager.newRelation(viatraInstance, value);
				modelManager.newInstanceOf(viatraStructuralFeature, newRel);
			}

			// Set the name of the entity in case name attribute
			// TODO : Check whether the value is not null. AND IT HAS TO BE DONE
			// GLOBALLY
			if (getNameForViatraStructuralFeature(viatraStructuralFeature).toLowerCase().equals("shortname")) {
				if (value.getValue() != "") {
					modelManager.setName(viatraInstance, value.getValue());
				}
			}

			// move the value under viatraInstance if the
			// viatraStructuralFeature is
			// attribute but not Enum or reference with container = true
			IModelElement referencedType = viatraStructuralFeature.getTo();
			boolean isEEnum = referencedType.isInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM));
			boolean isAttribute = viatraStructuralFeature.isInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE));
			boolean isReferenceAndContainer = viatraStructuralFeature.isInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE))
					&& isContainmentForViatraEReference(viatraStructuralFeature) == true;
			if ((isAttribute && !isEEnum) || isReferenceAndContainer) {
				modelManager.moveEntityTo(value, viatraInstance);
			}
		} finally {
			transactionalManager.commitTransaction();
		}
	}

	public void vAdd(IEntity viatraInstance, IRelation viatraStructuralFeature, int index, IEntity value)
			throws VPMCoreException {

		if (index == NO_INDEX) {
			throw new UnsupportedOperationException();
		}
		if (index > vGetSize(viatraInstance, viatraStructuralFeature)) {
			throw new IndexOutOfBoundsException();
		}

		try {
			transactionalManager.beginTransaction();
			IRelation actualRelAtPos = vGetRelation(viatraInstance, viatraStructuralFeature, index);

			IRelation newRel = modelManager.newRelation(viatraInstance, value);

			if (actualRelAtPos != null) {
				// Redirect next from actual relation at index
				IRelation nextTo = getNextTo(actualRelAtPos);
				modelManager.setRelationTo(nextTo, newRel);
				// Create new next from new relation to actual relation
				IRelation newNextFrom = modelManager.newRelation(newRel, actualRelAtPos);
				modelManager.newInstanceOf(emfCoreManager
						.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT), newNextFrom);
			} else {
				if (index > 0) {
					IRelation prevRelation = vGetRelation(viatraInstance, viatraStructuralFeature, index - 1);
					// Create new next from previous relation to new relation
					IRelation newNextFrom = modelManager.newRelation(prevRelation, newRel);
					modelManager.newInstanceOf(emfCoreManager
							.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT),
							newNextFrom);
				}
			}

			// Note: We have to set the instance after the setting/modifying of
			// the next edges
			modelManager.newInstanceOf(viatraStructuralFeature, newRel);

			// move the value under viatraInstance if the
			// viatraStructuralFeature is
			// attribute but not Enum or reference with container = true
			IModelElement referencedType = viatraStructuralFeature.getTo();
			boolean isEEnum = referencedType.isInstanceOf(emfCoreManager
					.getIEntityForEcoreElement(EMFCoreEntConst.ECORE_DATATYPES_EENUM));
			boolean isAttribute = viatraStructuralFeature.isInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE));
			boolean isReferenceAndContainer = viatraStructuralFeature.isInstanceOf(emfCoreManager
					.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE))
					&& isContainmentForViatraEReference(viatraStructuralFeature) == true;
			if ((isAttribute && !isEEnum) || isReferenceAndContainer) {
				modelManager.moveEntityTo(value, viatraInstance);
			}

		} finally {
			transactionalManager.commitTransaction();
		}

	}

	public void vAddToResource(IEntity viatraInstance) throws VPMCoreException {
		transactionalManager.beginTransaction();
		modelManager.moveEntityTo(viatraInstance, emfCoreManager.getIEntityForEcoreElement(EMFCoreEntConst.RESOURCES));
		transactionalManager.commitTransaction();
	}

	public void vRemoveFromResource(IEntity viatraInstance) throws VPMCoreException {
		transactionalManager.beginTransaction();
		// TODO : ez a szemantika kell?
		modelManager.deleteElement(viatraInstance, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
		transactionalManager.commitTransaction();
	}

	public int vGetSize(IEntity viatraInstance, IRelation viatraStructuralFeature) {
		if (viatraStructuralFeature.isSupertypeOf(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION))) {
			return vGetAll(viatraInstance, viatraStructuralFeature).size();
		} else {
			// Megj.: Most ua. Nagy valoszinuseggel az is marad
			return vGetAll(viatraInstance, viatraStructuralFeature).size();
		}
	}

	public IEntity createData(IEntity type, String value) throws VPMCoreException {
		transactionalManager.beginTransaction();
		IEntity newData = modelManager.newEntity(NameUtility.convertToValidViatraName(value), value);
		modelManager.newInstanceOf(type, newData);
		transactionalManager.commitTransaction();
		return newData;
	}

	public IEntity createEObject(IEntity type) throws VPMCoreException {
		transactionalManager.beginTransaction();
		IEntity newEntity = modelManager.newEntity();
		modelManager.newInstanceOf(type, newEntity);
		// modelManager.newSupertypeOf(emfCoreManager.
		// getIEntityForEcoreElement(
		// EMFCoreEntConst.ECORE_EOBJECT), newEntity);
		transactionalManager.commitTransaction();
		return newEntity;
	}

	// Map for performance tuning
	HashMap<IEntity, HashMap<IRelation, ArrayList<IEntity>>> cache_entityList;

	public ArrayList<IEntity> vGetAll(IEntity viatraInstance, IRelation viatraRelationType) {

		ArrayList<IEntity> results = null;

		HashMap<IRelation, ArrayList<IEntity>> relToList = null;
		if (cacheForvGetAll) {
			relToList = cache_entityList.get(viatraInstance);
			if (relToList == null) {
				relToList = new HashMap<IRelation, ArrayList<IEntity>>();
				cache_entityList.put(viatraInstance, relToList);
			}
			results = relToList.get(viatraRelationType);
		}

		if (results == null) {
			results = new ArrayList<IEntity>();
			if (relToList != null) {
				relToList.put(viatraRelationType, results);
			}
			for (SingleStructuralFeature feature : vGetAllStructuralFeatures(viatraInstance, viatraRelationType)) {
				results.add(feature.getValueInstance());
			}
		}

		return results;
	}

	// Map for performance tuning
	HashMap<IEntity, HashMap<IRelation, ArrayList<IRelation>>> cache_relationList;

	private ArrayList<IRelation> vGetAllRelations(IEntity viatraInstance, IRelation viatraRelationType) {

		ArrayList<IRelation> results = null;

		HashMap<IRelation, ArrayList<IRelation>> relToList = null;
		if (cacheForvGetAllRelations) {
			relToList = cache_relationList.get(viatraInstance);
			if (relToList == null) {
				relToList = new HashMap<IRelation, ArrayList<IRelation>>();
				cache_relationList.put(viatraInstance, relToList);
			}
			results = relToList.get(viatraRelationType);
		}

		if (results == null) {
			results = new ArrayList<IRelation>();
			if (relToList != null) {
				relToList.put(viatraRelationType, results);
			}
			for (SingleStructuralFeature feature : vGetAllStructuralFeatures(viatraInstance, viatraRelationType)) {
				results.add(feature.getRelationInstance());
			}
		}

		return results;
	}

	private String getReferencedAttributeValueForViatraEntity(IModelElement viatraElement, IRelation relationType) {
		String nameStr = null;
		Collection<IModelElement> names = viatraElement.getAllRelationTargetByType(relationType);
		if (names != null && names.size() == 1) {
			IEntity nameEnt = (IEntity) names.toArray()[0];
			nameStr = nameEnt.getValue();
		}
		return nameStr;
	}

	private boolean getReferencedBooleanAttributeValueForViatraEntity(IModelElement viatraElement,
			IRelation relationType) {
		IEntity nameEnt = null;
		Collection<IModelElement> names = viatraElement.getAllRelationTargetByType(relationType);
		if (names != null && names.size() == 1) {
			nameEnt = (IEntity) names.toArray()[0];
		} else {
			// throw ecteption!!!
			return false;
		}
		return nameEnt.equals(modelManager.getElementByName("datatypes.Boolean.true")) ? true : false;
	}

	public String getNameForViatraNamedElement(IEntity namedElement) {
		return getReferencedAttributeValueForViatraEntity(namedElement, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ENAMEDELEMENT_NAME));
	}

	public String getNameForViatraStructuralFeature(IRelation structuralFeature) {
		return getReferencedAttributeValueForViatraEntity(structuralFeature, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_NAME));
	}

	public boolean isContainmentForViatraEReference(IRelation eReference) {
		return getReferencedBooleanAttributeValueForViatraEntity(eReference, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE_CONTAINMENT));
	}

	public boolean isManyForViatraEStructuralFeature(IRelation eReference) {
		return getReferencedBooleanAttributeValueForViatraEntity(eReference, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_ESTRUCTURALFEATURE_MANY));
	}

	public String getNsUriForViatraPackageElement(IEntity viatraPackage) {
		return getReferencedAttributeValueForViatraEntity(viatraPackage, emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EPACKAGE_NSURI));
	}

	public IEntity vGetContainer(IEntity entity) {
		if (vGetContainingRelationType(entity) == null) {
			return null;
		} else {
			return entity.getParent();
		}
	}

	public IRelation vGetContainingRelationType(IEntity viatraInstance) {
		IEntity container = viatraInstance.getParent();
		Collection<IRelation> relationsFrom = container.getRelationsFrom();

		for (IRelation rel : relationsFrom) {
			if (viatraInstance.equals(rel.getTo())) {
				return (IRelation) rel.getTypes().toArray()[0];
			}
		}

		return null;
	}

	// Viatra model manipulation stuffs:

	public void newSupertypeOf(IModelElement sup, IModelElement sub) {
		try {
			modelManager.newSupertypeOf(sup, sub);
		} catch (VPMCoreException e) {
			e.printStackTrace();
		}

	}

	public IRelation newRelation(IModelElement source, IModelElement target) {
		try {
			IRelation element = modelManager.newRelation(source, target);
			return element;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	public IRelation newRelation(IModelElement source, IModelElement target, String name) {
		try {
			IRelation element = modelManager.newRelation(source, target);
			if (name != null) {
				modelManager.setName(element, NameUtility.convertToValidViatraName(name));
			}
			return element;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	public void newInstanceOf(IModelElement type, IModelElement instance) {
		try {
			modelManager.newInstanceOf(type, instance);
		} catch (VPMCoreException e) {
			e.printStackTrace();
		}
	}

	public IEntity newEntity(IEntity container) {
		try {
			IEntity element = modelManager.newEntity(container);
			return element;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	public IEntity newEntity(IEntity container, String name) {
		try {
			IEntity element = modelManager.newEntity(container);
			if (name != null) {
				modelManager.setName(element, NameUtility.convertToValidViatraName(name));
			}
			return element;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	public IEntity newEntity(IEntity container, String name, String value) {
		try {
			IEntity element = modelManager.newEntity(container);
			if (name != null) {
				modelManager.setName(element, NameUtility.convertToValidViatraName(name));
			}
			modelManager.setValue(element, value);
			return element;
		} catch (VPMCoreException e) {
			e.printStackTrace();
			return null;
		}
	}

	// Csak "datatypes.Boolean.true" jelleg�t k�r�nk igy ki!
	// Azokra kell egyedi
	public IEntity getEntityByName(String name) {
		IEntity element = modelManager.getEntityByName(name);
		return element;
	}

	public void vClear(IEntity viatraInstance, IRelation viatraRelationType) throws VPMCoreException {
		for (IModelElement featureToDelete : viatraInstance.getAllRelationFromByType(viatraRelationType)) {
			removeStructuralFeatureInstance((IRelation) featureToDelete);
		}
	}

	private void removeStructuralFeatureInstance(IRelation featureToDelete) throws VPMCoreException {
		transactionalManager.beginTransaction();

		IEntity entityToDelete = (IEntity) featureToDelete.getTo();

		if (featureToDelete.getTypes().size() == 0)
			return;

		IRelation viatraRelationType = (IRelation) featureToDelete.getTypes().toArray()[0];

		if (featureToDelete.isInstanceOf(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION))) {
			IRelation relNextFrom = getNextFrom(featureToDelete);
			IRelation relNextTo = getNextTo(featureToDelete);
			IRelation nextRel = getNext(featureToDelete);

			if (relNextTo != null && nextRel != null) {
				// Set the next point to featureToDelete for the next of
				// featureToDelete
				modelManager.setRelationTo(relNextTo, nextRel);
			} else if (relNextTo != null && nextRel == null) {
				modelManager.deleteElement(relNextTo, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
			}

			if (relNextFrom != null) {
				modelManager.deleteElement(relNextFrom, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
			}
		}

		// If the relation was containment or an attribute, delete the
		// entity otherwise just the relation
		boolean isAttribute = viatraRelationType.isInstanceOf(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EATTRIBUTE));
		boolean isReferenceAndContainer = viatraRelationType.isInstanceOf(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_ECLASS_EREFERENCE))
				&& isContainmentForViatraEReference(viatraRelationType) == true;
		if (isAttribute || isReferenceAndContainer) {
			modelManager.deleteElement(entityToDelete, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
		} else {
			modelManager.deleteElement(featureToDelete, EDeleteSemantics.DELETE_SEMANTICS_BRANCH_FORCE);
		}

		transactionalManager.commitTransaction();
	}

	public boolean vContains(IEntity viatraInstance, IRelation viatraRelationType, IEntity entityValue) {
		return vGetAll(viatraInstance, viatraRelationType).contains(entityValue);
	}

	public void vRemove(IEntity viatraInstance, IRelation viatraStructuralFeature, int index) throws VPMCoreException {
		if (vGetSize(viatraInstance, viatraStructuralFeature) <= index) {
			throw new IndexOutOfBoundsException();
		}
		transactionalManager.beginTransaction();
		IRelation rel = vGetRelation(viatraInstance, viatraStructuralFeature, index);

		removeStructuralFeatureInstance(rel);

		transactionalManager.commitTransaction();
	}

	public int vIndexOf(IEntity viatraInstance, IRelation viatraRelationType, IEntity entityValue) {
		return vGetAll(viatraInstance, viatraRelationType).indexOf(entityValue);
	}

	public int vLastIndexOf(IEntity viatraInstance, IRelation viatraRelationType, IEntity entityValue) {
		return vGetAll(viatraInstance, viatraRelationType).lastIndexOf(entityValue);
	}

	public void vUnset(IEntity viatraInstance, IRelation viatraRelationType) throws VPMCoreException {
		vClear(viatraInstance, viatraRelationType);
	}

	public boolean vIsEmpty(IEntity viatraInstance, IRelation viatraStructuralFeature) {
		return vGetSize(viatraInstance, viatraStructuralFeature) == 0 ? true : false;
	}

	public boolean vIsSet(IEntity viatraInstance, IRelation viatraStructuralFeature) {
		return vGetSize(viatraInstance, viatraStructuralFeature) == 0 ? false : true;
	}

	// Getter methods for previous and next elements ///////////////////////////
	private IRelation getNextTo(IRelation orderedRelation) {
		return getNextTo(orderedRelation, null, null);
	}

	private IRelation getNextTo(IRelation orderedRelation, IRelation excludingNextRel, IRelation includingNextRel) {
		IRelation result = null;

		// if (orderedRelation.isInstanceOf(emfCoreManager
		// .getIRelationForEcoreElement(EMFCoreRelConst.
		// ECORE_EOBJECT_ORDEREDRELATION))) {
		Collection<IRelation> nextTo = orderedRelation.getAllRelationToByType(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT));
		if (excludingNextRel != null && nextTo.contains(excludingNextRel)) {
			nextTo.remove(excludingNextRel);
		}
		if (includingNextRel != null && !nextTo.contains(includingNextRel)) {
			nextTo.add(includingNextRel);
		}
		if (nextTo.size() == 1 && nextTo.toArray()[0] instanceof IRelation) {
			result = (IRelation) nextTo.toArray()[0];
		}
		// }

		return result;
	}

	private IRelation getNextFrom(IRelation orderedRelation) {
		return getNextFrom(orderedRelation, null, null);
	}

	private IRelation getNextFrom(IRelation orderedRelation, IRelation excludingNextRel, IRelation includingNextRel) {
		IRelation result = null;

		// if (orderedRelation.isInstanceOf(emfCoreManager
		// .getIRelationForEcoreElement(EMFCoreRelConst.
		// ECORE_EOBJECT_ORDEREDRELATION))) {
		Collection<IRelation> nextFrom = orderedRelation.getAllRelationFromByType(emfCoreManager
				.getIRelationForEcoreElement(EMFCoreRelConst.ECORE_EOBJECT_ORDEREDRELATION_NEXT));
		if (excludingNextRel != null && nextFrom.contains(excludingNextRel)) {
			nextFrom.remove(excludingNextRel);
		}
		if (includingNextRel != null && !nextFrom.contains(includingNextRel)) {
			nextFrom.add(includingNextRel);
		}
		if (nextFrom.size() == 1 && nextFrom.toArray()[0] instanceof IRelation) {
			result = (IRelation) nextFrom.toArray()[0];
		}
		// }

		return result;
	}

	public IRelation getPrev(IRelation orderedRelation) {
		return getPrev(orderedRelation, null, null);
	}

	public IRelation getPrev(IRelation orderedRelation, IRelation excludingNextRel, IRelation includingNextRel) {
		IRelation nextTo = getNextTo(orderedRelation, excludingNextRel, includingNextRel);
		if (nextTo != null && nextTo.getFrom() instanceof IRelation) {
			return (IRelation) nextTo.getFrom();
		}
		return null;
	}

	public IRelation getNext(IRelation orderedRelation) {
		return getNext(orderedRelation, null, null);
	}

	public IRelation getNext(IRelation orderedRelation, IRelation excludingNextRel, IRelation includingNextRel) {
		IRelation nextTo = getNextFrom(orderedRelation, excludingNextRel, includingNextRel);
		if (nextTo != null && nextTo.getTo() instanceof IRelation) {
			return (IRelation) nextTo.getTo();
		}
		return null;
	}
}
