/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Balogh and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Balogh - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.imports.vtml;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.core.simple.SimpleModelManager;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.logger.Logger;

import java.util.*;
/**
 * @author babo
 *
*/
public class ASTBuilder {


	Vector imps;
	String namespac = null;
	public Logger log;
	Vector machines;
	
	IEntity ent;
	//Vector elemdefs;
	Vector reldefs;
	IModelSpace ms;
	
	HashSet<EntityDefinition> entities = null;
	
	HashSet<EntityDefinition> entity_tmp = null;
	HashSet<RelationDefinition> relation_tmp = null;
	
	
	
	public String getNamespace()
	{
		return namespac;
	}

	public void init(Logger l, IModelSpace m)
	{
		ms = m;
		imps = new Vector();
		
		entities = new HashSet<EntityDefinition>();
		entity_tmp = new HashSet<EntityDefinition>();
		relation_tmp =new HashSet<RelationDefinition>();
		//elemdefs = new Vector();
		reldefs = new Vector();
		machines = new Vector();
		
		namespac = "";
		
		mm = m.getModelManager();
		log=l;
		
		try
		{
		VTMLMetaModelHelper.init(l,m,this);
		}
		catch (Exception e)
		{
			log.fatal("VTML Importer initialization failed. All further imports will fail. Cause : "+e.getLocalizedMessage());
		}
	}
	

	public void addSupertypeDefinition(int l, String ns, String m, String s)
	{
		
			reldefs.add(new SupertypeDefinition(l,ns,m,s));
		
	}

	public void addTypeDefinition(int l, String ns, String m, String s)
	{
		
			reldefs.add(new TypeDefinition(l,ns,m,s));
		
	}	
	
	public void addIsFinalTypeDefinition(int l, String ns,String n, boolean b)
	{
		reldefs.add(new FinalTypeDefinition(l,ns,n,b));
	}
	
	public void addIsAggregationDefinition(int l,String ns, String n, boolean b)
	{
		reldefs.add(new AggregationDefinition(l,ns,n,b));
	}	
	
	public void addIsAnySourceDefinition(int l,String ns, String n, boolean b)
	{
		reldefs.add(new AnySourceDefinition(l,ns,n,b));
	}	
	
	public void addIsAnyTargetDefinition(int l,String ns, String n, boolean b)
	{
		reldefs.add(new AnyTargetDefinition(l,ns,n,b));
	}	
	
	public void addInverseDefinition(int l, String ns,String n, String i)
	{
		reldefs.add(new InverseDefinition(l,ns,n,i));
	}	
	
	public void addMultiplicityDefinition(int l, String ns,String n, String i)
	{
		reldefs.add(new MultiplicityDefinition(l,ns,n,i));
	}
	
	public IEntity lookupNamespace(String ns)
	{
		IEntity e = null;
		if (ns.equals("")) return mm.getRoot(); 
		e = mm.getEntityByName(ns);
		if (e!=null) return e;
		//if (!namespac.equals("")) e = mm.getEntityByName(namespac+"."+ns);
		//if (e!=null) return e;
		for (int i=0;i<imps.size();i++)
		{
			NamespaceImport ni = (NamespaceImport)imps.get(i);
			e = mm.getEntityByName(ni.getName()+"."+ns);
			if (e!=null) return e;
		}
		
		return null;
	}
	private boolean preamb(String n, String e)
	{
		return ((n.startsWith(e)) && (n.length()>e.length()) && (n.charAt(e.length())=='.'));
	}
	private EntityDefinition getEntityByQN(String n, HashSet<EntityDefinition> hs)
	{
		for (EntityDefinition e : hs)
		{
			//log.info("n = '"+n+"'  e.fqn='"+e.getFQN()+"'");
			if (n.equals(e.getFQN())) return e;
			if (preamb(n,e.getFQN()))
			{
				return getEntityByQN(n,e.getContent());
			}
		}
		return null;
	}
	
	public EntityDefinition lookupDuplicateEntity(EntityDefinition ed)
	{
		return getEntityByQN(ed.getFQN(),entities,ed);
	}
	private EntityDefinition getEntityByQN(String n, HashSet<EntityDefinition> hs,EntityDefinition self)
	{
		for (EntityDefinition e : hs)
		{
			if ((n.equals(e.getFQN())) && (e!=self)) return e;
			if (preamb(n,e.getFQN()))
			{
				return getEntityByQN(n,e.getContent(),self);
			}
		}
		return null;
	}
	
	public EntityDefinition getEntityWithLookup(String ns,String n)
	{
		log.debug("called entityWithlookup: "+ns+" name: "+n);
		log.debug("lookup ns: "+ns+" name: "+n);
		EntityDefinition ed = null;
		if (n==null) return null;
		if (n.indexOf('.')>=0) 
		{
			ed = getEntityByQN(n,entities);
			log.debug("lookup: "+n+" res: "+ed);
			if  (ed!=null) return ed;
		}
		while ((ed==null)&& ( ns!=null))
		{
			
			ed = getEntityByQN(fqn(ns,n),entities);
			if (ed!=null) return ed;
			ns = parentName(ns);
		}
		IEntity ee = mm.getEntityByName(fqn(ns,n));
		if (ee!=null)
		{
			ed = new EntityDefinition(ee);
			try
			{
			buildEntityTree(ed);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			return ed;
		}
		for (int i=0;i<imps.size();i++)
		{
			NamespaceImport ni = (NamespaceImport)imps.get(i);
			log.debug("lookup import: "+ni.getName()+" name: "+n);
			IEntity e = mm.getEntityByName(ni.getName()+"."+n);
			if (e!=null)
			{
				ed = new EntityDefinition(e);
				try
				{
				buildEntityTree(ed);
				}
				catch (Exception exc)
				{
					//TODO
					exc.printStackTrace();
				}
				return ed;
			}
		}
		
		
		return null;
	}
	
	
	public ElementDefinition getElementWithLookup(String ns, String n)
	{
		ElementDefinition ed = getEntityWithLookup(ns,n);
		if (ed!=null) return ed;
		return getRelationWithLookup(ns,n);
	}
	public RelationDefinition getRelationWithLookup(String ns,String n)
	{
		RelationDefinition ed = null;
		if (n.indexOf('.')>=0) 
			{
			ElementDefinition ted = getElementByQN(n,entities);
			if ((ted!=null) && (ted instanceof RelationDefinition)) return (RelationDefinition)ted;
			
			}
		String nstmp = ns;
		while ((ed==null)&& ( nstmp!=null))
		{
			
			ElementDefinition ted = getElementByQN(fqn(nstmp,n),entities);
			if ((ted!=null) && (ted instanceof RelationDefinition)) return (RelationDefinition)ted;
			nstmp = parentName(nstmp);
		}
		
		IRelation ee = mm.getRelationByName(fqn(ns,n));
		if (ee!=null)
		{
			ed = new RelationDefinition(ee);
			try
			{
			addRelationToTree(ed);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			return ed;
		}
		ee = mm.getRelationByName(n);
		if (ee!=null)
		{
			ed = new RelationDefinition(ee);
			try
			{
			addRelationToTree(ed);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			return ed;
		}
		for (int i=0;i<imps.size();i++)
		{
			NamespaceImport ni = (NamespaceImport)imps.get(i);
			IRelation e = mm.getRelationByName(ni.getName()+"."+n);
			if (e!=null)
			{
				ed = new RelationDefinition(e);
				try
				{
					addRelationToTree(ed);
				}
				catch (Exception exc)
				{
					//TODO
				}
				return ed;
			}
		}
		
		
		return null;
	}
	/*
	 * 		for (int i=0;i<imps.size();i++)
		{
			NamespaceImport ni = (NamespaceImport)imps.get(i);
			e = mm.getEntityByName(ni.getName()+"."+n);
			if (e!=null) return e;
		}
	 */
	
	private RelationDefinition getRelationByQN(String n, HashSet<RelationDefinition> hs)
	{
		for (RelationDefinition e : hs)
		{
			if (n.equals(e.getFQN())) return e;
			if (preamb(n,e.getFQN()))
			{
				return getRelationByQN(n,e.getRelations());
			}
		}
		return null;
	}
	
	private ElementDefinition getElementByQN(String n, HashSet<EntityDefinition> hs)
	{
		for (EntityDefinition e : hs)
		{
			if (n.equals(e.getFQN())) return e;
			//String tmp = n.substring(e.getFQN().length());
			
			if (n.startsWith(e.getFQN()))
			{
				ElementDefinition ed =  getElementByQN(n,e.getContent());
				if (ed==null)
				{
					for (RelationDefinition r : e.relations)
					{
						if (r.getFQN().equals(n)) return r;
						if (r.getFQN().startsWith(n))
						{
							RelationDefinition rtmp = getRelationByQN(n,r.getRelations());
							if (rtmp!=null) return rtmp;
						}
					}
				} else return ed;
			}
		}
		return null;
	}
	public boolean namespaceExists(String ns)
	{
		if (ns.equals("")) return true;
		if (lookupNamespace(ns)!=null) return true;
		
		return (getEntityByQN(ns,entities)!=null);
		
	}

	

	
	

	
	public String fqn(String ns, String n)
	{
		if (ns==null) return n;
		if (ns.length()==0) return n;
		//if (n.indexOf('.')>=0) return n;
		return ns+"."+n;
	}
	public void addEntityDef(int l, String ns, String n, String t, String v)
	{
	
			
		 entity_tmp.add(new EntityDefinition(l,ns,n,t,v));
		
	}
	public void addRelationDef(int l, String ns, String n, String t, String s,String ta)
	{
		RelationDefinition rd = new RelationDefinition(l,ns,n,t,s,ta);
	
			relation_tmp.add(rd);
		
	}
	
	
	public void addImport(int l,String s)
	{ 
		imps.add(new NamespaceImport(l,s));
	}
	
	public void setNamespace(String s)
	{
		namespac = s;
	}
	
	public void debug()
	{
		log.debug("Module namespace : "+namespac);
		log.debug("\nimports:");
		printEntities(entities,0);
		for (int i=0;i<imps.size();i++) log.debug("     "+((NamespaceImport)imps.get(i)).getName());
		//for (int i=0;i<elemdefs.size();i++) log.debug("Elem : "+elemdefs.get(i).toString());
	}
	
	public void printEntities(HashSet<EntityDefinition> e, int i)
	{
		for (EntityDefinition ed : e)
		{
			log.debug("                                                                                                                    ".substring(0,i)+ed.getFQN());
			printEntities(ed.getContent(),i+2);
		}
	}
	
	IModelManager mm = null;
	
	public String parentName(String fqn)
	{
		if (fqn==null) return null;
		if (fqn.equals("")) return null;
		int i = fqn.lastIndexOf('.');
		if (i<0) return "";
		return fqn.substring(0,i);
	}
	
	public EntityDefinition getFromTmp(String _fqn)
	{
		for (EntityDefinition e : entity_tmp)
		{
			if (_fqn.equals(fqn(e.getNamespace(),e.getName()))) return e;
		}
		
		return null;
	}
	public void buildEntityTree(EntityDefinition ed) throws VPMRuntimeException
	{
		String ns = ed.getNamespace();
		String n = ed.getName();
		String fqn = fqn(ns,n);
		ed.setProcessed(true);
		
		if ((mm.getEntityByName(fqn)!=null) && ((!ed.isNative)))
		{
		
			log.error("duplicate entity found! ("+fqn+") error in line "+ed.getLine());
			throw new VPMRuntimeException("duplicate entity found! ("+fqn+") error in line "+ed.getLine());
		}
		EntityDefinition etmp = getEntityByQN(fqn,entities);
		if ((etmp!=null) && (ed!=etmp) && !(ed.isNative && etmp.isNative))
		{
			//debug();
			log.error("duplicate entity found! ("+fqn+") error in line "+ed.getLine());
			throw new VPMRuntimeException("duplicate entity found! ("+fqn+") error in line "+ed.getLine());
		}
		
		if (etmp!=null) return;
		else 
		{
			String par = parentName(fqn);
			if ((par==null) || (par.equals("")))
			{
				entities.add(ed);
				return;
			}
			EntityDefinition ep = getFromTmp(par);

			if (ep==null) ep = getEntityByQN(par,entities);
			if (ep==null)
			{
				IEntity ee = mm.getEntityByName(par);
				if (ee==null)
				{
					log.error("parent entity "+par+" for entity "+fqn+" is undefined.");
					throw new VPMRuntimeException("entity undefined: "+fqn);
				}
				ep = new EntityDefinition(ee);
				buildEntityTree(ep);
				
			}
			
			ep.addContent(ed);
		}
		
	}
	
	public String localName(String s)
	{
		int i = s.lastIndexOf('.');
		if (i<0) return s;
		return s.substring(i+1);
	}
	/**
	 * @param d
	 * @throws VPMRuntimeException
	 */
	private void addRelationToTree(RelationDefinition d) throws VPMRuntimeException
	{
		String ns = d.getNamespace();
		String n  = d.getName();
		String en = d.getSource();
		String sfqn = fqn(ns,en);
		
		String tmpns = ns;
		String realFqn = fqn(sfqn,localName(n));
		ElementDefinition ed = null;
		d.setProcessed(true);
		ed = getElementByQN(fqn(ns,n),entities);
		
		if ((mm.getRelationByName(realFqn)!=null) && (!d.isNativ))
		{
			log.error("Duplicate relation ("+fqn(ns,n)+") found. Error in line "+d.getLine());
		}
		if ((ed!=null) && ((ed!=d)))
		{
			log.error("Duplicate relation ("+ed.getFQN()+") found. Error in line "+d.getLine());
		}
		
		while ((ed==null) && (tmpns!=null))
		{
			
			ed = getElementByQN(fqn(tmpns,en),entities);
			
			tmpns=parentName(tmpns);
			
		}
		
		if (ed==null)
		{
			for (RelationDefinition rr : relation_tmp)
			{
				if (!rr.isProcessed())
				{
				
				if (rr.getName().equals(localName(en)))
				{
					addRelationToTree(rr);
					ed = getElementByQN(en,entities);
					if (ed!=null) break;
				}
				}
			}
			if (ed==null)
			{
				
				
				ed = getElementWithLookup(ns,en);
				
				if (ed==null)
				{
					log.error("Source ("+sfqn+") for relation "+n+" undefined.");
					throw new VPMRuntimeException("error. relation source undefined.");
				}
//				if (ee instanceof IEntity)
//				{
//					ed = new EntityDefinition((IEntity)ee);
//					buildEntityTree((EntityDefinition)ed);
//				}
//				else
//				{
//					ed = new RelationDefinition((IRelation)ee);
//					addRelationToTree((RelationDefinition)ed);
//				}
			}
		}
		ed.addRelation(d);
	}
	public boolean checkDefinition()
	{
		
		
		
		if (mm.getEntityByName(namespac)==null)
		{
			log.error("Namespace "+namespac+" not found. Invalid namespace reference.");
			return false;
		}
		
		for (int i=0;i<imps.size();i++)
		{
			NamespaceImport ni = (NamespaceImport)imps.get(i);
			if (!ni.checkDefintion(this,ms)) return false;
		}
		for (EntityDefinition e : entity_tmp)
		{
			e.normalizeName(this);
		}
		
		for (EntityDefinition e : entity_tmp)
		{
			try
			{
			 if (!e.isProcessed()) buildEntityTree(e);
			}
			catch (VPMRuntimeException ex)
			{
				return false;
			}
		}
		
		for (RelationDefinition r : relation_tmp)
		{
			try
			{
				if (!r.isProcessed()) addRelationToTree(r);
			}
			catch (VPMRuntimeException ex)
			{
				return false;
			}
		}
		
		Collection<EntityDefinition> eetmp = (Collection<EntityDefinition>)entities.clone();	
		for (EntityDefinition e : eetmp)
		{
			
				if (!e.checkDefintion(this,ms)) return false;
			
		
		}
		
		for (int i=0;i<reldefs.size();i++)
		{
			if (reldefs.get(i) instanceof RelationshipDefinition)
			{
				RelationshipDefinition ni = (RelationshipDefinition)reldefs.get(i);
				if (!ni.checkDefintion(this,ms)) return false;
			}
		}
		log.debug("end "+mm.getElementByName("reldb"));
		return true;
	}

	public void generateElements() throws VPMCoreException
	{
		//IEntity tmp = mm.newEntity();

		for (EntityDefinition e : entities)
		{
			e.updateModelspace(this,ms);
		}
	
		for (int i=0;i<reldefs.size();i++)
		{
			if (reldefs.get(i) instanceof RelationshipDefinition)
			{
				RelationshipDefinition ni = (RelationshipDefinition)reldefs.get(i);
				ni.resolveReferences(this);
			}
		}	
	}
	
	
	
	int nameID= 1;
	public String nameGenerator()
	{
		return "_tmp"+((nameID++));
	}
}
