/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Balogh and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Balogh - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.imports.vtml;

import org.eclipse.viatra2.logger.Logger;

import java.util.*;
import java.io.InputStream;
import java.io.FileInputStream;

public class VTMLParser implements VTMLParserConstants {



  public static final String progname = "VPM Parser Version 3.0.1";

  public VTMLParser()
  {

  }

  static VTMLParser parser = null;
  static org.eclipse.viatra2.logger.Logger log;

  static int line = 0;
  static ASTBuilder ab = null;

    public void processFile(InputStream inp,ASTBuilder a, Logger l) throws Exception
   {
         log=l;
         ab = a;


                 if (parser==null)
                 parser = new VTMLParser(inp);
                 else parser.ReInit(inp);

           try {
                 parser.specification();
                 log.info(progname+":  Program parsed successfully.");
                 inp.close();
                 ab = null;
                 log = null;
           } catch (ParseException e) {
                 log.error(progname+":  error parsing file. Cause: "+e.getMessage());
                 inp.close();
                 ab = null;
                 log = null;
                 throw new Exception();
           }
           catch (Throwable e)
           {
            log.error("Parser error in line: "+line);
            log.error("Message: "+e.getMessage());
                         log = null;
            ab = null;
            throw new Exception();
           }
                 ab = null;
                 log = null;

   }
   public void processFile(String f,ASTBuilder a, Logger l) throws Exception
  {
        log=l;
        ab = a;
        FileInputStream inp = null;
  try {
        inp = new java.io.FileInputStream(f);
                if (parser==null)
                parser = new VTMLParser(inp);
                else parser.ReInit(inp);
          } catch (java.io.FileNotFoundException e) {
                log.error(progname+":  File " + f + " not found.");
                ab = null;
                return;
  }
          try {
                parser.specification();
                log.info(progname+":  Program parsed successfully.");
                inp.close();
                ab = null;
                log = null;
          } catch (ParseException e) {
                log.error(progname+":  error parsing file. Cause: "+e.getMessage());
                inp.close();
                ab = null;
                log = null;
                throw new Exception();
          }
          catch (Throwable e)
          {
           log.error("Parser error in line: "+line);
           log.error("Message: "+e.getMessage());
                        log = null;
           ab = null;
           throw new Exception();
          }
                ab = null;
                log = null;

  }

/* Production 1 */
  static final public void specification() throws ParseException {
 String s;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case 10:
      namespac();
      break;
    default:
      jj_la1[0] = jj_gen;
      ;
    }
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 8:
        ;
        break;
      default:
        jj_la1[1] = jj_gen;
        break label_1;
      }
      namespace_import();
    }
    label_2:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 16:
      case 17:
      case 18:
      case 19:
      case 20:
      case 23:
      case 24:
      case 25:
      case 26:
      case 27:
      case ID:
      case VARIABLE:
      case ANYID:
        ;
        break;
      default:
        jj_la1[2] = jj_gen;
        break label_2;
      }
    s=ab.getNamespace();
      definition(s);
    }
  }

/* Namespace import handling... */
  static final public void namespace_import() throws ParseException {
 String s;
    jj_consume_token(8);
    s = name();
    jj_consume_token(9);
                          ab.addImport(line,s);
  }

/* Default namespace definition */
  static final public void namespac() throws ParseException {
 String s;
    jj_consume_token(10);
    s = name();
    jj_consume_token(9);
                             ab.setNamespace(s);
  }

  static final public void definition(String s) throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ID:
    case VARIABLE:
    case ANYID:
      elemdef(s);
      break;
    case 16:
    case 17:
    case 18:
    case 19:
    case 20:
    case 23:
    case 24:
    case 25:
    case 26:
    case 27:
      relshipdef(s);
      break;
    default:
      jj_la1[3] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

/* VPM element definitions */
  static final public void elemdef(String ns) throws ParseException {
 String t,u,v=null,s;
    t = nameref();
    jj_consume_token(11);
    u = name();
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case 12:
      jj_consume_token(12);
      v = nameref();
      jj_consume_token(12);
      s = nameref();
      jj_consume_token(13);
                                                     ab.addRelationDef(line,ns,u,t,v,s);
      jj_consume_token(9);
      break;
    case 13:
      jj_consume_token(13);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case IMPLOP:
        jj_consume_token(IMPLOP);
        v = value();
        break;
      default:
        jj_la1[4] = jj_gen;
        ;
      }
                                         ab.addEntityDef(line, ns, u, t, v);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 14:
        jj_consume_token(14);
               if (ns.length()>0) ns = ns+"."+u; else ns = u;
        label_3:
        while (true) {
          switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
          case 16:
          case 17:
          case 18:
          case 19:
          case 20:
          case 23:
          case 24:
          case 25:
          case 26:
          case 27:
          case ID:
          case VARIABLE:
          case ANYID:
            ;
            break;
          default:
            jj_la1[5] = jj_gen;
            break label_3;
          }
          definition(ns);
        }
        jj_consume_token(15);
        break;
      case 9:
        jj_consume_token(9);
        break;
      default:
        jj_la1[6] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      break;
    default:
      jj_la1[7] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  static final public void relshipdef(String s) throws ParseException {
 String t,u;boolean bb;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case 16:
      jj_consume_token(16);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = nameref();
      jj_consume_token(13);
      jj_consume_token(9);
                                                            ab.addTypeDefinition(line,s,u,t);
      break;
    case 17:
      jj_consume_token(17);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = nameref();
      jj_consume_token(13);
      jj_consume_token(9);
                                                        ab.addTypeDefinition(line,s,t,u);
      break;
    case 18:
      jj_consume_token(18);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = nameref();
      jj_consume_token(13);
      jj_consume_token(9);
                                                             ab.addSupertypeDefinition(line,s,t,u);
      break;
    case 19:
      jj_consume_token(19);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = nameref();
      jj_consume_token(13);
      jj_consume_token(9);
                                                          ab.addSupertypeDefinition(line,s,u,t);
      break;
    case 20:
      jj_consume_token(20);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 21:
        jj_consume_token(21);
                                               bb=true;
        break;
      case 22:
        jj_consume_token(22);
                                                                    bb=false;
        break;
      default:
        jj_la1[8] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(13);
      jj_consume_token(9);
                                                                                         ab.addIsFinalTypeDefinition(line,s,t,bb);
      break;
    case 23:
      jj_consume_token(23);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 21:
        jj_consume_token(21);
                                                 bb=true;
        break;
      case 22:
        jj_consume_token(22);
                                                                      bb=false;
        break;
      default:
        jj_la1[9] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(13);
      jj_consume_token(9);
                                                                                           ab.addIsAggregationDefinition(line,s,t,bb);
      break;
    case 24:
      jj_consume_token(24);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 21:
        jj_consume_token(21);
                                               bb=true;
        break;
      case 22:
        jj_consume_token(22);
                                                                    bb=false;
        break;
      default:
        jj_la1[10] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(13);
      jj_consume_token(9);
                                                                                         ab.addIsAnySourceDefinition(line,s,t,bb);
      break;
    case 25:
      jj_consume_token(25);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 21:
        jj_consume_token(21);
                                               bb=true;
        break;
      case 22:
        jj_consume_token(22);
                                                                    bb=false;
        break;
      default:
        jj_la1[11] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      jj_consume_token(13);
      jj_consume_token(9);
                                                                                         ab.addIsAnyTargetDefinition(line,s,t,bb);
      break;
    case 26:
      jj_consume_token(26);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = nameref();
      jj_consume_token(13);
      jj_consume_token(9);
                                                       ab.addInverseDefinition(line,s,t,u);
      break;
    case 27:
      jj_consume_token(27);
      jj_consume_token(11);
      t = nameref();
      jj_consume_token(12);
      u = multiplicity_kind();
      jj_consume_token(13);
      jj_consume_token(9);
                                                                      ab.addMultiplicityDefinition(line,s,t,u);
      break;
    default:
      jj_la1[12] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
  }

  static final public String multiplicity_kind() throws ParseException {
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case 28:
      jj_consume_token(28);
                {if (true) return "one_to_one" ;}
      break;
    case 29:
      jj_consume_token(29);
                {if (true) return "one_to_many";}
      break;
    case 30:
      jj_consume_token(30);
                 {if (true) return "many_to_one";}
      break;
    case 31:
      jj_consume_token(31);
                  {if (true) return "many_to_many";}
      break;
    default:
      jj_la1[13] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    throw new Error("Missing return statement in function");
  }

/* Values */
  static final public String variable() throws ParseException {
 Token t;
    t = jj_consume_token(VARIABLE);
                  line = t.beginLine; {if (true) return t.toString();}
    throw new Error("Missing return statement in function");
  }

  static final public String value() throws ParseException {
  Token t;
  String z;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case STRING:
      t = jj_consume_token(STRING);
                line = t.beginLine;z=t.toString();z=z.substring(1,z.length()-1);  {if (true) return z;}
      break;
    case OCTALINT:
      t = jj_consume_token(OCTALINT);
                 line = t.beginLine; {if (true) return t.toString();}
      break;
    case DECIMALINT:
      t = jj_consume_token(DECIMALINT);
                   line = t.beginLine; {if (true) return t.toString();}
      break;
    case HEXADECIMALINT:
      t = jj_consume_token(HEXADECIMALINT);
                       line = t.beginLine; {if (true) return t.toString();}
      break;
    case FLOATONE:
      t = jj_consume_token(FLOATONE);
                 line = t.beginLine; {if (true) return t.toString();}
      break;
    case FLOATTWO:
      t = jj_consume_token(FLOATTWO);
                 line = t.beginLine; {if (true) return t.toString();}
      break;
    default:
      jj_la1[14] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    throw new Error("Missing return statement in function");
  }

/* name references */
  static final public String nameref() throws ParseException {
 String s="";
 Token t;
    s = name();
             {if (true) return s;}
    throw new Error("Missing return statement in function");
  }

/* FQN names */
  static final public String name() throws ParseException {
  Token t;
String s,z;
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case ANYID:
      t = jj_consume_token(ANYID);
      break;
    case ID:
      t = jj_consume_token(ID);
      break;
    case VARIABLE:
      t = jj_consume_token(VARIABLE);
      break;
    default:
      jj_la1[15] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
                z = t.toString();
                line = t.beginLine;
                if (z.charAt(0)=='\'') z=z.substring(1,z.length()-1);
                if (z.charAt(0)=='"') z=z.substring(1,z.length()-1);
                s=z;
    label_4:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case 32:
        ;
        break;
      default:
        jj_la1[16] = jj_gen;
        break label_4;
      }
      jj_consume_token(32);
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ID:
        t = jj_consume_token(ID);
        break;
      case ANYID:
        t = jj_consume_token(ANYID);
        break;
      case VARIABLE:
        t = jj_consume_token(VARIABLE);
        break;
      default:
        jj_la1[17] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
                z = t.toString();
                line = t.beginLine;
                if (z.charAt(0)=='\'') z=z.substring(1,z.length()-1);
                if (z.charAt(0)=='"') z=z.substring(1,z.length()-1);
                s=s+"."+z;
    }
     {if (true) return s;}
    throw new Error("Missing return statement in function");
  }

  static private boolean jj_initialized_once = false;
  static public VTMLParserTokenManager token_source;
  static JavaCharStream jj_input_stream;
  static public Token token, jj_nt;
  static private int jj_ntk;
  static private int jj_gen;
  static final private int[] jj_la1 = new int[18];
  static private int[] jj_la1_0;
  static private int[] jj_la1_1;
  static {
      jj_la1_0();
      jj_la1_1();
   }
   private static void jj_la1_0() {
      jj_la1_0 = new int[] {0x400,0x100,0xf9f0000,0xf9f0000,0x0,0xf9f0000,0x4200,0x3000,0x600000,0x600000,0x600000,0x600000,0xf9f0000,0xf0000000,0x0,0x0,0x0,0x0,};
   }
   private static void jj_la1_1() {
      jj_la1_1 = new int[] {0x0,0x0,0x40c0,0x40c0,0x20,0x40c0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x3f00,0x40c0,0x1,0x40c0,};
   }

  public VTMLParser(java.io.InputStream stream) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    jj_input_stream = new JavaCharStream(stream, 1, 1);
    token_source = new VTMLParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  static public void ReInit(java.io.InputStream stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  public VTMLParser(java.io.Reader stream) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    jj_input_stream = new JavaCharStream(stream, 1, 1);
    token_source = new VTMLParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  static public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  public VTMLParser(VTMLParserTokenManager tm) {
    if (jj_initialized_once) {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  public void ReInit(VTMLParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 18; i++) jj_la1[i] = -1;
  }

  static final private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }

  static final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

  static final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  static final private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  static private java.util.Vector jj_expentries = new java.util.Vector();
  static private int[] jj_expentry;
  static private int jj_kind = -1;

  static public ParseException generateParseException() {
    jj_expentries.removeAllElements();
    boolean[] la1tokens = new boolean[47];
    for (int i = 0; i < 47; i++) {
      la1tokens[i] = false;
    }
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 18; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
          if ((jj_la1_1[i] & (1<<j)) != 0) {
            la1tokens[32+j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 47; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.addElement(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.elementAt(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  static final public void enable_tracing() {
  }

  static final public void disable_tracing() {
  }

  }
