/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.viatra2.codegen.CodeOutputPluginFactory;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.properties.IViatraPropertyProvider;
import org.eclipse.viatra2.imports.NativeImporterFactory;
import org.eclipse.viatra2.interpreters.ModelInterpreterFactory;
import org.eclipse.viatra2.loaders.LoaderFactory;
import org.eclipse.viatra2.logger.EclipseLogger;
import org.eclipse.viatra2.logger.Logger;
import org.eclipse.viatra2.logger.LoggerFactory;
import org.eclipse.viatra2.natives.ASMNativeFunction;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class ViatraPlugin extends AbstractUIPlugin// Plugin
{
	// The shared instance.
	private static ViatraPlugin plugin;

	// Resource bundle.
//	private ResourceBundle resourceBundle;

	/**
	 * The constructor.
	 */

	public ViatraPlugin() {

		super();

		plugin = this;
/*		try {
			resourceBundle = ResourceBundle
					.getBundle("org.eclipse.viatra2.ViatraPluginResources");

		} catch (MissingResourceException x) {
			resourceBundle = null;

		}
		*/
		FrameworkManager.getInstance().setExtensionProvider(
				new EclipseExtensionProvider());
	}

	/*
	 * public ViatraPlugin(IPluginDescriptor pdesc) {
	 * 
	 * super(pdesc);
	 * 
	 * 
	 * plugin = this; try { resourceBundle =
	 * ResourceBundle.getBundle("org.eclipse.viatra2.ViatraPluginResources");
	 * 
	 * } catch (MissingResourceException x) { resourceBundle = null;
	 * 
	 * } }
	 */

	static final String import_ext_id = "org.eclipse.viatra2.core2.modelimport";

	static final String loader_ext_id = "org.eclipse.viatra2.core2.programloader";

	static final String native_ext_id = "org.eclipse.viatra2.core2.nativefunction";

	static final String codeout_ext_id = "org.eclipse.viatra2.core2.codeout";

	static final String logger_ext_id = "org.eclipse.viatra2.core2.logger";

	static final String interpreter_ext_id = "org.eclipse.viatra2.core2.interpreter";

	static final String properties_ext_id = "org.eclipse.viatra2.core2.properties";

	private List<IViatraPropertyProvider> initPropertyProviders() {
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint poi = reg.getExtensionPoint(properties_ext_id);
		ArrayList<IViatraPropertyProvider> r = new ArrayList<IViatraPropertyProvider>();
		if (poi == null)
			return r;
		IExtension[] exts = poi.getExtensions();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					Object o = el[j].createExecutableExtension("class");
					if (o instanceof IViatraPropertyProvider) {
						r.add((IViatraPropertyProvider) o);
					}
				} catch (Exception e) {

				}
			}
		}

		initPrefs(r);

		return r;
	}

	List<IViatraPropertyProvider> propProviders;

	public List<IViatraPropertyProvider> getPropertyProviders() {
		if (propProviders == null)
			propProviders = initPropertyProviders();
		return propProviders;
	}

	/**
	 * Load default if not yet present.
	 */
	private void initPrefs(List<IViatraPropertyProvider> providers) {
		IPreferenceStore prefs = ViatraPlugin.getDefault().getPreferenceStore();
		for (IViatraPropertyProvider p : providers) {
			for (String prop_id : p.getAllPropertyIds()) {
				if (!prefs.contains(p.getProviderID() + "::" + prop_id)) {
					switch (p.getPropertyKind(prop_id)) {
					case STRING:
						prefs.setDefault(p.getProviderID() + "::" + prop_id, p
								.getDefaultPropertyValue(prop_id));
						break;
					case INTEGER:
						prefs.setDefault(p.getProviderID() + "::" + prop_id,
								Integer.parseInt(p
										.getDefaultPropertyValue(prop_id)));
						break;
					case BOOLEAN:
						prefs.setDefault(p.getProviderID() + "::" + prop_id,
								Boolean.parseBoolean(p
										.getDefaultPropertyValue(prop_id)));
						break;
					case PATH:
						prefs.setDefault(p.getProviderID() + "::" + prop_id, p
								.getDefaultPropertyValue(prop_id));
						break;
					}
				}
			}

		}
	}

	public List<ASMNativeFunction> initNativeFunctionPlugins() {
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint poi = reg.getExtensionPoint(native_ext_id);
		ArrayList<ASMNativeFunction> ret = new ArrayList<ASMNativeFunction>();
		if (poi == null)
			return ret;
		IExtension[] exts = poi.getExtensions();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					Object o = el[j].createExecutableExtension("class");
					if (o instanceof ASMNativeFunction) {
						ret.add((ASMNativeFunction) o);
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return ret;
	}

	public Vector<LoaderFactory> initLoaderPlugins() {
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint poi = reg.getExtensionPoint(loader_ext_id);
		Vector<LoaderFactory> v = new Vector<LoaderFactory>();
		if (poi == null)
			return v;
		IExtension[] exts = poi.getExtensions();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					String name = el[j].getAttribute("name");
					String id = el[j].getAttribute("id");
					String fileExtensionList = el[j]
							.getAttribute("fileExtensionList");
					String[] list = new String[0];
					try {
						StringTokenizer tok = new StringTokenizer(
								fileExtensionList, ",");
						list = new String[tok.countTokens()];
						for (int k = 0; k < list.length; ++k) {
							list[k] = tok.nextToken();
						}
					} catch (Exception e) {

					}
					LoaderFactory fact = new GeneralLoaderFactory(id, name,
							el[j], list);
					v.add(fact);
				} catch (Exception e) {
					e.printStackTrace();
					// TODO megjeleniteni valahogy
				}
			}
		}
		return v;
	}

	public Vector<NativeImporterFactory> initModelImportPlugins() {
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint poi = reg.getExtensionPoint(import_ext_id);
		Vector<NativeImporterFactory> v = new Vector<NativeImporterFactory>();
		if (poi == null)
			return v;
		IExtension[] exts = poi.getExtensions();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					String name = el[j].getAttribute("name");
					String id = el[j].getAttribute("id");
					String fileExtensionList = el[j]
							.getAttribute("fileExtensionList");
					String[] list = new String[0];
					try {
						StringTokenizer tok = new StringTokenizer(
								fileExtensionList, ",");
						list = new String[tok.countTokens()];
						for (int k = 0; k < list.length; ++k) {
							list[k] = tok.nextToken();
						}
					} catch (Exception e) {

					}
					NativeImporterFactory fact = new GeneralNativeImporterFactory(
							id, name, el[j], list);
					v.add(fact);
				} catch (Exception e) {
					e.printStackTrace();
					// TODO megjeleniteni valahogy
				}
			}
		}
		return v;
	}

	public Vector<CodeOutputPluginFactory> initCodeOutPlugins() {
		// TODO
		IExtensionRegistry reg = Platform.getExtensionRegistry();

		IExtensionPoint poi = reg.getExtensionPoint(codeout_ext_id);

		if (poi == null)
			return new Vector<CodeOutputPluginFactory>();
		IExtension[] exts = poi.getExtensions();

		Vector<CodeOutputPluginFactory> v = new Vector<CodeOutputPluginFactory>();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					// CodeOutputPlugin o =
					// (CodeOutputPlugin)el[j].createExecutableExtension
					// ("class");
					String id = el[j].getAttribute("id");
					String name = el[j].getAttribute("name");
					GeneralCodeoutFactory fact = new GeneralCodeoutFactory(id,
							name, el[j]);
					v.add(fact);
				} catch (Exception e) {
					e.printStackTrace();
					// TODO megjeleniteni valahogy
				}
			}
		}
		return v;
	}

	public Vector<LoggerFactory> initLoggerPlugins() {
		// TODO
		IExtensionRegistry reg = Platform.getExtensionRegistry();

		IExtensionPoint poi = reg.getExtensionPoint(logger_ext_id);

		if (poi == null)
			return new Vector<LoggerFactory>();
		IExtension[] exts = poi.getExtensions();

		Vector<LoggerFactory> v = new Vector<LoggerFactory>();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					LoggerFactory o = (LoggerFactory) el[j]
							.createExecutableExtension("class");
					v.add(o);
				} catch (Exception e) {
					e.printStackTrace();
					// TODO megjeleniteni valahogy
				}
			}
		}
		return v;
	}

	public Vector<ModelInterpreterFactory> initInterpreterPlugins() {
		// TODO
		IExtensionRegistry reg = Platform.getExtensionRegistry();

		IExtensionPoint poi = reg.getExtensionPoint(interpreter_ext_id);

		if (poi == null)
			return new Vector<ModelInterpreterFactory>();
		IExtension[] exts = poi.getExtensions();

		Vector<ModelInterpreterFactory> v = new Vector<ModelInterpreterFactory>();
		for (int i = 0; i < exts.length; i++) {
			IConfigurationElement[] el = exts[i].getConfigurationElements();
			for (int j = 0; j < el.length; j++) {
				try {
					// (ModelInterpreterFactory)el[j].createExecutableExtension(
					// "class")
					ModelInterpreterFactory o = new GeneralInterpreterFactory(
							el[j]);
					v.add(o);
				} catch (Exception e) {
					e.printStackTrace();
					// TODO megjeleniteni valahogy
				}
			}
		}
		return v;
	}

	/**
	 * This method is called upon plug-in activation
	 */

	public ILog log;

	List<NativeImporterFactory> nativeImporterPlugins;

	List<LoaderFactory> loaderPlugins;

//	List<ASMNativeFunction> nativeFunctions;

	private BundleContext context;

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		this.context = context;
		log = Platform.getLog(context.getBundle());
		try {
			// FrameWorkManager fw = FrameWorkManager.getInstance();
			nativeImporterPlugins = initModelImportPlugins();
			loaderPlugins = initLoaderPlugins();
//			nativeFunctions = initNativeFunctionPlugins();
		} catch (Exception e) {
			logError("error : " + e.getLocalizedMessage());
		}
	}

	public NativeImporterFactory[] getImporterPlugins() {
		return nativeImporterPlugins.toArray(new NativeImporterFactory[0]);
	}

	public LoaderFactory[] getLoaderPlugins() {
		return loaderPlugins.toArray(new LoaderFactory[0]);
	}

	public void logError(String s) {
		Status st = new Status(Status.ERROR, "ViatraCore", 1, s, null);
		log.log(st);
	}

	public void logInfo(String s) {
		Status st = new Status(Status.INFO, "ViatraCore", 1, s, null);
		log.log(st);

	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static ViatraPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle, or 'key' if not
	 * found.
	 */
	/*
	public static String getResourceString(String key) {
		ResourceBundle bundle = ViatraPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}
*/
	/**
	 * Returns the plugin's resource bundle,
	 */
//	public ResourceBundle getResourceBundle() {
//		return resourceBundle;
//	}

	public Logger getEclipseLogger() {
		ILog elog = ViatraPlugin.getDefault().log;
		Logger eclLogger = new EclipseLogger(elog);
		eclLogger.init(null);
		// XXX Eclipse logger is inited with null properties
		return eclLogger;
	}

	/*
	 * public InputStream getPropertyStream () throws IOException { return
	 * openStream(new Path("vpm.properties")); }
	 */
	/*
	public List<ASMNativeFunction> getNativeFunctions() {
		// null would be returned if VIATRA is running in a console
		// environment... Andras Balogh
		if (nativeFunctions == null)
			nativeFunctions = new ArrayList<ASMNativeFunction>();
		return nativeFunctions;
	}
*/
	public BundleContext getContext() {
		return context;
	}
}
