/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.core;

import java.util.List;

import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.logger.Logger;

/**
 * This interface defines the undo support for the Viatra core.
 * 
 * @author Andras Balogh
 * 
 */
public interface IUndoManager {
	/**
	 * Initializes the undo manager.
	 * 
	 * @param l
	 *            - the framework logger
	 * @param m
	 *            - the owning modelspace
	 * @throws VPMRuntimeException
	 */
	public void init(Logger l, IModelSpace m) throws VPMRuntimeException;

	/**
	 * Registers a new undo block with a descriptive name. (Checkpoint)
	 * 
	 * @param name
	 *            - name of undo block
	 */
	public void nextUndoBlock(String name);

	/**
	 * Checks whether the undo buffer is empty
	 * 
	 * @return true, if there are operations in the undo buffer
	 */
	public boolean canUndo();

	/**
	 * Get the first n undo marks' name.
	 * 
	 * @param n
	 *            Limit of undo marks.
	 * @return List of strings
	 */
	public List<String> getUndoList(int n);

	/**
	 * Flushes the undo buffer.
	 * 
	 */
	public void flushUndoBuffer();

	/**
	 * Undoes the last operation
	 */
	public void undo();

	/**
	 * Undoes the last n operations
	 * 
	 * @param n
	 *            number of operations to be undone.
	 */
	public void undo(int n);

	/**
	 * Undoes all operations made after the last checkpoint with given name.
	 * 
	 * @param markName
	 */
	public void undo(String markName);

	/**
	 * Undoes all operations made after the n. checkpoint with given name.
	 * 
	 * @param markName
	 * @param n
	 */
	public void undo(String markName, int n);

	/**
	 * Gets a list of undoable operations currently in the undo buffer.
	 * 
	 * @return array of operation names
	 */
	public List<String> getUndoableList();

}
