/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.core.constraint;

/**
 * @author Andras Schmidt
 * 
 */
public interface IConstraint {
	/** This is an element and tg is linked to this on from side */
	final static int REL_FROM = 0;

	/** This is an element and tg is linked to this on to side */
	final static int REL_TO = 1;

	/**
	 * This is a relation and tg is the end point of the relation on the from
	 * side
	 */
	final static int ENTITY_REL_FROM = 2;

	/**
	 * This is a relation and tg is the end point of the relation on the to side
	 */
	final static int ENTITY_REL_TO = 3;

	final static int OVER = 4;

	/** tg is child of src */
	final static int CHILD = 5;

	/** tg is parent of src */
	final static int PARENT = 6;

	/** tg is subtype of source */
	final static int SUBTYPE = 7;

	/** tg is supertype of source */
	final static int SUPERTYPE = 8;

	/** source is type of tg */
	final static int INSTANCE = 9;

	/** source is instance of tg */
	final static int TYPE = 10;

	/** source is entity tg is null */
	final static int ENTITY = 11;

	/** source is relation tg is null */
	final static int RELATION = 12;

	/** src is below tg */
	final static int BELOW = 13;

	final static int N_CONSTRAINTS = 14;

	final static int DTYPE = 14;

	final static int DSUBTYPE = 15;

	final static int DINSTANCE = 16;

	final static int DSUPERTYPE = 17;

	final static String[] namesOfConstraints = { "is source of",
			"is target of", "starts from", "ends at", "is over of",
			"is grandchild of", "is child of", "is supertype of",
			"is subtype of", "is type of", "is instance of", "is an entity",
			"is a relation", "is below of", "ERROR" };

	// IPatternVariable getSourceVar();
	// IPatternVariable getTargetVar();
	int getType();

	EConstraint getType2();

	Object getTag();

	void setTag(Object o);

	// int typeFromOtherSide() throws PatternMatcherException;
	/**
	 * Return the type of the constraint if seen from the other side. Returns -1
	 * (not a real type) if no other side is present.
	 * 
	 * @return
	 */
	int typeFromOtherSideNoException();
	/**
	 * In a pattern constraints can be given from both side. So one constraint
	 * can have two representations in constraint list. This method will return
	 * true if this is the main representation of the constraint.
	 * 
	 * @return
	 */
	// boolean isOtherSide();
}
