/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.core.simple;

import java.util.Random;

import org.eclipse.viatra2.logger.Logger;

/**
 * This class provides unique names. All used names must be notified to this
 * object. The returned name is guaranteed to be unique.
 * 
 * @author Andras Schmidt
 * 
 */
public class UniqueNameProvider {
	Random random;



	public UniqueNameProvider() {
		random = new Random();
		runningInstanceStamp = "_" + randomString(1);
	}

	public String byteToHex(byte _b) {
		int b = _b;
		b &= 0xFF; // automatic conversion converts bytes as if they were signed
		String ret = Integer.toHexString(b);
		if (ret.length() == 1)
			ret = '0' + ret;
		return ret;
	}

	public String randomString(int len) {
		String retStr = "";
		byte[] bytes = new byte[len];
		random.nextBytes(bytes);
		for (int i = 0; i < len; ++i) {
			retStr += byteToHex(bytes[i]);
		}
		return retStr;
	}

	public String uniqueName(String s) {
		return s + uniqueName();
	}

	public String uniqueName() {
		// TODO no guarantee, just probable
		return "uN" + (counter++) + runningInstanceStamp;
	}

	private int counter = 0;

	private String runningInstanceStamp = "";

}
