/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.editor.text;

import java.util.HashSet;

import org.eclipse.viatra2.editor.text.light.VTCLEditor;
import org.eclipse.viatra2.editor.text.light.VTMLEditor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.framework.IFrameworkListChangedListener;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin implements IFrameworkListChangedListener {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.eclipse.viatra2.editor.text";

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}

	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				"org.eclipse.viatra2.editor.text", path);
	}
	
	boolean listenerRegistered = false;
	
	HashSet<VTCLEditor> editors = new HashSet<VTCLEditor>();
	
	protected VTCLEditor iCurrentVTCLEditor;
	
	protected VTMLEditor iCurrentVTMLEditor;
	
	public VTCLEditor getCurrentVTCLEditor()
	{
		return iCurrentVTCLEditor;
	}
	
	public void setCurrentVTCLEditor(VTCLEditor ed)
	{
		if (!listenerRegistered)
		{
			FrameworkManager.getInstance().addFrameworkListChangedListener(this);
			listenerRegistered = true;
		}
		iCurrentVTCLEditor = ed;
		if (!editors.contains(ed))
		{
			addEditor(ed);
		}
	}
	
	public void addEditor(VTCLEditor ed)
	{
		editors.add(ed);
	}
	
	public void removeEditor(VTCLEditor ed)
	{
		editors.remove(ed);
	}
	
	public VTMLEditor getCurrentVTMLEditor()
	{
		return iCurrentVTMLEditor;
	}
	
	public void setCurrentVTMLEditor(VTMLEditor ed)
	{
		iCurrentVTMLEditor = ed;
	}

	public void frameworkAdded(IFramework f) {
		/*
		for (VTCLEditor e : editors)
		{
			if (e.getAttachedFramework()==null)
			{
				e.setAttachedFramework(f);
				//e.attachEditor(); // refresh title
				e.refreshTitle();
				e.parseOnSave();
			}
		}
		*/
	}

	public void frameworkRemoved(IFramework f) 
	{
		for (VTCLEditor e : editors)
		{
			if (f.equals(e.getAttachedFramework()))
			{
				e.setAttachedFramework(null); // signal editor that the framework has been removed
				//e.attachEditor(); // refresh title
				e.refreshTitle();
			}
		}
	}
	
}
