/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui;

import java.util.Collection;
import java.util.Vector;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.frameworkgui.actions.AbstractFrameworkGUIAction;
import org.eclipse.viatra2.frameworkgui.actions.extend.ContributedActionContainer;
import org.eclipse.viatra2.frameworkgui.content.IFrameworkContentProviderExtension;
import org.eclipse.viatra2.frameworkgui.views.console.commands.IVIATRAConsoleCommandProviderFactory;
import org.eclipse.viatra2.frameworkgui.wizards.model.FileTemplateExtension;
import org.osgi.framework.BundleContext;

/**
 * Summary: GUI Plugin class for the Framework tree view.
 * 
 * Description: This is an automatically generated class by Eclipse.
 * 
 * @author Istvan Rath
 * 
 */
public class FrameworkGUIPlugin extends AbstractUIPlugin {

	// The shared instance.
	private static FrameworkGUIPlugin plugin;

	/**
	 * The constructor.
	 */
	public FrameworkGUIPlugin() {
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static FrameworkGUIPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path.
	 * 
	 * @param path
	 *            the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				"org.eclipse.viatra2.gui", path);
	}

	public Collection<IFrameworkContentProviderExtension> getContentProviders() {
		Vector<IFrameworkContentProviderExtension> v = new Vector<IFrameworkContentProviderExtension>();
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint ep = reg
				.getExtensionPoint("org.eclipse.viatra2.gui.content");
		for (IExtension e : ep.getExtensions()) {
			for (IConfigurationElement ce : e.getConfigurationElements()) {
				if (!ce.getName().equalsIgnoreCase("content_provider"))
					continue;
				try {
					Object o = ce.createExecutableExtension("class");
					if (o instanceof IFrameworkContentProviderExtension) {
						v.add((IFrameworkContentProviderExtension) o);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		return v;
	}
	
	public Collection<IVIATRAConsoleCommandProviderFactory> getCommandProviderFactories() {
		Vector<IVIATRAConsoleCommandProviderFactory> v = new Vector<IVIATRAConsoleCommandProviderFactory>();
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint ep = reg.getExtensionPoint("org.eclipse.viatra2.gui.consoleCommandProvider");
		for (IExtension e : ep.getExtensions()) {
			for (IConfigurationElement ce : e.getConfigurationElements()) {
				if (!ce.getName().equalsIgnoreCase("commandProviderFactory"))
					continue;
				try {
					Object o = ce.createExecutableExtension("class");
					if (o instanceof IVIATRAConsoleCommandProviderFactory) {
						v.add((IVIATRAConsoleCommandProviderFactory) o);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		return v;
	}

	public Collection<ContributedActionContainer> getContributedActions() 
	{
		Vector<ContributedActionContainer> v = new Vector<ContributedActionContainer>();
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint ep = reg
				.getExtensionPoint("org.eclipse.viatra2.gui.contributedAction");
		for (IExtension e : ep.getExtensions()) {
			for (IConfigurationElement ce : e.getConfigurationElements()) {
				if (!ce.getName().equalsIgnoreCase("contributedAction"))
					continue;
				try {
					Object o = ce.createExecutableExtension("class");
					if (o instanceof AbstractFrameworkGUIAction) 
					{
						ContributedActionContainer c = new ContributedActionContainer();
						c.action = (AbstractFrameworkGUIAction) o;
						c.enablesFor = Class.forName(ce.getAttribute("enablesForType"));
						v.add(c);
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		return v;
	}
	
	public Collection<FileTemplateExtension> getFileTemplates() {
		Vector<FileTemplateExtension> v = new Vector<FileTemplateExtension>();
		IExtensionRegistry reg = Platform.getExtensionRegistry();
		IExtensionPoint ep = reg
				.getExtensionPoint("org.eclipse.viatra2.gui.fileTemplate");
		for (IExtension e : ep.getExtensions()) {
			for (IConfigurationElement ce : e.getConfigurationElements()) {
				if (!ce.getName().equalsIgnoreCase("templateReference"))
					continue;
				try {
					FileTemplateExtension ext = new FileTemplateExtension();
					ext.extension = ce.getAttribute("extension");
					ext.name = ce.getAttribute("name");
					ext.templateFile = ce.getAttribute("templateFile");
					if (ce.getAttribute("description") != null)
						ext.desc = ce.getAttribute("description");
					// process params
					for (IConfigurationElement p : ce.getChildren()) {
						if (p.getName().equals("templateParameter")) {
							FileTemplateExtension.TemplateParameter _p = ext.new TemplateParameter();
							_p.name = p.getAttribute("name");
							_p.defaultValue = p.getAttribute("defaultValue");
							_p.value = _p.defaultValue;
							if (p.getAttribute("description") != null)
								_p.description = p.getAttribute("description");
							ext.params.add(_p);
						}
					}
					ext.contributorID = e.getNamespaceIdentifier();
					v.add(ext);
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		}
		return v;
	}

	public static Shell getActiveWorkbenchShell() {
		IWorkbenchWindow workBenchWindow = getActiveWorkbenchWindow();
		if (workBenchWindow == null) {
			return null;
		}
		return workBenchWindow.getShell();
	}

	/**
	 * Returns the active workbench window
	 * 
	 * @return the active workbench window
	 */
	public static IWorkbenchWindow getActiveWorkbenchWindow() {
		if (plugin == null) {
			return null;
		}
		IWorkbench workBench = plugin.getWorkbench();
		if (workBench == null) {
			return null;
		}
		return workBench.getActiveWorkbenchWindow();
	}

	public static IWorkbenchPage getActivePage() {
		IWorkbenchWindow activeWorkbenchWindow = getActiveWorkbenchWindow();
		if (activeWorkbenchWindow == null) {
			return null;
		}
		return activeWorkbenchWindow.getActivePage();
	}

	public static String getPluginId() {
		return getDefault().getBundle().getSymbolicName();
	}

	public static void log(Throwable e) {
		log(new Status(IStatus.ERROR, getPluginId(), IStatus.ERROR, "Error", e)); //$NON-NLS-1$
	}

	public static void log(IStatus status) {
		getDefault().getLog().log(status);
	}
	
	
	// Runnable support
	
	public static void runAsLongTransaction(final IRunnableWithProgress r, final String taskName, final IFramework fw)
	{
		final Display display = Display.getDefault();
		
		if (r!=null)
		{
			display.syncExec(new Runnable() {
				public void run() {
					ProgressMonitorDialog d= new ProgressMonitorDialog(display.getActiveShell());
					try {
						d.run(true, false, r);
					} catch (Exception e) {
						fw.getLogger().fatal("Error executing task '"+taskName+"' ("+e.getClass().getCanonicalName()+") ["+e.getMessage()+"]");
						e.printStackTrace();
					}
				}
			});
		}
	}
	
}
