/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.viatra2.frameworkgui.FrameworkGUIPlugin;
import org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView;

public class MergeFileAction extends AbstractFrameworkGUIAction {

	public MergeFileAction(FrameworkTreeView part) {
		super();
		setupInternals(part);
		setText("Merge a VPML file");
		setToolTipText("Merge a VPML file with the Framework's modelspace");
		setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
	}

	public void run() {
		refreshSelection();
		FileDialog fpd = new FileDialog(iFT.getSite().getShell(), SWT.OPEN);
		fpd.setText("Choose a file to merge");
		fpd.setFilterNames(new String[] { "Viatra VPML files" });
		fpd.setFilterExtensions(new String[] { "*.vpml" });
		final String fn = fpd.open();
		if (fn != null) 
		{
			IRunnableWithProgress runnable = new IRunnableWithProgress()
			{
				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
					try 
					{
						monitor.beginTask("VPM merge: "+fn, 2);
						iViatraFramework.mergeFile(fn);
						monitor.done();
					} 
					catch (Exception e)
					{
						throw new InvocationTargetException(e);
					}
				}
			};
			
			FrameworkGUIPlugin.runAsLongTransaction(runnable, "VPM merge: "+fn, iViatraFramework);

		}
	}
}
