/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.actions;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import org.eclipse.viatra2.frameworkgui.FrameworkGUIPlugin;
import org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView;

public class NativeImportAction extends AbstractFrameworkGUIAction {

	private String iImporterID;

	public NativeImportAction(FrameworkTreeView part, String importerid) {
		super();
		setupInternals(part);
		iImporterID = importerid;
		setText("Native import");
		setToolTipText("Import a file into the modelspace using a Viatra native importer");
		setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
	}

	public void run() {
		refreshSelection();
		FileDialog fop = new FileDialog(iFT.getSite().getShell(), SWT.OPEN);
		fop.setText("Choose a file to import");
		fop.setFilterExtensions(new String[] { "*.*" });
		fop.setFilterNames(new String[] { "All files" });
		//fop.setFilterNames( );
		final String fn = fop.open();
		if (fn != null) 
		{
			IRunnableWithProgress runnable = new IRunnableWithProgress()
			{

			public void run(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
				try {
					monitor.beginTask("Native import: "+fn, 3);
					Long c = System.currentTimeMillis();
					monitor.worked(1);
					iViatraFramework.nativeImport(fn, iImporterID);
					monitor.worked(2);
					Long c2 = System.currentTimeMillis();
					iViatraFramework.getLogger().info("Native import lasted for "+(new Long(c2-c)).toString()+" ms");
					monitor.done();
				} catch (Exception e) {
					throw new InvocationTargetException(e);
					//iFT.showMessage("Error performing native import with "+ iImporterID);
				}	
			}
			};
			
			FrameworkGUIPlugin.runAsLongTransaction(runnable, "Native import: "+fn, iViatraFramework);

		}
	}

}
