/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.property;


import java.util.ArrayList;
import java.util.Set;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource2;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.eclipse.viatra2.framework.IFramework;

public class FrameworkGUIPropertySource implements IPropertySource2 {

	private IFramework iViatraFramework;

	private String iProviderID;
	
	public FrameworkGUIPropertySource(IFramework fw, String providerid) {
		super();
		iViatraFramework = fw;
		iProviderID = providerid;
	}

	public boolean isPropertyResettable(Object id) {
		return false;
	}

	public boolean isPropertySet(Object id) {
		if (iViatraFramework != null) {
			return iViatraFramework.getProperties().isRuntimePropertySet(iProviderID, id.toString());
		}
		return false;
	}

	public Object getEditableValue() {
		if (iViatraFramework != null)
			return iViatraFramework.getId();
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (iViatraFramework != null) {
			ArrayList a = new ArrayList();
			Set props = iViatraFramework.getProperties().getAllRuntimePropertyIDs(iProviderID);
			for (Object n : props.toArray())
			{
				a.add(new TextPropertyDescriptor(n, n.toString()));
			}
			IPropertyDescriptor[] a2 = new IPropertyDescriptor[a.size()];
			for (int i = 0; i < a.size(); a2[i] = (IPropertyDescriptor) a
					.get(i++))
				;
			return a2;
		}
		return null;
	}

	public Object getPropertyValue(Object id) {
		if (iViatraFramework != null)
			return iViatraFramework.getProperties().getRuntimeProperty(iProviderID, (String) id);
		return null;
	}

	public void resetPropertyValue(Object id) {
		// TODO Istvan Rath implement this.
	}

	public void setPropertyValue(Object id, Object value) {
		if (iViatraFramework != null)
			iViatraFramework.getProperties().setRuntimeProperty(iProviderID, (String) id, (String) value);
	}

}
