/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.runner;

import java.util.HashMap;
import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.viatra2.framework.FrameworkException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.frameworkgui.views.console.commands.xform.TransformationExecutionHelper;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;

/**
 * Helper class to facilitate the execution of VIATRA transformations
 * from the GUI.
 * @author istvan rath
 *
 */
public class TransformationRunner 
{
	/**
	 * Cache to store run history.
	 */
	private static HashMap<String, Vector<String>> runDialogEntries 
									= new HashMap<String, Vector<String>>();
	
	public static void addRunDialogEntry(String frameworkId, String entry)
	{
		Vector<String> target = runDialogEntries.get(frameworkId);
		if (target==null)
		{
			target = new Vector<String>();
		}
		if (!target.contains(entry))
			target.add(entry);
		runDialogEntries.put(frameworkId, target);
	}
	
	public static Object[] getRunDialogEntries(String frameworkId)
	{
		Vector<String> r = runDialogEntries.get(frameworkId);
		if (r!=null)
			return r.toArray();
		return null;
	}
	

	/**
	 * Invoked when a particular machine is to be run.
	 * @param m
	 * @param fw
	 * @param shell
	 */
	public static void run(final Machine m, final IFramework fw, final Shell shell)
	{
		// set-up phase
		TreeMap<String, Object> params_map = new TreeMap<String, Object>();
		try
		{
			if (fw.isRunnable(m))
			{
				String[] params = fw.getEntrypointParameters(m);
				if (params.length > 0) 
				{
					// there are parameters
					String s_params = "";
					for (String param : params) s_params += param+", ";
					s_params = s_params.substring(0,s_params.length()-2); // assemble information string 
				
					ValidatingRunnerInputDialog dialog = new ValidatingRunnerInputDialog(
							shell, 
							fw.getId(),
							"Set run parameters",
							"Enter the desired values for run parameters: "
									+ s_params, "", params);
					if (dialog.open() == 0) 
					{
						for (int i = 0; i < params.length; i++) 
						{
							String val = dialog.parsed_params[i];
							params_map.put(params[i], TransformationExecutionHelper.convertToPassableType(val));
						}
						//TransformationRunner.addRunDialogEntry(fw.getId(),dialog.getData()); <-- moved to Dialog class.
					}
					else
					{
						// user pressed cancel
						return;
					}
					
				}
			}
		}
		catch (FrameworkException fwe)
		{
			MessageDialog.openInformation(shell, "VIATRA2 R3", fwe.getMessage());
			return;
		}
		// set-up phase complete
		TransformationExecutionHelper.runMachine(fw, m, params_map, false);
	}
	
	
}
