/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.frameworkgui.views;

import java.lang.reflect.InvocationTargetException;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.ui.dialogs.ListDialog;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.frameworkgui.FrameworkGUIPlugin;
import org.eclipse.viatra2.frameworkgui.content.FrameworkContainer;

public class FrameworkDropTargetAdapter extends DropTargetAdapter {

	private FrameworkTreeView iFT;

	static FileTransfer transfer = FileTransfer.getInstance();

	public FrameworkDropTargetAdapter(FrameworkTreeView part) {
		super();
		iFT = part;
	}

	public static Transfer[] getTransfers() {
		Transfer[] transfers = new Transfer[1];
		transfers[0] = transfer;
		return transfers;
	}

	public void drop(DropTargetEvent event) {
		final String[] filenames = (String[]) event.data;
		if (event.item != null) {
			Object target = event.item.getData();
			if (target != null && target instanceof FrameworkContainer) {
				for (int i = 0; i < filenames.length; i++) {
					final IFramework fw = ((FrameworkContainer) target)
							.getFramework();
					String extension = filenames[i].substring(filenames[i]
							.lastIndexOf(".") + 1);
					final int _i = i;
					IRunnableWithProgress targetRunnable = null;

					// merge
					final IRunnableWithProgress runnable_merge = new IRunnableWithProgress() {

						public void run(IProgressMonitor monitor)
								throws InvocationTargetException,
								InterruptedException {
							try {
								monitor.beginTask(
										"VPM merge: " + filenames[_i], 2);
								fw.mergeFile(filenames[_i]);
								monitor.done();
							} catch (Exception e) {
								throw new InvocationTargetException(e);
							}
						}
					};

					// machine load
					// compute target loader first
					String loaderID = "";

					final Set<String> loaders = fw
							.getLoadersForExtension(extension);
					if (loaders.size() > 1) {
						ListDialog dialog = new ListDialog(iFT.getSite()
								.getShell());
						dialog.setTitle("Select loader to use for "
								+ filenames[i]);
						dialog.setAddCancelButton(true);
						dialog.setBlockOnOpen(true);
						dialog.setLabelProvider(new LabelProvider());
						dialog
								.setContentProvider(new IStructuredContentProvider() {
									public Object[] getElements(
											Object inputElement) {
										return loaders.toArray();
									}

									public void dispose() {
									}

									public void inputChanged(Viewer viewer,
											Object oldInput, Object newInput) {
									}
								});
						dialog.setInput("blablabla");
						int result = dialog.open();
						if (result == ListDialog.OK) {
							Object[] o = dialog.getResult();
							if (o != null && o.length > 0) {
								loaderID = o[0].toString();
							}
						}
					} else if (loaders.size() == 1) {
						loaderID = loaders.iterator().next().toString();
					}

					final String _loaderID = loaderID;
					IRunnableWithProgress runnable_load = new IRunnableWithProgress() {

						public void run(IProgressMonitor monitor)
								throws InvocationTargetException,
								InterruptedException {
							try {
								monitor.beginTask("Machine load: "
										+ filenames[_i], 2);
								Long c = System.currentTimeMillis();
								fw.loadMachine(filenames[_i], _loaderID);
								Long c2 = System.currentTimeMillis();
								fw.getLogger().info(
										"Machine loading using '" + _loaderID
												+ "' lasted for "
												+ (new Long(c2 - c)).toString()
												+ " ms");
								monitor.done();
							} catch (Exception e) {
								throw new InvocationTargetException(e);
							}
						}
					};

					// try importers
					String importerID = "";

					final Set<String> importers = fw
							.getNativeImportersForExtension(extension);
					if (importers.size() > 1) {
						ListDialog dialog = new ListDialog(iFT.getSite()
								.getShell());
						dialog.setTitle("Select native importer to use for "
								+ filenames[i]);
						dialog.setAddCancelButton(true);
						dialog.setBlockOnOpen(true);
						dialog.setLabelProvider(new LabelProvider());
						dialog
								.setContentProvider(new IStructuredContentProvider() {
									public Object[] getElements(
											Object inputElement) {
										return importers.toArray();
									}

									public void dispose() {
									}

									public void inputChanged(Viewer viewer,
											Object oldInput, Object newInput) {
									}
								});
						dialog.setInput("blablabla");
						int result = dialog.open();
						if (result == ListDialog.OK) {
							Object[] o = dialog.getResult();
							if (o != null && o.length > 0) {
								importerID = o[0].toString();
							}
						}
					} else if (importers.size() == 1) {
						importerID = importers.iterator().next().toString();
					}

					final String _importerID = importerID;
					final IRunnableWithProgress runnable_import = new IRunnableWithProgress() {

						public void run(IProgressMonitor monitor)
								throws InvocationTargetException,
								InterruptedException {
							try {
								monitor.beginTask("Native import: "
										+ filenames[_i], 2);
								Long c = System.currentTimeMillis();
								fw.nativeImport(filenames[_i], _importerID);
								Long c2 = System.currentTimeMillis();
								fw.getLogger().info(
										"Native import using '" + _importerID
												+ "' lasted for "
												+ (new Long(c2 - c)).toString()
												+ " ms");
								monitor.done();
							} catch (Exception e) {
								throw new InvocationTargetException(e);
							}
						}
					};

					if (extension.equalsIgnoreCase("vpml")) {
						targetRunnable = runnable_merge;
					} else if (loaderID.length() > 0) {
						targetRunnable = runnable_load;
					} else if (importerID.length() > 0) {
						targetRunnable = runnable_import;
					}

					FrameworkGUIPlugin.runAsLongTransaction(targetRunnable,
							"Drag-and-drop operation", fw);

				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.swt.dnd.DropTargetAdapter#dragEnter(org.eclipse.swt.dnd.
	 * DropTargetEvent)
	 */
	@Override
	public void dragEnter(DropTargetEvent event) {
		dragOver(event);
	}

	public void dragOver(DropTargetEvent event) {
		if (event.item == null || event.item.getData() == null
				|| !(event.item.getData() instanceof FrameworkContainer)) {
			event.detail = DND.DROP_NONE;
		} else {
			event.detail = DND.DROP_COPY;
		}
		event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_SCROLL;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.swt.dnd.DropTargetAdapter#dragOperationChanged(org.eclipse
	 * .swt.dnd.DropTargetEvent)
	 */
	@Override
	public void dragOperationChanged(DropTargetEvent event) {
		dragOver(event);
	}

}
