/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.views.code;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.viatra2.buffers.BufferStore;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.IFramework;

public class SaveAllBuffersAction extends Action {

	CodeBufferView cbv;
	
	public SaveAllBuffersAction(CodeBufferView v)
	{
		cbv = v;
		this.setToolTipText("Save the content of all buffers to a directory in the workspace");
		this.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_OBJ_FOLDER));
		this.setText("Save outputs");
	}
	
	@Override
	public void run() 
	{
		ContainerSelectionDialog dd = new ContainerSelectionDialog(
				cbv.getViewSite().getShell(),
				ResourcesPlugin.getWorkspace().getRoot(),
				true,
				"Select destination folder:");
		
		dd.setBlockOnOpen(true);
		dd.setTitle("Select folder to save buffers to");
		// check whether there are buffers to the current modelspace
		IFramework fw = FrameworkManager.getInstance().getFramework(cbv.getCurrentOuterPage().frameworkId);
		if (BufferStore.getAllCoreBuffers(fw.getTopmodel())!=null && Window.OK == dd.open())
		{
			try
			{
				// save output
				Object[] dirs = dd.getResult();
				for (Object _p : dirs) // support multiple output directories
				{
					IPath p = (IPath)_p;
					// iterate over buffers and save
					for (Map.Entry<String, StringWriter> buf : BufferStore.getAllCoreBuffers(fw.getTopmodel()))
					{
						// calculate target file path
						String fileName = buf.getKey().replaceFirst("core://", "");
						IFile ifile = ResourcesPlugin.getWorkspace().getRoot().getFile(p.append("/"+fileName));
						FileWriter wr = new FileWriter(new File(ifile.getLocation().toOSString()));
						try {
							wr.write(buf.getValue().getBuffer().toString());
						}
						finally {
							wr.flush();
							wr.close();
						}
					}
				}
				ResourcesPlugin.getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
			}
			catch (IOException e)
			{
				fw.getLogger().fatal(e.getMessage());
			} 
			catch (CoreException e) 
			{
				fw.getLogger().fatal(e.getMessage());
			}
		}
		
	}
	
	
}
