/*******************************************************************************
 * Copyright (c) 2008-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.frameworkgui.views.console;

import java.awt.Dimension;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;

/**
 * AWT-style border layout for SWT.
 * 
 * Completely rewritten by Istvan Rath, April 26, 2010.
 * 
 */
public class SimpleBorderLayout extends Layout {

	
	protected Point getPreferredSize(Control control, int wHint, int hHint, boolean changed) {
		Dimension dim = (Dimension) control.getData("preferredSize");
		if (dim != null) {
			return new Point(dim.width, dim.height);
		}
		return control.computeSize(wHint, hHint, changed);
	}
	
	public final static Integer CENTER = 0;
	public final static Integer EAST = 1;
	public final static Integer NORTH = 2;
	public final static Integer SOUTH = 3;
	public final static Integer WEST = 4;
	
	private Control centerElem, eElem, nElem, sElem, wElem;

	public SimpleBorderLayout() {
		super();
	}

	/**
	 * Computes the size of our composite.
	 */
	protected Point computeSize(Composite composite, int wHint, int hHint, boolean flushCache) {
		// init references to children
		getChildLayout(composite);
		Point base = new Point(0, 0);
		Point pref;
		if (sElem != null) 
		{
			pref=getPreferredSize(sElem, wHint, SWT.DEFAULT, flushCache);
			base.y+=pref.y;
		}
		if (nElem != null) 
		{
			pref=getPreferredSize(nElem, wHint, SWT.DEFAULT, flushCache);
			base.y+=pref.y;
		}
		if (eElem != null) 
		{
			pref=getPreferredSize(eElem, SWT.DEFAULT, hHint, flushCache);
			base.x+=pref.x;
		}
		if (wElem != null) 
		{
			pref=getPreferredSize(wElem, SWT.DEFAULT, hHint, flushCache);
			base.x+=pref.x;
		}
		if (centerElem != null) 
		{
			pref=getPreferredSize(centerElem, wHint, hHint, flushCache);
			base.x+=pref.x;
			base.y+=pref.y;
		}
		return base;
	}

	protected void layout(Composite composite, boolean flushCache) 
	{
		getChildLayout(composite);
		Rectangle bounds = composite.getClientArea();
		int t = bounds.y;
		int b = bounds.y + bounds.height;
		int l = bounds.x;
		int r = bounds.x + bounds.width;

		Point preferredSize;
		if (nElem != null) {
			preferredSize = getPreferredSize(nElem, bounds.width,SWT.DEFAULT, flushCache);
			nElem.setBounds(l, t, r-l, preferredSize.y);
			t+=preferredSize.y;
		}
		if (sElem != null) {
			preferredSize = getPreferredSize(sElem, bounds.width, SWT.DEFAULT, flushCache);
			sElem.setBounds(l, b-preferredSize.y, r-l,preferredSize.y);
			b -= preferredSize.y;
		}
		if (wElem != null) {
			preferredSize = getPreferredSize(wElem, SWT.DEFAULT, b-t, flushCache);
			wElem.setBounds(l, t, preferredSize.x, b-t);
			l += preferredSize.x;
		}
		if (eElem != null) {
			preferredSize = getPreferredSize(eElem, SWT.DEFAULT, b-t, flushCache);
			eElem.setBounds(r - preferredSize.x, t, preferredSize.x,b-t);
			r -= preferredSize.x;
		}
		if (centerElem != null) {
			centerElem.setBounds(l, t, r-l, b-t);
		}
	}

	/**
	 * Initialize member references to children controls.
	 */
	private void getChildLayout(Composite composite) {
		nElem = sElem = eElem = wElem = centerElem = null;
		Control[] children = composite.getChildren();
		for (int i = 0; i < children.length; i++) {
			Integer layoutData = (Integer) children[i].getLayoutData();
			switch (layoutData) {
			default:
				case 0: // center 				
					centerElem = children[i]; break;
				case 1: // east
					eElem = children[i]; break;
				case 2: // north
					nElem = children[i]; break;
				case 3: // south
					sElem = children[i]; break;
				case 4: // west
					wElem = children[i]; break;
			}
		}
	}




}
