/*******************************************************************************
 * Copyright (c) 2008-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.frameworkgui.views.console.commands.builtin;

import java.util.List;

import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.frameworkgui.views.console.commands.CommandExecutor;
import org.eclipse.viatra2.frameworkgui.views.console.commands.IVIATRAConsoleCommandProvider;

/**
 * Supertrivial implementation for the "about" command.
 * @author istvan
 *
 */
public class Help implements IVIATRAConsoleCommandProvider {

	public void executeCommand(IFramework fw, List<String> parameters) {
		if (parameters.size()>0) {
			IVIATRAConsoleCommandProvider cp = CommandExecutor.getCommandByName(fw, parameters.get(0));
			if (cp!=null) {
				fw.getLogger().info("Help text for the '"+cp.getClass().getSimpleName()+ "' command:");
				fw.getLogger().info(cp.getHelpText());
			}
			else {
				fw.getLogger().error("No command found by the name "+parameters.get(0));	
			}
		}
		else {
			fw.getLogger().error("help requires a command's name as a parameter; try the \"listcommands\" command to see what commands are available.");
		}
	}

	public String getCommandSignature() {
		return "help(commandname)";
	}

	public String getDescription() {
		return "Prints help information about commands.";
	}

	public String getHelpText() {
		return "Use help(<<commandname>>) to print out the help text of a given command.";
	}

}
