/*******************************************************************************
 * Copyright (c) 2008-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.frameworkgui.views.console.commands.xform;

import java.util.Map;

import org.eclipse.viatra2.framework.FrameworkException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Rule;

public abstract class AbstractRunnerCommand {

	
	protected void commenceRunning(IFramework fw, Object entrypoint, String paramStr, String epName, boolean undoable) {
		try {
			if (!fw.isRunnable(entrypoint)) {
				fw.getLogger().error("[RunRule] entrypoint "+epName+" is not runnable.");
				return;
			}
			} catch (FrameworkException e) {
				fw.getLogger().error("[RunRule] entrypoint "+epName+" is not runnable.");
				fw.getLogger().printStackTrace(e);
				return;
			}
			// 2. parse parameters
			Map<String, Object> params = TransformationExecutionHelper.parseParameters(paramStr);
			// 3. check parameter compatibility
			if (!TransformationExecutionHelper.checkParameterCompatibility(fw, entrypoint, params)) {
				fw.getLogger().error("[RunnerCommands] transformation parameters not compatible with machine.");
				fw.getLogger().info("[RunnerCommands] parsed parameters: "+params.toString());
				return;
			}
			// 4. run machine or rule depending on case
			if (entrypoint instanceof Machine) {
				TransformationExecutionHelper.runMachine(fw, (Machine)entrypoint, params, undoable);
			}
			else if (entrypoint instanceof Rule) {
				TransformationExecutionHelper.runRule(fw, (Rule)entrypoint, params, undoable);
			}
	}
	
}
