/*******************************************************************************
 * Copyright (c) 2004-2008 Akos Horvath, Gergely Varro and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Akos Horvath, Gergely Varro - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.impl.gtmatcher.extension;

import org.eclipse.emf.common.util.EList;

import org.eclipse.viatra2.gtasm.interpreter.exception.ASMInterpreterErrorStrings;
import org.eclipse.viatra2.gtasm.interpreter.exception.ViatraTransformationException;
import org.eclipse.viatra2.gtasm.interpreter.executionEnvironment.IExecutionEnvironment;
import org.eclipse.viatra2.gtasm.patternmatcher.exceptions.PatternMatcherCompileTimeException;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.gtmatcher.exceptions.GraphTransformationException;
import org.eclipse.viatra2.gtasm.patternmatcher.patterns.IPatternMatcher;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.core.RuntimeAnnotation;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.core.RuntimeAnnotationElement;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTPattern;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTRule;


/** Provides the registered alternative pattern matchers for the GT Rule matcher.
 * @author Akos Horvath
 *
 */
public class AlternativePatternMactherProvider {
	
	private static AlternativePatternMactherProvider  _instance = new AlternativePatternMactherProvider ();
	private static String patternMathcerIDToUse = VIATRAGTMatcherPlugin.DEFAULT_ALTERNATIVE_PM_FACTORY_ID;
	
	
	public static AlternativePatternMactherProvider getInstance(){
		return _instance;
	}
	
	/** Sets the pattern matcher ID to use
	 * @param id The new ID to be used
	 */
	public static void setPatternMatcherIDToUse(String id){
	 	patternMathcerIDToUse = id;
	}
	
	/** Returns the actual pattern matcher ID
	 * @return ID of the PM
	 */
	public static String getPatternMatcherIDToUse(){
	 	return patternMathcerIDToUse;
	}
	
	/** Returns a pattern matcher to work with for the LHS
	 * @param org.eclipse.viatra2.gtasm.interpreter.executionEnvironment the execution environment of the invocation context
	 * @param gtPattern the invoked pattern
	 * @return a patternMatcher for the pattern
	 * @throws PatternMatcherCompileTimeException
	 */
	public IPatternMatcher getPatternMatcher(IExecutionEnvironment environment, GTRule gtRule) throws ViatraTransformationException
	{	
		GTPattern gtPattern = gtRule.getPrecondition().getCalledPattern();
		//EList rtAnnotations = gtRule.getRuntimeAnnotations();
		EList rtAnnotations = gtRule.getPrecondition().getCalledPattern().getRuntimeAnnotations();
		
		if (rtAnnotations!=null && rtAnnotations.size() > 0) {
			for (Object obj: rtAnnotations) {
				RuntimeAnnotation an = null;
				if (obj instanceof RuntimeAnnotation) {
					an =(RuntimeAnnotation)obj;
				} else {
					String[] context = {obj.toString()};
					throw new GraphTransformationException(ASMInterpreterErrorStrings.ANNOTATION_ILLEGAL
							,context
							,gtRule);
				}
				if ("@incremental".equals( an.getAnnotationName().toLowerCase())) {
//					boolean parallel = false;
//					for (Object o2 : an.getElements()) {
//						if (o2 instanceof RuntimeAnnotationElement) {
//							RuntimeAnnotationElement rae = (RuntimeAnnotationElement) o2;
//							if ("parallel".equals(rae.getKey()))
//							{
//								String lcValue = rae.getValue().toLowerCase();
//								parallel = "true".equals(lcValue) || "parallel".equals(lcValue) || "1".equals(lcValue); 
//								break;
//							}
//						}
//					}
//					if (parallel)
//						return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(VIATRAGTMatcherPlugin.INCREMENTAL_PM_FACTORY_ID).getPatternMatcher(environment, gtPattern);
//					else
					return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(VIATRAGTMatcherPlugin.INCREMENTAL_PM_FACTORY_ID).getPatternMatcher(environment, gtPattern);
				}
			}
		}
		
//		//Annotation about LHS pattern matching
//		EList ptCallAnnotations = gtRule.getPrecondition().getRuntimeAnnotations();
//		EList ptAnnotations = gtRule.getPrecondition().getCalledPattern().getRuntimeAnnotations();
//		
//		//it got priority if it is locally annotated by the pattern call
//		if (ptCallAnnotations!=null && ptCallAnnotations.size()>0)
//			{
//				for(Object obj: ptCallAnnotations)
//					{
//					RuntimeAnnotation an = null;
//					if(obj instanceof RuntimeAnnotation)
//						an =(RuntimeAnnotation)obj;
//					else
//						throw new GTASMException(ASMInterpreterErrorStrings.ANNOTATION_ILLEGAL,gtRule);
//					if ( "@incremental".equals( an.getAnnotationName().toLowerCase() ))
//						return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(VIATRAGTMatcherPlugin.INCREMENTAL_PM_FACTORY_ID).getPatternMatcher(environment, gtPattern);
//					}
//			}
//		
//		if (ptAnnotations!=null && ptAnnotations.size()>0)
//		{
//			for(Object obj: ptAnnotations)
//				{
//				RuntimeAnnotation an = null;
//				if(obj instanceof RuntimeAnnotation)
//					an =(RuntimeAnnotation)obj;
//				else
//					throw new GTASMException(ASMInterpreterErrorStrings.ANNOTATION_ILLEGAL,gtRule);
//				if ( "@incremental".equals( an.getAnnotationName().toLowerCase() ))
//					return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(VIATRAGTMatcherPlugin.INCREMENTAL_PM_FACTORY_ID).getPatternMatcher(environment, gtPattern);
//				}
//		}
		//could not initialize the alternative pattern matcher --> switching to default
		return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(patternMathcerIDToUse).getPatternMatcher(environment, gtPattern);
	}
	
	public IPatternMatcher getPatternMatcherbyID(IExecutionEnvironment environment, GTRule gtRule, String ID)
		throws ViatraTransformationException
	{
		GTPattern gtPattern = gtRule.getPrecondition().getCalledPattern();
		return VIATRAGTMatcherPlugin.getDefault().getPatternMatcherFactory(ID).getPatternMatcher(environment, gtPattern);
	}
	
	/** Returns true if there is a registered alternative pattern matcher
	 * @return
	 */
	public boolean hasAlternatePatternMatcher(){
		return VIATRAGTMatcherPlugin.getDefault().hasAlternativePatternMatcher();
	}
}
