/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.adapters;

import java.util.Map;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.CodegenRecorderBuildable;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.CodegenRecordingCoordinator;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.RetePatternBuildException;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.Stub;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.terms.Term;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTPattern;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.PatternVariable;



/**
 * 
 * @author Bergmann Gábor
 *
 */
public abstract class CodegenRecorderGTASMBuildable extends
		CodegenRecorderBuildable<GTPattern> implements GTASMBuildable<String, String> {


	public CodegenRecorderGTASMBuildable(CodegenRecordingCoordinator<GTPattern> coordinator,
			String indent, String baseName, String instanceSuffix)  
	{
		super(coordinator, indent, baseName, instanceSuffix);
	}

	public Stub<String> buildGTASMTermChecker(Term term,
			Map<String, Integer> variableIndices,
			Map<String, String> variableEquivalence, Integer rhsIndex, Stub<String> stub)
			throws RetePatternBuildException 
	{
		int[] affectedIndices = new int[variableIndices.values().size()];
		int k = 0;
		for (Integer index : variableIndices.values()) affectedIndices[k++] = index;	
		
//		Map<String, String> variableGen = new HashMap<String, String>();
//		for (term.)
//			UnionFind<String> a; a.
		
		StringBuilder innerClass = new StringBuilder();
		innerClass.append("new AbstractEvaluator(){ \n");
		innerClass.append(indent + "\t@Override \n");
		innerClass.append(indent + "\tpublic Object doEvaluate(Tuple tuple) throws Exception { \n");
		generateTermEvalCode(innerClass, indent+"\t\t", term, variableIndices,variableEquivalence );
		//innerClass.append(indent + "\treturn null");
		innerClass.append(indent + "\t}\n");
		innerClass.append(indent + "}");
		String evaluator = declareNewValue("AbstractEvaluator", innerClass.toString());

		String[] arguments = {
			evaluator, gen(rhsIndex),
			gen(affectedIndices), gen(stub)
		};
		String resultVar = emitFunctionCall(coordinator.stubType, "buildPredicateChecker", arguments);
		
		return new Stub<String>(stub, resultVar);

		
	}
	
	
	public abstract void generateTermEvalCode(StringBuilder innerClass, String indent,
			Term term, Map<String, Integer> variableIndices,
			Map<String, String> variableEquivalence);

	public String gen(PatternVariable variable) {
		return "\"" + variable.getName() + "\"";
	}
	
	@Override
	public String genPattern(GTPattern desc) {
		return "\"" + desc.getFqn() + "\"";
	}
	@Override
	public String genPosMap(GTPattern desc) {
		String resultVar = declareNewValue("HashMap<Object, Integer>", "new HashMap<Object, Integer>()");
		Integer k=0;
		for (Object o : desc.getSymParameters()) {
			emitLine(resultVar + ".put(" + gen((PatternVariable)o) + ", " + k++ + ");");
		}
		return resultVar;
	}
	
}
