/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/


package org.eclipse.viatra2.gtasm.patternmatcher.incremental.adapters;

import java.lang.ref.WeakReference;
import java.util.Collection;
import java.util.LinkedList;

import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.boundary.IManipulationListener;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherBuilderContext;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherRuntimeContext;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.ReteEngine;
import org.eclipse.viatra2.logger.Logger;



public abstract class VPMContext<PatternDescription> 
	implements IPatternMatcherBuilderContext<PatternDescription>,
		IPatternMatcherRuntimeContext<PatternDescription>
{
	WeakReference<IFramework> framework;
	IModelManager manager; 
	/**
	 * @param framework
	 */
	public VPMContext(IFramework framework) {
		super();
		this.framework = new WeakReference<IFramework>(framework);
		this.manager = framework.getTopmodel().getModelManager();
	}

	public EdgeInterpretation edgeInterpretation() {
		return EdgeInterpretation.TERNARY;
	}

	public GeneralizationQueryDirection allowedGeneralizationQueryDirection() {
		return GeneralizationQueryDirection.BOTH;
	};
	
	public Collection<? extends Object> enumerateAllUnaries() {
		return manager.getEntities();
	}

	public Collection<Pair<Object, Object>> enumerateAllBinaryEdges() {
		throw new UnsupportedOperationException();
	}

	public Collection<? extends Object> enumerateAllTernaryEdges() {
		return manager.getRelations();
	}
	
	public Collection<? extends Object> enumerateDirectSupertypes(Object typeObject) {
		return ((IModelElement)typeObject).getSupertypes();
	}
	
	public Collection<? extends Object> enumerateDirectSubtypes(Object typeObject) {
		return ((IModelElement)typeObject).getSubtypes();
	}
	
	public Collection<? extends Object> enumerateDirectUnaryInstances(Object typeObject) {
		return ((IEntity)typeObject).getInstances();
	}

	public Collection<? extends Object> enumerateDirectUnarySubtypes(Object typeObject) {
		return ((IEntity)typeObject).getSubtypes();
	}
	
	public Collection<? extends Object> enumerateDirectUnarySupertypes(Object typeObject) {
		return ((IEntity)typeObject).getSupertypes();
	}
	
	public Collection<Pair<Object, Object>> enumerateDirectBinaryEdgeInstances(
			Object typeObject) {
		throw new UnsupportedOperationException();
	}
	public Collection<? extends Object> enumerateDirectBinaryEdgeSubtypes(
			Object typeObject) {
		throw new UnsupportedOperationException();
	}
	public Collection<? extends Object> enumerateDirectBinaryEdgeSupertypes(
			Object typeObject) {
		throw new UnsupportedOperationException();
	}
	
	public Collection<? extends Object> enumerateDirectTernaryEdgeInstances(Object typeObject) {
		return ((IRelation)typeObject).getInstances();
	}
	public Collection<? extends Object> enumerateDirectTernaryEdgeSubtypes(Object typeObject) {
		return ((IRelation)typeObject).getSubtypes();
	}
	public Collection<? extends Object> enumerateDirectTernaryEdgeSupertypes(
			Object typeObject) {
		return ((IRelation)typeObject).getSupertypes();
	}
	
	public IFramework getFramework() {
		return framework.get();
	}

	public boolean inputConstructionEmbargo() {
		return false;
	}

	public Object ternaryEdgeSource(Object relation) {
		return ((IRelation)relation).getFrom();
	}

	public Object ternaryEdgeTarget(Object relation) {
		return ((IRelation)relation).getTo();
	}

	public Object ternaryEdgeSourceType(Object typeObject) {
		IRelation relation = (IRelation)typeObject;
		return relation.getIsAnyFrom() ? null : relation.getFrom();

	}

	public Object ternaryEdgeTargetType(Object typeObject) {
		IRelation relation = (IRelation)typeObject;
		return relation.getIsAnyTo() ? null : relation.getTo();

	}
	
	public Object binaryEdgeSourceType(Object typeObject) {
		throw new UnsupportedOperationException();
	}

	public Object binaryEdgeTargetType(Object typeObject) {
		throw new UnsupportedOperationException();
	}

	
	public Object retrieveUnaryType(String fullyQualifiedName) {
		return manager.getElementByName(fullyQualifiedName);
	}

	public Object retrieveBinaryEdgeType(String fullyQualifiedName) {
		throw new UnsupportedOperationException(); // no references over VPM
		//return framework.get().getTopmodel().getModelManager().getElementByName(fullyQualifiedName);
	}

	public Object retrieveTernaryEdgeType(String fullyQualifiedName) {
		return manager.getElementByName(fullyQualifiedName);
	}

	
	public String retrieveUnaryTypeFQN(Object typeObject) {
		return ((IEntity)typeObject).getFullyQualifiedName();
	}

	public String retrieveBinaryEdgeTypeFQN(Object typeObject) {
		throw new UnsupportedOperationException(); // no references over VPM
	}

	public String retrieveTernaryEdgeTypeFQN(Object typeObject) {
		return ((IRelation)typeObject).getFullyQualifiedName();
	}

	public Object resolveConstant(String fullyQualifiedName) {
		return manager.getElementByName(fullyQualifiedName);
	}

	public Collection<Pair<Object, Object>> enumerateAllUnaryContainments() {
		Collection<Pair<Object, Object>> collector = new LinkedList<Pair<Object, Object>>();
		for (IEntity container : framework.get().getTopmodel()
				.getModelManager().getEntities()) 
			for (IModelElement element : container.getContents())
				if (element instanceof IEntity)
					collector.add(new Pair<Object, Object>(container, element));
		return collector;
	}

	public Collection<Pair<Object, Object>> enumerateAllInstantiations() {
		Collection<Pair<Object, Object>> collector = new LinkedList<Pair<Object, Object>>();
		for (IModelElement type : framework.get().getTopmodel()
				.getModelManager().getElements()) {
			for (IModelElement instance : type.getInstances()) {
				collector.add(new Pair<Object, Object>(type, instance));
			}
		}
		return collector;		
	}

	public Collection<Pair<Object, Object>> enumerateAllGeneralizations() {
		Collection<Pair<Object, Object>> collector = new LinkedList<Pair<Object, Object>>();
		for (IModelElement superType : framework.get().getTopmodel()
				.getModelManager().getElements()) {
			for (IModelElement subType : superType.getSubtypes()) {
				collector.add(new Pair<Object, Object>(superType, subType));
			}
		}
		return collector;		
	}


	public void modelReadLock() {
		framework.get().getTopmodel().getModelManager().getLock().readLock().lock();
	}

	public void modelReadUnLock() {
		framework.get().getTopmodel().getModelManager().getLock().readLock().unlock();
	}

	public IManipulationListener subscribePatternMatcherForUpdates(ReteEngine<PatternDescription> engine) {
		return new GlobalModelspaceListener(engine, framework.get());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherContext#logDebug(java.lang.String)
	 */
	public void logDebug(String message) {
		framework.get().getLogger().debug(message);
	}
	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherContext#logError(java.lang.String)
	 */
	public void logError(String message) {
		framework.get().getLogger().error(message);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherContext#logError(java.lang.String, java.lang.Throwable)
	 */
	public void logError(String message, Throwable cause) {
		framework.get().getLogger().message(Logger.ERROR, message, cause);
	}
	
//	public boolean checkBelowContainer(Object container, Object contained) {
//		return ((IModelElement)contained).isBelowNamespace((IModelElement)container);
//	}
//
//
//	public boolean checkInContainer(Object container, Object contained) {
//		return container.equals(((IModelElement) contained).getNamespace());
//	}

}
