/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.boundary;

import java.util.Collection;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherRuntimeContext;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.network.ReteContainer;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.network.StandardNode;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.tuple.FlatTuple;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.tuple.Tuple;


public class RelationFeederNode extends StandardNode {

	protected Object typeObject;
	protected IPatternMatcherRuntimeContext<?> context;
	protected ReteBoundary<?> boundary;

	/**
	 * typeObject = null means feeder for all relations regardless of type
	 */
	public RelationFeederNode(IPatternMatcherRuntimeContext<?> context,
			ReteContainer reteNet, ReteBoundary<?> boundary, Object typeObject) {
		super(reteNet);
		// this.engine = engine;
		// this.typeObject = supplierKey;
		this.typeObject = typeObject;
//		type = (typeObject == null) ? null : framework.get().getTopmodel()
//				.getModelManager().getRelationByName(typeObject);
		this.context = context;
		this.boundary = boundary;
	}

	public void pullInto(Collection<Tuple> collector) {
		for (Object instance : instances()) {
			Object relation = instance;
			Object from = context.ternaryEdgeSource(relation);					
			Object to = context.ternaryEdgeTarget(relation);
			collector.add(new FlatTuple(boundary.wrapElement(relation),
					boundary.wrapElement(from), boundary
							.wrapElement(to)));
		}
	}

	Collection<? extends Object> instances() {
		if (typeObject != null)
			return context.enumerateDirectTernaryEdgeInstances(typeObject);
		else
			return context.enumerateAllTernaryEdges();
	}

}
