/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.LinkedHashSet;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherContext;



/**
 * 
 * @author Bergmann Gábor
 *
 */
public class CodegenRecordingCoordinator<PatternDescription> {
	static final String varPrefix = "var";
	static final String buildablePrefix = "buildable";
	static final String collectorPrefix = "production";
	
	public Long nextIdentifier; /**/
	public PrintStream code; /**/
	public String stubType; /**/
	public String collectorType; /**/
	public String buildableType; /**/
	HashMap<PatternDescription, String> collectors; /**/
	LinkedHashSet<PatternDescription> unbuilt; /**/
	public IPatternMatcherContext<PatternDescription> targetContext; /**/

	public CodegenRecordingCoordinator(IPatternMatcherContext<PatternDescription> targetContext,
			PrintStream code,
			String stubType, String collectorType, String buildableType) 
	{
		super();
		this.targetContext = targetContext;
		this.code = code;
		this.nextIdentifier = 0L;
		this.stubType = stubType;
		this.collectorType = collectorType;
		this.buildableType = buildableType;
		
		this.collectors = new HashMap<PatternDescription, String>();
		this.unbuilt = new LinkedHashSet<PatternDescription>();
	}
	
	String newIdentifier(String prefix){
		return prefix + "_"+(nextIdentifier++).toString();
	}

	String newVariableIdentifier(){
		return newIdentifier(varPrefix);
	}
	String newBuildableIdentifier(){
		return newIdentifier(buildablePrefix);
	}
	String newCollectorIdentifier(){
		return newIdentifier(collectorPrefix);
	}
	
	void emitLine(String indent, String line) {
		code.println(indent + line);
	}
	
	String allocateNewCollector(PatternDescription pattern) {
		if (collectors.containsKey(pattern)) 
			throw new UnsupportedOperationException("Duplicate production nodes unsupported in RETE code generation");
		String prod = newCollectorIdentifier();
		collectors.put(pattern, prod);
		unbuilt.remove(pattern);
		return prod;
	}
	
	/**
	 * @pre isComplete()
	 */
	public void printMembers(String indent) {
		for (String collector : collectors.values()) {
			emitLine(indent, collectorType + " " + collector + ";");
		}
	}

	public boolean isComplete() {
		return unbuilt.isEmpty();
	}
	public PatternDescription nextUnbuilt() {
		return unbuilt.iterator().next();	
	}	
}
