/*******************************************************************************
 * Copyright (c) 2004-2009 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher;

import java.util.Collection;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.boundary.IManipulationListener;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.boundary.IPredicateTraceListener;



/**
 * Represents all knowledge of the outside world, that is needed durin runtime operation, towards the pattern matcher.
 *  
 * @author Bergmann Gábor
 *
 */
public interface IPatternMatcherRuntimeContext<PatternDescription> extends
		IPatternMatcherContext<PatternDescription> {

	// ---------------------------------------------------------------------------------	
	
	
	/**
	 * @pre: network, framework, boundary, disconnectables initialised
	 */
	IManipulationListener subscribePatternMatcherForUpdates(ReteEngine<PatternDescription> engine);
	/**
	 * @pre: boundary, disconnectables initialised
	 */
	IPredicateTraceListener subscribePatternMatcherForTraceInfluences(ReteEngine<PatternDescription> engine);

	Object ternaryEdgeTarget(Object relation);
	Object ternaryEdgeSource(Object relation);

	Collection<? extends Object> enumerateAllUnaries();
	Collection<? extends Object> enumerateAllTernaryEdges();
	Collection<Pair<Object, Object>> enumerateAllBinaryEdges();	// first=from, second=to

	Collection<? extends Object> enumerateDirectUnaryInstances(Object typeObject);
	Collection<? extends Object> enumerateDirectTernaryEdgeInstances(Object typeObject);
	Collection<Pair<Object, Object>> enumerateDirectBinaryEdgeInstances(Object typeObject);	// first=from, second=to	

	Collection<Pair<Object, Object>> enumerateAllUnaryContainments();	// first=container, second=contained
	Collection<Pair<Object, Object>> enumerateAllInstantiations();	// first=type, second=instance
	Collection<Pair<Object, Object>> enumerateAllGeneralizations();	// first=supertype, second=subtype

	void modelReadLock();
	void modelReadUnLock();

}
