/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.simple;

import java.util.HashMap;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.adapters.GTASMBuildable;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.IRetePatternBuilder;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.construction.RetePatternBuildException;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.IPatternMatcherBuilderContext;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTPattern;



// import org.eclipse.viatra2.gtasm.patternmatcher.term.TermHandler;

/**
 * @author Gabor Bergmann
 * 
 */
public class SimpleReteBuilder<StubHandle, Collector> implements IRetePatternBuilder<GTPattern, StubHandle, Collector> {
	protected GTASMBuildable<StubHandle, Collector> buildable;
	protected IPatternMatcherBuilderContext<GTPattern> context;

	/**
	 * Prerequisite: engine has its network and boundary fields initialized
	 * 
	 * @param logger
	 */
	public SimpleReteBuilder(GTASMBuildable<StubHandle, Collector> buildable, IPatternMatcherBuilderContext<GTPattern> context) {
		super();
		this.buildable = buildable;
		this.context = context;
		//refresh();
	}

	public void refresh() {
		buildable.reinitialize();
//		this.context = engine.getContext();		
//		this.reteNet = engine.getReteNet();
//		this.headContainer = reteNet.getHeadContainer();
//		this.boundary = engine.getBoundary();
	}

	public Collector construct(GTPattern gtPattern) throws RetePatternBuildException {
		Collector production = buildable.patternCollector(gtPattern);
		return new PatternScaffold<StubHandle, Collector>(this, gtPattern, production).run();
	}

//	public Address<? extends Production> constructScoper(
//			Collector unscopedProduction,
//			Map<Integer, Scope> additionalScopeMap,
//			Collector production)
//			throws PatternMatcherCompileTimeException {
//
//		return new ScopeExtensionScaffold(this, unscopedProduction, additionalScopeMap, production).run();
//	}

	public HashMap<Object, Integer> getPosMapping(GTPattern gtPattern) {
		HashMap<Object, Integer> posMapping = new HashMap<Object, Integer>();
		int l = 0;
		for (Object o : gtPattern.getSymParameters()) {
			posMapping.put(o, l++);
		}
		return posMapping;
	}

}
