/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Okros, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Okros - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.gtasm.trigger.engine;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.framework.IFrameworkListChangedListener;

/**
 * A singleton manager class for managing the framework - trigger execution engine associations.
 * @author Andr�s �kr�s
 */
public class TriggerEngineManager implements IFrameworkListChangedListener
{
	
	private static TriggerEngineManager _instance = new TriggerEngineManager();
	private Map<IFramework, TriggerExecutionEngine> engines = new HashMap<IFramework, TriggerExecutionEngine>();
	
	public static TriggerEngineManager getInstance() {
		return _instance;
	}
	
	private TriggerEngineManager() 
	{
		FrameworkManager.getInstance().addFrameworkListChangedListener(this);
	}
	
	public TriggerExecutionEngine getTriggerEngine(IFramework fw)
	{
		if (!engines.containsKey(fw))
		{
			TriggerExecutionEngine engine = new TriggerExecutionEngine(fw);
			engines.put(fw, engine);
			return engine;
		} else {
			TriggerExecutionEngine engine = engines.get(fw);
			return engine;
		}
	}

//	//Framework Listener
//	public void frameworkAdded(IFramework fw) {
//		if (!engines.containsKey(fw)) 
//		{
//			TriggerExecutionEngine engine = new TriggerExecutionEngine(fw);
//			engines.put(fw, engine);
//		}
//	}
	
	//Framework Listener
	public void frameworkAdded(IFramework f) {
		
	}
	
	//Framework Listener
	public void frameworkRemoved(IFramework fw) {
		if (engines.containsKey(fw)) 
		{
			engines.remove(fw);
		}
	}


		
}
