/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Okros, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Okros - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.gtasm.trigger.model;

import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.matcher.RetePatternMatcher;
import org.eclipse.viatra2.gtasm.patternmatcher.incremental.rete.misc.DeltaMonitor;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTRule;

/**
 * Model class for triggers.
 * @author Andr�s �kr�s
 */
public class Trigger 
{
	protected String name;
	protected int priority;
	protected String mode;  // always (default) | once
	protected String sensitivity; // rise (default) | fall
	protected String startup; // active (default)  | passive
	protected String execution; // forall (default) | iterate
	protected Machine machine;
	protected GTRule gtrule;
	protected RetePatternMatcher retepatternmatcher;
	protected DeltaMonitor receiver;
	protected boolean isRunning;
	protected boolean isASMFunctionsSet;
	protected boolean isReteEngineSet;
		
	public Trigger(String aName)
	{
		// Set the trigger's name:
		name = aName;
		
		// Initialize the default settings for the trigger:
		priority = 0;			// Other option: any integer number
		mode = "always";		// Other option: once
		sensitivity = "rise";	// Other option: fall
		startup = "active";		// Other option: passive
		execution = "forall";	// Other option: iterate
		isASMFunctionsSet = false;
		isReteEngineSet = false;
		isRunning = false;
	}
	
	//Getters & Setters:
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public GTRule getGtrule() {
		return gtrule;
	}

	public void setGtrule(GTRule gtrule) {
		this.gtrule = gtrule;
	}

	public Machine getMachine() {
		return machine;
	}

	public void setMachine(Machine machine) {
		this.machine = machine;
	}

	public String getMode() {
		return mode;
	}

	public void setMode(String mode) {
		this.mode = mode;
	}

	public int getPriority() {
		return priority;
	}

	public void setPriority(int priority) {
		this.priority = priority;
	}

	public String getSensitivity() {
		return sensitivity;
	}

	public void setSensitivity(String sensitivity) {
		this.sensitivity = sensitivity;
	}

	public String getStartup() {
		return startup;
	}

	public void setStartup(String startup) {
		this.startup = startup;
	}

	public RetePatternMatcher getRetepatternmatcher() {
		return retepatternmatcher;
	}

	public void setRetepatternmatcher(RetePatternMatcher retepatternmatcher) {
		this.retepatternmatcher = retepatternmatcher;
	}

	public DeltaMonitor getReceiver() {
		return receiver;
	}

	public void setReceiver(DeltaMonitor receiver) {
		this.receiver = receiver;
	}

	public boolean isASMFunctionsSet() {
		return isASMFunctionsSet;
	}

	public void setASMFunctionsSet(boolean isASMFunctionsSet) {
		this.isASMFunctionsSet = isASMFunctionsSet;
	}

	public String getExecution() {
		return execution;
	}

	public void setExecution(String execution) {
		this.execution = execution;
	}

	public boolean isRunning() {
		return isRunning;
	}

	public void setRunning(boolean isRunning) {
		this.isRunning = isRunning;
	}

	public boolean isReteEngineSet() {
		return isReteEngineSet;
	}

	public void setReteEngineSet(boolean isReteEngineSet) {
		this.isReteEngineSet = isReteEngineSet;
	}

}
