/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gui.perspectives;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * Class that customizes the Eclipse UI to form the VIATRA perspective.
 * @author istvan rath
 *
 */
public class VIATRAPerspective implements IPerspectiveFactory {

	private IPageLayout factory;

	public VIATRAPerspective() {
		super();
	}

	public void createInitialLayout(IPageLayout factory) {
		this.factory = factory;
		addViews();
		addNewWizardShortcuts();
		addViewShortcuts();
	}

	private void addViews() {
		// Creates the overall folder layout. 
		// Note that each new Folder uses a percentage of the remaining EditorArea.
		


		IFolderLayout topLeft =
			factory.createFolder(
				"topLeft", //NON-NLS-1
				IPageLayout.LEFT,
				0.25f,
				factory.getEditorArea());
		topLeft.addView(IPageLayout.ID_RES_NAV);
		
		IFolderLayout bottomLeft =
			factory.createFolder(
				"bottomLeft", //NON-NLS-1
				IPageLayout.BOTTOM,
				0.75f,
				"topLeft");
		bottomLeft.addView("org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView"); //NON-NLS-1
		
		
		IFolderLayout bottom =
			factory.createFolder(
				"bottomRight", //NON-NLS-1
				IPageLayout.BOTTOM,
				0.75f,
				factory.getEditorArea());
		bottom.addView("org.eclipse.viatra2.gui.codebufferview");
		
		bottom.addView("org.eclipse.viatra2.gui.console");
		bottom.addView(IPageLayout.ID_PROP_SHEET);
		bottom.addView(IPageLayout.ID_PROBLEM_VIEW);
	
		IFolderLayout topRight =
			factory.createFolder(
				"topRight", //NON-NLS-1
				IPageLayout.RIGHT,
				0.80f,
				factory.getEditorArea());
		topRight.addView(IPageLayout.ID_OUTLINE);
		
		
		//factory.addFastView("org.eclipse.team.ccvs.ui.RepositoriesView",0.50f); //NON-NLS-1
		factory.addFastView("org.eclipse.team.sync.views.SynchronizeView", 0.50f); //NON-NLS-1
		factory.addFastView("org.eclipse.pde.runtime.LogView");
	}

	private void addNewWizardShortcuts() {
		factory.addNewWizardShortcut("org.eclipse.viatra2.editor.newVPMLWizard");//NON-NLS-1
		factory.addNewWizardShortcut("org.eclipse.viatra2.editor.text.wizard.vtcl");//NON-NLS-1
		factory.addNewWizardShortcut("org.eclipse.viatra2.editor.text.wizard.vtml");//NON-NLS-1
	}

	private void addViewShortcuts() {
		factory.addShowViewShortcut("org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView"); //NON-NLS-1
		factory.addShowViewShortcut("org.eclipse.viatra2.gui.codebufferview"); //NON-NLS-1
		factory.addShowViewShortcut("org.eclipse.viatra2.gui.console"); //NON-NLS-1
		
		//factory.addShowViewShortcut(JavaUI.ID_PACKAGES);
		factory.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);
		factory.addShowViewShortcut(IPageLayout.ID_PROBLEM_VIEW);
		factory.addShowViewShortcut(IPageLayout.ID_OUTLINE);
	}

}
