/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports;

import java.io.InputStream;

import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.properties.VPMProperties;

/**
 * Interface for implementations of an importer that imports the whole VPM model
 * from one file.
 * 
 * @author Andras Schmidt
 * 
 */
public interface IVPMImporter {

	/**
	 * Open a stream and import its content into the model
	 * 
	 * @param is
	 *            stream that contains the model
	 * @param topmodel
	 *            a new instance of IModelSpace
	 * @param properties
	 *            properties to load into
	 */
	void process(InputStream is, IModelSpace topmodel, VPMProperties props)
			throws VPMRuntimeException;

	/**
	 * Open a file and import its content into the model
	 * 
	 * @param initialModel
	 *            file name of the model
	 * @param topmodel
	 *            a new instance of IModelSpace
	 * @param properties
	 *            properties to load into
	 */
	void process(String initialModel, IModelSpace topmodel, VPMProperties props)
			throws VPMRuntimeException;

}
