/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports;

import org.eclipse.viatra2.errors.VPMRuntimeException;

/**
 * Interface of importer creator classes.
 * 
 * @author Andras Balogh
 * 
 */
public interface NativeImporterFactory {

	/**
	 * Descriptive name of importer.
	 * 
	 * @return
	 */
	public String getImporterName();

	/**
	 * Initialise the importer.
	 * 
	 * @param f
	 * @return
	 * @throws VPMRuntimeException
	 */
	public NativeImporter getImporterInstance() throws VPMRuntimeException;

	/**
	 * Returns a preferred id for the given importer.
	 * 
	 * @return
	 */
	public String getId();

	/**
	 * Get the set of file extensions that are handled by this importer.
	 * 
	 * @return the set of file extensions that are handled by this importer
	 */
	public String[] getFileExtensionList();

}
