/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports;

import org.eclipse.core.resources.IResource;
import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.logger.Logger;

/**
 * Summary: An extended native importer interface supporting imports from
 * Eclipse Resources.
 * 
 * Description: This interface, as of the initial version, is a proposal. As the
 * native importer functionality only works if Viatra2 is being run within
 * Eclipse, it seems logical to provide a way for passing IResources to native
 * importers, since all files/java code snippets/EMF models exist as IResource
 * instances within the workspace.
 * 
 * @author Istvan Rath
 * 
 */
public interface NativeResourceImporter extends NativeImporter {

	/**
	 * Processes an IResource instance for native importing.
	 * 
	 * @param r
	 *            IResource to process
	 * @param m
	 *            The current modelspace
	 * @param l
	 *            The system logger
	 * @throws VPMRuntimeException
	 */
	public void processResource(IResource r, IModelSpace m, IFramework fw,
			Logger l) throws VPMRuntimeException;

}
