/*******************************************************************************
 * Copyright (c) 2009-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports.uml2.galileo.modelexporter;

import java.io.IOException;
import java.util.HashSet;


import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.treeeditor.ViatraTreeEditor;
import org.eclipse.viatra2.treeeditor.actions.ViatraTreeEditorSelectionAction;


public class UmlModelExportAction extends ViatraTreeEditorSelectionAction{

	public static final String ID = "org.eclipse.viatra2.imports.uml2.galileo.modelexporter";
	static HashSet<String> packageFQNs = new HashSet<String>();
	static {
	 packageFQNs.add("uml.metamodel.uml");
	}

	private ViatraTreeEditor iVTE;	
	
	public UmlModelExportAction() {
		this(null);
	}
	
	public UmlModelExportAction(IWorkbenchPart part) {
		super(part);
		iVTE = (ViatraTreeEditor) part;
		setId(ID);
		setText("Export as .uml");
		setToolTipText("Export selected model element as a Uml EMF model");
		setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_ETOOL_SAVE_EDIT));

	}
	
	@Override
	public void setWorkbenchPart(IWorkbenchPart part) {
		super.setWorkbenchPart(part);
		iVTE = (ViatraTreeEditor) part;
	}
	
	//IWorkbenchPart targetPart;
	IEntity subject;
	
	public void run() {
		
		FileDialog fd = new FileDialog(iVTE.getEditorSite().getShell(),SWT.SAVE);
		//final String[] extensions = {"*.uml"};
		//fd.setFilterExtensions(extensions);
		fd.setFileName(subject.getName()+".uml");
		String fn = fd.open();
		if (fn!=null)
		{
		UmlModelExportModel emod = new UmlModelExportModel();
		emod.getPackageMap().put(subject.getModelSpace().getModelManager().getEntityByName("uml.metamodel.uml")
				,org.eclipse.uml2.uml.UMLPackage.eINSTANCE);
		EObject oo = emod.exportModel(subject);
		
		Resource res = new ResourceSetImpl().createResource(URI.createFileURI(fn));
		res.getContents().add(oo);
		try {
			res.save(null);
		} catch (IOException e) {
			e.printStackTrace();
		}
		}
		Shell shell = new Shell();
		MessageDialog.openInformation(
			shell,
			"EMF Exporter Plug-in",
			"Model exported successfully.");
	}
	


	@Override
	public void updateSelf() {
		subject=null;
		if (getSelectedObjects().size() == 1)
		{
			Object o = getSelectedObjects().getFirstElement();
			if (o instanceof IEntity)
			{
				subject=(IEntity)o;
			}
		}
		if (subject==null )
		{
			setEnabled(false);
		}
		else
		{
			for (IModelElement type: subject.getAllTypes())
			{
				if (type instanceof IEntity && packageFQNs.contains(((IEntity)type).getParent().getFullyQualifiedName())) {
					setEnabled(true);
					return;
				}
			}
			setEnabled(false);
			
		}
	}

}
