/*******************************************************************************
 * Copyright (c) 2009-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports.uml2.galileo.modelimporter;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.Enumerator;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;
import org.eclipse.viatra2.imports.uml2.galileo.Activator;
import org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.*;
import org.eclipse.viatra2.imports.uml2.preferences.PreferenceConstants;
import org.eclipse.viatra2.logger.Logger;
import org.eclipse.uml2.uml.*;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.internal.impl.PropertyImpl;
import org.eclipse.uml2.uml.internal.impl.UMLPackageImpl;
/* @generated */
public class UmlModelImporter implements NativeImporter {

	public String convertName(String s)
	{
		if (s==null) return "nullName";
		return s.replaceAll("\\.", "_");
	}
	public List<RelationReference> references = new ArrayList<RelationReference>();
	public Map<java.lang.Object,IEntity> elemref = new HashMap<java.lang.Object, IEntity>();

	public void process(InputStream f, IFramework fw)
	throws VPMRuntimeException {
		// TODO Auto-generated method stub

	}

/******************* handwritten code *******************/	
	
	
	List<String> profiles = new ArrayList<String>();
	/*	{
	profiles.add("Ecore");
	profiles.add("ecore");
	profiles.add("uml");
	profiles.add("Standard");
	profiles.add("UML");
	profiles.add("Business Modeling");
	profiles.add("CORBA Transformation");
	profiles.add("CPPTransformation");
	profiles.add("Default");
	profiles.add("Deployment");
	profiles.add("DoDAF");
	profiles.add("EJBTransformProfile");
	profiles.add("JavaTransformation5.0");
	profiles.add("JavaTransformation1.4");
	profiles.add("LogicalDataModel");
	profiles.add("ProfileBase");
	profiles.add("RUPAnalysis");
	profiles.add("Software Services Profile");
	profiles.add("XSDProfile");

}*/
	boolean ignoreProfiles = true;
	
	private void initOptions()
	{
		ignoreProfiles = Activator.getDefault().getPreferenceStore().getBoolean(PreferenceConstants.P_IGNORE);
		if (ignoreProfiles)
		{
			String s = Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.P_PROFILES);
			String[] tmp = s.split("\\|");
			for (String y : tmp) profiles.add(y);
		}
	}
	
/************************** end of hand written code *********************/	
	
	
	

private void resolveReferences() throws VPMCoreException, VPMRuntimeException
	{
		// 'references' may grow in the meantime !
		for(int currentRef = 0; currentRef < references.size(); currentRef++)
		{
			RelationReference r = references.get(currentRef);
			IEntity ee = elemref.get(r.getTarget());
			if (ee==null)
			{
				if (r.getTarget() instanceof EObject)
				{
					if (((EObject)r.getTarget()).eResource()==resource)
					{
						ee = routeProcessing(r.getTarget(),cont);
						if (ee!=null)
						{
							IRelation rel = mm.newRelation(r.getElement(),ee);
							mm.newInstanceOf(r.getType(), rel);
						} else {
							log.debug
							("[UML2 importer] Failed to map element (owner:"+r.getElement().getFullyQualifiedName()+",type: "+r.getType().getFullyQualifiedName()+", target: "+r.getTarget().toString()+")");
						}
					}
					else {
						log
						.debug("[UML2 importer] Found an invalid reference for "+r.getElement().getFullyQualifiedName()+" of type "+r.getType().getFullyQualifiedName()+" pointing to outside resource: "
								+((EObject)r.getTarget()).eResource().toString());
						
					}
				} 
				else
				{
					ee = routeProcessing(r.getTarget(),cont);
					if (ee!=null)
					{
						IRelation rel = mm.newRelation(r.getElement(),ee);
						mm.newInstanceOf(r.getType(), rel);
					} else {
						log.debug
						("[UML2 importer] Failed to map element (owner:"+r.getElement().getFullyQualifiedName()+",type: "+r.getType().getFullyQualifiedName()+", target: "+r.getTarget().toString()+")");
					}
			
				//	log
				//	.warning("Found an invalid reference for "+r.getElement().getFullyQualifiedName()+" of type "+r.getType().getFullyQualifiedName()+" pointing to a non-EObject instance: "+r.getTarget().toString());
				}
			} 
			else
			{
				IRelation rel = mm.newRelation(r.getElement(),ee);
				mm.newInstanceOf(r.getType(), rel);
			}
		}
	}
	public UmlModelReference ref;
	public IModelManager mm;
	IEntity cont;
	Resource resource;
	
	Logger log;
	public void processFile(String f, IFramework fw) throws VPMRuntimeException {

		
		try
		{
			// hand written code
			initOptions();
			// end of hand written code
			mm = fw.getTopmodel().getModelManager();
		ref = new UmlModelReference(mm);
		
		log = mm.getRoot().getModelSpace().getFramework().getLogger();
		
		registerProcessors();
		// hand-written code
		UMLPackageImpl.init();
		// end of hand-written code

		//TODO initialization
		
		resource = new ResourceSetImpl().getResource(URI.createFileURI(f), true);

		java.lang.Object root = (java.lang.Object)resource.getContents().get(0);

		/************* hand-written code start ************/
		if (root instanceof Package)
		{	
			Package m = (Package) root;

			
			EList profs = m.getAllAppliedProfiles();
			if (profs!=null)
			{
				for (int i=0;i<profs.size();i++)
					((Profile)profs.get(i)).getName();
			}
			if (ignoreProfiles) {
				log.warning("[UML2 importer] ignoring profiles...");
			}			
			
			String tmpname =null;
			try
			{
				tmpname= new File(f).getName();
				tmpname=convertName(tmpname);
			} catch (Exception e){}
			if (tmpname==null || tmpname.length()==0) tmpname="model_x";
			cont = mm.newEntity(tmpname, mm.getEntityByName("uml.models"));
			
			// go on for processing additional resources
			EList reslist = resource.getResourceSet().getResources();
			for (int r=0;r<reslist.size();r++)
			{
				EList ctlist =((Resource) reslist.get(r)).getContents();
				if (!ctlist.isEmpty())
				{
					Object o = ctlist.get(0);
					if (o instanceof Package) {
						if (!ignoreProfiles) {
							routeProcessing(o, cont);					
						}
						else {
							// we are ignoring some profiles
							if (profiles.contains(((NamedElement)o).getName())) {
								log.warning("[UML2 importer] ignoring and not importing: " + ((Package)o).getQualifiedName());
							//	routeProcessing(o, cont);
							}
							else {
								log.warning("[UML2 importer] NOT ignoring and importing reference: "+ ((Package)o).getQualifiedName());
								routeProcessing(o, cont);
							}
						}
					}
					else {
						log.error("[UML2 importer] encountered a non-package element in the resource, skipping: " + o.toString());
					}
				}
			}
			resolveReferences();

			elemref.clear();
			processors.clear();
			references.clear();
			reslist = resource.getResourceSet().getResources();
			for (int r=0;r<reslist.size();r++)
			{
				((Resource)reslist.get(r)).unload();
			}
			ref=null;
		}
		else
		{
			throw new VPMRuntimeException("UML model contents not supported.");
		}
	} catch (Exception ex)
	{
		fw.getLogger().fatal("Error while importing UML model: "+ex.getMessage());
		ex.printStackTrace();
	}
	/****************** hand-written code end ***************/	
		
/*		
		String tmpname =null;
		try
		{
		tmpname= f.substring(0,f.lastIndexOf("."));
		tmpname= tmpname.substring(f.lastIndexOf("\\")+1);
		tmpname=convertName(tmpname);
		} catch (Exception e){}
		if (tmpname==null || tmpname.length()==0) tmpname="model_x";
		cont = mm.newEntity(tmpname, mm.getEntityByName("uml.models"));
		routeProcessing(m,cont);

		resolveReferences();

		elemref.clear();
		processors.clear();
		references.clear();

		EList reslist = resource.getResourceSet().getResources();
		for (int r=0;r<reslist.size();r++)
		{
			((Resource)reslist.get(r)).unload();
		}
		ref=null;
		} catch (Exception ex)
		{
			ex.printStackTrace();
		}
		*/

	}


	private Map<String, AbstractProcessor> processors = new HashMap<String, AbstractProcessor>();

	public IEntity routeProcessLocal(String s,java.lang.Object o, IEntity e,IEntity cont)  throws VPMRuntimeException,VPMCoreException
	{
		AbstractProcessor p = processors.get(s);
		if (p!=null) return p.processLocal(o, e, cont);
		//else java.lang.System.out.println("no processor for type: "+((EObject)o).eClass().getName());
		return e;
	}

	/* hand-coded shortcuts for primitive types */
	
	private IEntity processInteger(Integer c,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
	{
		if (e==null)
		{
			e = mm.newEntity(container);
			mm.newInstanceOf(ref.ENT_UML_INTEGER,e);
			mm.setValue(e,c.toString());
		}

		return e;
	}
	private IEntity processBoolean(Boolean c,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
	{
		if (e==null)
		{
			e = mm.newEntity(container);
			mm.newInstanceOf(ref.ENT_UML_BOOLEAN,e);
			mm.setValue(e,c.toString());
		}

		return e;
	}
	private IEntity processString(String c,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
	{
		if (e==null)
		{
			e = mm.newEntity(container);
			mm.newInstanceOf(ref.ENT_UML_STRING,e);
			mm.setValue(e,c.toString());
		}

		return e;
	}
	private IEntity processUnlimitedNatural(Double c,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
	{
		if (e==null)
		{
			e = mm.newEntity(container);
			mm.newInstanceOf(ref.ENT_UML_UNLIMITEDNATURAL,e);
			mm.setValue(e,new Integer(c.intValue()).toString());

		}

		return e;
	}
	
	
	/* end of hand-coded shortcuts */
	
	
	public IEntity routeProcessing(java.lang.Object o, IEntity cont)  throws VPMRuntimeException,VPMCoreException
	{
		if (elemref.get(o)!=null) return elemref.get(o);
		// hand-coded part to add shortcuts to primitive type processors
		if (o instanceof Integer) return processInteger((Integer)o,null,cont); else
			if (o instanceof Boolean) return processBoolean((Boolean)o,null,cont); else
				if (o instanceof String) return processString((String)o,null,cont); else
					if (o instanceof Double) return processUnlimitedNatural((Double)o,null,cont); else
					{ // end of hand-written code

						if (o instanceof EObject)
						{
							AbstractProcessor p = processors.get(((EObject)o).eClass().getName());
							if (p!=null) return p.process(o, null, cont);
							//else java.lang.System.out.println("no processor for type: "+((EObject)o).eClass().getName());
						}
						else if (o instanceof java.lang.Class<?>)
						{
							java.lang.Class<?> _o = (java.lang.Class<?>) o;
							String tmp = _o.getSimpleName();
							tmp = tmp.substring(0, tmp.indexOf("Impl"));
							AbstractProcessor p = processors.get(tmp);
							if (p!=null) return p.process(o, null, cont);
							else {
								log.debug("[UML2 importer] no processor for type: "+(tmp));
							}
						}
						else //if (o instanceof Property)
						{
							String tmp = o.getClass().getSimpleName();
							AbstractProcessor p = processors.get(tmp);
							if (p!=null) return p.process(o, null, cont);
							else {
								log.debug("[UML2 importer] no processor for type: "+(tmp));
							}
						}
//						else
//						{
//							log.debug("unexpected element: "+o.getClass().getCanonicalName());
//						}
					}
		return null;
	}
private void registerProcessors()
{
	processors.put("Comment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CommentProcessor(mm,this));
	processors.put("Element", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ElementProcessor(mm,this));
	processors.put("Package", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PackageProcessor(mm,this));
	processors.put("PackageableElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PackageableElementProcessor(mm,this));
	processors.put("NamedElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.NamedElementProcessor(mm,this));
	processors.put("Dependency", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DependencyProcessor(mm,this));
	processors.put("DirectedRelationship", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DirectedRelationshipProcessor(mm,this));
	processors.put("Relationship", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RelationshipProcessor(mm,this));
	processors.put("Namespace", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.NamespaceProcessor(mm,this));
	processors.put("ElementImport", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ElementImportProcessor(mm,this));
	processors.put("PackageImport", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PackageImportProcessor(mm,this));
	processors.put("Constraint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConstraintProcessor(mm,this));
	processors.put("ValueSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ValueSpecificationProcessor(mm,this));
	processors.put("TypedElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TypedElementProcessor(mm,this));
	processors.put("Type", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TypeProcessor(mm,this));
	processors.put("Association", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AssociationProcessor(mm,this));
	processors.put("Classifier", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClassifierProcessor(mm,this));
	processors.put("RedefinableElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RedefinableElementProcessor(mm,this));
	processors.put("TemplateableElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TemplateableElementProcessor(mm,this));
	processors.put("TemplateBinding", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TemplateBindingProcessor(mm,this));
	processors.put("TemplateSignature", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TemplateSignatureProcessor(mm,this));
	processors.put("TemplateParameter", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TemplateParameterProcessor(mm,this));
	processors.put("ParameterableElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ParameterableElementProcessor(mm,this));
	processors.put("TemplateParameterSubstitution", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TemplateParameterSubstitutionProcessor(mm,this));
	processors.put("Generalization", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.GeneralizationProcessor(mm,this));
	processors.put("GeneralizationSet", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.GeneralizationSetProcessor(mm,this));
	processors.put("Feature", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.FeatureProcessor(mm,this));
	processors.put("Substitution", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SubstitutionProcessor(mm,this));
	processors.put("Realization", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RealizationProcessor(mm,this));
	processors.put("Abstraction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AbstractionProcessor(mm,this));
	processors.put("OpaqueExpression", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OpaqueExpressionProcessor(mm,this));
	processors.put("Parameter", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ParameterProcessor(mm,this));
	processors.put("MultiplicityElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MultiplicityElementProcessor(mm,this));
	processors.put("ConnectableElement", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConnectableElementProcessor(mm,this));
	processors.put("ConnectorEnd", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConnectorEndProcessor(mm,this));
	processors.put("Property", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PropertyProcessor(mm,this));
	processors.put("DeploymentTarget", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DeploymentTargetProcessor(mm,this));
	processors.put("Deployment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DeploymentProcessor(mm,this));
	processors.put("DeployedArtifact", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DeployedArtifactProcessor(mm,this));
	processors.put("DeploymentSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DeploymentSpecificationProcessor(mm,this));
	processors.put("Artifact", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ArtifactProcessor(mm,this));
	processors.put("Manifestation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ManifestationProcessor(mm,this));
	processors.put("Operation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OperationProcessor(mm,this));
	processors.put("BehavioralFeature", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.BehavioralFeatureProcessor(mm,this));
	processors.put("Behavior", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.BehaviorProcessor(mm,this));
	processors.put("Class", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClassProcessor(mm,this));
	processors.put("BehavioredClassifier", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.BehavioredClassifierProcessor(mm,this));
	processors.put("InterfaceRealization", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InterfaceRealizationProcessor(mm,this));
	processors.put("Interface", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InterfaceProcessor(mm,this));
	processors.put("Reception", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReceptionProcessor(mm,this));
	processors.put("Signal", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SignalProcessor(mm,this));
	processors.put("ProtocolStateMachine", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ProtocolStateMachineProcessor(mm,this));
	processors.put("StateMachine", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StateMachineProcessor(mm,this));
	processors.put("Region", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RegionProcessor(mm,this));
	processors.put("Vertex", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.VertexProcessor(mm,this));
	processors.put("Transition", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TransitionProcessor(mm,this));
	processors.put("Trigger", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TriggerProcessor(mm,this));
	processors.put("Event", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.EventProcessor(mm,this));
	processors.put("Port", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PortProcessor(mm,this));
	processors.put("State", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StateProcessor(mm,this));
	processors.put("ConnectionPointReference", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConnectionPointReferenceProcessor(mm,this));
	processors.put("Pseudostate", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PseudostateProcessor(mm,this));
	processors.put("ProtocolConformance", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ProtocolConformanceProcessor(mm,this));
	processors.put("EncapsulatedClassifier", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.EncapsulatedClassifierProcessor(mm,this));
	processors.put("StructuredClassifier", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StructuredClassifierProcessor(mm,this));
	processors.put("Connector", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConnectorProcessor(mm,this));
	processors.put("Extension", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExtensionProcessor(mm,this));
	processors.put("ExtensionEnd", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExtensionEndProcessor(mm,this));
	processors.put("Stereotype", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StereotypeProcessor(mm,this));
	processors.put("Image", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ImageProcessor(mm,this));
	processors.put("Profile", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ProfileProcessor(mm,this));
	processors.put("Model", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ModelProcessor(mm,this));
	processors.put("ParameterSet", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ParameterSetProcessor(mm,this));
	processors.put("DataType", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DataTypeProcessor(mm,this));
	processors.put("OperationTemplateParameter", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OperationTemplateParameterProcessor(mm,this));
	processors.put("StructuralFeature", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StructuralFeatureProcessor(mm,this));
	processors.put("ConnectableElementTemplateParameter", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConnectableElementTemplateParameterProcessor(mm,this));
	processors.put("CollaborationUse", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CollaborationUseProcessor(mm,this));
	processors.put("Collaboration", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CollaborationProcessor(mm,this));
	processors.put("UseCase", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.UseCaseProcessor(mm,this));
	processors.put("Include", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.IncludeProcessor(mm,this));
	processors.put("Extend", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExtendProcessor(mm,this));
	processors.put("ExtensionPoint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExtensionPointProcessor(mm,this));
	processors.put("RedefinableTemplateSignature", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RedefinableTemplateSignatureProcessor(mm,this));
	processors.put("ClassifierTemplateParameter", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClassifierTemplateParameterProcessor(mm,this));
	processors.put("StringExpression", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StringExpressionProcessor(mm,this));
	processors.put("Expression", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExpressionProcessor(mm,this));
	processors.put("Usage", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.UsageProcessor(mm,this));
	processors.put("PackageMerge", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PackageMergeProcessor(mm,this));
	processors.put("ProfileApplication", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ProfileApplicationProcessor(mm,this));
	processors.put("Enumeration", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.EnumerationProcessor(mm,this));
	processors.put("EnumerationLiteral", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.EnumerationLiteralProcessor(mm,this));
	processors.put("InstanceSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InstanceSpecificationProcessor(mm,this));
	processors.put("Slot", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SlotProcessor(mm,this));
	processors.put("PrimitiveType", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PrimitiveTypeProcessor(mm,this));
	processors.put("LiteralSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralSpecificationProcessor(mm,this));
	processors.put("LiteralInteger", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralIntegerProcessor(mm,this));
	processors.put("LiteralString", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralStringProcessor(mm,this));
	processors.put("LiteralBoolean", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralBooleanProcessor(mm,this));
	processors.put("LiteralNull", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralNullProcessor(mm,this));
	processors.put("InstanceValue", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InstanceValueProcessor(mm,this));
	processors.put("LiteralUnlimitedNatural", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LiteralUnlimitedNaturalProcessor(mm,this));
	processors.put("OpaqueBehavior", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OpaqueBehaviorProcessor(mm,this));
	processors.put("FunctionBehavior", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.FunctionBehaviorProcessor(mm,this));
	processors.put("OpaqueAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OpaqueActionProcessor(mm,this));
	processors.put("Action", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActionProcessor(mm,this));
	processors.put("ExecutableNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExecutableNodeProcessor(mm,this));
	processors.put("ActivityNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityNodeProcessor(mm,this));
	processors.put("StructuredActivityNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StructuredActivityNodeProcessor(mm,this));
	processors.put("ActivityGroup", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityGroupProcessor(mm,this));
	processors.put("Activity", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityProcessor(mm,this));
	processors.put("Variable", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.VariableProcessor(mm,this));
	processors.put("ActivityEdge", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityEdgeProcessor(mm,this));
	processors.put("ActivityPartition", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityPartitionProcessor(mm,this));
	processors.put("InterruptibleActivityRegion", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InterruptibleActivityRegionProcessor(mm,this));
	processors.put("ExceptionHandler", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExceptionHandlerProcessor(mm,this));
	processors.put("ObjectNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ObjectNodeProcessor(mm,this));
	processors.put("OutputPin", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OutputPinProcessor(mm,this));
	processors.put("Pin", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PinProcessor(mm,this));
	processors.put("InputPin", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InputPinProcessor(mm,this));
	processors.put("CallAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CallActionProcessor(mm,this));
	processors.put("InvocationAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InvocationActionProcessor(mm,this));
	processors.put("SendSignalAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SendSignalActionProcessor(mm,this));
	processors.put("CallOperationAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CallOperationActionProcessor(mm,this));
	processors.put("CallBehaviorAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CallBehaviorActionProcessor(mm,this));
	processors.put("SequenceNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SequenceNodeProcessor(mm,this));
	processors.put("ControlNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ControlNodeProcessor(mm,this));
	processors.put("ControlFlow", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ControlFlowProcessor(mm,this));
	processors.put("InitialNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InitialNodeProcessor(mm,this));
	processors.put("ActivityParameterNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityParameterNodeProcessor(mm,this));
	processors.put("ValuePin", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ValuePinProcessor(mm,this));
	processors.put("Message", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MessageProcessor(mm,this));
	processors.put("MessageEnd", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MessageEndProcessor(mm,this));
	processors.put("Interaction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InteractionProcessor(mm,this));
	processors.put("InteractionFragment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InteractionFragmentProcessor(mm,this));
	processors.put("Lifeline", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LifelineProcessor(mm,this));
	processors.put("PartDecomposition", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.PartDecompositionProcessor(mm,this));
	processors.put("InteractionUse", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InteractionUseProcessor(mm,this));
	processors.put("Gate", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.GateProcessor(mm,this));
	processors.put("GeneralOrdering", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.GeneralOrderingProcessor(mm,this));
	processors.put("OccurrenceSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.OccurrenceSpecificationProcessor(mm,this));
	processors.put("InteractionOperand", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InteractionOperandProcessor(mm,this));
	processors.put("InteractionConstraint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InteractionConstraintProcessor(mm,this));
	processors.put("ExecutionSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExecutionSpecificationProcessor(mm,this));
	processors.put("StateInvariant", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StateInvariantProcessor(mm,this));
	processors.put("ActionExecutionSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActionExecutionSpecificationProcessor(mm,this));
	processors.put("BehaviorExecutionSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.BehaviorExecutionSpecificationProcessor(mm,this));
	processors.put("ExecutionEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExecutionEventProcessor(mm,this));
	processors.put("CreationEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CreationEventProcessor(mm,this));
	processors.put("DestructionEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DestructionEventProcessor(mm,this));
	processors.put("SendOperationEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SendOperationEventProcessor(mm,this));
	processors.put("MessageEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MessageEventProcessor(mm,this));
	processors.put("SendSignalEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SendSignalEventProcessor(mm,this));
	processors.put("MessageOccurrenceSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MessageOccurrenceSpecificationProcessor(mm,this));
	processors.put("ExecutionOccurrenceSpecification", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExecutionOccurrenceSpecificationProcessor(mm,this));
	processors.put("ReceiveOperationEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReceiveOperationEventProcessor(mm,this));
	processors.put("ReceiveSignalEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReceiveSignalEventProcessor(mm,this));
	processors.put("Actor", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActorProcessor(mm,this));
	processors.put("CallEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CallEventProcessor(mm,this));
	processors.put("ChangeEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ChangeEventProcessor(mm,this));
	processors.put("SignalEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SignalEventProcessor(mm,this));
	processors.put("AnyReceiveEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AnyReceiveEventProcessor(mm,this));
	processors.put("ForkNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ForkNodeProcessor(mm,this));
	processors.put("FlowFinalNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.FlowFinalNodeProcessor(mm,this));
	processors.put("FinalNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.FinalNodeProcessor(mm,this));
	processors.put("CentralBufferNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CentralBufferNodeProcessor(mm,this));
	processors.put("MergeNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.MergeNodeProcessor(mm,this));
	processors.put("DecisionNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DecisionNodeProcessor(mm,this));
	processors.put("ObjectFlow", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ObjectFlowProcessor(mm,this));
	processors.put("ActivityFinalNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActivityFinalNodeProcessor(mm,this));
	processors.put("ComponentRealization", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ComponentRealizationProcessor(mm,this));
	processors.put("Component", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ComponentProcessor(mm,this));
	processors.put("Node", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.NodeProcessor(mm,this));
	processors.put("CommunicationPath", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CommunicationPathProcessor(mm,this));
	processors.put("Device", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DeviceProcessor(mm,this));
	processors.put("ExecutionEnvironment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExecutionEnvironmentProcessor(mm,this));
	processors.put("CombinedFragment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CombinedFragmentProcessor(mm,this));
	processors.put("Continuation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ContinuationProcessor(mm,this));
	processors.put("ConsiderIgnoreFragment", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConsiderIgnoreFragmentProcessor(mm,this));
	processors.put("CreateObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CreateObjectActionProcessor(mm,this));
	processors.put("DestroyObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DestroyObjectActionProcessor(mm,this));
	processors.put("TestIdentityAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TestIdentityActionProcessor(mm,this));
	processors.put("ReadSelfAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadSelfActionProcessor(mm,this));
	processors.put("StructuralFeatureAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StructuralFeatureActionProcessor(mm,this));
	processors.put("ReadStructuralFeatureAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadStructuralFeatureActionProcessor(mm,this));
	processors.put("WriteStructuralFeatureAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.WriteStructuralFeatureActionProcessor(mm,this));
	processors.put("ClearStructuralFeatureAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClearStructuralFeatureActionProcessor(mm,this));
	processors.put("RemoveStructuralFeatureValueAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RemoveStructuralFeatureValueActionProcessor(mm,this));
	processors.put("AddStructuralFeatureValueAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AddStructuralFeatureValueActionProcessor(mm,this));
	processors.put("LinkAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LinkActionProcessor(mm,this));
	processors.put("LinkEndData", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LinkEndDataProcessor(mm,this));
	processors.put("QualifierValue", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.QualifierValueProcessor(mm,this));
	processors.put("ReadLinkAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadLinkActionProcessor(mm,this));
	processors.put("LinkEndCreationData", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LinkEndCreationDataProcessor(mm,this));
	processors.put("CreateLinkAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CreateLinkActionProcessor(mm,this));
	processors.put("WriteLinkAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.WriteLinkActionProcessor(mm,this));
	processors.put("DestroyLinkAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DestroyLinkActionProcessor(mm,this));
	processors.put("LinkEndDestructionData", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LinkEndDestructionDataProcessor(mm,this));
	processors.put("ClearAssociationAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClearAssociationActionProcessor(mm,this));
	processors.put("BroadcastSignalAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.BroadcastSignalActionProcessor(mm,this));
	processors.put("SendObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.SendObjectActionProcessor(mm,this));
	processors.put("ValueSpecificationAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ValueSpecificationActionProcessor(mm,this));
	processors.put("TimeExpression", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TimeExpressionProcessor(mm,this));
	processors.put("Observation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ObservationProcessor(mm,this));
	processors.put("Duration", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DurationProcessor(mm,this));
	processors.put("DurationInterval", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DurationIntervalProcessor(mm,this));
	processors.put("Interval", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.IntervalProcessor(mm,this));
	processors.put("TimeConstraint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TimeConstraintProcessor(mm,this));
	processors.put("IntervalConstraint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.IntervalConstraintProcessor(mm,this));
	processors.put("TimeInterval", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TimeIntervalProcessor(mm,this));
	processors.put("DurationConstraint", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DurationConstraintProcessor(mm,this));
	processors.put("TimeObservation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TimeObservationProcessor(mm,this));
	processors.put("DurationObservation", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DurationObservationProcessor(mm,this));
	processors.put("FinalState", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.FinalStateProcessor(mm,this));
	processors.put("TimeEvent", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.TimeEventProcessor(mm,this));
	processors.put("VariableAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.VariableActionProcessor(mm,this));
	processors.put("ReadVariableAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadVariableActionProcessor(mm,this));
	processors.put("WriteVariableAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.WriteVariableActionProcessor(mm,this));
	processors.put("ClearVariableAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClearVariableActionProcessor(mm,this));
	processors.put("AddVariableValueAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AddVariableValueActionProcessor(mm,this));
	processors.put("RemoveVariableValueAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RemoveVariableValueActionProcessor(mm,this));
	processors.put("RaiseExceptionAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.RaiseExceptionActionProcessor(mm,this));
	processors.put("ActionInputPin", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ActionInputPinProcessor(mm,this));
	processors.put("InformationItem", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InformationItemProcessor(mm,this));
	processors.put("InformationFlow", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.InformationFlowProcessor(mm,this));
	processors.put("ReadExtentAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadExtentActionProcessor(mm,this));
	processors.put("ReclassifyObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReclassifyObjectActionProcessor(mm,this));
	processors.put("ReadIsClassifiedObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadIsClassifiedObjectActionProcessor(mm,this));
	processors.put("StartClassifierBehaviorAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StartClassifierBehaviorActionProcessor(mm,this));
	processors.put("ReadLinkObjectEndAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadLinkObjectEndActionProcessor(mm,this));
	processors.put("ReadLinkObjectEndQualifierAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReadLinkObjectEndQualifierActionProcessor(mm,this));
	processors.put("CreateLinkObjectAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.CreateLinkObjectActionProcessor(mm,this));
	processors.put("AcceptEventAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AcceptEventActionProcessor(mm,this));
	processors.put("AcceptCallAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AcceptCallActionProcessor(mm,this));
	processors.put("ReplyAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReplyActionProcessor(mm,this));
	processors.put("UnmarshallAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.UnmarshallActionProcessor(mm,this));
	processors.put("ReduceAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ReduceActionProcessor(mm,this));
	processors.put("StartObjectBehaviorAction", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.StartObjectBehaviorActionProcessor(mm,this));
	processors.put("JoinNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.JoinNodeProcessor(mm,this));
	processors.put("DataStoreNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.DataStoreNodeProcessor(mm,this));
	processors.put("ConditionalNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ConditionalNodeProcessor(mm,this));
	processors.put("Clause", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ClauseProcessor(mm,this));
	processors.put("LoopNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.LoopNodeProcessor(mm,this));
	processors.put("ExpansionNode", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExpansionNodeProcessor(mm,this));
	processors.put("ExpansionRegion", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ExpansionRegionProcessor(mm,this));
	processors.put("ProtocolTransition", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.ProtocolTransitionProcessor(mm,this));
	processors.put("AssociationClass", new org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml.AssociationClassProcessor(mm,this));
}
		}
