/*******************************************************************************
 * Copyright (c) 2009-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;
import org.eclipse.viatra2.imports.uml2.galileo.modelimporter.*;
import org.eclipse.uml2.uml.*;
/* @generated */
public class ClassifierProcessor extends AbstractProcessor {
public ClassifierProcessor(IModelManager mm, UmlModelImporter im) {
	super(mm,im); }

public IEntity process(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof Classifier)) return null;
	Classifier c = (Classifier)oc;
	if (e==null)
	{
		e = mm.newEntity(container);
		mm.newInstanceOf(imp.ref.ENT_UML_CLASSIFIER,e);
		imp.elemref.put(c,e);
		if (c.getName()!=null) mm.setValue(e,c.getName());
		if (c.getName()!=null) mm.setName(e,imp.convertName(c.getName()));
	}

	imp.routeProcessLocal("EObject",c,e,container);
	imp.routeProcessLocal("EModelElement",c,e,container);
	imp.routeProcessLocal("Element",c,e,container);
	imp.routeProcessLocal("NamedElement",c,e,container);
	imp.routeProcessLocal("Namespace",c,e,container);
	imp.routeProcessLocal("RedefinableElement",c,e,container);
	imp.routeProcessLocal("ParameterableElement",c,e,container);
	imp.routeProcessLocal("PackageableElement",c,e,container);
	imp.routeProcessLocal("Type",c,e,container);
	imp.routeProcessLocal("TemplateableElement",c,e,container);

	processLocal(c,e,container);

	return e;
}

public IEntity processLocal(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof Classifier)) return null;
	Classifier c = (Classifier)oc;

	IEntity attr;
	IRelation arel;
	Iterator it;


	if (c.isAbstract())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isAbstract()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_CLASSIFIER_ISABSTRACT,arel);}
	}


	if (c.getGeneralizations()!=null)
	{
		it = c.getGeneralizations().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_CLASSIFIER_GENERALIZATION,arel);}
		}
	}


	if (c.getPowertypeExtents()!=null)
	{
		it = c.getPowertypeExtents().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_POWERTYPEEXTENT,it.next()));
		}
	}


	if (c.getFeatures()!=null)
	{
		it = c.getFeatures().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_FEATURE,it.next()));
		}
	}


	if (c.getInheritedMembers()!=null)
	{
		it = c.getInheritedMembers().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_INHERITEDMEMBER,it.next()));
		}
	}


	if (c.getRedefinedClassifiers()!=null)
	{
		it = c.getRedefinedClassifiers().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_REDEFINEDCLASSIFIER,it.next()));
		}
	}


	if (c.getGenerals()!=null)
	{
		it = c.getGenerals().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_GENERAL,it.next()));
		}
	}


	if (c.getSubstitutions()!=null)
	{
		it = c.getSubstitutions().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_CLASSIFIER_SUBSTITUTION,arel);}
		}
	}


	if (c.getAttributes()!=null)
	{
		it = c.getAttributes().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_ATTRIBUTE,it.next()));
		}
	}


	if (c.getRepresentation()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_REPRESENTATION,c.getRepresentation()));
	}


	if (c.getCollaborationUses()!=null)
	{
		it = c.getCollaborationUses().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_CLASSIFIER_COLLABORATIONUSE,arel);}
		}
	}


	if (c.getOwnedUseCases()!=null)
	{
		it = c.getOwnedUseCases().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_CLASSIFIER_OWNEDUSECASE,arel);}
		}
	}


	if (c.getUseCases()!=null)
	{
		it = c.getUseCases().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_CLASSIFIER_USECASE,it.next()));
		}
	}
	return e;
}
}
