/*******************************************************************************
 * Copyright (c) 2009-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;
import org.eclipse.viatra2.imports.uml2.galileo.modelimporter.*;
import org.eclipse.uml2.uml.*;
/* @generated */
public class PropertyProcessor extends AbstractProcessor {
public PropertyProcessor(IModelManager mm, UmlModelImporter im) {
	super(mm,im); }

public IEntity process(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof Property)) return null;
	Property c = (Property)oc;
	if (e==null)
	{
		e = mm.newEntity(container);
		mm.newInstanceOf(imp.ref.ENT_UML_PROPERTY,e);
		imp.elemref.put(c,e);
		if (c.getName()!=null) mm.setValue(e,c.getName());
		if (c.getName()!=null) mm.setName(e,imp.convertName(c.getName()));
	}

	imp.routeProcessLocal("EObject",c,e,container);
	imp.routeProcessLocal("EModelElement",c,e,container);
	imp.routeProcessLocal("Element",c,e,container);
	imp.routeProcessLocal("NamedElement",c,e,container);
	imp.routeProcessLocal("RedefinableElement",c,e,container);
	imp.routeProcessLocal("Feature",c,e,container);
	imp.routeProcessLocal("TypedElement",c,e,container);
	imp.routeProcessLocal("MultiplicityElement",c,e,container);
	imp.routeProcessLocal("StructuralFeature",c,e,container);
	imp.routeProcessLocal("ParameterableElement",c,e,container);
	imp.routeProcessLocal("ConnectableElement",c,e,container);
	imp.routeProcessLocal("DeploymentTarget",c,e,container);

	processLocal(c,e,container);

	return e;
}

public IEntity processLocal(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof Property)) return null;
	Property c = (Property)oc;

	IEntity attr;
	IRelation arel;
	Iterator it;


	if (c.isDerived())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isDerived()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_ISDERIVED,arel);}
	}


	if (c.isDerivedUnion())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isDerivedUnion()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_ISDERIVEDUNION,arel);}
	}


	if (c.getDefault()!=null)
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.getDefault()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.String"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_DEFAULT,arel);}
	}


	if (c.getAggregation()!=null)
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.getAggregation()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.AggregationKind"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_AGGREGATION,arel);}
	}


	if (c.isComposite())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isComposite()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_ISCOMPOSITE,arel);}
	}


	if (c.getClass()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_CLASS,c.getClass()));
	}


	if (c.getDatatype()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_DATATYPE,c.getDatatype()));
	}


	if (c.getRedefinedProperties()!=null)
	{
		it = c.getRedefinedProperties().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_REDEFINEDPROPERTY,it.next()));
		}
	}


	if (c.getOwningAssociation()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_OWNINGASSOCIATION,c.getOwningAssociation()));
	}


	if (c.getDefaultValue()!=null)
	{
		attr = imp.routeProcessing(c.getDefaultValue(),e);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_DEFAULTVALUE,arel);}
	}


	if (c.getOpposite()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_OPPOSITE,c.getOpposite()));
	}


	if (c.getSubsettedProperties()!=null)
	{
		it = c.getSubsettedProperties().iterator();
		while(it.hasNext())
		{
			imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_SUBSETTEDPROPERTY,it.next()));
		}
	}


	if (c.getAssociation()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_ASSOCIATION,c.getAssociation()));
	}


	if (c.getQualifiers()!=null)
	{
		it = c.getQualifiers().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_PROPERTY_QUALIFIER,arel);}
		}
	}


	if (c.getAssociationEnd()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_PROPERTY_ASSOCIATIONEND,c.getAssociationEnd()));
	}
	return e;
}
}
