/*******************************************************************************
 * Copyright (c) 2009-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.imports.uml2.galileo.modelimporter.uml;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.errors.VPMCoreException;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.imports.NativeImporter;
import org.eclipse.viatra2.imports.uml2.galileo.modelimporter.*;
import org.eclipse.uml2.uml.*;
/* @generated */
public class StateProcessor extends AbstractProcessor {
public StateProcessor(IModelManager mm, UmlModelImporter im) {
	super(mm,im); }

public IEntity process(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof State)) return null;
	State c = (State)oc;
	if (e==null)
	{
		e = mm.newEntity(container);
		mm.newInstanceOf(imp.ref.ENT_UML_STATE,e);
		imp.elemref.put(c,e);
		if (c.getName()!=null) mm.setValue(e,c.getName());
		if (c.getName()!=null) mm.setName(e,imp.convertName(c.getName()));
	}

	imp.routeProcessLocal("EObject",c,e,container);
	imp.routeProcessLocal("EModelElement",c,e,container);
	imp.routeProcessLocal("Element",c,e,container);
	imp.routeProcessLocal("NamedElement",c,e,container);
	imp.routeProcessLocal("Namespace",c,e,container);
	imp.routeProcessLocal("RedefinableElement",c,e,container);
	imp.routeProcessLocal("Vertex",c,e,container);

	processLocal(c,e,container);

	return e;
}

public IEntity processLocal(java.lang.Object oc,IEntity e,IEntity container) throws VPMRuntimeException,VPMCoreException
{
	if (!(oc instanceof State)) return null;
	State c = (State)oc;

	IEntity attr;
	IRelation arel;
	Iterator it;


	if (c.isComposite())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isComposite()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_ISCOMPOSITE,arel);}
	}


	if (c.isOrthogonal())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isOrthogonal()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_ISORTHOGONAL,arel);}
	}


	if (c.isSimple())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isSimple()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_ISSIMPLE,arel);}
	}


	if (c.isSubmachineState())
	{
		attr = mm.newEntity(e);
		mm.setValue(attr,((java.lang.Object)c.isSubmachineState()).toString());
		mm.newInstanceOf(mm.getEntityByName("uml.metamodel.uml.Boolean"),attr);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_ISSUBMACHINESTATE,arel);}
	}


	if (c.getSubmachine()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_STATE_SUBMACHINE,c.getSubmachine()));
	}


	if (c.getConnections()!=null)
	{
		it = c.getConnections().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_STATE_CONNECTION,arel);}
		}
	}


	if (c.getConnectionPoints()!=null)
	{
		it = c.getConnectionPoints().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_STATE_CONNECTIONPOINT,arel);}
		}
	}


	if (c.getRedefinedState()!=null)
	{
		imp.references.add(new RelationReference(e,imp.ref.REL_UML_STATE_REDEFINEDSTATE,c.getRedefinedState()));
	}


	if (c.getStateInvariant()!=null)
	{
		attr = imp.routeProcessing(c.getStateInvariant(),e);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_STATEINVARIANT,arel);}
	}


	if (c.getEntry()!=null)
	{
		attr = imp.routeProcessing(c.getEntry(),e);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_ENTRY,arel);}
	}


	if (c.getExit()!=null)
	{
		attr = imp.routeProcessing(c.getExit(),e);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_EXIT,arel);}
	}


	if (c.getDoActivity()!=null)
	{
		attr = imp.routeProcessing(c.getDoActivity(),e);
		if (attr!=null) {arel = mm.newRelation(e,attr);
		mm.newInstanceOf(imp.ref.REL_UML_STATE_DOACTIVITY,arel);}
	}


	if (c.getDeferrableTriggers()!=null)
	{
		it = c.getDeferrableTriggers().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_STATE_DEFERRABLETRIGGER,arel);}
		}
	}


	if (c.getRegions()!=null)
	{
		it = c.getRegions().iterator();
		while(it.hasNext())
		{
			attr=imp.routeProcessing(it.next(),e);
			if (attr!=null) {arel = mm.newRelation(e,attr);
			mm.newInstanceOf(imp.ref.REL_UML_STATE_REGION,arel);}
		}
	}
	return e;
}
}
