/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.interpreters;

/**
 * Describes a model interpreter factory, a stub capable of instantiating a
 * given type of interpreter for the Viatra2 framework.
 * 
 * @author Istvan Rath
 * 
 */
public interface ModelInterpreterFactory {

	/**
	 * The unique ID of the interpreter.
	 */
	public String getId();

	/**
	 * The name of the interpreter.
	 */
	public String getName();

	/**
	 * The textual description of the interpreter.
	 */
	public String getDescription();

	/**
	 * Instantiates the interpreter type associated to this factory.
	 * 
	 */
	public ModelInterpreter getInterpreter();

	/**
	 * The fully qualified Java class names of objects which this interpreter
	 * can handle.
	 */
	public String[] getInterpretedClasses();
}
