/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.loaders;

import java.io.InputStream;

import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.IFramework;

/**
 * Interface to modules which load program models into a framework's machine
 * model storage
 * 
 * @author Istvan Rath
 */
public interface Loader {

	/**
	 * Processes the given file. It is the Loader's responsibility to add the
	 * resulting program model instance to the Framework's program model
	 * registry using the appropriate FQN.
	 * 
	 * @param f
	 *            File name
	 * @param m
	 *            The current modelspace
	 * @param l
	 *            THe system logger
	 * @throws VPMRuntimeException
	 */
	public void processFile(String fileName, IFramework fw)
			throws VPMRuntimeException;

	/**
	 * Processes the given stream It is the Loader's responsibility to add the
	 * resulting program model instance to the Framework's program model
	 * registry using the appropriate FQN.
	 * 
	 * @param f
	 *            stream to process
	 * @param m
	 *            The current modelspace
	 * @param l
	 *            THe system logger
	 * @throws VPMRuntimeException
	 */
	public void process(InputStream f, IFramework fw)
			throws VPMRuntimeException;

}
