/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Schmidt, Andras Balogh, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Schmidt, Andras Balogh, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.loaders;

import org.eclipse.viatra2.errors.VPMRuntimeException;

/**
 * Interface of loader creator classes.
 * 
 * @author Istvan Rath
 * 
 */
public interface LoaderFactory {

	/**
	 * Descriptive name of loader.
	 * 
	 * @return
	 */
	public String getLoaderName();

	/**
	 * Initialise the loader.
	 * 
	 * @param f
	 * @return
	 * @throws VPMRuntimeException
	 */
	public Loader getLoaderInstance() throws VPMRuntimeException;

	/**
	 * Returns a preferred id for the given loader.
	 * 
	 * @return
	 */
	public String getId();

	/**
	 * Get the set of file extensions that are handled by this loader.
	 * 
	 * @return the set of file extensions that are handled by this loader
	 */
	public String[] getFileExtensionList();

}
