/**
* Copyright (c) 2008 - 2010 OptXware Research and Development LLC.
* All rights reserved.   This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*   Daniel Varro - Initial API and implementation
*/
package org.eclipse.viatra2.lpgparser.loader;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.framework.IFrameworkService;

public class VTCLParserManager implements IFrameworkService {
	private Map<String,VTCLParseController> parse_controllers = new HashMap<String, VTCLParseController>();
	private IFramework framework;
	
	
	/**
	 * Creates a new {@link VTCLParseController} for a given file
	 * @param vtclFileName
	 * @return {@link VTCLParseController}
	 */
	public VTCLParseController createParseController(String vtclFileName) {
		VTCLParseController parseController = parse_controllers.get(vtclFileName);		
		if (parseController == null) {
			parseController = new VTCLParseController();
			parse_controllers.put(vtclFileName, parseController);			
			// Initialize parse controller
			parseController.initialize(vtclFileName, framework);
		} // do nothing if a parse controller is already registered for the file
		return parseController;
	}

	/**
	 * Deletes a new {@link VTCLParseController} for a given file
	 * @param vtclFileName
	 */
	public void deleteParseController(String vtclFileName) {
		VTCLParseController parseController = parse_controllers.get(vtclFileName); 
		if (parseController != null) {
			// TODO: Check if a terminate() method is needed for parseController
			parse_controllers.remove(vtclFileName);			
		}
	}

	/**
	 * Looks up the {@link VTCLParseController} associated to a given file name
	 */
	public VTCLParseController lookupParseController(String vtclFileName) {
		VTCLParseController parseController = parse_controllers.get(vtclFileName);
		if (parseController != null) {
			return parseController;
		}
		else {
			return null;
		}
		
	}

	/**
	 * Looks up the {@link VTCLParseController} associated to a given file, and
	 * creates a new one if none exists.
	 * @param vtclFileName
	 * @return {@link VTCLParseController}
	 */
	public VTCLParseController lookupAndCreateParseController(String vtclFileName) {
		VTCLParseController parseController = lookupParseController(vtclFileName); 
		if (parseController != null) {
			parseController.initialize(vtclFileName, framework);
			return parseController;
		}
		else {
			return createParseController(vtclFileName);
		}
	}
	
	public void init(IFramework fw) {
		this.framework = fw;
		
	}


}
