/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.properties.util;

import org.eclipse.viatra2.treeeditor.properties.VPMProperties;
import org.eclipse.viatra2.treeeditor.properties.VPMPropertyCategory;

public class LabelTranslator 
{

	/* Categories */
	
	
	static final String PROP_CAT_NAMING = "Naming & values";
	static final String PROP_CAT_TYPES = "Types & supertypes";
	static final String PROP_CAT_RELATIONS = "Relations";
	static final String PROP_CAT_TAGS = "Tags";
		
	
	public static VPMPropertyCategory getPropCategory(String name)
	{
		if (PROP_CAT_NAMING.equals(name))
			return VPMPropertyCategory.NAMING_VALUES;
		else if (PROP_CAT_RELATIONS.equals(name))
			return VPMPropertyCategory.RELATIONS;
		else if (PROP_CAT_TAGS.equals(name))
			return VPMPropertyCategory.TAGS;
		else //if (PROP_CAT_TYPES.equals(name))
			return VPMPropertyCategory.TYPES_SUPERTYPES;
	}
	
	public static String getCategoryName(VPMPropertyCategory p)
	{
		if (p.equals(VPMPropertyCategory.NAMING_VALUES))
			return PROP_CAT_NAMING;
		else if (p.equals(VPMPropertyCategory.RELATIONS))
			return PROP_CAT_RELATIONS;
		else if (p.equals(VPMPropertyCategory.TAGS))
			return PROP_CAT_TAGS;
		else //if (p.equals(VPMPropertyCategory.TYPES_SUPERTYPES))
			return PROP_CAT_TYPES;
	}
	
	
	
	/* Properties */
	
	public static final String PROP_P_NAME = "Name";
	public static final String PROP_P_FQN = "Fully qualified name";
	public static final String PROP_P_VALUE = "Value";
	public static final String PROP_P_VIEWINFO = "Comments";
	public static final String PROP_P_TYPE = "Type(s)";
	public static final String PROP_P_SUPERTYPE = "Supertype(s)";
	public static final String PROP_P_INSTANCE = "Instance(s)";
	public static final String PROP_P_SUBTYPE = "Subtype(s)";
	public static final String PROP_P_ISFINALTYPE = "Final";
	public static final String PROP_P_SOURCE = "Source";
	public static final String PROP_P_TARGET = "Target";
	public static final String PROP_P_ISAGGREGATION = "Aggregation";
	public static final String PROP_P_MULTIPLICITY = "Multiplicity";
	public static final String PROP_P_WARNINGS = "Problem tags";
	public static final String PROP_P_INFOTAGS = "Information tags";
	
	
	
	public static VPMProperties getVPMProperty(String name)
	{
		if (PROP_P_NAME.equals(name))
			return VPMProperties.NAME;
		else if (PROP_P_FQN.equals(name))
			return VPMProperties.FQN;
		else if (PROP_P_VALUE.equals(name))
			return VPMProperties.VALUE;
		else if (PROP_P_VIEWINFO.equals(name))
			return VPMProperties.VIEWINFO;
		else if (PROP_P_TYPE.equals(name))
			return VPMProperties.TYPE;
		else if (PROP_P_SUPERTYPE.equals(name))
			return VPMProperties.SUPERTYPE;
		else if (PROP_P_INSTANCE.equals(name))
			return VPMProperties.INSTANCE;
		else if (PROP_P_SUBTYPE.equals(name))
			return VPMProperties.SUBTYPE;
		else if (PROP_P_ISFINALTYPE.equals(name))
			return VPMProperties.ISFINALTYPE;
		else if (PROP_P_SOURCE.equals(name))
			return VPMProperties.SOURCE;
		else if (PROP_P_TARGET.equals(name))
			return VPMProperties.TARGET;
		else if (PROP_P_ISAGGREGATION.equals(name))
			return VPMProperties.ISAGGREGATION;
		else if (PROP_P_WARNINGS.equals(name))
			return VPMProperties.WARNINGS;
		else if (PROP_P_INFOTAGS.equals(name))
			return VPMProperties.INFOTAGS;
		else //if (PROP_P_MULTIPLICITY.equals(name))
			return VPMProperties.MULTIPLICITY;
	}
	
	public static String getPropertyName(VPMProperties p)
	{
		if (VPMProperties.NAME.equals(p))
			return PROP_P_NAME;
		else if (VPMProperties.FQN.equals(p))
			return PROP_P_FQN;
		else if (VPMProperties.VALUE.equals(p))
			return PROP_P_VALUE;
		else if (VPMProperties.VIEWINFO.equals(p))
			return PROP_P_VIEWINFO;
		else if (VPMProperties.TYPE.equals(p))
			return PROP_P_TYPE;
		else if (VPMProperties.SUPERTYPE.equals(p))
			return PROP_P_SUPERTYPE;
		else if (VPMProperties.INSTANCE.equals(p))
			return PROP_P_INSTANCE;
		else if (VPMProperties.SUBTYPE.equals(p))
			return PROP_P_SUBTYPE;
		else if (VPMProperties.ISFINALTYPE.equals(p))
			return PROP_P_ISFINALTYPE;
		else if (VPMProperties.SOURCE.equals(p))
			return PROP_P_SOURCE;
		else if (VPMProperties.TARGET.equals(p))
			return PROP_P_TARGET;
		else if (VPMProperties.ISAGGREGATION.equals(p))
			return PROP_P_ISAGGREGATION;
		else if (VPMProperties.WARNINGS.equals(p))
			return PROP_P_WARNINGS;
		else if (VPMProperties.INFOTAGS.equals(p))
			return PROP_P_INFOTAGS;
		else //if (VPMProperties.MULTIPLICITY.equals([p))
			return PROP_P_MULTIPLICITY;
		
	}
}
