/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.transfer;



import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.treeeditor.ViatraTreeEditor;
import org.eclipse.viatra2.treeeditor.commands.AddSupertypeCommand;
import org.eclipse.viatra2.treeeditor.commands.AddTypeCommand;
import org.eclipse.viatra2.treeeditor.commands.MoveModelElementCommand;
import org.eclipse.viatra2.treeeditor.commands.RetargetRelationCommand;
import org.eclipse.viatra2.treeeditor.commands.ViatraCompoundCommand;

public class ViatraEditorDropTargetAdapter extends DropTargetAdapter {

	
	protected ViatraTreeEditor iVTE;
	
	public ViatraEditorDropTargetAdapter(ViatraTreeEditor vte) {
		super();
		iVTE = vte;
	}
	
	
	public static Transfer[] getTransfers() {
		Transfer[] r = new Transfer[1];
		r[0] = ViatraTransfer.getInstance();
		return r;
	}
	
	
	
	@Override
	public void drop(DropTargetEvent event) {
		//System.out.println("Dropped "+(event!=null&&event.data!=null?event.data.toString()+" onto "+event.getSource().toString():"something"));
		if (event!=null && event.data!=null && event.item!=null && event.data instanceof ViatraTransferData[]) {
			// System.out.println("Code: " + event.detail);
			ViatraTransferData[] data = (ViatraTransferData[])event.data;
			// added by XMI: 
			// precheck, must make sure the target is not one of the items.
			// this has disastrous consequences on the entire modelspace if it could sneak by
			for (int i=0; i<data.length; i++) {
				IModelElement target = (IModelElement)event.item.getData();
				String fqn = data[i].fullyQualifiedName;
				IModelElement item = target.getModelSpace().getModelManager().getElementByName(fqn);
				if (target.equals(item))
					return;
			}
			ViatraCompoundCommand ccmd = new ViatraCompoundCommand();
			for (int i=0; i<data.length; i++) {
				String fqn = data[i].fullyQualifiedName;
				IModelElement target = (IModelElement)event.item.getData();
				IModelElement item = target.getModelSpace().getModelManager().getElementByName(fqn);
				if (target.equals(item))
					return;
				// Added by dave for supertypeOf and instanceOf DND semantics
				if (event.detail == DND.DROP_COPY)
				{
					// Copy (CTRL) is new instanceOf relation (type is dragged onto instance)
					AddTypeCommand cmd = new AddTypeCommand();
					cmd.setTypeElem(item);
					cmd.setTargetElem(target);
					ccmd.add(cmd);
				}
				else if (event.detail == DND.DROP_LINK)
				{
					// Link (CTRL+SHIFT) is new supertypeOf relation (supertype is dragged onto subtype)
					AddSupertypeCommand cmd = new AddSupertypeCommand();
					cmd.setSupertypeElem(item);
					cmd.setTargetElem(target);
					ccmd.add(cmd);
				}
				else
				{
					// End of added code
					if (target instanceof IEntity) {
						// either reparent or resource
						MoveModelElementCommand cmd = new MoveModelElementCommand();
						if (item instanceof IEntity) {
							// reparent
							cmd.setEntity((IEntity)item,(IEntity)target);
						}
						else if (item instanceof IRelation) {
							cmd.setRelation((IRelation)item,target);
						}
						ccmd.add(cmd);
					}
					else if (target instanceof IRelation) {
						// retarget or retarget relation
						if (item instanceof IRelation) {
							// resource item
							MoveModelElementCommand cmd = new MoveModelElementCommand();
							cmd.setRelation((IRelation)item,target);
							ccmd.add(cmd);
						}
						else if (item instanceof IEntity) {
							// retarget target
							RetargetRelationCommand cmd = new RetargetRelationCommand();
							cmd.setRelation((IRelation)target,item);
							ccmd.add(cmd);
						}
					}
				}
			}
			if (!ccmd.isEmpty())
			{
				iVTE.getCommandStack().execute(ccmd);
				iVTE.updateActions();
			}
		}
	}
}
