/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.services.IDisposable;
import org.eclipse.viatra2.visualisation.layouts.IViatraLayoutAlgorithm;

/**
 * This interface describes a visualisation task for the view. It also details
 * menu and toolbar contributions.
 * @author Zoltan Ujhelyi
 */
public interface IVisualisationDescriptor extends IDisposable, IAdaptable {

	/**
	 * Adds the menu contributions related to the visualisation to a selected
	 * manager.
	 * @param manager
	 */
	public void addMenuContributions(IMenuManager manager);

	/**
	 * Adds the toolbar contributions related to the visualisation to a selected
	 * manager.
	 * @param manager
	 */
	public void addToolbarContributions(IToolBarManager manager);

	/**
	 * Creates the item to be displayed in the form. It also initializes the
	 * graph viewer.
	 * @param parent the parent element to create the components in.
	 */
	public void createFormContributions(Composite parent);

	/**
	 * Returns the viewer described by the descriptor.
	 * @return the graph viewer component
	 */
	public ViatraGraphViewer getViewer();

	/**
	 * Clears the described graph model
	 */
	void clearGraphModel();

	/**
	 * Removes the selected elements from the graph model
	 */
	void removeSelection();

	public Object getAdapter(Class adapter);
	
	/**
	 * Sets a layout algorithm for the visualisation.
	 * @param algorithm
	 */
	public void setLayoutAlgorithm(IViatraLayoutAlgorithm algorithm);
}
