/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.color;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemePreview;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.visualisation.modelspace.ModelSpaceGraphViewer;
import org.eclipse.viatra2.visualisation.modelspace.datasource.GraphArc;
import org.eclipse.zest.core.viewers.IGraphEntityRelationshipContentProvider;
import org.eclipse.zest.core.widgets.ZestStyles;
import org.eclipse.zest.layouts.LayoutStyles;
import org.eclipse.zest.layouts.algorithms.RadialLayoutAlgorithm;

/**
 * @author Zoltan Ujhelyi
 *
 */
public class ColorPreviewFrame implements IThemePreview {
	
	ModelSpaceGraphViewer gv;
	ITheme theme;
	IPropertyChangeListener listener = new IPropertyChangeListener() {
		
		public void propertyChange(PropertyChangeEvent event) {
			reloadTheme();
		}
	};

	/* (non-Javadoc)
	 * @see org.eclipse.ui.themes.IThemePreview#createControl(org.eclipse.swt.widgets.Composite, org.eclipse.ui.themes.ITheme)
	 */
	public void createControl(Composite parent, ITheme currentTheme) {
		this.theme = currentTheme;
		currentTheme.addPropertyChangeListener(listener);
		gv = new ModelSpaceGraphViewer(parent, SWT.NONE);
		gv.setLayoutAlgorithm(new RadialLayoutAlgorithm(LayoutStyles.NO_LAYOUT_NODE_RESIZING|ZestStyles.IGNORE_INVISIBLE_LAYOUT), true);
		gv.loadTheme(currentTheme);
		gv.setConnectionStyle(ZestStyles.CONNECTIONS_DIRECTED);
		gv.setContentProvider(new IGraphEntityRelationshipContentProvider() {
			PreviewEntity entity1 = new PreviewEntity("Entity1");
			PreviewEntity entity2 = new PreviewEntity("Entity2");
			PreviewRelation relation = new PreviewRelation("Relation");
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
		
			public void dispose() {}
		
			public Object[] getElements(Object inputElement) {
				return new Object[] {entity1, entity2, relation};
			}
		
			public Object[] getRelationships(Object source, Object dest) {
				if (!(source.equals(entity1) && dest.equals(relation)) 
						&& !(source.equals(relation) && dest.equals(entity2))) return null;
				GraphArc arc = new GraphArc((IModelElement)source,(IModelElement)dest, "relation");
				return new Object[] {arc};
			}
		});
		gv.setInput(null);
	}
	
	void reloadTheme() {
		gv.refreshTheme();
		gv.setInput(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.themes.IThemePreview#dispose()
	 */
	public void dispose() {
		theme.removePropertyChangeListener(listener);
	}

}
