/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.layouts.algorithms;

import org.eclipse.viatra2.visualisation.layouts.AbstractViatraLayoutAlgorithm;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.ICriteria;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.SimulatedCooling;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria.ArcAngleDistribution;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria.DistinctNodes;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria.EdgeIntersection;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria.EdgeLength;
import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria.InBounds;
import org.eclipse.zest.layouts.LayoutAlgorithm;

/**
 * An algorithm using a simulated cooling based optimalization of various properties of the layout.
 * @author Zoltan Ujhelyi
 *
 */
public class SimulatedCoolingLayout extends AbstractViatraLayoutAlgorithm {

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.visualisation.layouts.IViatraLayoutAlgorithm#getAlgorithm()
	 */
	public LayoutAlgorithm getAlgorithm() {
		SimulatedCooling coolingAlgorithm = new SimulatedCooling(
				layoutStyle,
				new ICriteria[] { new InBounds(), /*new PointDistribution(10),*/
						new ArcAngleDistribution(1000), new EdgeLength(0.0001),
						new EdgeIntersection(1000), new DistinctNodes(1000000) }, 15);
		return coolingAlgorithm;
	}

}
