/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.layouts.simulatedcooling;

import org.eclipse.zest.layouts.LayoutEntity;
import org.eclipse.zest.layouts.LayoutRelationship;

/**
 * A criteria generates a double value from a graph configuration. The lower the
 * value, the better the graph by this criteria.
 */
public interface ICriteria {

	/**
	 * Apply the criteria with the given graph configuration.
	 * 
	 * The graph bounds are given by four values: (x,y,w,h). This gives the
	 * position and size of the place inside where the graph elements are
	 * visible.
	 * 
	 * @param entities
	 *            nodes in the graph
	 * @param relationships
	 *            edges in the graph
	 * @param x
	 *            X coordinate of the upper-left corner
	 * @param y
	 *            Y coordinate of the upper-left corner
	 * @param w
	 *            Width
	 * @param h
	 *            Height
	 * @return punishment of the current graph layout by this criteria
	 */
	public double apply(LayoutEntity[] entities,
			LayoutRelationship[] relationships, double x, double y, double w,
			double h);
}
