/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria;

import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.ICriteria;
import org.eclipse.zest.layouts.LayoutEntity;
import org.eclipse.zest.layouts.LayoutRelationship;

/**
 * This criteria punishes long edges by multiplying edge length by the given
 * values. The value of punishment is linear to the edge length.
 */
public class EdgeLength implements ICriteria {

	private final double factor;

	/**
	 * Creates a criteria which punishes long edges
	 * 
	 * @param factor
	 *            a linear factor to determine punishment, which is
	 *            factor*length, where length is the length of the edge
	 */
	public EdgeLength(double factor) {
		this.factor = factor;
	}

	/**
	 * {@inheritDoc}
	 */
	public double apply(LayoutEntity[] entities,
			LayoutRelationship[] relationships, double x, double y, double w,
			double h) {

		double result = 0;

		for (LayoutRelationship r : relationships) {
			LayoutEntity i = r.getSourceInLayout();
			LayoutEntity j = r.getDestinationInLayout();

			double d = Math.pow(i.getXInLayout() - j.getXInLayout(), 2)
					+ Math.pow(i.getYInLayout() - j.getYInLayout(), 2);

			result += Math.sqrt(d) * factor;
		}

		return result;
	}

}
