/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria;

import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.ICriteria;
import org.eclipse.zest.layouts.LayoutEntity;
import org.eclipse.zest.layouts.LayoutRelationship;

/**
 * This criteria punishes out-bounded nodes with a large value. A node is
 * out-bound if its position is not in the bounds of the graph.
 * 
 */
public class InBounds implements ICriteria {

	/**
	 * {@inheritDoc}
	 */
	public double apply(LayoutEntity[] entities,
			LayoutRelationship[] relationships, double x, double y, double w,
			double h) {

		double result = 0;

		for (LayoutEntity e : entities) {
			double xe = e.getXInLayout();
			double ye = e.getYInLayout();
			double we = e.getWidthInLayout();
			double he = e.getHeightInLayout();
			if (xe < x)
				result += 1000000 + 100 * (x - xe);
			if (xe + we > w)
				result += 1000000 + 100 * (xe + we - w);
			if (ye < y)
				result += 1000000 + 100 * (y - ye);
			if (ye + he > h)
				result += 1000000 + 100 * (ye + he - h);
		}

		return result;
	}

}

