/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.layouts.simulatedcooling.criteria;

import org.eclipse.viatra2.visualisation.layouts.simulatedcooling.ICriteria;
import org.eclipse.zest.layouts.LayoutEntity;
import org.eclipse.zest.layouts.LayoutRelationship;

/**
 * This criteria punishes a graph layout if the nodes are too close to each
 * other. Every node pair is punished by a value of factor/d, where d is the
 * distance between the nodes.
 * 
 */
public class PointDistribution implements ICriteria {

	private final double factor;

	/**
	 * Creates a criteria which punishes nodes which are too close to each
	 * other.
	 * 
	 * @param factor
	 *            a linear factor to determine punishment, which is factor/d,
	 *            where d is the distance between the nodes.
	 */
	public PointDistribution(double factor) {
		this.factor = factor;
	}

	/**
	 * {@inheritDoc}
	 */
	public double apply(LayoutEntity[] entities,
			LayoutRelationship[] relationships, double x, double y, double w,
			double h) {

		double result = 0;

		for (LayoutEntity i : entities)
			for (LayoutEntity j : entities)
				if (!i.equals(j)) {
					double d = Math.pow(i.getXInLayout() - j.getXInLayout(), 2)
							+ Math.pow(i.getYInLayout() - j.getYInLayout(), 2);

					if (d == 0)
						d = 0.00001;

					result += factor / d;
				}

		return result;
	}

}

