/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.viatra2.visualisation.ViatraGraphViewer;
import org.eclipse.viatra2.visualisation.modelspace.datasource.ModelSpaceLabelProvider;

/**
 * This GraphViewer component is the same as the component present in Zest
 * except of the selection handling.
 * @author Zoltan Ujhelyi
 */
public class ModelSpaceGraphViewer extends ViatraGraphViewer {

	private ModelSpaceLabelProvider labelProvider = new ModelSpaceLabelProvider();

	/**
	 * Initializes the graph viewer
	 * @param composite
	 * @param style
	 *            the style for the viewer and for the layout algorithm
	 * @see SWT#V_SCROLL
	 * @see SWT#H_SCROLL
	 */
	public ModelSpaceGraphViewer(Composite composite, int style) {
		super(composite, style);
		setLabelProvider(labelProvider);
	}

	/**
	 * Loads the color information from a theme
	 * @param theme
	 */
	public void loadTheme(ITheme theme) {
		labelProvider.loadTheme(theme);
	}

	/**
	 * Notifies the Label Provider to reload the colors according the settings
	 * XXX it should not be needed but without this call the color preview is
	 * not in sync with the settings
	 */
	public void refreshTheme() {
		labelProvider.reloadColors();
	}

}
