/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.visualisation.modelspace.datasource.filter.PreFilter;

public class ModelspaceTransactionChanges {

	/**
	 * This list stores the elements created during the last transaction for
	 * calculating whether a redraw is needed.
	 */
	private Set<IModelElement> newElements = new HashSet<IModelElement>();
	/**
	 * This list stores the elements deleted during the last transaction for
	 * calculating whether a redraw is needed.
	 */
	private Set<IModelElement> deletedElements = new HashSet<IModelElement>();
	
	private PreFilter filter;

	public ModelspaceTransactionChanges(PreFilter filter) {
		super();
		this.filter = filter;
	}

	/**
	 * @return the newElements
	 */
	public Collection<IModelElement> getNewElements() {
		return newElements;
	}

	public void addNewElement(IModelElement element) {
		newElements.add(element);
	}

	public void addNewElements(Collection<IModelElement> elements) {
		newElements.addAll(elements);
	}

	/**
	 * @return the deletedElements
	 */
	public Collection<IModelElement> getDeletedElements() {
		return deletedElements;
	}

	public void addRemovedElement(IModelElement element) {
		deletedElements.add(element);
		for (IRelation rel : element.getRelations())
			addRemovedElement(rel);
	}

	public void addRemovedElements(Collection<IModelElement> elements) {
		deletedElements.addAll(elements);
	}
	
	public void handleLooseEdges() {
		HashSet<IModelElement> deletes = new HashSet<IModelElement>();
		newElements.removeAll(deletedElements);
		for (IModelElement element : deletedElements) {
			for (IRelation rel : element.getRelations())
				deletes.add(rel);
		}
		addRemovedElements(deletes);
		HashSet<IModelElement> news = new HashSet<IModelElement>();
		for (IModelElement element : newElements) {
			if (element instanceof IRelation) {
				IRelation rel = (IRelation) element;
				news.add(rel.getFrom());
				filter.includeElement(rel.getFrom());
				news.add(rel.getTo());
				filter.includeElement(rel.getTo());
			}
		}
		addNewElements(news);
	}

}
