/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.PlatformUI;
import org.eclipse.viatra2.visualisation.modelspace.ModelSpaceVisualisationDescriptor;
import org.eclipse.viatra2.visualisation.view.ViatraVisualisationView;

/**
 * This Action allows the selection of a filter
 * @author Zoltan Ujhelyi
 *
 */
public class ToggleFilterAction extends Action {
	
	private IFilterManipulator manipulator;

	/**
	 * Initializes a Toggle Filter Action
	 * @param name the name of the action in the user interface
	 */
	public ToggleFilterAction(String name) {
		super(name,AS_CHECK_BOX);
		setToolTipText("Toggles the " + name + " filter");
		addPropertyChangeListener(new IPropertyChangeListener() {
			
			public void propertyChange(PropertyChangeEvent event) {
				// listening for changes in the property named "checked"
				if (event.getProperty().equals("checked") && manipulator != null) {
					manipulator.setFilterProperty((Boolean)event.getNewValue());
				}
					
			}
		});
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.Action#run()
	 */
	@Override
	public void run() {
		String viewId = "org.eclipse.viatra2.visualisation.view";
		ViatraVisualisationView view = (ViatraVisualisationView) PlatformUI
		.getWorkbench().getActiveWorkbenchWindow().getActivePage()
			.findViewReference(viewId).getView(false);
		((ModelSpaceVisualisationDescriptor)view.getDescriptor()).redrawGraph();
	}

	/**
	 * @param manipulator the manipulator to set
	 */
	public void setManipulator(IFilterManipulator manipulator) {
		this.manipulator = manipulator;
	}
}
