/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.actions.groups;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.actions.ActionGroup;
import org.eclipse.viatra2.visualisation.modelspace.actions.IFilterManipulator;
import org.eclipse.viatra2.visualisation.modelspace.actions.ToggleFilterAction;
import org.eclipse.viatra2.visualisation.modelspace.datasource.filter.PostFilter;

/**
 * @author Zoltan Ujhelyi
 *
 */
public class FilterGroup extends ActionGroup {

	private PostFilter filter;

	/**
	 * @return the filter
	 */
	public PostFilter getFilter() {
		return filter;
	}

	/**
	 * @param filter the filter to set
	 */
	public void setFilter(PostFilter filter) {
		this.filter = filter;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.ActionGroup#fillContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void fillContextMenu(IMenuManager manager) {
		//Creating items
		IMenuManager menu = new MenuManager("Filters");
		manager.add(menu);
		ToggleFilterAction containmentFilter = new ToggleFilterAction("Hide Containment Relations");
		containmentFilter.setManipulator(new IFilterManipulator() {
		
			public void setFilterProperty(boolean newValue) {
				filter.setHideContainment(newValue);
		
			}
		});
		containmentFilter.setChecked(true);
		menu.add(containmentFilter);
		ToggleFilterAction inheritanceFilter = new ToggleFilterAction("Hide Inheritance Relations");
		inheritanceFilter.setManipulator(new IFilterManipulator() {
			
			public void setFilterProperty(boolean newValue) {
				filter.setHideInheritance(newValue);
		
			}
		});
		menu.add(inheritanceFilter);
		inheritanceFilter.setChecked(true);
		ToggleFilterAction instanceFilter = new ToggleFilterAction("Hide Instance Relations");
		instanceFilter.setManipulator(new IFilterManipulator() {
			
			public void setFilterProperty(boolean newValue) {
				filter.setHideInstantiation(newValue);
				
			}
		});
		menu.add(instanceFilter);
		instanceFilter.setChecked(true);
	}
	
	
}
