/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.datasource;

import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.visualisation.modelspace.datasource.filter.PreFilter;

/**
 * This class represents an abstract content provider for the graph viewer.
 * It is possible to use any of the possible Graph Content Providers from Zest,
 * but all the descendants should provide the same IContentProvider descendant interface
 * (otherwise a Zest runtime exception is thrown).
 * @author Zoltan Ujhelyi
 *
 */
public abstract class AbstractModelSpaceProvider implements IContentProvider{

	protected IModelManager modelManager;
	protected PreFilter filter; 

	/**
	 * Sets the viatra model manager from which the content provider extracts the graph data
	 * @param modelManager the modelManager to set
	 */
	public void setModelManager(IModelManager modelManager) {
		this.modelManager = modelManager;
	}

	/**
	 * Sets a filter to reduce the number of constraints to generate
	 * @param filter the filter to set
	 */
	public void setFilter(PreFilter filter) {
		this.filter = filter;
	}

}