/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.datasource;

import java.util.ArrayList;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.zest.core.viewers.IGraphEntityRelationshipContentProvider;

/**
 * Acts as a Model Space Provider for the Graph Viewer,
 * represents the Relations as Nodes.
 * 
 * @author Zoltan Ujhelyi
 *
 */
public class FullModelSpaceProvider extends AbstractModelSpaceProvider implements IGraphEntityRelationshipContentProvider{
	
	/* (non-Javadoc)
	 * @see org.eclipse.zest.core.viewers.IGraphEntityRelationshipContentProvider#getRelationships(java.lang.Object, java.lang.Object)
	 */
	public Object[] getRelationships(Object source, Object dest) {
		if (modelManager==null) return new Object[0];
		Assert.isTrue(source instanceof IModelElement && dest instanceof IModelElement);
		ArrayList<GraphArc> relations = new ArrayList<GraphArc>();
		if (source instanceof IRelation) {
			IRelation src = (IRelation) source;
			IModelElement dst = (IModelElement)dest;
			if (src.getTo().compareTo(dst)==0) {
				GraphArc relation = new GraphArc();
				relation.setEntity1(src);
				relation.setEntity2(dst);
				relation.setMultiplicity(1);
				relations.add(relation);
			}
		}
		if (dest instanceof IRelation) {
			IRelation dst = (IRelation) dest;
			IModelElement src = (IModelElement)source;
			if (dst.getFrom().compareTo(src)==0) {
				GraphArc relation = new GraphArc();
				relation.setEntity1(src);
				relation.setEntity2(dst);
				relation.setMultiplicity(1);
				relations.add(relation);
			}
		}
		IModelElement src = (IModelElement) source;
		IModelElement dst = (IModelElement) dest;
		if (src.isDirectSupertypeOf(dst) && src.compareTo(dst) != 0) {
			GraphArc relation = new GraphArc();
			relation.setEntity1(src);
			relation.setEntity2(dst);
			relation.setMultiplicity(1);
			relation.setName("supertypeOf");
			relations.add(relation);
		}
		if (dst.isDirectTypeOf(src) && src.compareTo(dst) != 0) {
			GraphArc relation = new GraphArc();
			relation.setEntity1(src);
			relation.setEntity2(dst);
			relation.setMultiplicity(1);
			relation.setName("instanceOf");
			relations.add(relation);
		}
		if (src instanceof IEntity && ((IEntity)src).getContents().contains(dst) && src.compareTo(dst) != 0) {
			GraphArc relation = new GraphArc();
			relation.setEntity1(src);
			relation.setEntity2(dst);
			relation.setMultiplicity(1);
			relation.setName("contains");
			relations.add(relation);
		}
		return relations.toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		if (modelManager != null) {
			ArrayList<IModelElement> elements = new ArrayList<IModelElement>();
			for (IModelElement element : modelManager.getElements()) {
				if (!filter.isFiltered(element))
					elements.add(element);
			}
			return elements.toArray();
		} else return new ArrayList<Object>().toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {

	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}

	/**
	 * Used to return the child elements - required for nested elements using INestedContentProvider (Zest Galileo)
	 * @param element the object to check
	 * @return the array of children
	 */
	public Object[] getChildren(Object element) {
		if (element instanceof IEntity) {
			return ((IEntity) element).getContents().toArray();
		}
		return null;
	}

	/**
	 * Used to check whether a node has child nodes - required for nested elements using INestedContentProvider (Zest Galileo) 
	 * @param element the object to check
	 * @return true if there are subnodes
	 */
	public boolean hasChildren(Object element) {
		if (element instanceof IEntity) {
			return !((IEntity) element).getContents().isEmpty();
			//.getAllElementsInNamespace().isEmpty();
		}
		return false;
	}

/*
 * Used in IGraphEntityContentProvider - not a full provider!
 * 	public Object[] getConnectedTo(Object entity) {
		if (entity instanceof IEntity) {
			return ((IEntity) entity).getRelationsFrom().toArray();
		}
		if (entity instanceof IRelation) {
			return new Object[] {((IRelation) entity).getTo()};
		}
		return null;
	}*/

}
