/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.datasource;

import org.eclipse.viatra2.core.IModelElement;

/**
 * A Graph Arc object represents a graph arc. It is used in order to assign arcs
 * to such components that are not directly present in the Viatra model space.
 * @author Zoltan Ujhelyi
 */
public class GraphArc {

	private IModelElement entity1, entity2;
	private Integer multiplicity;
	private String name;

	/**
	 * Initializes a Graph Arc
	 */
	public GraphArc() {

	}

	/**
	 * Initializes a directed Graph Arc between the two elements, and assigns
	 * the name to it
	 * @param source
	 *            the source model element
	 * @param destination
	 *            the destination model element
	 * @param name
	 *            the label of the arc
	 */
	public GraphArc(IModelElement source, IModelElement destination, String name) {
		this.entity1 = source;
		this.entity2 = destination;
		multiplicity = 1;
		this.name = name;
	}

	/**
	 * @return the entity1
	 */
	public IModelElement getEntity1() {
		return entity1;
	}

	/**
	 * @param entity1
	 *            the entity1 to set
	 */
	public void setEntity1(IModelElement entity1) {
		this.entity1 = entity1;
	}

	/**
	 * @return the entity2
	 */
	public IModelElement getEntity2() {
		return entity2;
	}

	/**
	 * @param entity2
	 *            the entity2 to set
	 */
	public void setEntity2(IModelElement entity2) {
		this.entity2 = entity2;
	}

	/**
	 * @return the multiplicity
	 */
	public Integer getMultiplicity() {
		return multiplicity;
	}

	/**
	 * @param multiplicity
	 *            the multiplicity to set
	 */
	public void setMultiplicity(Integer multiplicity) {
		this.multiplicity = multiplicity;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

}
