/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.datasource;

import java.util.Set;

import org.eclipse.draw2d.IFigure;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IModelSpace;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.tags.ITag;
import org.eclipse.viatra2.visualisation.ViatraColoredLabelProvider;
import org.eclipse.zest.core.viewers.IConnectionStyleProvider;
import org.eclipse.zest.core.viewers.IEntityStyleProvider;
import org.eclipse.zest.core.widgets.ZestStyles;

/**
 * This class is used as a label provider for the graph viewer. It defines node
 * and arc labels and style.
 * @author Zoltan Ujhelyi
 */
public class ModelSpaceLabelProvider extends ViatraColoredLabelProvider
		implements IEntityStyleProvider, IConnectionStyleProvider {

	@Override
	public String getText(Object element) {
		String name;
		if (element instanceof IModelElement) {
			IModelElement modelElement = (IModelElement) element;
			name = modelElement.getName();
			if (!modelElement.getTypes().isEmpty()) {
				name = name.concat(" : ");
				name = name.concat(modelElement.getTypes().iterator().next()
						.getName());
			}
		} else if (element instanceof GraphArc) {
			GraphArc arc = (GraphArc) element;
			name = arc.getName();
		} else
			return null;
		return name;
	}

	public boolean fisheyeNode(Object entity) {
		return true;// entity instanceof IRelation;
	}

	public Color getBackgroundColour(Object entity) {
		// Istvan: add support for recognizing tag color information
		IModelSpace modelSpace = ((IModelElement) entity).getModelSpace();
		// The model space might be null, because the Property page uses a dummy entity implementation
		if (modelSpace != null) {
			IFramework fw = modelSpace.getFramework();
			if (fw != null) {
				Set<ITag> tags = fw.getTagManager().getTagsForModelElement(
						(IModelElement) entity);
				if (tags != null && !tags.isEmpty()) {
					return Display.getDefault().getSystemColor(
							tags.iterator().next().getBackgroundColor());
				}
			}
		}
		if (entity instanceof IRelation) {
			return RELATION_BG;
		}
		return ENTITY_BG;
	}

	public Color getBorderColor(Object entity) {
		if (entity instanceof IRelation) {
			return RELATION_BO;
		}
		return ENTITY_BO;
	}

	public Color getBorderHighlightColor(Object entity) {
		if (entity instanceof IRelation) {
			return RELATION_BOHL;
		}
		return ENTITY_BOHL;
	}

	public int getBorderWidth(Object entity) {
		return 1;
	}

	public Color getForegroundColour(Object entity) {
		if (entity instanceof IRelation) {
			return RELATION_FG;
		}
		return ENTITY_FG;
	}

	public Color getNodeHighlightColor(Object entity) {
		if (entity instanceof IRelation) {
			return RELATION_HL;
		}
		return ENTITY_HL;
	}

	public IFigure getTooltip(Object entity) {
		// return new TextFlow("Hello");
		return null;
	}

	public Color getColor(Object rel) {
		return ARC;
	}

	public int getConnectionStyle(Object rel) {
		if (rel instanceof GraphArc && !(rel instanceof GraphRelationArc)) {
			return ZestStyles.CONNECTIONS_DOT;
		}
		return 0;
	}

	public Color getHighlightColor(Object rel) {
		return ARC_HL;
	}

	public int getLineWidth(Object rel) {
		return 1;
	}

}