/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.modelspace.datasource.filter;

import java.util.Iterator;
import java.util.TreeSet;

/**
 * Represents a filter condition
 * @author Zoltan Ujhelyi
 *
 */
class FilterCondition {
	
	protected TreeSet<String> subtreeFilters = new TreeSet<String>();
	protected TreeSet<String> elementFilters = new TreeSet<String>();

	/**
	 * Add an element to the filter
	 * @param id the id string of the selected item
	 */
	void addSubtreeFilterCondition(String id) {
		if (!subtreeFilters.contains(id)) subtreeFilters.add(id);
		
	}
	
	void addElementFilterCondition(String id) {
		if (!elementFilters.contains(id)) elementFilters.add(id);
	}
	
	boolean containsID(String id) {
		return elementFilters.contains(id) || subtreeFilters.contains(id);
	}
	
	boolean elementContainsID(String id) {
		return elementFilters.contains(id);
	}
	void removeFilter(String id, boolean isSubTree) {
		if (id.contentEquals("")) {
			subtreeFilters.clear();
			return;
		}
		Iterator<String> it;
		if (isSubTree) {
			it = subtreeFilters.iterator();
			while (it.hasNext()) {
				if (it.next().startsWith(id)) it.remove();
			}
		}
		it = elementFilters.iterator();
		while (it.hasNext()) {
			if (it.next().startsWith(id)) it.remove();
		}
	}
	/**
	 * Decides whether a selected item is filtered or not
	 * @param id  the id string of the selected item
	 * @return true, if the selected item is filtered
	 */
	boolean isMatched(String id) {
		if (id == null || id.equals("")) return subtreeFilters.contains("");
		for (String filter : elementFilters) {
			if (id.contentEquals(filter)) return true;
		}
		for (String filter : subtreeFilters) {
			if (id.startsWith(filter)) return true;
		}
		return false;
	};
	
	/**
	 * Collects the most specific condition of the variable
	 * @param id the id string of the selected item
	 * @return the id of the most specific condition stored in this object
	 */
	String getMostSpecificMatch(String id) {
		if (subtreeFilters.contains(id) || id.equals("")) return id;
		if (elementFilters.contains(id)) return id;
		String[] elements = id.split("\\.");
		String newId = "";
		for (int i=0; i<elements.length-1; i++) {
			newId=newId.concat(elements[i]);
			if (i != elements.length-2) newId = newId.concat(".");
		}
		return getMostSpecificMatch(newId);
	}
}
