/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.patterns.properties;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTPattern;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.PatternVariable;
import org.eclipse.viatra2.visualisation.patterns.viewmodel.ViewPattern;

/**
 * A Property Page Section describing the list of potential parameters of a
 * visualised call.
 * @author Zoltan Ujhelyi
 */
public class ParametersPropertySection extends AbstractPropertySection {
	// private List parameterList;
	private TableViewer parameterList;
	IStructuredContentProvider contentProvider = new IStructuredContentProvider() {

		GTPattern input;

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			input = (GTPattern) newInput;

		}

		public void dispose() {
		}

		public Object[] getElements(Object inputElement) {
			return input.getSymParameters().toArray();
		}
	};
	ILabelProvider labelProvider = new ILabelProvider() {

		public void removeListener(ILabelProviderListener listener) {
		}

		public boolean isLabelProperty(Object element, String property) {
			return false;
		}

		public void dispose() {
		}

		public void addListener(ILabelProviderListener listener) {
		}

		public String getText(Object element) {
			if (element instanceof PatternVariable)
				return ((PatternVariable) element).getName();
			return null;
		}

		public Image getImage(Object element) {
			return null;
		}
	};

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#createControls
	 * (org.eclipse.swt.widgets.Composite,
	 * org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
	 */
	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage aTabbedPropertySheetPage) {
		super.createControls(parent, aTabbedPropertySheetPage);
		Composite composite = getWidgetFactory()
				.createFlatFormComposite(parent);
		composite.setLayout(new RowLayout());
		getWidgetFactory().createCLabel(composite, "Parameters:");
		Table t = getWidgetFactory().createTable(composite, SWT.NONE);
		// TODO more dynamic sizing of the table needed
		t.setSize(400, 300);
		parameterList = new TableViewer(t);
		parameterList.setContentProvider(contentProvider);
		parameterList.setLabelProvider(labelProvider);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#setInput
	 * (org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void setInput(IWorkbenchPart part, ISelection selection) {
		super.setInput(part, selection);
		Object input = ((IStructuredSelection) selection).getFirstElement();
		GTPattern pattern = ((ViewPattern) input).getPattern();
		parameterList.setInput(pattern);
	}

}
