/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.patterns.sources;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.viatra2.gtasmmodel.vpm.editmodel.Entity;
import org.eclipse.viatra2.visualisation.patterns.viewmodel.AbstractViewModelElement;
import org.eclipse.viatra2.visualisation.patterns.viewmodel.ViewPattern;
import org.eclipse.viatra2.visualisation.patterns.viewmodel.ViewPatternBody;
import org.eclipse.viatra2.visualisation.patterns.viewmodel.ViewPatternModelElement;

/**
 * A content provider for creating a reduced graph. The reduced graph displays
 * relations as arcs. This way the resulting graphs are smaller, but it cannot
 * display relations of relations.
 * @author Zoltan Ujhelyi
 */
public class PatternReducedProvider extends PatternContentProvider {

	public Object[] getRelationships(Object arg0, Object arg1) {
		ArrayList<AbstractViewModelElement> relations = new ArrayList<AbstractViewModelElement>();
		if (machine != null && pattern != null) {
			if (arg0 instanceof ViewPatternModelElement
					&& arg1 instanceof ViewPatternModelElement) {
				ViewPatternModelElement element1 = (ViewPatternModelElement) arg0;
				ViewPatternModelElement element2 = (ViewPatternModelElement) arg1;
				if (element1.getBody().equals(element2.getBody())) {
					relations.addAll(loadRelationsBetween(element1, element2));
				}
			}
			if (arg0 instanceof ViewPatternModelElement
					&& arg1 instanceof ViewPattern) {
				relations.addAll(loadPatternParameters((ViewPattern) arg1,
						(ViewPatternModelElement) arg0));

			}
		}
		return relations.toArray();
	}

	void fillPatternGraph(Entity patternGraph, ViewPatternBody body,
			List<Object> elements) {

		for (Entity element : patternGraph.getComponents()) {
			ViewPatternModelElement entity = new ViewPatternModelElement();
			entity.setModelElement(element);
			entity.setBody(body);
			elements.add(entity);
			body.addEntity(entity);
			viewItems.put(element.getName(), entity);
		}
	}
}
