/*******************************************************************************
 * Copyright (c) 2009 Zoltan Ujhelyi and Daniel Varro.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Zoltan Ujhelyi - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.visualisation.patterns.viewmodel;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.gt.GTPattern;

/**
 * A view model element representing a GT Pattern from the model space.
 * @author Zoltan Ujhelyi
 */
public class ViewPattern extends AbstractViewModelElement{

	/**
	 * @return the pattern
	 */
	public GTPattern getPattern() {
		return pattern;
	}

	private PatternTypes type;
	private GTPattern pattern;
	private ArrayList<ViewPatternBody> bodies = new ArrayList<ViewPatternBody>();
	private Hashtable<String, ViewPatternModelElement> callParameters = new Hashtable<String, ViewPatternModelElement>();
	
	/**
	 * Initializes a GT Pattern in the view model.
	 * @param pattern the pattern to initialize.
	 * @param type the type of pattern to call.
	 */
	public ViewPattern(GTPattern pattern, PatternTypes type) {
		this.type = type;
		this.pattern = pattern;
	}

	/**
	 * Decides whether the pattern represents a positive or a negative pattern
	 * call.
	 * @return true if the pattern is a negative call.
	 */
	public PatternTypes getType() {
		return type;
	}
	
	/**
	 * Adds a pattern body to the view model.
	 * @param body
	 */
	public void addBody(ViewPatternBody body) {
		if (!bodies.contains(body)) {
			bodies.add(body);
		}
	}

	/**
	 * Return the bodies of the graph pattern
	 * @return the bodies
	 */
	public ArrayList<ViewPatternBody> getBodies() {
		return bodies;
	}

	@Override
	public String getName() {
		return pattern.getName();
	}

	@Override
	public String getFQName() {
		return pattern.getFqn();
	}
	
	/**
	 * Adds a parameter to the pattern with a name
	 * @param name the symbolic name of the parameter 
	 * @param parameter the parameter variable
	 */
	public void addParameter(String name, ViewPatternModelElement parameter) {
		callParameters.put(name, parameter);
	}
	
	/**
	 * Returns the list of symbolic names related to a parameter variable
	 * @param parameter the parameter variable
	 * @return the list of symbolic names
	 */
	public List<String> getParametersName(ViewPatternModelElement parameter) {
		ArrayList<String> foundParams = new ArrayList<String>();
		for (String key : callParameters.keySet()) {
			if (callParameters.get(key).equals(parameter)) {
				foundParams.add(key);
			}
		}
		return foundParams;
	}

	@Override
	public String getTypeString() {
		return "Pattern";
	}
}
