/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.editor.text.light.vtcl;

import org.eclipse.viatra2.editor.text.Activator;
import org.eclipse.viatra2.editor.text.light.VTCLEditor;
import org.eclipse.viatra2.frameworkgui.runner.TransformationRunner;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;
import org.eclipse.jface.action.Action;
import org.eclipse.viatra2.framework.FrameworkManager;

public class QuickRunVTCLAction extends Action
{

	public QuickRunVTCLAction()
	{
		setActionDefinitionId("org.eclipse.viatra2.editor.text.runCommand");
	}
	
	
	@Override
	public void run() {
		// check if we can load at all (there is at least one framework instance)
		if (FrameworkManager.getInstance().getAllFrameWorks().length<1)
			return;

		// proceed
		VTCLEditor iEditor = Activator.getDefault().getCurrentVTCLEditor();
		if (iEditor!=null)
		{
			// attach editor (if already attached, this call will do nothing)
			iEditor.attachEditor();
			// if dirty, save (& load) first
			if (iEditor.isDirty())
			{
					iEditor.doSave(null);
					iEditor.parseOnSave();
			}
			//if (iEditor.getAttachedFramework().getMachineByFQN(machineFqn))
			// check if save - parse - load was successful
			if (iEditor.getModule()!=null)
			{
				// TODO: This code should be rewritten to accommodate the new MachineWrapper
				Machine m = (Machine)iEditor.getAttachedFramework().getMachineByFQN(
						((Machine)iEditor.getModule().getMachine().get(0)).getFqn() );
				if (m!=null)
				{
					// try running
					TransformationRunner.run(m, iEditor.getAttachedFramework(), iEditor.getEditorSite().getShell());
				}						
			}
		}
	}
		
}
