/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.editor.text.light.vtcl;

//import org.eclipse.viatra2.texteditor_light.VTCLEditor;
//import org.eclipse.viatra2.lpgparser.loader.ErrorInformation.ErrorSeverity;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.viatra2.editor.text.markers.IVTCLErrorReporter;
import org.eclipse.viatra2.errors.info.ErrorInformation;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.texteditor.MarkerAnnotation;

public class VTCLMarkerManager implements IVTCLErrorReporter {
	
	protected ISourceViewer sourceViewer;
	protected IFile file;	
	
	public ISourceViewer getSourceViewer() 
	{
		return sourceViewer;
	}

	public void initialize(ISourceViewer sourceViewer, IFile file) {
		this.sourceViewer = sourceViewer;
		this.file = file;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.editor.text.light.vtcl.IVTCLErrorReporter#reportError(org.eclipse.viatra2.errors.info.ErrorInformation)
	 */
	public void reportError(ErrorInformation info)
	{
		try
		{
			int offset;
			if (info.getLocation().getBeginColumn() == 0 || info.getLocation().getEndColumn() == 0)
			{
				offset = 0;
			}
			else
			{
				offset = findOffset(info.getLocation().getBeginLine(), 
						info.getLocation().getBeginColumn(), getSourceViewer());
			}
			
//			int length =  info.getLocation().getEndColumn() - info.getLocation().getBeginColumn();
			int endOffset = findOffset(info.getLocation().getEndLine(), 
					info.getLocation().getEndColumn(), getSourceViewer());
			int length =  endOffset - offset + 1;
			if (length <= 0) length = 1;
			int severity;
			switch (info.getErrorSeverity()) {
			case INFO:
				severity = IMarker.SEVERITY_INFO;
				break;
			case WARNING:
				severity = IMarker.SEVERITY_WARNING;
				break;
			case ERROR:
				severity = IMarker.SEVERITY_ERROR;
				break;

			default:
				severity = IMarker.SEVERITY_ERROR;
				break;
			}
			//System.out.println("Offset: " + offset);
			//System.out.println("Length: " + length);
			addAnnotation(info.getMessage(), info.getMarkerId(), severity, new Position(offset,length), info.getLocation().getBeginLine());
		}
		catch (Exception e)
		{
			System.out.println("Editor::reportError exception occured!!!!");
		}
	}

	public boolean addAnnotation(String message, String markerId, int severity, Position pos, int line) 
	{
		IAnnotationModel annotationModel = getSourceViewer().getAnnotationModel();
		
		if ((annotationModel != null) && (file != null)) 
		{
			try 
			{				
				IMarker marker = file.createMarker(markerId);
				marker.setAttribute(IMarker.SEVERITY, severity);
				marker.setAttribute(IMarker.MESSAGE, message);
				//marker.setAttribute(IMarker.TEXT, message);
				marker.setAttribute(IMarker.LINE_NUMBER, line);
				//marker.setAttribute(IMarker., value)
				if (pos.offset != 0)
				{
					marker.setAttribute(IMarker.CHAR_START,pos.offset);
					marker.setAttribute(IMarker.CHAR_END,pos.offset+pos.length);
				}
				MarkerAnnotation annotation = new MarkerAnnotation(markerId, marker);
				annotationModel.addAnnotation(annotation, pos);					
				return true;
			} 
			catch (Exception exc) 
			{
				//System.err.println("exc");
				exc.printStackTrace(System.err);
			}
		}		
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.editor.text.light.vtcl.IVTCLErrorReporter#deleteMarkers(org.eclipse.core.resources.IResource, java.lang.String)
	 */
	public void deleteMarkers(IResource resource, String type) 
	{
		if (resource == null)
			return;
		try 
		{
			resource.deleteMarkers(type, true, IResource.DEPTH_INFINITE);
		} 
		catch (CoreException e) 
		{				
		}
	}

	public int findOffset(int line, int column, ISourceViewer viewer) 
	{
		IDocument doc = viewer.getDocument();
		int tabwidth = viewer.getTextWidget().getTabs();
		
		int offset = 0;
		if (line > 0) 
		{
			try 
			{
				offset = doc.getLineOffset(line - 1);
				if (column > 0) 
				{
					char c;
					for (int i = 0; i < column - 1;) 
					{
						c = doc.getChar(offset);
						++offset;
						if (c == '\t')
						{
							i = (i - (i % tabwidth)) + tabwidth;
							
						}
						else
						{
							++i;
						}
					}
				}
			} 
			catch (BadLocationException e) 
			{
				System.out.println(".findOffset(): Bad location: line="
						+ line
						+ ", column="
						+ column);
			}
		}
		return offset;
	}


}
