/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.actions;

import org.eclipse.viatra2.frameworkgui.content.FrameworkContainer;
import org.eclipse.viatra2.frameworkgui.content.ITreeObject;
import org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.viatra2.framework.IFramework;

/**
 * Summary:
 *  Abstract action class.
 *  
 * Description:
 *  This class supports internal references for the Viatra2 Framework instance
 *  and the Framework tree view, and built-in enabledness calculation.
 *  
 * @author Istvan Rath
 * 
 */
public abstract class AbstractFrameworkGUIAction extends Action 
{

	protected IFramework iViatraFramework;

	protected FrameworkTreeView iFT;

	public void setupInternals(FrameworkTreeView part) {
		iFT = part;
		refreshSelection();
	}

	protected void refreshSelection()
	{
		ISelection selection = iFT.getSelection();
		Object obj = ((IStructuredSelection) selection).getFirstElement();
		
		if (obj instanceof ITreeObject)
		{
			iViatraFramework = ((ITreeObject)obj).getFrameworkContainer().getFramework();
		}
		else if (obj instanceof FrameworkContainer)
		{
			iViatraFramework = ((FrameworkContainer)obj).getFramework();
		}
	}
	
	protected Object getFirstSelected()
	{
		IStructuredSelection s = (IStructuredSelection) iFT.getSelection();
		return s.getFirstElement();
	}
	
	public boolean isEnabled() {
		if (iFT != null) 
		{
			ISelection selection = iFT.getSelection();
			Object obj = ((IStructuredSelection) selection).getFirstElement();
			if (obj instanceof FrameworkContainer)
			{
				return true;
			}
		}
		return false;
	}

}
