/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.content.transformation;

import org.eclipse.viatra2.frameworkgui.actions.AbstractFrameworkGUIAction;
import org.eclipse.viatra2.frameworkgui.content.transformation.TransformationContent.MachineDummy;
import org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;

import java.io.IOException;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.SaveAsDialog;

public class SaveModelAction extends AbstractFrameworkGUIAction {

	public SaveModelAction(FrameworkTreeView part) {
		super();
		setupInternals(part);
		setText("Save as module");
		setToolTipText("Saves the program model as an XMI file");
		setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
	}
	
	@Override
	public boolean isEnabled() {
		return true;
	}
	
	@Override
	public void run() {
		refreshSelection();
		try 
		{
			Object c = getFirstSelected();
			if (c instanceof MachineDummy)
			{
				Machine m = ((MachineDummy)c).machine;
				SaveAsDialog d = new SaveAsDialog(iFT.getViewSite().getShell());
				d.setOriginalName(m.getName()+".module");
				if (d.open()==Window.OK)
				{
					IPath result = d.getResult();
					ResourceSet rs = new ResourceSetImpl();
					IWorkspace ws = ResourcesPlugin.getWorkspace();
					String wsroot = ws.getRoot().getRawLocation().toOSString();
					Resource res = rs.createResource(URI.createFileURI(wsroot+result.toOSString()));
					res.getContents().add(m.getModule());
					res.save(null);
					ResourcesPlugin.getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
				}
				
			}
		}
		catch (Exception e) 
		{
			iFT.showMessage(e.getMessage());
		}
		
	}
	
}
