/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.loaders;

import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.core.CorePackage;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Machine;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Module;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Transformation;
import org.eclipse.viatra2.errors.VPMRuntimeException;
import org.eclipse.viatra2.framework.FrameworkException;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.loaders.Loader;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

public class XMITransformationLoader implements Loader {

	public void process(InputStream f, IFramework fw)	throws VPMRuntimeException 
	{
		// return null;
	}

	public void processFile(String fileName, IFramework fw) throws VPMRuntimeException 
	{
		ResourceSet rs = new ResourceSetImpl();
		rs.getPackageRegistry().put(CorePackage.eNS_URI, CorePackage.eINSTANCE);
		Resource res = rs.getResource(URI.createFileURI(fileName), true);
		try {
			res.load(null);
		} catch (IOException e) {
			throw new VPMRuntimeException("XMI transformation load error");
		}
		if (res.getContents().get(0) instanceof Transformation)
		{
			Transformation t = (Transformation)res.getContents().get(0);
			try {
				for (Object _mod : t.getModules())
				{
					Machine m = (Machine) ((Module)_mod).getMachine().get(0);
					fw.addMachine(m.getFqn(), m);
				}
				
			} catch (Exception e) {
				// should never happen
				throw new VPMRuntimeException(e.getMessage());
			}
		}
	}

}
