/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.PathEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.viatra2.ViatraPlugin;
import org.eclipse.viatra2.framework.properties.IViatraPropertyProvider;
import org.eclipse.viatra2.framework.properties.VIATRAPreferences;

public class VIATRAPreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {

	public VIATRAPreferencePage() 
	{
		super(GRID);
		setPreferenceStore(ViatraPlugin.getDefault().getPreferenceStore());
		setDescription("Changes will be applied for newly loaded VPML files.");
		setMessage("VIATRA2 Framework");
		//initPrefs();
	}
	

	
	@Override
	protected void createFieldEditors() {
		addField(
				new BooleanFieldEditor(
				VIATRAPreferences.DISABLE_VPML_OVERRIDE,
				"&Disable override by custom preferences in VPML files",
				getFieldEditorParent()
				)
		);
		
		// add contributed field editors
		for (IViatraPropertyProvider p : ViatraPlugin.getDefault().getPropertyProviders())
		{
			Label l = new Label(getFieldEditorParent(),SWT.BOLD);
			l.setText(p.getProviderID());
			l.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_DARK_BLUE));
			//l.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			
			
			Label sep = new Label(getFieldEditorParent(), SWT.NONE);//SWT.HORIZONTAL|SWT.SEPARATOR);
			sep.setText("");
			sep.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_DARK_BLUE));
			//sep.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			
			
			for (String prop_id : p.getAllPropertyIds())
			{
				switch (p.getPropertyKind(prop_id))
				{
				case STRING:
					addField(
							new StringFieldEditor(
									p.getProviderID()+"::"+prop_id,
									prop_id,
									getFieldEditorParent()
									)
							);
					break;
				case INTEGER:
					addField(
							new IntegerFieldEditor(
									p.getProviderID()+"::"+prop_id,
									prop_id,
									getFieldEditorParent()
									)
							);
					break;
				case BOOLEAN:
					addField(
							new BooleanFieldEditor(
									p.getProviderID()+"::"+prop_id,
									"&"+prop_id,
									getFieldEditorParent()
									)
							);
					break;
				case PATH:
					addField(
							new PathEditor(
									p.getProviderID()+"::"+prop_id,
									prop_id,
									"Choose path",
									getFieldEditorParent()
									)
							);
					break;
				}
			}
			
			// just to ensure there is enough distance to the next category
			Label placeHolder0 = new Label(getFieldEditorParent(),SWT.BOLD);
			placeHolder0.setText("\n\n");
			Label placeHolder1 = new Label(getFieldEditorParent(), SWT.NONE);//SWT.HORIZONTAL|SWT.SEPARATOR);
			placeHolder1.setText("\n\n");
			
			
		}
		// just to ensure there is enough distance to the next category
		Label placeHolder0 = new Label(getFieldEditorParent(),SWT.BOLD);
		placeHolder0.setText("\n\n");
		Label placeHolder1 = new Label(getFieldEditorParent(), SWT.NONE);//SWT.HORIZONTAL|SWT.SEPARATOR);
		placeHolder1.setText("\n\n");
	}


	public void init(IWorkbench workbench) 
	{
		
	}

}
