/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.property;

import org.eclipse.viatra2.frameworkgui.content.FrameworkContainer;
import org.eclipse.viatra2.frameworkgui.content.ITreeObject;

import java.util.Vector;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.IPropertySourceProvider;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.viatra2.framework.IFramework;

public class MultiPropertySheetPage implements IPropertySheetPage {

	private TabFolder tabFolder;

	
	private Vector<PropertySheetPage> pages;
	
	private PropertySheetPage singlePage;
	
	/**
	 * Points to the selected Viatra framework.
	 */
	private IFramework viatraFramework;
	
	public MultiPropertySheetPage(IWorkbenchPart part)
	{
		pages = new Vector<PropertySheetPage>();
		//singlePage = new PropertySheetPage();
		selectionChanged(part,part.getSite().getSelectionProvider().getSelection());		
	}
	
	public void createControl(Composite parent) {
		tabFolder = new TabFolder(parent,SWT.BOTTOM);
		initTabs();
	}
	
	private void initSinglePage(ITreeObject treeObject)
	{
		if (viatraFramework==null)
			return;
		for (PropertySheetPage p : pages)
		{
			p.dispose(); // remove all propertysheetpages
		}
		for (TabItem i : tabFolder.getItems())
		{
			i.dispose(); // remove all tabitems
		}
		pages.clear();
		
		TabItem item = new TabItem(tabFolder,SWT.NONE);
		singlePage = new PropertySheetPage();
		singlePage.createControl(tabFolder);
		//final ITreeObject _to = treeObject;
		singlePage.setPropertySourceProvider(new IPropertySourceProvider(){

			public IPropertySource getPropertySource(Object object) 
			{
				if (object instanceof ITreeObject) 
				{
					return ((ITreeObject)object).getPropertySource();
				}
				return null;
			}
			
		});
		item.setText(treeObject.getLabelText());
		item.setControl(singlePage.getControl());
	}
	
	private void initTabs()
	{
		if (viatraFramework==null)
			return;
		for (PropertySheetPage p : pages)
		{
			p.dispose(); // remove all propertysheetpages
		}
		for (TabItem i : tabFolder.getItems())
		{
			i.dispose(); // remove all tabitems
		}
		pages.clear();
		for (String provID : viatraFramework.getProperties().getAllProviderIDs())
		{
			if (!viatraFramework.getProperties().isProviderActive(provID))
				continue; // FIXME remove me for all prop. prov. display
			TabItem item = new TabItem(tabFolder,SWT.NONE);
			PropertySheetPage page = new PropertySheetPage();
			page.createControl(tabFolder);
			page.setPropertySourceProvider(new FrameworkGUIPPropertySourceProvider(provID));
			pages.add(page);
			item.setText(provID);
			item.setControl(page.getControl());
		}
	}
/*
 * TODO 
 * 3. uj property felvetele dinamikusan? + notification?
 *    
 */
	
	
	public void dispose() {
		tabFolder.dispose();
	}

	public Control getControl() {
		return tabFolder;
	}

	public void setActionBars(IActionBars actionBars) {
		
	}

	public void setFocus() {
		
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (selection instanceof IStructuredSelection)
		{
			IStructuredSelection _s = (IStructuredSelection)selection;
			if (_s.getFirstElement() instanceof FrameworkContainer)
			{
				viatraFramework = ((FrameworkContainer)_s.getFirstElement()).getFramework();
				if (tabFolder != null)
				{
					initTabs();
					for (PropertySheetPage p : pages)
					{
						p.selectionChanged(part,selection);
					}
				}
			}
			else if (_s.getFirstElement() instanceof ITreeObject)
			{
				ITreeObject to = (ITreeObject) _s.getFirstElement();
				viatraFramework = to.getFrameworkContainer().getFramework();
				if (tabFolder!=null && to.getPropertySource()!=null)
				{
					initSinglePage(to);
					singlePage.selectionChanged(part, selection);
				}
			}
		}
	}



}
