/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.runner;

import java.util.StringTokenizer;

import org.eclipse.core.databinding.AggregateValidationStatus;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.observable.ChangeEvent;
import org.eclipse.core.databinding.observable.IChangeListener;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.core.databinding.validation.IValidator;
import org.eclipse.core.databinding.validation.ValidationStatus;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.bindings.keys.ParseException;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IElementComparer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.framework.FrameworkManager;
import org.eclipse.viatra2.framework.IFramework;
import org.eclipse.viatra2.frameworkgui.FrameworkGUIPlugin;
import org.eclipse.viatra2.frameworkgui.content.transformation.ViatraTreeviewSorter;

/**
 * @author Istvan Rath
 * 
 */
public class ValidatingRunnerInputDialog extends Dialog {
	private final String message;

	//private String value;

	private final String title;

	/**
	 * Used to select a model element from the tree view.
	 */
	private Button modelButton;

	private Button runButton;
	
	protected Label errorMessageLabel;

	private TreeViewer modelSpaceViewer;

	public String[] params;

	public String[] parsed_params;

	private Text valuetext;

	private final WritableValue parameters;

	private DataBindingContext context;

	// private FrameworkContainer frameworkContainer;

	private final String frameworkId;

	private AggregateValidationStatus aggregateValidationStatus;
	private IChangeListener listener;
	
	/**
	 * @param parentshell
	 *            The Shell
	 * @param t
	 *            The title of the dialog window
	 * @param m
	 *            The message above the edit box
	 * @param i
	 *            The initial value of the edit box
	 */
	public ValidatingRunnerInputDialog(Shell parentshell, // FrameworkContainer
															// cont,
			String fwid, String t, String m, String i, String[] p) {
		super(parentshell);
		// frameworkContainer = cont;
		frameworkId = fwid;
		title = t;
//		value = i;
		message = m;

		params = p;

		if (Realm.getDefault() == null)
			// SWTObservables.getRealm(Display.getCurrent());
			SWTObservables.getRealm(Display.getDefault());

		parameters = WritableValue.withValueType(String.class);
	}

	@Override
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(title);
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		runButton = createButton(parent, 0, IDialogConstants.OK_LABEL, true);
		runButton.setEnabled(false);
		createButton(parent, 1, IDialogConstants.CANCEL_LABEL, false);
		modelButton = createButton(parent, 2, "Select model element", false);
	}

	private class RunContentProposalAdapter implements IContentProposalProvider {
		public IContentProposal[] getProposals(String contents, int position) {
			Object[] entries = TransformationRunner.getRunDialogEntries(frameworkId);
			if (entries!=null)
			{
				IContentProposal[] ret = new IContentProposal[entries.length];
				for (int i = 0; i < ret.length; i++) {
					final String entry = (String) TransformationRunner
							.getRunDialogEntries(frameworkId)[ret.length - i - 1]; // inverse
					ret[i] = new IContentProposal() {
						public String getContent() {
							return entry;
						}
	
						public int getCursorPosition() {
							return entry.length();
						}
	
						public String getDescription() {
							return null;
						}
	
						public String getLabel() {
							return null;
						}
					};
				}
				return ret;
			}
			return new IContentProposal[]{};
		}
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		Label label = new Label(composite, SWT.HORIZONTAL | SWT.LEFT);
		label.setText(message);
		
		Label label2 = new Label(composite, SWT.HORIZONTAL | SWT.LEFT);
		label2.setText("Hint: use characters [,;] and space to separate parameters.");
		
		errorMessageLabel = new Label(composite, SWT.HORIZONTAL | SWT.LEFT);
		errorMessageLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_RED));
		GridData _gd = new GridData();
		_gd.widthHint = 400;
		errorMessageLabel.setLayoutData(_gd);
		errorMessageLabel.setText("No parameters read.");

		valuetext = new Text(composite, 2052);
		GridData gd = new GridData();
		gd.widthHint = 513; // magic number woohoohoohoo
		valuetext.setLayoutData(gd);
//		valuetext.setText(value);
		valuetext.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_RED));

		// content proposal based on history
		try {
			ContentProposalAdapter adapter = new ContentProposalAdapter(
					valuetext, new TextContentAdapter(),
					// new SimpleContentProposalProvider(new String [] {"A",
					// "B", "C"}),
					new RunContentProposalAdapter(), KeyStroke
							.getInstance("Ctrl+Space"), null);
		} catch (ParseException e) {
		}

		// place treeviewer
		modelSpaceViewer = new TreeViewer(composite, SWT.BORDER);
		GridData gd2 = new GridData();
		gd2.heightHint = 300;
		gd2.widthHint = 500;

		modelSpaceViewer.getTree().setLayoutData(gd2);
		modelSpaceViewer.setContentProvider(new ModelspaceContentProvider());
		modelSpaceViewer.setLabelProvider(new ModelspaceLabelProvider());
		// modelSpaceViewer.setInput(this.frameworkContainer.getFramework());
		modelSpaceViewer.setInput(FrameworkManager.getInstance().getFramework(
				this.frameworkId));
		modelSpaceViewer.setSorter(new ViatraTreeviewSorter());
		modelSpaceViewer.setComparer(new IElementComparer() {
			public boolean equals(Object a, Object b) {
				if (a == null && b == null)
					return true;
				if (a == null || b == null)
					return false;
				if (a instanceof IModelElement && b instanceof IModelElement)
					return a.equals(b);
				// if (a instanceof ViatraRootProvider && b instanceof
				// ViatraRootProvider)
				// return (a == b);
				return false;
			}

			public int hashCode(Object element) {
				return element == null ? 0 : element.hashCode();
			}
		});

		modelSpaceViewer.addDoubleClickListener(new IDoubleClickListener(){
			public void doubleClick(DoubleClickEvent event) 
			{
				buttonPressed(2);
			}
		});
		
		context = initDataBindings();

		return composite;
	}

	public class ModelspaceContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof IModelElement) {
				return ((IModelElement) parentElement).getElementsInNamespace()
						.toArray();
			} else
				return null;
		}

		public Object getParent(Object element) {
			if (element instanceof IModelElement) {
				return ((IModelElement) element).getNamespace();
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof IModelElement) {
				return !((IModelElement) element).getElementsInNamespace()
						.isEmpty();
			}
			return false;
		}

		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof IFramework) {
				return ((IFramework) inputElement).getTopmodel()
						.getModelManager().getRoot().getContents().toArray();
			} else
				return null;
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}

	}

	public static Image im_entity = FrameworkGUIPlugin.getImageDescriptor(
			"icons/entity_transparent.png").createImage();

	public static Image im_relation = FrameworkGUIPlugin.getImageDescriptor(
			"icons/relation_transparent.png").createImage();

	public class ModelspaceLabelProvider implements ILabelProvider {

		public Image getImage(Object element) {
			if (element instanceof IRelation)
				return im_relation;
			else if (element instanceof IEntity)
				return im_entity;
			return null;
		}

		public String getText(Object element) {
			return ((IModelElement) element).getName() + getTypesAsString((IModelElement)element);
		}

		public void addListener(ILabelProviderListener listener) {
		}

		public void dispose() {
		}

		public boolean isLabelProperty(Object element, String property) {
			return true;
		}

		public void removeListener(ILabelProviderListener listener) {
		}

		
		public String getTypesAsString(IModelElement element) {
			String r = " : ";
			for (IModelElement type : element.getTypes())
			{
				r+=type.getName()+", ";
			}
			if (r.length()>2)
			{
				r=r.substring(0, r.length()-2);
				return r;
			}
			else {
			 return "";
			}
		}
		
	}

	@Override
	protected void buttonPressed(int buttonId) {
		switch (buttonId) {
		case 0: 
			TransformationRunner.addRunDialogEntry(frameworkId,valuetext.getText());
			break;
		case 2:
			// use model element selection
			valuetext.setText(valuetext.getText()
					+ (this.valuetext.getText().length()>0?"; ":"")
					+ ((IStructuredSelection) modelSpaceViewer.getSelection())
							.getFirstElement().toString());
			break;
		}
		super.buttonPressed(buttonId);
	}
	
	@Override
	public boolean close() {
		defuseDataBindings();
		return super.close();
	}
	
//	public String getData() {
//		return value;
//	}

	protected DataBindingContext initDataBindings() {
		IObservableValue value = SWTObservables.observeText(this.valuetext,
				SWT.Modify);
		DataBindingContext bindingContext = new DataBindingContext();
		bindingContext.bindValue(value, parameters, new UpdateValueStrategy()
				.setAfterConvertValidator(new ParameterValidator()), null);

		final AggregateValidationStatus aggregateValidationStatus = new AggregateValidationStatus(
				bindingContext.getBindings(),
				AggregateValidationStatus.MAX_SEVERITY);
		this.aggregateValidationStatus = aggregateValidationStatus;

		this.listener = new IChangeListener() {
			public void handleChange(ChangeEvent event) {
				Object value = aggregateValidationStatus.getValue();
				if (value instanceof IStatus) {
					IStatus status = (IStatus) value;
					if (status.getSeverity() == IStatus.INFO) {
						// enable pressing run
						runButton.setEnabled(true);
						valuetext.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
						errorMessageLabel.setText("The number of parameters is correct.");
						errorMessageLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GREEN));

					} else {
						// disable running
						runButton.setEnabled(false);
						valuetext.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_RED));
						errorMessageLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_RED));
						errorMessageLabel.setText(status.getMessage());
					}
				}
			}
		};
		
		aggregateValidationStatus.addChangeListener(this.listener);
		return bindingContext;
	}

	void defuseDataBindings() {
		aggregateValidationStatus.removeChangeListener(this.listener);
	}
	
	protected class ParameterValidator implements IValidator {
		public IStatus validate(Object value) {
			boolean isValid = false;

			StringTokenizer st = new StringTokenizer((String) value, " ,;:");
			isValid = (st.countTokens() == params.length);
			if (isValid) {
				// perform parsing
				parsed_params = new String[params.length];
				int i = 0;
				while (st.hasMoreTokens())
					parsed_params[i++] = st.nextToken();
			}
			return isValid ? ValidationStatus.info("") : ValidationStatus
					.error("Read " + st.countTokens() + " parameters, but expected " + params.length+".");
		}
	}

}
