/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.views;

import org.eclipse.viatra2.frameworkgui.FrameworkGUIPlugin;
import org.eclipse.viatra2.frameworkgui.actions.DisposeFrameworkAction;
import org.eclipse.viatra2.frameworkgui.content.FrameworkViewContentProvider;
import org.eclipse.viatra2.frameworkgui.content.FrameworkViewLabelProvider;
import org.eclipse.viatra2.frameworkgui.content.ITreeObject;
import org.eclipse.viatra2.frameworkgui.content.NameSorter;
import org.eclipse.viatra2.frameworkgui.property.MultiPropertySheetPage;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.viatra2.framework.FrameworkManager;

/**
 * A treeview-based GUI for accessing Viatra Framework-specific functionality.
 * 
 * @author Istvan Rath
 */

public class FrameworkTreeView extends ViewPart implements ISelectionListener
{
	
	private TreeViewer viewer;

	private DrillDownAdapter drillDownAdapter;

	private DisposeFrameworkAction deleteAction;

	private Action refreshAction;

	private Action doubleClickAction;

	private FrameworkManager iFrameworkManager;

	public FrameworkManager getFrameWorkManager() {
		return iFrameworkManager;
	}

//	private Object refreshParameter;
	
	public void refreshViewer(final Object o) 
	{
//		refreshParameter = o;
		Display.getDefault().syncExec(new Runnable(){

			public void run() {
				if (!viewer.getTree().isDisposed())
				{
				if (o == null)
					viewer.refresh();
				else
				{
					viewer.refresh(o, true);
					viewer.expandToLevel(o, AbstractTreeViewer.ALL_LEVELS);
				}
				}
			}
			
		});
	}
	
	/**
	 * The constructor.
	 */
	public FrameworkTreeView() {
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	public void createPartControl(Composite parent) {
		iFrameworkManager = FrameworkManager.getInstance();
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		drillDownAdapter = new DrillDownAdapter(viewer);
		viewer.setContentProvider(new FrameworkViewContentProvider(this));
		viewer.setLabelProvider(new FrameworkViewLabelProvider());
		viewer.setSorter(new NameSorter());
		viewer.setInput(getViewSite());
		getSite().setSelectionProvider(viewer);
		getSite().getWorkbenchWindow().getSelectionService()
				.addSelectionListener(this);
		viewer.addDropSupport(org.eclipse.swt.dnd.DND.DROP_COPY
				| org.eclipse.swt.dnd.DND.DROP_MOVE, FrameworkDropTargetAdapter
				.getTransfers(), new FrameworkDropTargetAdapter(this));
		makeStaticActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				FrameworkTreeView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	public void updateLocalToolBar() {
		getViewSite().getActionBars().getToolBarManager().update(true);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) 
	{
		
	}

	private void fillContextMenu(IMenuManager manager) 
	{
		IStructuredSelection s = (IStructuredSelection)getSelection();
		Object first = s.getFirstElement();
		if (first instanceof ITreeObject)
		{
			if (((ITreeObject)first).getActions()!=null)
			{
				for (IAction a : ((ITreeObject)first).getActions())
				{
					manager.add(a);
				}
			}
			//manager.add(new Separator());
			if (((ITreeObject)first).getContributionItems()!=null)
			{
				for (IContributionItem i : ((ITreeObject)first).getContributionItems())
				{
					manager.add(i);
				}
			}
		}
		
		// drillDownAdapter.addNavigationActions(manager);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.removeAll();
		manager.add(refreshAction);
		manager.add(deleteAction);
		drillDownAdapter.addNavigationActions(manager);
	}

	private void makeStaticActions() {
		deleteAction = new DisposeFrameworkAction(this);
		refreshAction = new Action() {
			public void run() {
				refreshViewer(null);
				System.gc();
			}
		};
		refreshAction.setText("Refresh");
		refreshAction.setToolTipText("Refresh view");
		refreshAction.setImageDescriptor(FrameworkGUIPlugin.getImageDescriptor("icons/refresh.gif"));
		doubleClickAction = new Action() {
			public void run() {
				ISelection selection = viewer.getSelection();
				Object obj = ((IStructuredSelection) selection).getFirstElement();
				if (obj instanceof ITreeObject)
				{
					ITreeObject to = (ITreeObject) obj;
					if (to.getDoubleClickAction()!=null)
					{
						to.getDoubleClickAction().run();
					}
				}
			}
		};
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				doubleClickAction.run();
			}
		});
	}

	public void showMessage(String message) {
		MessageDialog.openInformation(viewer.getControl().getShell(),
				"VIATRA2 R3", message);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public ISelection getSelection() {
		return viewer.getSelection();
	}


	
	public Object getAdapter(Class type) {
		if (type == IPropertySheetPage.class) {
			//PropertySheetPage page = new PropertySheetPage();
			//page.setPropertySourceProvider(new FrameworkGUIPPropertySourceProvider());
			//return page;
			return new MultiPropertySheetPage(this);
		}
		return super.getAdapter(type);
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if (this.equals(getSite().getPage().getActivePart())) {
			if (deleteAction != null)
				deleteAction.calculateEnabled();
		}
	}
	
}
