/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.frameworkgui.views.code;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.viatra2.framework.FrameworkManager;

public class SaveOutputAction extends Action {

	CodeBufferView cbv;
	
	public SaveOutputAction(CodeBufferView v)
	{
		cbv = v;
		this.setToolTipText("Save current buffer content to a file in the workspace");
		this.setImageDescriptor(PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
		this.setText("Save output");
	}
	
	@Override
	public void run() 
	{
		String target = null;
		String targetName = null;
		CodeBufferView.DummyPage.DummyBufferPage pp = cbv.getCurrentInnerPage();
		if (pp==null)
		{
			// default buffer should be written
			target = cbv.getCurrentOuterPage().defaultBuffer.toString();
			if (target==null || target.length()<1)
			{
				target = cbv.getCurrentOuterPage().defaultDisplay.getText();
			} // fix for bug #266484
			targetName = "default";
		}
		else
		{
			target = pp.text.getText();
			targetName = pp.key.replaceFirst("core://", "");
		}
		SaveAsDialog d = new SaveAsDialog(cbv.getViewSite().getShell());
		d.setBlockOnOpen(true);
		d.setTitle("Save buffer output");
		d.setOriginalName(targetName);
		if (Window.OK == d.open())
		{
			try
			{
				// save output
				IFile ifile = ResourcesPlugin.getWorkspace().getRoot().getFile(d.getResult());
				FileWriter wr = new FileWriter(new File(ifile.getLocation().toOSString()));
				try {
					wr.write(target);
				}
				finally {
					wr.flush();
					wr.close();
				}
				ResourcesPlugin.getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, new NullProgressMonitor());
			}
			catch (IOException e)
			{
				FrameworkManager.getInstance().getFramework(cbv.getCurrentOuterPage().frameworkId).getLogger().fatal(e.getMessage());
			} catch (CoreException e) 
			{
				FrameworkManager.getInstance().getFramework(cbv.getCurrentOuterPage().frameworkId).getLogger().fatal(e.getMessage());
			}
		}
		
	}
	
	
}
