/*******************************************************************************
 * Copyright (c) 2008-2010, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.frameworkgui.views.console.commands;

import java.util.List;

import org.eclipse.viatra2.framework.IFramework;

/**
 * Interface for VIATRA console command providers.
 * 
 * Command providers are identified by their Java class names.
 * 
 * E.g. to invoke a commandprovider implemented in SomeCommand.java, you would need
 * to type "SomeCommand" or "someCommand" or "somecommand" or "SoMeCommAnd" etc. into the console. 
 * 
 * @author istvan rath
 *
 */
public interface IVIATRAConsoleCommandProvider {
	
	public String getCommandSignature();
	
	public String getDescription();
	
	public String getHelpText();
	
	/**
	 * Implementation should care about exception handling by e.g. logging as an error.
	 * @param fw
	 * @param parameters
	 */
	public void executeCommand(IFramework fw, List<String> parameters);
	
}
