/*******************************************************************************
 * Copyright (c) 2004-2008 Peter Pasztor, Akos Horvath, Gergely Varro, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Peter Pasztor, Akos Horvath, Gergely Varro, Istvan Rath - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.interpreter.impl.rules;

import org.eclipse.emf.common.util.EList;
import org.eclipse.viatra2.gtasm.interpreter.exception.ViatraTransformationException;
import org.eclipse.viatra2.gtasm.interpreter.executionEnvironment.IExecutionEnvironment;
import org.eclipse.viatra2.gtasm.interpreter.impl.executionEnvironment.ExecutionEnvironment;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.compoundRules.NestedRule;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.compoundRules.ParallelRule;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.compoundRules.RandomRule;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.compoundRules.SequentialRule;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.simpleRules.ASMRuleInvocation;
import org.eclipse.viatra2.interpreters.IProgressReport;


public class NestedRuleInterpreter extends RuleInterpreter {

	private static NestedRuleInterpreter _instance = new NestedRuleInterpreter();

	
	public static NestedRuleInterpreter getInstance() { 
		return _instance; 
	} 
	
	@Override
	public Boolean interpretRule(IExecutionEnvironment executionEnvironment, ASMRuleInvocation ruleToBeInterpreted, final IProgressReport pr) 
		throws ViatraTransformationException
	{
		NestedRule nestedRule = (NestedRule)ruleToBeInterpreted;
		if(nestedRule instanceof SequentialRule)
		{
			for(int i=0;i<((SequentialRule)nestedRule).getSubrules().size();i++)
			{
				if(!(RuleInterpreter.getInstance().
						interpretRule(executionEnvironment,
								((SequentialRule)nestedRule).getSubrules().get(i),pr)))
					return Boolean.FALSE;
			}
			return Boolean.TRUE;
			
		}
		else if(nestedRule instanceof RandomRule)
		{
			int i=0;
			i= (int)Math.round(Math.random()*(((RandomRule)nestedRule).getSubrules().size()-1));
			return RuleInterpreter.getInstance().
					interpretRule(executionEnvironment,
							((RandomRule)nestedRule).getSubrules().get(i),pr);
		}
		else if(nestedRule instanceof ParallelRule)
		{
			EList<ASMRuleInvocation> subRules = ((ParallelRule)nestedRule).getSubrules(); 
			Thread[] threads = new Thread[subRules.size()];
			final boolean[] returnValues = new boolean[subRules.size()];
			final IExecutionEnvironment[] eEnvs = new IExecutionEnvironment[subRules.size()];
			for (int i=0; i<subRules.size(); i++) eEnvs[i] = new ExecutionEnvironment(executionEnvironment);
			//for (int i=0; i<subRules.size(); i++) eEnvs[i] = org.eclipse.viatra2.gtasm.interpreter.executionEnvironment;
			
			for(int _i=0;_i<subRules.size();_i++)
			{
				final int i = _i;
				final ASMRuleInvocation target = subRules.get(i);
				threads[i] = new Thread(new Runnable(){
					public void run() 
					{
						try
						{
							returnValues[i] = RuleInterpreter.getInstance().interpretRule(eEnvs[i], target, pr);
						}
						catch (ViatraTransformationException e)
						{
							e.printStackTrace();
							//TODO: istvan proper error handling required 
						}
						
					}
				}, "ParallelVIATRA_R3");
				//if(!(org.eclipse.viatra2.gtasm.interpreter.executionEnvironment.getRuleInterpreter().interpretRule(org.eclipse.viatra2.gtasm.interpreter.executionEnvironment,(Rule)((ParallelRule)nestedRule).getSubrules().get(i))))
				//	return Boolean.FALSE;
			}
			
			// start org.eclipse.viatra2.gtasm.interpreter.impl.threads
			for (Thread t : threads) t.start();
			
			// join org.eclipse.viatra2.gtasm.interpreter.impl.threads
			try
			{
				for (Thread t : threads) t.join();
			} catch (InterruptedException e) { e.printStackTrace(); }
			
			// calculate return value
			boolean ret = Boolean.TRUE;
			for (boolean b : returnValues) ret &= b;
			return ret;
		}
		return Boolean.FALSE;
	}
}
