/*******************************************************************************
 * Copyright (c) 2004-2008 Akos Horvath, Gergely Varro and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Akos Horvath, Gergely Varro - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.callgraph;

import java.util.Collection;

import org.eclipse.viatra2.core.IModelManager;
import org.eclipse.viatra2.gtasm.patternmatcher.exceptions.PatternMatcherCompileTimeException;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.callgraph.FlattenedPattern.Pair;
import org.eclipse.viatra2.logger.Logger;


public interface IFlattenedPatternElement {
    /** Adds the local variables of the pattern body to the search graph of its container search graph
     * @param pattern The container flattened pattern 
     * @throws PatternMatcherCompileTimeException
     */
    void addLocalVariables(FlattenedPattern pattern)
        throws PatternMatcherCompileTimeException;
    /** Evaluates the passed parameters of the flattened pattern body invocation and adds them to the search graph of its container flattened pattern
     * @param pattern the container flattened pattern
     * @throws PatternMatcherCompileTimeException
     */
    void addFormalParameters(FlattenedPattern pattern)
        throws PatternMatcherCompileTimeException;
    /** Adds the variable assignment related elements to the search graph of its containing flattened pattern
     * @param pattern The container flattened pattern
     * @throws PatternMatcherCompileTimeException
     */
   Collection<Pair> processVariableAssignments(FlattenedPattern pattern)
        throws PatternMatcherCompileTimeException;
    /** Adds the NAC check operation to its flattened pattern
     * @param pattern The container flattened pattern
     * @param logger The logger to be used in case of execution
     * @param manager The model manager used to match the NAC pattern
     * @throws PatternMatcherCompileTimeException
     * @return The collection of the assigned variables pairs
     */
   void processNegativeApplicationConditions(FlattenedPattern pattern, Logger logger, IModelManager manager)
        throws PatternMatcherCompileTimeException;
    /** Adds a check operation to the operation plan of its flattened pattern.
     * @param pattern The container flattened pattern
     * @param logger The logger to be used when the operation is executed
     * @throws PatternMatcherCompileTimeException
     */
    void processCheckExpressions(FlattenedPattern pattern, Logger logger)
        throws PatternMatcherCompileTimeException;
    /**Builds the Search graph of the input flattened pattern
     * @param pattern The flattened pattern whose search graph is created
     * @throws PatternMatcherCompileTimeException
     */
    void buildSearchGraph(FlattenedPattern pattern)
        throws PatternMatcherCompileTimeException;
    /** Sets the current location ID to the input value
     * @param currentLocation
     */
    void setCurrentLocation(int currentLocation);
    
    /** Adds the injectivity constraints defined in the pattern to the search operations
     * @param pattern The container flattened pattern
     * @throws PatternMatcherCompileTimeException
     * @return The collection of the assigned variables pairs 
     */
    public Collection<Pair> processInjectivityAssignments(FlattenedPattern pattern) throws PatternMatcherCompileTimeException ;
	
    /**Generates the injectivity constraints based on its container graph pattern's setting (sharable) and the actual variable assignment and non equality operations
	 * @param flattenedPattern the container flattened pattern
     * @param localAdditionalExclusionInjectivityPairs Variable pair indexes that were included in a variable assignment operation (in the actual pattern body)
	 * @param localAdditionalInclusionInjectivityPairs Variable pair indexes that were included in a non equality  operation (in the actual pattern body)
   */
	public void generateElementInjectivityConstraints(FlattenedPattern flattenedPattern,
			Collection<Pair> localAdditionalExclusionInjectivityPairs,
			Collection<Pair> localAdditionalInclusionInjectivityPairs) throws PatternMatcherCompileTimeException; 
}
