/*******************************************************************************
 * Copyright (c) 2004-2008 Akos Horvath, Gergely Varro and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Akos Horvath, Gergely Varro - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.operation;

import java.util.Set;

import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.gtasm.patternmatcher.exceptions.PatternMatcherRuntimeException;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.gtmatcher.exceptions.GTErrorStrings;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.gtmatcher.exceptions.GTOperationException;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.MatchingFrame;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.PatternMatcherErrorStrings;
import org.eclipse.viatra2.gtasm.patternmatcher.impl.patternmatcher.internal.searchgraph.SearchGraphEdge;
import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.core.AnnotatedElement;


public class VariableAssignmentCheckOperation extends CheckOperation {
    private Integer source;
    private Integer target;
    private SearchGraphEdge relatedSearchGraphEdge;
    
    public VariableAssignmentCheckOperation(Integer source, 
            Integer target, SearchGraphEdge edge) {
        this.source = source;
        this.target = target;
        this.relatedSearchGraphEdge = edge;
    }

    protected boolean check(MatchingFrame frame) 
        throws PatternMatcherRuntimeException {
        IModelElement sourceElement = null;
        IModelElement targetElement = null;
        try {
            sourceElement = (IModelElement) frame.getValue(source);
        } catch (ClassCastException e) {
        	String[] context = {relatedSearchGraphEdge.getSourceNode().getName(), getClass().getSimpleName()};
			throw new PatternMatcherRuntimeException(GTErrorStrings.NOT_A_MODELELEMENT
					,context
					,relatedSearchGraphEdge.getSourceNode().getTraceabilityElement().getRepresentativeEMFElement());

        }
        try {
            targetElement = (IModelElement) frame.getValue(target);
        } catch (ClassCastException e) {
        	String[] context = {relatedSearchGraphEdge.getTargetNode().getName(), getClass().getSimpleName()};
			throw new PatternMatcherRuntimeException(GTErrorStrings.NOT_A_MODELELEMENT
					,context
					,relatedSearchGraphEdge.getTargetNode().getTraceabilityElement().getRepresentativeEMFElement());

        }
        if (sourceElement == null) {
        	String[] context = {relatedSearchGraphEdge.getSourceNode().getName(),getClass().getSimpleName()};
			throw new PatternMatcherRuntimeException(PatternMatcherErrorStrings.INTERNAL_EMPTY_VALUE
					,context
					,relatedSearchGraphEdge.getSourceNode().getTraceabilityElement().getRepresentativeEMFElement());
	
        }
        if (targetElement == null) {
        	String[] context = {relatedSearchGraphEdge.getTargetNode().getName(),getClass().getSimpleName()};
			throw new PatternMatcherRuntimeException(PatternMatcherErrorStrings.INTERNAL_EMPTY_VALUE
					,context
					,relatedSearchGraphEdge.getTargetNode().getTraceabilityElement().getRepresentativeEMFElement());
	
        }
        return sourceElement.equals(targetElement);
    }

    public String toString() {
		return getClass().getSimpleName() + " : " + 
            source + " == " + target;
	}
    
    public void calculateSidewaysPassedVariables(Set<Integer> variables) {
    	variables.add(source);
    	variables.add(target);
    }
    
    public void calculateLocalVariables(Set<Integer> variables) {
    	variables.add(source);
    	variables.add(target);
    }

	public AnnotatedElement getErrorfulElement(MatchingFrame frame) {
		return relatedSearchGraphEdge.getTraceabilityElement().getRepresentativeEMFElement();
	}
}
