/*******************************************************************************
 * Copyright (c) 2004-2010 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/
package org.eclipse.viatra2.gtasm.patternmatcher.incremental;

import org.eclipse.viatra2.framework.properties.VIATRAPropertyKind;
import org.eclipse.viatra2.framework.properties.providers.DefaultPropertyProvider;

public class IncrementalPMPropertyProvider extends DefaultPropertyProvider {

	public final static String VIATRA_INCREMENTAL_PROVIDER_ID = "Incremental Pattern Matcher";
	public final static String PROP_THREADS = "Number of parallel threads";// (0: single-threaded; 1: asynchronous PM; n>1: n parallel containers; auto: adjust to resources)";
	
	public IncrementalPMPropertyProvider() {
		super(VIATRA_INCREMENTAL_PROVIDER_ID);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.framework.properties.providers.DefaultPropertyProvider#init()
	 */
	@Override
	public void init() {
		allids.add(PROP_THREADS);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.framework.properties.IViatraPropertyProvider#getDefaultPropertyValue(java.lang.String)
	 */
	public String getDefaultPropertyValue(String id) {
		if (id.equals(PROP_THREADS)) return "0"; 
		else return "default";
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.framework.properties.IViatraPropertyProvider#getPropertyKind(java.lang.String)
	 */
	public VIATRAPropertyKind getPropertyKind(String id) {
		return VIATRAPropertyKind.STRING;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.viatra2.framework.properties.IViatraPropertyProvider#isActive()
	 */
	public boolean isActive() {
		return true;
	}

	public static Integer interpretThreads(String threadStr) {
		Integer threads = null;
		try {
			threads = Integer.parseInt(threadStr);
		} catch (NumberFormatException e) {
			String lcThreads = threadStr.toLowerCase();
			if ("true".equals(lcThreads) || "parallel".equals(lcThreads)) threads = 1;
			if ("auto".equals(lcThreads)) threads = Runtime.getRuntime().availableProcessors();
			// default to null
		}
		return threads;
	}
}
