/*******************************************************************************
 * Copyright (c) 2004-2008 Gabor Bergmann and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Gabor Bergmann - initial API and implementation
 *******************************************************************************/

package org.eclipse.viatra2.gtasm.patternmatcher.incremental.simple;

import java.util.HashSet;
import java.util.LinkedList;

import org.eclipse.viatra2.gtasmmodel.gtasm.metamodel.asm.definitions.Variable;
import org.eclipse.viatra2.gtasmmodel.vpm.editmodel.ModelElement;

class PatternNode implements Comparable<PatternNode> {
	/**
	 * The name of the pattern variable. This is the unique key of the pattern node.
	 */
	public String name;
	
	/**
	 * An editmodel element corresponding to this variable, if there are any.
	 */
	public ModelElement source; 

	// public boolean isEntity; 
	/**
	 * if a node is not a symbolic parameter nor is it touched (quantified) by any
	 * nonnegative constraints, it should be inversely quantified
	 */
	public boolean isTouched; 
	/**
	 * virtual pNodes are nodes that do not correspond to actual pattern
	 * variables; they represent constants or Term substitutes
	 */
	public boolean isVirtual; 
	/**
	 * true if node is explicitly made exempt from injectivity
	 */
	public boolean isExemptFromInjectivity = false; 
	
	/**
	 * Counts the number of hitherto unmatched constraints that can restrict the type of the node
	 */
	public int lingeringTypeInfoCounter; 
	/**
	 * types explicitly asserted of this node
	 */
	public LinkedList<Object> assertedTypes;
	/**
	 * types automatically implied by other constraints
	 */
	public HashSet<Object> impliedTypes;

	
	/**
	 * @param name
	 * @param entity
	 */
	private PatternNode(String name, /* boolean isEntity, */boolean isVirtual,
			boolean isTouched) {
		super();
		this.name = name;
		this.source = null; // (entities==null)? new LinkedList<Entity>() :
							// entities;
		// this.isEntity = isEntity;
		this.isVirtual = isVirtual;
		this.isTouched = isTouched;
		
		this.assertedTypes = new LinkedList<Object>();
		this.impliedTypes = new HashSet<Object>();
		this.lingeringTypeInfoCounter = 0;
	}

	private PatternNode(String name/* , boolean isEntity */) {
		this(name, /* isEntity, */false, false);
	}

	/*
	 * public PatternNode(Entity entity) { this(entity.getName(), null, true,
	 * false, false); entities.add(entity); }
	 */

	public PatternNode(Variable variable/* , boolean isEntity */) {
		this(variable.getName()/* , isEntity */);
	}

	// virtual variable: constant or term substitution
	public PatternNode(int newId) {
		this(".virtual{" + newId + "}", /* false, */true, true /*
																 * virtual nodes
																 * are
																 * considered
																 * touched
																 */);
		// this.entities = new LinkedList<Entity>();
		// this.isEntity = false;
	}

	/*
	 * public boolean isEntity() { return isEntity; }
	 */
	public boolean isVirtual() {
		return isVirtual;
	}
	
	public boolean isExemptFromInjectivity() {
		return isVirtual || isExemptFromInjectivity;
	}
	public void makeExemptFromInjectivity() {
		isExemptFromInjectivity = true;
	}
	
	/**
	 * @param entity
	 *            the entity to set
	 */
	public void setSource(ModelElement source) {
		this.source = source;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final PatternNode other = (PatternNode) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	public int compareTo(PatternNode arg0) {
		return name.compareTo(arg0.name);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return name;// + ":PatternNode";
	}

}
