/*******************************************************************************
 * Copyright (c) 2004-2008 Andras Okros, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Andras Okros - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.gtasm.trigger.view;

import org.eclipse.viatra2.frameworkgui.content.FrameworkContainer;
import org.eclipse.viatra2.frameworkgui.content.IFrameworkContentProviderExtension;
import org.eclipse.viatra2.frameworkgui.content.ITreeObject;
import org.eclipse.viatra2.frameworkgui.content.ITreeParent;
import org.eclipse.viatra2.frameworkgui.views.FrameworkTreeView;
import org.eclipse.viatra2.gtasm.trigger.Activator;
import org.eclipse.viatra2.gtasm.trigger.engine.ITriggerSetChangedListener;
import org.eclipse.viatra2.gtasm.trigger.engine.TriggerEngineManager;
import org.eclipse.viatra2.gtasm.trigger.engine.TriggerExecutionEngine;
import org.eclipse.viatra2.gtasm.trigger.model.Trigger;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource2;
import org.eclipse.viatra2.framework.IFramework;

/**
 * An extension for the Viatra framework's model space view.
 * @author Andr�s �kr�s
 */
public class FrameworkViewExtension 
	implements IFrameworkContentProviderExtension, ITreeParent, ITriggerSetChangedListener
{
	private DeleteTriggerAction deleteAction;
	private StartTriggerAction startAction;
	private StopTriggerAction stopAction;
	private SwitchExecutionModeAction switchAction;
	private Vector<IAction> actions;
	private Vector<IAction> sActions;
	private FrameworkTreeView viewer;
	private IFramework fw;
	private TriggerExecutionEngine tee;
	private FrameworkContainer cont;
	private Map<Object, ITreeObject> triggers;
	//private static Image trigger_image = Activator.getImageDescriptor("icons/trigger_transparent.png").createImage();
	private static Image trigger_loaded = Activator.getImageDescriptor("icons/trigger_loaded.png").createImage();
	private static Image trigger_running = Activator.getImageDescriptor("icons/trigger_running.png").createImage();
	private static Image trigger_stopped = Activator.getImageDescriptor("icons/trigger_stopped.png").createImage();
		
	
	public FrameworkViewExtension() 
	{
		triggers = new HashMap<Object, ITreeObject>();
		actions = new Vector<IAction>();
		sActions = new Vector<IAction>();
	}
	
	public void init(FrameworkTreeView fv, IFramework f, FrameworkContainer c) 
	{
		viewer = fv; 
		fw = f;
		cont = c;

		tee = TriggerEngineManager.getInstance().getTriggerEngine(fw);
		deleteAction = new DeleteTriggerAction(fv);
		startAction = new StartTriggerAction(fv);
		stopAction = new StopTriggerAction(fv);
		switchAction = new SwitchExecutionModeAction(fv,fw,tee);
		actions.add(deleteAction);
		actions.add(startAction);
		actions.add(stopAction);
		sActions.add(switchAction);
		initTriggers();
		tee.addTriggerSetListener(this);
		
	}
	
	//First time run.. At this moment it won't find anything;
	//(No trigger can be in the system at the start...)
	private void initTriggers()
	{
		for (Object _m : tee.getTriggers())
		{
			if (_m instanceof Trigger)
				triggers.put(_m, new TriggerDummy((Trigger)_m));
		}
	}
	
	public Collection<IAction> getActions()
	{ 
		return sActions;
	}
	
	public Collection<IContributionItem> getContributionItems() 
	{
		return null;
	}

	public ITreeObject getTopLevelEntry() 
	{
		return this;
	}
	
	public void dispose() 
	{
		tee.removeTriggerSetListener(this);
	}

	public Set<ITreeObject> getChildren() 
	{
		return new HashSet<ITreeObject>(triggers.values());
	}

	public boolean hasChildren() 
	{
		return !triggers.isEmpty();
	}

	public FrameworkContainer getFrameworkContainer() 
	{
		return cont;
	}

	public Image getLabelImage() 
	{
		// TODO
		return null;
	}

	public String getLabelText() 
	{
		String s;
		if (tee.isSerialExecutionMode())
		{
			s = "Triggers (serial execution mode)";
		} else {
			s = "Triggers (parallel execution mode)";
		}
		return s;
	}

	public ITreeParent getParent() 
	{
		return cont;
	}

	public IPropertySource2 getPropertySource() 
	{
		return null;
	}
	
	public class TriggerDummy implements ITreeObject 
	{
		private Trigger trigger;
		
		public TriggerDummy(Trigger t) 
		{
			trigger = t;
		}
		
		public Trigger getTrigger()
		{
			return trigger;
		}
		
		public FrameworkContainer getFrameworkContainer() {
			return cont;
		}

		public Image getLabelImage() 
		{
			if (trigger.isReteEngineSet()==false)
				return trigger_loaded;
			else if (trigger.isRunning())
				return trigger_running;
			else
				return trigger_stopped;
			//return trigger_image;
		}

		public String getLabelText() 
		{
			return trigger.getName();
		}

		public ITreeParent getParent() 
		{
			return FrameworkViewExtension.this;
		}
	
		public IPropertySource2 getPropertySource() {
			TriggerPropertySource tps = new TriggerPropertySource(trigger);
			return tps;
		}

		public Collection<IContributionItem> getContributionItems() {
			return null;
		}
		
		public Collection<IAction> getActions() 
		{
			return actions;
		}

		public IAction getDoubleClickAction() {
			return null;
		}
	}
	
	public void triggerAdded(Object trigger) 
	{
		if (trigger instanceof Trigger)
			triggers.put(trigger , new TriggerDummy((Trigger)trigger));	
		viewer.refreshViewer(this);
	}

	public void triggerRemoved(Object trigger) 
	{
		if (trigger instanceof Trigger)
			triggers.remove(trigger);
		viewer.refreshViewer(this);
	}
	
	public void triggerRefreshed(Object trigger) 
	{
		if (trigger instanceof Trigger){
			triggers.remove(trigger);			
			triggers.put(trigger , new TriggerDummy((Trigger)trigger));
		}	
		viewer.refreshViewer(this);
	}

	public IAction getDoubleClickAction() {
		return null;
	}

}
