/**
* Copyright (c) 2007 - 2009 OptXware Research and Development LLC.
* All rights reserved.   This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*   Daniel Varro - Initial API and implementation
*
* This file was generated automatically based upon the VTCL LPG Parser
* VTCLParser.g 
*/

package org.eclipse.viatra2.lpgparser;

import org.eclipse.viatra2.lpgparser.ast.*;
import lpg.lpgjavaruntime.*;
import org.eclipse.viatra2.errors.info.ErrorInformation;
import org.eclipse.viatra2.errors.info.Location;
import org.eclipse.viatra2.lpgparser.loader.VTCLMessages;
import org.eclipse.viatra2.errors.info.ErrorInformation.ErrorKind;
import org.eclipse.viatra2.errors.info.ErrorInformation.ErrorSeverity;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.eclipse.viatra2.lpgparser.loader.ExplanationGenerator;
  
public class VTCLParser extends PrsStream implements RuleAction {
    final String PARSER_MARKER = "org.eclipse.viatra2.loaders.vtclparsermarker";

    ExplanationGenerator generator = new ExplanationGenerator(this);

    private java.util.List<ErrorInformation> allErrorInfos = new java.util.ArrayList<ErrorInformation>();
    public java.util.List<ErrorInformation> getAllErrorInfos() {
        return allErrorInfos;
    }
    
    public void clearParseErrorList() {
    if (allErrorInfos == null)
	    allErrorInfos = new java.util.ArrayList<ErrorInformation>();
    else
	    allErrorInfos.clear();
}

    private static ParseTable prs = new VTCLParserprs();
    private BacktrackingParser btParser;

    public BacktrackingParser getParser() { return btParser; }
    private void setResult(Object object) { btParser.setSym1(object); }
    public Object getRhsSym(int i) { return btParser.getSym(i); }

    public int getRhsTokenIndex(int i) { return btParser.getToken(i); }
    public IToken getRhsIToken(int i) { return super.getIToken(getRhsTokenIndex(i)); }
    
    public int getRhsFirstTokenIndex(int i) { return btParser.getFirstToken(i); }
    public IToken getRhsFirstIToken(int i) { return super.getIToken(getRhsFirstTokenIndex(i)); }

    public int getRhsLastTokenIndex(int i) { return btParser.getLastToken(i); }
    public IToken getRhsLastIToken(int i) { return super.getIToken(getRhsLastTokenIndex(i)); }

    public int getLeftSpan() { return btParser.getFirstToken(); }
    public IToken getLeftIToken()  { return super.getIToken(getLeftSpan()); }

    public int getRightSpan() { return btParser.getLastToken(); }
    public IToken getRightIToken() { return super.getIToken(getRightSpan()); }

    public int getRhsErrorTokenIndex(int i) {
        int index = btParser.getToken(i);
        IToken err = super.getIToken(index);
        return (err instanceof ErrorToken ? index : 0);
    }
    public ErrorToken getRhsErrorIToken(int i) {
        int index = btParser.getToken(i);
        IToken err = super.getIToken(index);
        return (ErrorToken) (err instanceof ErrorToken ? err : null);
    }

    public VTCLParser(LexStream lexStream) {
        super(lexStream);

        try {
            super.remapTerminalSymbols(orderedTerminalSymbols(), VTCLParserprs.EOFT_SYMBOL);
        } catch(NullExportedSymbolsException e) {
        } catch(NullTerminalSymbolsException e) {
        } catch(UnimplementedTerminalsException e) {
            java.util.ArrayList unimplemented_symbols = e.getSymbols();
            System.out.println("The Lexer will not scan the following token(s):");
            for (int i = 0; i < unimplemented_symbols.size(); i++) {
                Integer id = (Integer) unimplemented_symbols.get(i);
                System.out.println("    " + VTCLParsersym.orderedTerminalSymbols[id.intValue()]);               
            }
            System.out.println();                        
        }
        catch(UndefinedEofSymbolException e) {
            throw new Error(new UndefinedEofSymbolException
                                ("The Lexer does not implement the Eof symbol " +
                                 VTCLParsersym.orderedTerminalSymbols[VTCLParserprs.EOFT_SYMBOL]));
        } 
    }

    public String[] orderedTerminalSymbols() { return VTCLParsersym.orderedTerminalSymbols; }
    public String getTokenKindName(int kind) { return VTCLParsersym.orderedTerminalSymbols[kind]; }
    public int getEOFTokenKind() { return VTCLParserprs.EOFT_SYMBOL; }
    public PrsStream getParseStream() { return this; }
    

/**
 * Called when encountering a single bad character. Do not report
 * this error at this point.
 */
public void reportError(int left_char, int right_char) {
//		System.out.println("reportError(int, int)");	    
}

/**
 * Reports the parse error by setting the error to <code>errorString</code>
 * 
 * @param errorCode the error code
 * @param locationInfo
 * @param leftToken the token to the left of the error
 * @param rightToken the token to the right of the error
 * @param tokenText the text of the bad token
 */
public void reportError(int errorCode, String locationInfo, int leftToken, int rightToken, String tokenText) {

    int leftTokenLoc = (leftToken > rightToken ? rightToken : leftToken);
    int rightTokenLoc = rightToken;
    String errorString;

    if (getLine(leftTokenLoc)  <= 0) {
 	    errorString =  VTCLMessages.InvalidVTCL_ERROR_;
        allErrorInfos.add(new ErrorInformation(errorString, PARSER_MARKER, ErrorKind.PARSE_ERROR)); 
    } 
    else {
      Location location = new Location(
    		getLine(leftTokenLoc),
    		getColumn(leftTokenLoc), 
		getEndLine(rightTokenLoc),
		getEndColumn(rightTokenLoc) + 1
    		);
	
	switch (errorCode) {
	case EOF_CODE:
	case MISPLACED_CODE:
        case DELETION_CODE:
            errorString = " Syntax error: " + errorMsgText[errorCode] + " " +    
    		      	'"' + computeInputString(
    				getIToken(leftToken).getStartOffset(),
    				getIToken(rightToken).getEndOffset()) + '"';
            break;
        case INVALID_TOKEN_CODE:
             errorString = " Syntax error: Token" +  " " +    
    		          '"' + computeInputString(
    				getIToken(leftToken).getStartOffset(),
    				getIToken(rightToken).getEndOffset()) + '"'
                          + " " + errorMsgText[errorCode] + ".";
             String explanation = generator.calculateErrorExplanation(tokenText, leftToken, rightToken);
             if (explanation != "") {
    	     errorString = errorString + " Cause: " + explanation + ".";
            }
        break;

	case MERGE_CODE:
	case BEFORE_CODE:
	case INSERTION_CODE:
	case SCOPE_CODE:
	case SUBSTITUTION_CODE: // includes SECONDARY_CODE
	    errorString = " Syntax error: " + tokenText + " " + errorMsgText[errorCode] + " " + 
		'"' + computeInputString(
			getIToken(leftToken).getStartOffset(),
			getIToken(rightToken).getEndOffset()) + '"';
	    break;

	default:
	    errorString = " Syntax error: " + errorMsgText[errorCode] + " at token " + tokenText;
	break;
	}
	allErrorInfos.add(new 
		ErrorInformation(errorString,
			PARSER_MARKER,
			ErrorKind.PARSE_ERROR,
			location,
			ErrorSeverity.ERROR));
 }
    
}
 

/**
 * Returns a single line string representation of the input chars
 * for the given range.
 * 
 * @param left left most char index
 * @param right right most char index
 * @return a single line string representation of the input chars
 */
private String computeInputString(int left, int right) {
	StringBuffer result = new StringBuffer(right - left + 1);
	char[] chars = getInputChars();

	for (int i = left; i <= right; i++) {
		if (chars[i] == '\t') {
			result.append(' ');
		} else if (chars[i] == '\n' || chars[i] == '\r' || chars[i] == '\f') {
			if (i > 0) {
				if (!Character.isWhitespace(chars[i-1])) {
					result.append(' ');
				}
			}
		} else {
			result.append(chars[i]);
		}
			
	}
	return result.toString();
}


    
/**
 * Report error message for given error_token by calling 
 * method <code>reportError/5</code> is called finally
 * @param error_token 
 * @param msg 
 */
public final void reportErrorTokenMessage(int error_token, String msg)
{
    allErrorInfos.add(new 
	ErrorInformation(msg, 
			PARSER_MARKER,
		    ErrorKind.PARSE_ERROR,
			getLine(error_token),  
			getColumn(error_token),
			getEndLine(error_token),
			getEndColumn(error_token)));
}

/** 
 * This SymbolTable class is from the IMP project
 */
public static class SymbolTable {
    private Map<String,ASTNode> table = new HashMap<String,ASTNode>();

    public void addDefinition(String name, ASTNode def) { table.put(name, def); }
    public ASTNode lookup(String name) { return table.get(name); }
    public List<String> allSymbolsOfType(Class type) {
        List<String> result= new ArrayList<String>();

        for(String sym: table.keySet()) {
            ASTNode def= table.get(sym);

            if (type.isInstance(def))
                result.add(sym);
        }
        return result;
    }
    public <T> List<T> allDefsOfType(Class<T> type) {
        List<T> result = new ArrayList<T>();
        
        for(String sym: table.keySet()) {
            ASTNode def= table.get(sym);

            if (type.isInstance(def))
                result.add((T) def);
        }
        return result;
    }
    public Set<String> allSymbols() { return table.keySet(); }
}
//    protected static SymbolTable symtab;
//    { symtab = new SymbolTable(); }

    public ASTNode parser()
    {
        return parser(null, 0);
    }
    
    public ASTNode parser(Monitor monitor)
    {
        return parser(monitor, 0);
    }
    
    public ASTNode parser(int error_repair_count)
    {
        return parser(null, error_repair_count);
    }

    public ASTNode parser(Monitor monitor, int error_repair_count)
    {
        try
        {
            btParser = new BacktrackingParser(monitor, this, prs, this);
        }
        catch (NotBacktrackParseTableException e)
        {
            throw new Error(new NotBacktrackParseTableException
                                ("Regenerate VTCLParserprs.java with -BACKTRACK option"));
        }
        catch (BadParseSymFileException e)
        {
            throw new Error(new BadParseSymFileException("Bad Parser Symbol File -- VTCLParsersym.java"));
        }

        try
        {
            return (ASTNode) btParser.parse(error_repair_count);
        }
        catch (BadParseException e)
        {
            reset(e.error_token); // point to error token
            DiagnoseParser diagnoseParser = new DiagnoseParser(this, prs);
            diagnoseParser.diagnose(e.error_token);
        }

        return null;
    }


    public void ruleAction(int ruleNumber)
    {
        switch (ruleNumber)
        {
 
            //
            // Rule 1:  VTCLFile ::= NamespaceDefAST NamespaceImportsAST GTASMDefAST
            //
            case 1: {
                setResult(
                    new VTCLFile(getLeftIToken(), getRightIToken(),
                                 (NamespaceDefAST)getRhsSym(1),
                                 (NamespaceImportsAST)getRhsSym(2),
                                 (GTASMDefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 2:  NamespaceDefAST ::= $Empty
            //
            case 2: {
                setResult(null);
                break;
            } 
            //
            // Rule 3:  NamespaceDefAST ::= namespace QualifiedTypeNameAST SEMICOLON
            //
            case 3: {
                setResult(
                    new NamespaceDefAST(getLeftIToken(), getRightIToken(),
                                        (IQualifiedTypeNameAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 4:  NamespaceImportsAST ::= $Empty
            //
            case 4: {
                setResult(null);
                break;
            } 
            //
            // Rule 5:  NamespaceImportsAST ::= NamespaceImportsAST NamespaceImportAST
            //
            case 5: {
                setResult(
                    new NamespaceImportsAST(getLeftIToken(), getRightIToken(),
                                            (NamespaceImportsAST)getRhsSym(1),
                                            (INamespaceImportAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 6:  NamespaceImportAST ::= import QualifiedTypeNameAST SEMICOLON
            //
            case 6: {
                setResult(
                    new NamespaceImportAST0(getLeftIToken(), getRightIToken(),
                                            (IQualifiedTypeNameAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 7:  NamespaceImportAST ::= ERROR_TOKEN
            //
            case 7: {
                setResult(
                    new NamespaceImportAST1(VTCLParser.this, getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 8:  GTASMDefAST ::= OptAnnotationsAST machine TypeNameAST LBRACE MachineContentsAST RBRACE
            //
            case 8: {
                setResult(
                    new GTASMDefAST(getLeftIToken(), getRightIToken(),
                                    (IOptAnnotationsAST)getRhsSym(1),
                                    (ITypeNameAST)getRhsSym(3),
                                    (IMachineContentsAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 9:  MachineContentsAST ::= MachineContentAST
            //
            case 9:
                break; 
            //
            // Rule 10:  MachineContentsAST ::= MachineContentsAST MachineContentAST
            //
            case 10: {
                setResult(
                    new MachineContentsAST(getLeftIToken(), getRightIToken(),
                                           (IMachineContentsAST)getRhsSym(1),
                                           (IMachineContentAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 11:  MachineContentAST ::= GraphPatternDefAST
            //
            case 11:
                break; 
            //
            // Rule 12:  MachineContentAST ::= GTRuleDefAST
            //
            case 12:
                break; 
            //
            // Rule 13:  MachineContentAST ::= AsmFunctionDefAST
            //
            case 13:
                break; 
            //
            // Rule 14:  MachineContentAST ::= AsmRuleDefAST
            //
            case 14:
                break; 
            //
            // Rule 15:  MachineContentAST ::= ERROR_TOKEN
            //
            case 15: {
                setResult(
                    new MachineContentAST(VTCLParser.this, getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 16:  GraphPatternDefAST ::= OptAnnotationsAST OptShareableDefAST pattern TypeNameAST FormalParameterDefAST EQUAL PatternBodiesAST
            //
            case 16: {
                setResult(
                    new GraphPatternDefAST(getLeftIToken(), getRightIToken(),
                                           (IOptAnnotationsAST)getRhsSym(1),
                                           (OptShareableDefAST)getRhsSym(2),
                                           (ITypeNameAST)getRhsSym(4),
                                           (IFormalParameterDefAST)getRhsSym(5),
                                           (IPatternBodiesAST)getRhsSym(7))
                );
                break;
            } 
            //
            // Rule 17:  OptShareableDefAST ::= $Empty
            //
            case 17: {
                setResult(null);
                break;
            } 
            //
            // Rule 18:  OptShareableDefAST ::= shareable
            //
            case 18: {
                setResult(
                    new OptShareableDefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 19:  PatternBodiesAST ::= PatternBodyAST
            //
            case 19:
                break; 
            //
            // Rule 20:  PatternBodiesAST ::= PatternBodiesAST or PatternBodyAST
            //
            case 20: {
                setResult(
                    new PatternBodiesAST(getLeftIToken(), getRightIToken(),
                                         (IPatternBodiesAST)getRhsSym(1),
                                         (PatternBodyAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 21:  PatternBodyAST ::= LBRACE PatternBodyContentsAST RBRACE
            //
            case 21: {
                setResult(
                    new PatternBodyAST(getLeftIToken(), getRightIToken(),
                                       (PatternBodyContentsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 22:  PatternBodyContentsAST ::= $Empty
            //
            case 22: {
                setResult(null);
                break;
            } 
            //
            // Rule 23:  PatternBodyContentsAST ::= PatternBodyContentsAST PatternBodyContentDefAST
            //
            case 23: {
                setResult(
                    new PatternBodyContentsAST(getLeftIToken(), getRightIToken(),
                                               (PatternBodyContentsAST)getRhsSym(1),
                                               (IPatternBodyContentDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 24:  PatternBodyContentDefAST ::= PatternBodyContentAST SEMICOLON
            //
            case 24: {
                setResult(
                    new PatternBodyContentDefAST0(getLeftIToken(), getRightIToken(),
                                                  (IPatternBodyContentAST)getRhsSym(1))
                );
                break;
            } 
            //
            // Rule 25:  PatternBodyContentDefAST ::= neg GraphPatternDefAST
            //
            case 25: {
                setResult(
                    new PatternBodyContentDefAST1(getLeftIToken(), getRightIToken(),
                                                  (GraphPatternDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 26:  PatternBodyContentAST ::= LocalPatternBodyElementAST
            //
            case 26:
                break; 
            //
            // Rule 27:  PatternBodyContentAST ::= PatternCompositionAST
            //
            case 27:
                break; 
            //
            // Rule 28:  PatternBodyContentAST ::= NegativePatternAST
            //
            case 28:
                break; 
            //
            // Rule 29:  PatternBodyContentAST ::= PatternVariableAssignmentAST
            //
            case 29:
                break; 
            //
            // Rule 30:  PatternBodyContentAST ::= CheckConditionAST
            //
            case 30:
                break; 
            //
            // Rule 31:  PatternBodyContentAST ::= NonInjectivityConstraintAST
            //
            case 31:
                break; 
            //
            // Rule 32:  PatternBodyContentAST ::= ERROR_TOKEN
            //
            case 32: {
                setResult(
                    new PatternBodyContentAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 33:  LocalPatternBodyElementAST ::= ModelElementAST
            //
            case 33:
                break; 
            //
            // Rule 34:  LocalPatternBodyElementAST ::= RelationshipAST
            //
            case 34:
                break; 
            //
            // Rule 35:  NegativePatternAST ::= neg PatternCompositionAST
            //
            case 35: {
                setResult(
                    new NegativePatternAST(getLeftIToken(), getRightIToken(),
                                           (PatternCompositionAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 36:  PatternCompositionAST ::= find QualifiedTypeNameAST ActualPatternParameterDefAST OptMatchCountAST
            //
            case 36: {
                setResult(
                    new PatternCompositionAST(getLeftIToken(), getRightIToken(),
                                              (IQualifiedTypeNameAST)getRhsSym(2),
                                              (IActualPatternParameterDefAST)getRhsSym(3),
                                              (OptMatchCountAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 37:  PatternVariableAssignmentAST ::= PatternVariableRefAST EQUAL PatternVariableRefAST
            //
            case 37: {
                setResult(
                    new PatternVariableAssignmentAST(getLeftIToken(), getRightIToken(),
                                                     (PatternVariableRefAST)getRhsSym(1),
                                                     (PatternVariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 38:  NonInjectivityConstraintAST ::= PatternVariableRefAST NONINJECTIVE PatternVariableRefAST
            //
            case 38: {
                setResult(
                    new NonInjectivityConstraintAST(getLeftIToken(), getRightIToken(),
                                                    (PatternVariableRefAST)getRhsSym(1),
                                                    (PatternVariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 39:  CheckConditionAST ::= check LPAREN LogicalTermAST RPAREN
            //
            case 39: {
                setResult(
                    new CheckConditionAST(getLeftIToken(), getRightIToken(),
                                          (ILogicalTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 40:  OptMatchCountAST ::= $Empty
            //
            case 40: {
                setResult(null);
                break;
            } 
            //
            // Rule 41:  OptMatchCountAST ::= HASHMARK CounterVariableAST
            //
            case 41: {
                setResult(
                    new OptMatchCountAST(getLeftIToken(), getRightIToken(),
                                         (CounterVariableAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 42:  PatternVariableDefAST ::= UpperCaseLiteral
            //
            case 42: {
                setResult(
                    new PatternVariableDefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 43:  PatternVariableRefAST ::= UpperCaseLiteral
            //
            case 43: {
                setResult(
                    new PatternVariableRefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 44:  PatternVariableDefRefAST ::= UpperCaseLiteral
            //
            case 44: {
                setResult(
                    new PatternVariableDefRefAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 45:  PatternVariableDefRefAST ::= DummyVariableLiteral
            //
            case 45: {
                setResult(
                    new PatternVariableDefRefAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 46:  CounterVariableAST ::= UpperCaseLiteral
            //
            case 46: {
                setResult(
                    new CounterVariableAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 47:  GraphPatternCallAST ::= find QualifiedTypeNameAST ActualParameterDefAST OptMatchCountAST
            //
            case 47: {
                setResult(
                    new GraphPatternCallAST(getLeftIToken(), getRightIToken(),
                                            (IQualifiedTypeNameAST)getRhsSym(2),
                                            (IActualParameterDefAST)getRhsSym(3),
                                            (OptMatchCountAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 48:  ModelElementAST ::= EntityAST
            //
            case 48:
                break; 
            //
            // Rule 49:  ModelElementAST ::= RelationAST
            //
            case 49:
                break; 
            //
            // Rule 50:  EntityAST ::= QualifiedTypeNameAST EntityBody ContainmentConstraintOptAST
            //
            case 50: {
                setResult(
                    new EntityAST(getLeftIToken(), getRightIToken(),
                                  (IQualifiedTypeNameAST)getRhsSym(1),
                                  (EntityBody)getRhsSym(2),
                                  (IContainmentConstraintOptAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 51:  EntityBody ::= LPAREN PatternVariableDefRefAST RPAREN
            //
            case 51: {
                setResult(
                    new EntityBody(getLeftIToken(), getRightIToken(),
                                   (IPatternVariableDefRefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 52:  RelationAST ::= QualifiedTypeNameAST RelationBodyAST
            //
            case 52: {
                setResult(
                    new RelationAST(getLeftIToken(), getRightIToken(),
                                    (IQualifiedTypeNameAST)getRhsSym(1),
                                    (RelationBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 53:  RelationBodyAST ::= LPAREN PatternVariableDefRefAST COMMA PatternVariableRefAST COMMA PatternVariableRefAST RPAREN
            //
            case 53: {
                setResult(
                    new RelationBodyAST(getLeftIToken(), getRightIToken(),
                                        (IPatternVariableDefRefAST)getRhsSym(2),
                                        (PatternVariableRefAST)getRhsSym(4),
                                        (PatternVariableRefAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 54:  RelationshipAST ::= InheritanceAST
            //
            case 54:
                break; 
            //
            // Rule 55:  RelationshipAST ::= InstantiationAST
            //
            case 55:
                break; 
            //
            // Rule 56:  InheritanceAST ::= supertypeOf RelationshipBodyAST
            //
            case 56: {
                setResult(
                    new InheritanceAST0(getLeftIToken(), getRightIToken(),
                                        (RelationshipBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 57:  InheritanceAST ::= subtypeOf RelationshipBodyAST
            //
            case 57: {
                setResult(
                    new InheritanceAST1(getLeftIToken(), getRightIToken(),
                                        (RelationshipBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 58:  InstantiationAST ::= typeOf RelationshipBodyAST
            //
            case 58: {
                setResult(
                    new InstantiationAST0(getLeftIToken(), getRightIToken(),
                                          (RelationshipBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 59:  InstantiationAST ::= instanceOf RelationshipBodyAST
            //
            case 59: {
                setResult(
                    new InstantiationAST1(getLeftIToken(), getRightIToken(),
                                          (RelationshipBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 60:  RelationshipBodyAST ::= LPAREN PatternVariableRefAST COMMA PatternVariableRefAST RPAREN
            //
            case 60: {
                setResult(
                    new RelationshipBodyAST(getLeftIToken(), getRightIToken(),
                                            (PatternVariableRefAST)getRhsSym(2),
                                            (PatternVariableRefAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 61:  GTRuleDefAST ::= OptAnnotationsAST gtrule TypeNameAST DirectedFormalParameterDefAST EQUAL GTRuleBodyAST
            //
            case 61: {
                setResult(
                    new GTRuleDefAST(getLeftIToken(), getRightIToken(),
                                     (IOptAnnotationsAST)getRhsSym(1),
                                     (ITypeNameAST)getRhsSym(3),
                                     (IDirectedFormalParameterDefAST)getRhsSym(4),
                                     (IGTRuleBodyAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 62:  GTRuleBodyAST ::= LBRACE PreconditionDefAST PostconditionOptAST ActionOptAST RBRACE
            //
            case 62: {
                setResult(
                    new GTRuleBodyAST0(getLeftIToken(), getRightIToken(),
                                       (IPreconditionDefAST)getRhsSym(2),
                                       (IPostconditionOptAST)getRhsSym(3),
                                       (ActionOptAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 63:  GTRuleBodyAST ::= ERROR_TOKEN
            //
            case 63: {
                setResult(
                    new GTRuleBodyAST1(VTCLParser.this, getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 64:  PreconditionDefAST ::= precondition GraphPatternDefAST
            //
            case 64: {
                setResult(
                    new PreconditionDefAST0(getLeftIToken(), getRightIToken(),
                                            (GraphPatternDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 65:  PreconditionDefAST ::= precondition PatternCompositionAST
            //
            case 65: {
                setResult(
                    new PreconditionDefAST1(getLeftIToken(), getRightIToken(),
                                            (PatternCompositionAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 66:  PostconditionOptAST ::= $Empty
            //
            case 66: {
                setResult(null);
                break;
            } 
            //
            // Rule 67:  PostconditionOptAST ::= postcondition GraphPatternDefAST
            //
            case 67: {
                setResult(
                    new PostconditionOptAST0(getLeftIToken(), getRightIToken(),
                                             (GraphPatternDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 68:  PostconditionOptAST ::= postcondition PatternCompositionAST
            //
            case 68: {
                setResult(
                    new PostconditionOptAST1(getLeftIToken(), getRightIToken(),
                                             (PatternCompositionAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 69:  ActionOptAST ::= $Empty
            //
            case 69: {
                setResult(null);
                break;
            } 
            //
            // Rule 70:  ActionOptAST ::= action LBRACE AsmRulesAST RBRACE
            //
            case 70: {
                setResult(
                    new ActionOptAST(getLeftIToken(), getRightIToken(),
                                     (AsmRulesAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 71:  GTRuleCallAST ::= apply QualifiedTypeNameAST ActualParameterDefAST
            //
            case 71: {
                setResult(
                    new GTRuleCallAST(getLeftIToken(), getRightIToken(),
                                      (IQualifiedTypeNameAST)getRhsSym(2),
                                      (IActualParameterDefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 72:  LogicalTermAST ::= LogicalAndTermAST
            //
            case 72:
                break; 
            //
            // Rule 73:  LogicalTermAST ::= LogicalTermAST OR_OR LogicalAndTermAST
            //
            case 73: {
                setResult(
                    new LogicalTermAST0(getLeftIToken(), getRightIToken(),
                                        (ILogicalTermAST)getRhsSym(1),
                                        (ILogicalAndTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 74:  LogicalTermAST ::= LogicalTermAST xor LogicalAndTermAST
            //
            case 74: {
                setResult(
                    new LogicalTermAST1(getLeftIToken(), getRightIToken(),
                                        (ILogicalTermAST)getRhsSym(1),
                                        (ILogicalAndTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 75:  LogicalAndTermAST ::= EqualityTermAST
            //
            case 75:
                break; 
            //
            // Rule 76:  LogicalAndTermAST ::= LogicalAndTermAST AND_AND EqualityTermAST
            //
            case 76: {
                setResult(
                    new LogicalAndTermAST(getLeftIToken(), getRightIToken(),
                                          (ILogicalAndTermAST)getRhsSym(1),
                                          (IEqualityTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 77:  EqualityTermAST ::= RelationalTermAST
            //
            case 77:
                break; 
            //
            // Rule 78:  EqualityTermAST ::= EqualityTermAST EQUAL_EQUAL RelationalTermAST
            //
            case 78: {
                setResult(
                    new EqualityTermAST0(getLeftIToken(), getRightIToken(),
                                         (IEqualityTermAST)getRhsSym(1),
                                         (IRelationalTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 79:  EqualityTermAST ::= EqualityTermAST NOT_EQUAL RelationalTermAST
            //
            case 79: {
                setResult(
                    new EqualityTermAST1(getLeftIToken(), getRightIToken(),
                                         (IEqualityTermAST)getRhsSym(1),
                                         (IRelationalTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 80:  RelationalTermAST ::= ArithmeticTermAST
            //
            case 80:
                break; 
            //
            // Rule 81:  RelationalTermAST ::= RelationalTermAST RelationalOpAST ArithmeticTermAST
            //
            case 81: {
                setResult(
                    new RelationalTermAST(getLeftIToken(), getRightIToken(),
                                          (IRelationalTermAST)getRhsSym(1),
                                          (RelationalOpAST)getRhsSym(2),
                                          (IArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 82:  RelationalOpAST ::= LESS
            //
            case 82: {
                setResult(
                    new RelationalOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 83:  RelationalOpAST ::= GREATER
            //
            case 83: {
                setResult(
                    new RelationalOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 84:  RelationalOpAST ::= LESS_EQUAL
            //
            case 84: {
                setResult(
                    new RelationalOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 85:  RelationalOpAST ::= GREATER_EQUAL
            //
            case 85: {
                setResult(
                    new RelationalOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 86:  ArithmeticTermAST ::= MultArithmeticTermAST
            //
            case 86:
                break; 
            //
            // Rule 87:  ArithmeticTermAST ::= ArithmeticTermAST PLUS MultArithmeticTermAST
            //
            case 87: {
                setResult(
                    new ArithmeticTermAST0(getLeftIToken(), getRightIToken(),
                                           (IArithmeticTermAST)getRhsSym(1),
                                           (IMultArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 88:  ArithmeticTermAST ::= ArithmeticTermAST MINUS MultArithmeticTermAST
            //
            case 88: {
                setResult(
                    new ArithmeticTermAST1(getLeftIToken(), getRightIToken(),
                                           (IArithmeticTermAST)getRhsSym(1),
                                           (IMultArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 89:  MultArithmeticTermAST ::= UnaryArithmeticTermAST
            //
            case 89:
                break; 
            //
            // Rule 90:  MultArithmeticTermAST ::= MultArithmeticTermAST MULTIPLY UnaryArithmeticTermAST
            //
            case 90: {
                setResult(
                    new MultArithmeticTermAST0(getLeftIToken(), getRightIToken(),
                                               (IMultArithmeticTermAST)getRhsSym(1),
                                               (IUnaryArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 91:  MultArithmeticTermAST ::= MultArithmeticTermAST DIVIDE UnaryArithmeticTermAST
            //
            case 91: {
                setResult(
                    new MultArithmeticTermAST1(getLeftIToken(), getRightIToken(),
                                               (IMultArithmeticTermAST)getRhsSym(1),
                                               (IUnaryArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 92:  MultArithmeticTermAST ::= MultArithmeticTermAST REMAINDER UnaryArithmeticTermAST
            //
            case 92: {
                setResult(
                    new MultArithmeticTermAST2(getLeftIToken(), getRightIToken(),
                                               (IMultArithmeticTermAST)getRhsSym(1),
                                               (IUnaryArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 93:  UnaryArithmeticTermAST ::= BaseArithmeticTermAST
            //
            case 93:
                break; 
            //
            // Rule 94:  UnaryArithmeticTermAST ::= MINUS BaseArithmeticTermAST
            //
            case 94: {
                setResult(
                    new UnaryArithmeticTermAST0(getLeftIToken(), getRightIToken(),
                                                (IBaseArithmeticTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 95:  UnaryArithmeticTermAST ::= NOT BaseArithmeticTermAST
            //
            case 95: {
                setResult(
                    new UnaryArithmeticTermAST1(getLeftIToken(), getRightIToken(),
                                                (IBaseArithmeticTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 96:  BaseArithmeticTermAST ::= VariableRefAST
            //
            case 96:
                break; 
            //
            // Rule 97:  BaseArithmeticTermAST ::= ValueAST
            //
            case 97:
                break; 
            //
            // Rule 98:  BaseArithmeticTermAST ::= LPAREN LogicalTermAST RPAREN
            //
            case 98: {
                setResult(
                    new BaseArithmeticTermAST(getLeftIToken(), getRightIToken(),
                                              (ILogicalTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 99:  BaseArithmeticTermAST ::= AsmFunctionLocationAST
            //
            case 99:
                break; 
            //
            // Rule 100:  BaseArithmeticTermAST ::= GraphPatternCallAST
            //
            case 100:
                break; 
            //
            // Rule 101:  BaseArithmeticTermAST ::= PredefinedFunctionCallAST
            //
            case 101:
                break; 
            //
            // Rule 102:  AsmFunctionDefAST ::= OptAnnotationsAST asmfunction TypeNameAST ArityOrTypeDeclAST ReturnTypeOptAST InitialValuesOptAST
            //
            case 102: {
                setResult(
                    new AsmFunctionDefAST(getLeftIToken(), getRightIToken(),
                                          (IOptAnnotationsAST)getRhsSym(1),
                                          (ITypeNameAST)getRhsSym(3),
                                          (IArityOrTypeDeclAST)getRhsSym(4),
                                          (ReturnTypeOptAST)getRhsSym(5),
                                          (IInitialValuesOptAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 103:  ArityOrTypeDeclAST ::= DIVIDE ArityAST
            //
            case 103: {
                setResult(
                    new ArityOrTypeDeclAST0(getLeftIToken(), getRightIToken(),
                                            (ArityAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 104:  ArityOrTypeDeclAST ::= LPAREN TypeConstantsAST
            //
            case 104: {
                setResult(
                    new ArityOrTypeDeclAST1(getLeftIToken(), getRightIToken(),
                                            (ITypeConstantsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 105:  ArityAST ::= DecimalIntegerLiteral
            //
            case 105: {
                setResult(
                    new ArityAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 106:  InitialValuesOptAST ::= SEMICOLON
            //
            case 106: {
                setResult(
                    new InitialValuesOptAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 107:  InitialValuesOptAST ::= LBRACE InitialValuesAST RBRACE
            //
            case 107: {
                setResult(
                    new InitialValuesOptAST1(getLeftIToken(), getRightIToken(),
                                             (IInitialValuesAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 108:  InitialValuesAST ::= InitialValueAST
            //
            case 108:
                break; 
            //
            // Rule 109:  InitialValuesAST ::= InitialValuesAST InitialValueAST
            //
            case 109: {
                setResult(
                    new InitialValuesAST(getLeftIToken(), getRightIToken(),
                                         (IInitialValuesAST)getRhsSym(1),
                                         (InitialValueAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 110:  InitialValueAST ::= ActualParameterDefAST EQUAL ArithmeticTermAST SEMICOLON
            //
            case 110: {
                setResult(
                    new InitialValueAST(getLeftIToken(), getRightIToken(),
                                        (IActualParameterDefAST)getRhsSym(1),
                                        (IArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 111:  TypeConstantsAST ::= QualifiedTypeNameAST RPAREN
            //
            case 111: {
                setResult(
                    new TypeConstantsAST0(getLeftIToken(), getRightIToken(),
                                          (IQualifiedTypeNameAST)getRhsSym(1))
                );
                break;
            } 
            //
            // Rule 112:  TypeConstantsAST ::= QualifiedTypeNameAST COMMA TypeConstantsAST
            //
            case 112: {
                setResult(
                    new TypeConstantsAST1(getLeftIToken(), getRightIToken(),
                                          (IQualifiedTypeNameAST)getRhsSym(1),
                                          (ITypeConstantsAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 113:  AsmFunctionLocationAST ::= ASMFunctionNameAST ActualParameterDefAST
            //
            case 113: {
                setResult(
                    new AsmFunctionLocationAST(getLeftIToken(), getRightIToken(),
                                               (IASMFunctionNameAST)getRhsSym(1),
                                               (IActualParameterDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 114:  PredefinedFunctionCallAST ::= BuiltInFunctionNameAST LPAREN ArithmeticTermAST RPAREN
            //
            case 114: {
                setResult(
                    new PredefinedFunctionCallAST0(getLeftIToken(), getRightIToken(),
                                                   (BuiltInFunctionNameAST)getRhsSym(1),
                                                   (IArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 115:  PredefinedFunctionCallAST ::= ConversionFunctionNameAST LPAREN ArithmeticTermAST RPAREN
            //
            case 115: {
                setResult(
                    new PredefinedFunctionCallAST1(getLeftIToken(), getRightIToken(),
                                                   (ConversionFunctionNameAST)getRhsSym(1),
                                                   (IArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 116:  BuiltInFunctionNameAST ::= ref
            //
            case 116: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 117:  BuiltInFunctionNameAST ::= fqn
            //
            case 117: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 118:  BuiltInFunctionNameAST ::= name
            //
            case 118: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 119:  BuiltInFunctionNameAST ::= value
            //
            case 119: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 120:  BuiltInFunctionNameAST ::= source
            //
            case 120: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 121:  BuiltInFunctionNameAST ::= target
            //
            case 121: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 122:  BuiltInFunctionNameAST ::= multiplicity
            //
            case 122: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 123:  BuiltInFunctionNameAST ::= aggregation
            //
            case 123: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 124:  BuiltInFunctionNameAST ::= inverse
            //
            case 124: {
                setResult(
                    new BuiltInFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 125:  ConversionFunctionNameAST ::= toBoolean
            //
            case 125: {
                setResult(
                    new ConversionFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 126:  ConversionFunctionNameAST ::= toString
            //
            case 126: {
                setResult(
                    new ConversionFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 127:  ConversionFunctionNameAST ::= toInteger
            //
            case 127: {
                setResult(
                    new ConversionFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 128:  ConversionFunctionNameAST ::= toDouble
            //
            case 128: {
                setResult(
                    new ConversionFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 129:  ConversionFunctionNameAST ::= toMultiplicity
            //
            case 129: {
                setResult(
                    new ConversionFunctionNameAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 130:  AsmRuleDefAST ::= OptAnnotationsAST rule TypeNameAST DirectedFormalParameterDefAST EQUAL AsmRuleAST
            //
            case 130: {
                setResult(
                    new AsmRuleDefAST(getLeftIToken(), getRightIToken(),
                                      (IOptAnnotationsAST)getRhsSym(1),
                                      (ITypeNameAST)getRhsSym(3),
                                      (IDirectedFormalParameterDefAST)getRhsSym(4),
                                      (IAsmRuleAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 131:  AsmRulesAST ::= AsmRulesOptAST AsmRuleAST
            //
            case 131: {
                setResult(
                    new AsmRulesAST(getLeftIToken(), getRightIToken(),
                                    (AsmRulesOptAST)getRhsSym(1),
                                    (IAsmRuleAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 132:  AsmRulesOptAST ::= $Empty
            //
            case 132: {
                setResult(null);
                break;
            } 
            //
            // Rule 133:  AsmRulesOptAST ::= AsmRulesOptAST AsmRuleAST
            //
            case 133: {
                setResult(
                    new AsmRulesOptAST(getLeftIToken(), getRightIToken(),
                                       (AsmRulesOptAST)getRhsSym(1),
                                       (IAsmRuleAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 134:  AsmRuleAST ::= RegularAsmRuleAST
            //
            case 134:
                break; 
            //
            // Rule 135:  AsmRuleAST ::= IfRuleAST
            //
            case 135:
                break; 
            //
            // Rule 136:  AsmRuleAST ::= TryRuleAST
            //
            case 136:
                break; 
            //
            // Rule 137:  AsmRuleAST ::= ERROR_TOKEN
            //
            case 137: {
                setResult(
                    new AsmRuleAST(VTCLParser.this, getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 138:  RegularAsmRuleAST ::= SimpleRuleAST
            //
            case 138:
                break; 
            //
            // Rule 139:  RegularAsmRuleAST ::= CompoundRuleAST
            //
            case 139:
                break; 
            //
            // Rule 140:  RegularAsmRuleAST ::= ModelManipulationRuleAST
            //
            case 140:
                break; 
            //
            // Rule 141:  SimpleRuleAST ::= SkipRuleAST
            //
            case 141:
                break; 
            //
            // Rule 142:  SimpleRuleAST ::= FailRuleAST
            //
            case 142:
                break; 
            //
            // Rule 143:  SimpleRuleAST ::= CallRuleAST
            //
            case 143:
                break; 
            //
            // Rule 144:  SimpleRuleAST ::= UpdateRuleAST
            //
            case 144:
                break; 
            //
            // Rule 145:  SimpleRuleAST ::= PrintRuleAST
            //
            case 145:
                break; 
            //
            // Rule 146:  SimpleRuleAST ::= LogRuleAST
            //
            case 146:
                break; 
            //
            // Rule 147:  SkipRuleAST ::= skip SEMICOLON
            //
            case 147: {
                setResult(
                    new SkipRuleAST(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 148:  FailRuleAST ::= fail SEMICOLON
            //
            case 148: {
                setResult(
                    new FailRuleAST(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 149:  CallRuleAST ::= call QualifiedTypeNameAST ActualParameterDefAST SEMICOLON
            //
            case 149: {
                setResult(
                    new CallRuleAST(getLeftIToken(), getRightIToken(),
                                    (IQualifiedTypeNameAST)getRhsSym(2),
                                    (IActualParameterDefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 150:  UpdateRuleAST ::= update VariableRefAST EQUAL ArithmeticTermAST SEMICOLON
            //
            case 150: {
                setResult(
                    new UpdateRuleAST0(getLeftIToken(), getRightIToken(),
                                       (VariableRefAST)getRhsSym(2),
                                       (IArithmeticTermAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 151:  UpdateRuleAST ::= update AsmFunctionUpdateLocationAST EQUAL ArithmeticTermAST SEMICOLON
            //
            case 151: {
                setResult(
                    new UpdateRuleAST1(getLeftIToken(), getRightIToken(),
                                       (AsmFunctionUpdateLocationAST)getRhsSym(2),
                                       (IArithmeticTermAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 152:  AsmFunctionUpdateLocationAST ::= ASMFunctionNameAST ActualParameterDefAST
            //
            case 152: {
                setResult(
                    new AsmFunctionUpdateLocationAST(getLeftIToken(), getRightIToken(),
                                                     (IASMFunctionNameAST)getRhsSym(1),
                                                     (IActualParameterDefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 153:  PrintRuleAST ::= print LPAREN LogicalTermAST RPAREN SEMICOLON
            //
            case 153: {
                setResult(
                    new PrintRuleAST0(getLeftIToken(), getRightIToken(),
                                      (ILogicalTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 154:  PrintRuleAST ::= print LPAREN ArithmeticTermAST COMMA LogicalTermAST RPAREN SEMICOLON
            //
            case 154: {
                setResult(
                    new PrintRuleAST1(getLeftIToken(), getRightIToken(),
                                      (IArithmeticTermAST)getRhsSym(3),
                                      (ILogicalTermAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 155:  PrintRuleAST ::= println LPAREN LogicalTermAST RPAREN SEMICOLON
            //
            case 155: {
                setResult(
                    new PrintRuleAST2(getLeftIToken(), getRightIToken(),
                                      (ILogicalTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 156:  PrintRuleAST ::= println LPAREN ArithmeticTermAST COMMA LogicalTermAST RPAREN SEMICOLON
            //
            case 156: {
                setResult(
                    new PrintRuleAST3(getLeftIToken(), getRightIToken(),
                                      (IArithmeticTermAST)getRhsSym(3),
                                      (ILogicalTermAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 157:  LogRuleAST ::= log LPAREN LogLevelAST COMMA ArithmeticTermAST RPAREN SEMICOLON
            //
            case 157: {
                setResult(
                    new LogRuleAST(getLeftIToken(), getRightIToken(),
                                   (LogLevelAST)getRhsSym(3),
                                   (IArithmeticTermAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 158:  LogLevelAST ::= debug
            //
            case 158: {
                setResult(
                    new LogLevelAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 159:  LogLevelAST ::= info
            //
            case 159: {
                setResult(
                    new LogLevelAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 160:  LogLevelAST ::= warning
            //
            case 160: {
                setResult(
                    new LogLevelAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 161:  LogLevelAST ::= error
            //
            case 161: {
                setResult(
                    new LogLevelAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 162:  LogLevelAST ::= fatal
            //
            case 162: {
                setResult(
                    new LogLevelAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 163:  CompoundRuleAST ::= SequentialRuleAST
            //
            case 163:
                break; 
            //
            // Rule 164:  CompoundRuleAST ::= RandomRuleAST
            //
            case 164:
                break; 
            //
            // Rule 165:  CompoundRuleAST ::= LetRuleAST
            //
            case 165:
                break; 
            //
            // Rule 166:  CompoundRuleAST ::= IterateRuleAST
            //
            case 166:
                break; 
            //
            // Rule 167:  CompoundRuleAST ::= ChooseRuleAST
            //
            case 167:
                break; 
            //
            // Rule 168:  CompoundRuleAST ::= ForallRuleAST
            //
            case 168:
                break; 
            //
            // Rule 169:  CompoundRuleAST ::= ParallelRuleAST
            //
            case 169:
                break; 
            //
            // Rule 170:  CompoundRuleAST ::= WhenRuleAST
            //
            case 170:
                break; 
            //
            // Rule 171:  SequentialRuleAST ::= seq LBRACE AsmRulesAST RBRACE OptSemicolonAST
            //
            case 171: {
                setResult(
                    new SequentialRuleAST(getLeftIToken(), getRightIToken(),
                                          (AsmRulesAST)getRhsSym(3),
                                          (OptSemicolonAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 172:  ParallelRuleAST ::= parallel LBRACE AsmRulesAST RBRACE OptSemicolonAST
            //
            case 172: {
                setResult(
                    new ParallelRuleAST(getLeftIToken(), getRightIToken(),
                                        (AsmRulesAST)getRhsSym(3),
                                        (OptSemicolonAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 173:  RandomRuleAST ::= random LBRACE AsmRulesAST RBRACE OptSemicolonAST
            //
            case 173: {
                setResult(
                    new RandomRuleAST(getLeftIToken(), getRightIToken(),
                                      (AsmRulesAST)getRhsSym(3),
                                      (OptSemicolonAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 174:  OptSemicolonAST ::= $Empty
            //
            case 174: {
                setResult(null);
                break;
            } 
            //
            // Rule 175:  OptSemicolonAST ::= SEMICOLON
            //
            case 175: {
                setResult(
                    new OptSemicolonAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 176:  LetRuleAST ::= let VariableDefinitionsAST in AsmRuleAST
            //
            case 176: {
                setResult(
                    new LetRuleAST(getLeftIToken(), getRightIToken(),
                                   (IVariableDefinitionsAST)getRhsSym(2),
                                   (IAsmRuleAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 177:  VariableDefinitionsAST ::= VariableDefinitionAST
            //
            case 177:
                break; 
            //
            // Rule 178:  VariableDefinitionsAST ::= VariableDefinitionsAST COMMA VariableDefinitionAST
            //
            case 178: {
                setResult(
                    new VariableDefinitionsAST(getLeftIToken(), getRightIToken(),
                                               (IVariableDefinitionsAST)getRhsSym(1),
                                               (VariableDefinitionAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 179:  VariableDefinitionAST ::= VariableDefAST TypeOptAST EQUAL ArithmeticTermAST
            //
            case 179: {
                setResult(
                    new VariableDefinitionAST(getLeftIToken(), getRightIToken(),
                                              (VariableDefAST)getRhsSym(1),
                                              (TypeOptAST)getRhsSym(2),
                                              (IArithmeticTermAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 180:  IterateRuleAST ::= iterate AsmRuleAST
            //
            case 180: {
                setResult(
                    new IterateRuleAST(getLeftIToken(), getRightIToken(),
                                       (IAsmRuleAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 181:  ChooseRuleAST ::= choose ConstrainedVariablesOptAST with ConditionAST DoActionOptAST
            //
            case 181: {
                setResult(
                    new ChooseRuleAST(getLeftIToken(), getRightIToken(),
                                      (IConstrainedVariablesOptAST)getRhsSym(2),
                                      (IConditionAST)getRhsSym(4),
                                      (IDoActionOptAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 182:  ForallRuleAST ::= forall ConstrainedVariablesOptAST with ConditionAST DoActionOptAST
            //
            case 182: {
                setResult(
                    new ForallRuleAST(getLeftIToken(), getRightIToken(),
                                      (IConstrainedVariablesOptAST)getRhsSym(2),
                                      (IConditionAST)getRhsSym(4),
                                      (IDoActionOptAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 183:  ConditionAST ::= LogicalTermAST
            //
            case 183:
                break; 
            //
            // Rule 184:  ConditionAST ::= GTRuleCallAST
            //
            case 184:
                break; 
            //
            // Rule 185:  DoActionOptAST ::= SEMICOLON
            //
            case 185: {
                setResult(
                    new DoActionOptAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 186:  DoActionOptAST ::= do AsmRuleAST
            //
            case 186: {
                setResult(
                    new DoActionOptAST1(getLeftIToken(), getRightIToken(),
                                        (IAsmRuleAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 187:  IfRuleAST ::= if LPAREN LogicalTermAST RPAREN AsmRuleAST
            //
            case 187: {
                setResult(
                    new IfRuleAST0(getLeftIToken(), getRightIToken(),
                                   (ILogicalTermAST)getRhsSym(3),
                                   (IAsmRuleAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 188:  IfRuleAST ::= if LPAREN LogicalTermAST RPAREN AsmRuleAST else AsmRuleAST
            //
            case 188: {
                setResult(
                    new IfRuleAST1(getLeftIToken(), getRightIToken(),
                                   (ILogicalTermAST)getRhsSym(3),
                                   (IAsmRuleAST)getRhsSym(5),
                                   (IAsmRuleAST)getRhsSym(7))
                );
                break;
            } 
            //
            // Rule 189:  TryRuleAST ::= try AsmRuleAST
            //
            case 189: {
                setResult(
                    new TryRuleAST0(getLeftIToken(), getRightIToken(),
                                    (IAsmRuleAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 190:  TryRuleAST ::= try AsmRuleAST else AsmRuleAST
            //
            case 190: {
                setResult(
                    new TryRuleAST1(getLeftIToken(), getRightIToken(),
                                    (IAsmRuleAST)getRhsSym(2),
                                    (IAsmRuleAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 191:  WhenRuleAST ::= when LPAREN WhenConditionAST RPAREN do AsmRuleAST
            //
            case 191: {
                setResult(
                    new WhenRuleAST(getLeftIToken(), getRightIToken(),
                                    (IWhenConditionAST)getRhsSym(3),
                                    (IAsmRuleAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 192:  WhenConditionAST ::= WhenChangeAST
            //
            case 192:
                break; 
            //
            // Rule 193:  WhenConditionAST ::= WhenConditionAST COMMA WhenChangeAST
            //
            case 193: {
                setResult(
                    new WhenConditionAST(getLeftIToken(), getRightIToken(),
                                         (IWhenConditionAST)getRhsSym(1),
                                         (IWhenChangeAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 194:  WhenChangeAST ::= new LPAREN VariableRefAST RPAREN
            //
            case 194: {
                setResult(
                    new WhenChangeAST0(getLeftIToken(), getRightIToken(),
                                       (VariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 195:  WhenChangeAST ::= delete LPAREN VariableRefAST RPAREN
            //
            case 195: {
                setResult(
                    new WhenChangeAST1(getLeftIToken(), getRightIToken(),
                                       (VariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 196:  WhenChangeAST ::= changed LPAREN VariableRefAST RPAREN
            //
            case 196: {
                setResult(
                    new WhenChangeAST2(getLeftIToken(), getRightIToken(),
                                       (VariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 197:  ModelManipulationRuleAST ::= CreateRuleAST
            //
            case 197:
                break; 
            //
            // Rule 198:  ModelManipulationRuleAST ::= DeleteRuleAST
            //
            case 198:
                break; 
            //
            // Rule 199:  ModelManipulationRuleAST ::= CopyRuleAST
            //
            case 199:
                break; 
            //
            // Rule 200:  ModelManipulationRuleAST ::= MoveRuleAST
            //
            case 200:
                break; 
            //
            // Rule 201:  ModelManipulationRuleAST ::= ElementUpdateRuleAST
            //
            case 201:
                break; 
            //
            // Rule 202:  CreateRuleAST ::= new LPAREN CreateModelElementAST RPAREN SEMICOLON
            //
            case 202: {
                setResult(
                    new CreateRuleAST0(getLeftIToken(), getRightIToken(),
                                       (ICreateModelElementAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 203:  CreateRuleAST ::= new LPAREN CreateRelationshipAST RPAREN SEMICOLON
            //
            case 203: {
                setResult(
                    new CreateRuleAST1(getLeftIToken(), getRightIToken(),
                                       (ICreateRelationshipAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 204:  CreateModelElementAST ::= QualifiedTypeNameAST CreateEntityBodyAST InConstraintOptAST
            //
            case 204: {
                setResult(
                    new CreateModelElementAST0(getLeftIToken(), getRightIToken(),
                                               (IQualifiedTypeNameAST)getRhsSym(1),
                                               (CreateEntityBodyAST)getRhsSym(2),
                                               (InConstraintOptAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 205:  CreateModelElementAST ::= QualifiedTypeNameAST CreateRelationBodyAST
            //
            case 205: {
                setResult(
                    new CreateModelElementAST1(getLeftIToken(), getRightIToken(),
                                               (IQualifiedTypeNameAST)getRhsSym(1),
                                               (CreateRelationBodyAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 206:  CreateEntityBodyAST ::= LPAREN VariableRefAST RPAREN
            //
            case 206: {
                setResult(
                    new CreateEntityBodyAST(getLeftIToken(), getRightIToken(),
                                            (VariableRefAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 207:  CreateRelationBodyAST ::= LPAREN VariableDefRefAST COMMA ArithmeticTermAST COMMA ArithmeticTermAST RPAREN
            //
            case 207: {
                setResult(
                    new CreateRelationBodyAST(getLeftIToken(), getRightIToken(),
                                              (VariableDefRefAST)getRhsSym(2),
                                              (IArithmeticTermAST)getRhsSym(4),
                                              (IArithmeticTermAST)getRhsSym(6))
                );
                break;
            } 
            //
            // Rule 208:  InConstraintOptAST ::= $Empty
            //
            case 208: {
                setResult(null);
                break;
            } 
            //
            // Rule 209:  InConstraintOptAST ::= in ArithmeticTermAST
            //
            case 209: {
                setResult(
                    new InConstraintOptAST(getLeftIToken(), getRightIToken(),
                                           (IArithmeticTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 210:  CreateRelationshipAST ::= supertypeOf TwoParametersAST
            //
            case 210: {
                setResult(
                    new CreateRelationshipAST0(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 211:  CreateRelationshipAST ::= subtypeOf TwoParametersAST
            //
            case 211: {
                setResult(
                    new CreateRelationshipAST1(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 212:  CreateRelationshipAST ::= typeOf TwoParametersAST
            //
            case 212: {
                setResult(
                    new CreateRelationshipAST2(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 213:  CreateRelationshipAST ::= instanceOf TwoParametersAST
            //
            case 213: {
                setResult(
                    new CreateRelationshipAST3(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 214:  DeleteRuleAST ::= delete LPAREN DeleteContentsAST RPAREN SEMICOLON
            //
            case 214: {
                setResult(
                    new DeleteRuleAST0(getLeftIToken(), getRightIToken(),
                                       (DeleteContentsAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 215:  DeleteRuleAST ::= delete LPAREN DeleteRelationshipAST RPAREN SEMICOLON
            //
            case 215: {
                setResult(
                    new DeleteRuleAST1(getLeftIToken(), getRightIToken(),
                                       (IDeleteRelationshipAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 216:  DeleteContentsAST ::= ArithmeticTermAST DeleteSemanticsOptAST
            //
            case 216: {
                setResult(
                    new DeleteContentsAST(getLeftIToken(), getRightIToken(),
                                          (IArithmeticTermAST)getRhsSym(1),
                                          (DeleteSemanticsOptAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 217:  DeleteSemanticsOptAST ::= $Empty
            //
            case 217: {
                setResult(null);
                break;
            } 
            //
            // Rule 218:  DeleteSemanticsOptAST ::= COMMA DeleteSemanticsAST
            //
            case 218: {
                setResult(
                    new DeleteSemanticsOptAST(getLeftIToken(), getRightIToken(),
                                              (IDeleteSemanticsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 219:  DeleteSemanticsAST ::= delete_content
            //
            case 219: {
                setResult(
                    new DeleteSemanticsAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 220:  DeleteSemanticsAST ::= move_content
            //
            case 220: {
                setResult(
                    new DeleteSemanticsAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 221:  DeleteRelationshipAST ::= supertypeOf TwoParametersAST
            //
            case 221: {
                setResult(
                    new DeleteRelationshipAST0(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 222:  DeleteRelationshipAST ::= subtypeOf TwoParametersAST
            //
            case 222: {
                setResult(
                    new DeleteRelationshipAST1(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 223:  DeleteRelationshipAST ::= typeOf TwoParametersAST
            //
            case 223: {
                setResult(
                    new DeleteRelationshipAST2(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 224:  DeleteRelationshipAST ::= instanceOf TwoParametersAST
            //
            case 224: {
                setResult(
                    new DeleteRelationshipAST3(getLeftIToken(), getRightIToken(),
                                               (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 225:  CopyRuleAST ::= copy LPAREN CopyContentsAST RPAREN SEMICOLON
            //
            case 225: {
                setResult(
                    new CopyRuleAST(getLeftIToken(), getRightIToken(),
                                    (CopyContentsAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 226:  CopyContentsAST ::= ArithmeticTermAST COMMA ArithmeticTermAST COMMA VariableRefAST COMMA CopySemanticsAST
            //
            case 226: {
                setResult(
                    new CopyContentsAST(getLeftIToken(), getRightIToken(),
                                        (IArithmeticTermAST)getRhsSym(1),
                                        (IArithmeticTermAST)getRhsSym(3),
                                        (VariableRefAST)getRhsSym(5),
                                        (ICopySemanticsAST)getRhsSym(7))
                );
                break;
            } 
            //
            // Rule 227:  CopySemanticsAST ::= copy_boundary_edges
            //
            case 227: {
                setResult(
                    new CopySemanticsAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 228:  CopySemanticsAST ::= skip_boundary_edges
            //
            case 228: {
                setResult(
                    new CopySemanticsAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 229:  MoveRuleAST ::= move TwoParametersAST SEMICOLON
            //
            case 229: {
                setResult(
                    new MoveRuleAST(getLeftIToken(), getRightIToken(),
                                    (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 230:  TwoParametersAST ::= LPAREN ArithmeticTermAST COMMA ArithmeticTermAST RPAREN
            //
            case 230: {
                setResult(
                    new TwoParametersAST(getLeftIToken(), getRightIToken(),
                                         (IArithmeticTermAST)getRhsSym(2),
                                         (IArithmeticTermAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 231:  ElementUpdateRuleAST ::= ElementUpdateOpAST TwoParametersAST SEMICOLON
            //
            case 231: {
                setResult(
                    new ElementUpdateRuleAST(getLeftIToken(), getRightIToken(),
                                             (ElementUpdateOpAST)getRhsSym(1),
                                             (TwoParametersAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 232:  ElementUpdateOpAST ::= rename
            //
            case 232: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 233:  ElementUpdateOpAST ::= setValue
            //
            case 233: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 234:  ElementUpdateOpAST ::= setFrom
            //
            case 234: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 235:  ElementUpdateOpAST ::= setTo
            //
            case 235: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 236:  ElementUpdateOpAST ::= setMultiplicity
            //
            case 236: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 237:  ElementUpdateOpAST ::= setAggregation
            //
            case 237: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 238:  ElementUpdateOpAST ::= setInverse
            //
            case 238: {
                setResult(
                    new ElementUpdateOpAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 239:  QualifiedTypeNameAST ::= TypeNameAST
            //
            case 239:
                break; 
            //
            // Rule 240:  QualifiedTypeNameAST ::= QualifiedTypeNameAST DOT TypeNameAST
            //
            case 240: {
                setResult(
                    new QualifiedTypeNameAST(getLeftIToken(), getRightIToken(),
                                             (IQualifiedTypeNameAST)getRhsSym(1),
                                             (ITypeNameAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 241:  TypeNameAST ::= NameLiteral
            //
            case 241: {
                setResult(
                    new TypeNameAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 242:  TypeNameAST ::= IDENTIFIER
            //
            case 242: {
                setResult(
                    new TypeNameAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 243:  TypeNameAST ::= UpperCaseLiteral
            //
            case 243: {
                setResult(
                    new TypeNameAST2(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 244:  TypeNameAST ::= KeywordAsIdentifier
            //
            case 244:
                break; 
            //
            // Rule 245:  VariableRefAST ::= UpperCaseLiteral
            //
            case 245: {
                setResult(
                    new VariableRefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 246:  VariableDefRefAST ::= UpperCaseLiteral
            //
            case 246: {
                setResult(
                    new VariableDefRefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 247:  VariableDefAST ::= UpperCaseLiteral
            //
            case 247: {
                setResult(
                    new VariableDefAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 248:  ValueQualifiedNameAST ::= ValueLocalNameAST
            //
            case 248:
                break; 
            //
            // Rule 249:  ValueQualifiedNameAST ::= QualifiedTypeNameAST DOT TypeNameAST
            //
            case 249: {
                setResult(
                    new ValueQualifiedNameAST(getLeftIToken(), getRightIToken(),
                                              (IQualifiedTypeNameAST)getRhsSym(1),
                                              (ITypeNameAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 250:  ValueLocalNameAST ::= NameLiteral
            //
            case 250: {
                setResult(
                    new ValueLocalNameAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 251:  ValueLocalNameAST ::= IDENTIFIER
            //
            case 251: {
                setResult(
                    new ValueLocalNameAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 252:  ASMFunctionNameAST ::= ValueQualifiedNameAST
            //
            case 252:
                break; 
            //
            // Rule 253:  FormalParameterDefAST ::= LPAREN FormalParamsAST RPAREN
            //
            case 253: {
                setResult(
                    new FormalParameterDefAST0(getLeftIToken(), getRightIToken(),
                                               (IFormalParamsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 254:  FormalParameterDefAST ::= LPAREN RPAREN
            //
            case 254: {
                setResult(
                    new FormalParameterDefAST1(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 255:  FormalParamsAST ::= PatternVariableDefAST TypeOptAST
            //
            case 255: {
                setResult(
                    new FormalParamsAST0(getLeftIToken(), getRightIToken(),
                                         (PatternVariableDefAST)getRhsSym(1),
                                         (TypeOptAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 256:  FormalParamsAST ::= FormalParamsAST COMMA PatternVariableDefAST TypeOptAST
            //
            case 256: {
                setResult(
                    new FormalParamsAST1(getLeftIToken(), getRightIToken(),
                                         (IFormalParamsAST)getRhsSym(1),
                                         (PatternVariableDefAST)getRhsSym(3),
                                         (TypeOptAST)getRhsSym(4))
                );
                break;
            } 
            //
            // Rule 257:  DirectedFormalParameterDefAST ::= LPAREN DirectedFormalParamsAST RPAREN
            //
            case 257: {
                setResult(
                    new DirectedFormalParameterDefAST0(getLeftIToken(), getRightIToken(),
                                                       (IDirectedFormalParamsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 258:  DirectedFormalParameterDefAST ::= LPAREN RPAREN
            //
            case 258: {
                setResult(
                    new DirectedFormalParameterDefAST1(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 259:  DirectedFormalParamsAST ::= DirectionKindAST VariableDefAST TypeOptAST
            //
            case 259: {
                setResult(
                    new DirectedFormalParamsAST0(getLeftIToken(), getRightIToken(),
                                                 (IDirectionKindAST)getRhsSym(1),
                                                 (VariableDefAST)getRhsSym(2),
                                                 (TypeOptAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 260:  DirectedFormalParamsAST ::= DirectedFormalParamsAST COMMA DirectionKindAST VariableDefAST TypeOptAST
            //
            case 260: {
                setResult(
                    new DirectedFormalParamsAST1(getLeftIToken(), getRightIToken(),
                                                 (IDirectedFormalParamsAST)getRhsSym(1),
                                                 (IDirectionKindAST)getRhsSym(3),
                                                 (VariableDefAST)getRhsSym(4),
                                                 (TypeOptAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 261:  DirectionKindAST ::= in
            //
            case 261: {
                setResult(
                    new DirectionKindAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 262:  DirectionKindAST ::= out
            //
            case 262: {
                setResult(
                    new DirectionKindAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 263:  DirectionKindAST ::= inout
            //
            case 263: {
                setResult(
                    new DirectionKindAST2(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 264:  ActualParameterDefAST ::= LPAREN ActualParamsAST RPAREN
            //
            case 264: {
                setResult(
                    new ActualParameterDefAST0(getLeftIToken(), getRightIToken(),
                                               (IActualParamsAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 265:  ActualParameterDefAST ::= LPAREN RPAREN
            //
            case 265: {
                setResult(
                    new ActualParameterDefAST1(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 266:  ActualParamsAST ::= ArithmeticTermAST
            //
            case 266:
                break; 
            //
            // Rule 267:  ActualParamsAST ::= ActualParamsAST COMMA ArithmeticTermAST
            //
            case 267: {
                setResult(
                    new ActualParamsAST(getLeftIToken(), getRightIToken(),
                                        (IActualParamsAST)getRhsSym(1),
                                        (IArithmeticTermAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 268:  ActualPatternParameterDefAST ::= LPAREN ActualParamsPatternVariablesAST RPAREN
            //
            case 268: {
                setResult(
                    new ActualPatternParameterDefAST0(getLeftIToken(), getRightIToken(),
                                                      (IActualParamsPatternVariablesAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 269:  ActualPatternParameterDefAST ::= LPAREN RPAREN
            //
            case 269: {
                setResult(
                    new ActualPatternParameterDefAST1(getLeftIToken(), getRightIToken())
                );
                break;
            } 
            //
            // Rule 270:  ActualParamsPatternVariablesAST ::= PatternVariableRefAST
            //
            case 270:
                break; 
            //
            // Rule 271:  ActualParamsPatternVariablesAST ::= ActualParamsPatternVariablesAST COMMA PatternVariableRefAST
            //
            case 271: {
                setResult(
                    new ActualParamsPatternVariablesAST(getLeftIToken(), getRightIToken(),
                                                        (IActualParamsPatternVariablesAST)getRhsSym(1),
                                                        (PatternVariableRefAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 272:  ConstrainedVariablesOptAST ::= $Empty
            //
            case 272: {
                setResult(null);
                break;
            } 
            //
            // Rule 273:  ConstrainedVariablesOptAST ::= ConstrainedVariablesAST
            //
            case 273:
                break; 
            //
            // Rule 274:  ConstrainedVariablesAST ::= VariableDefAST TypeOptAST ContainmentConstraintOptAST
            //
            case 274: {
                setResult(
                    new ConstrainedVariablesAST0(getLeftIToken(), getRightIToken(),
                                                 (VariableDefAST)getRhsSym(1),
                                                 (TypeOptAST)getRhsSym(2),
                                                 (IContainmentConstraintOptAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 275:  ConstrainedVariablesAST ::= ConstrainedVariablesAST COMMA VariableDefAST TypeOptAST ContainmentConstraintOptAST
            //
            case 275: {
                setResult(
                    new ConstrainedVariablesAST1(getLeftIToken(), getRightIToken(),
                                                 (IConstrainedVariablesAST)getRhsSym(1),
                                                 (VariableDefAST)getRhsSym(3),
                                                 (TypeOptAST)getRhsSym(4),
                                                 (IContainmentConstraintOptAST)getRhsSym(5))
                );
                break;
            } 
            //
            // Rule 276:  ContainmentConstraintOptAST ::= $Empty
            //
            case 276: {
                setResult(null);
                break;
            } 
            //
            // Rule 277:  ContainmentConstraintOptAST ::= in ArithmeticTermAST
            //
            case 277: {
                setResult(
                    new ContainmentConstraintOptAST0(getLeftIToken(), getRightIToken(),
                                                     (IArithmeticTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 278:  ContainmentConstraintOptAST ::= below ArithmeticTermAST
            //
            case 278: {
                setResult(
                    new ContainmentConstraintOptAST1(getLeftIToken(), getRightIToken(),
                                                     (IArithmeticTermAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 279:  TypeOptAST ::= $Empty
            //
            case 279: {
                setResult(null);
                break;
            } 
            //
            // Rule 280:  TypeOptAST ::= COLON QualifiedTypeNameAST
            //
            case 280: {
                setResult(
                    new TypeOptAST(getLeftIToken(), getRightIToken(),
                                   (IQualifiedTypeNameAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 281:  ReturnTypeOptAST ::= $Empty
            //
            case 281: {
                setResult(null);
                break;
            } 
            //
            // Rule 282:  ReturnTypeOptAST ::= COLON QualifiedTypeNameAST
            //
            case 282: {
                setResult(
                    new ReturnTypeOptAST(getLeftIToken(), getRightIToken(),
                                         (IQualifiedTypeNameAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 283:  ValueAST ::= UndefValueAST
            //
            case 283:
                break; 
            //
            // Rule 284:  ValueAST ::= BooleanConstantAST
            //
            case 284:
                break; 
            //
            // Rule 285:  ValueAST ::= MultiplicityConstantAST
            //
            case 285:
                break; 
            //
            // Rule 286:  ValueAST ::= StringConstantAST
            //
            case 286:
                break; 
            //
            // Rule 287:  ValueAST ::= NumericConstantAST
            //
            case 287:
                break; 
            //
            // Rule 288:  ValueAST ::= ValueQualifiedNameAST
            //
            case 288:
                break; 
            //
            // Rule 289:  UndefValueAST ::= undef
            //
            case 289: {
                setResult(
                    new UndefValueAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 290:  BooleanConstantAST ::= false
            //
            case 290: {
                setResult(
                    new BooleanConstantAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 291:  BooleanConstantAST ::= true
            //
            case 291: {
                setResult(
                    new BooleanConstantAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 292:  MultiplicityConstantAST ::= one_to_one
            //
            case 292: {
                setResult(
                    new MultiplicityConstantAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 293:  MultiplicityConstantAST ::= one_to_many
            //
            case 293: {
                setResult(
                    new MultiplicityConstantAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 294:  MultiplicityConstantAST ::= many_to_one
            //
            case 294: {
                setResult(
                    new MultiplicityConstantAST2(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 295:  MultiplicityConstantAST ::= many_to_many
            //
            case 295: {
                setResult(
                    new MultiplicityConstantAST3(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 296:  StringConstantAST ::= StringLiteral
            //
            case 296: {
                setResult(
                    new StringConstantAST(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 297:  NumericConstantAST ::= DecimalIntegerLiteral
            //
            case 297: {
                setResult(
                    new NumericConstantAST0(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 298:  NumericConstantAST ::= IntegerLiteral
            //
            case 298: {
                setResult(
                    new NumericConstantAST1(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 299:  NumericConstantAST ::= FloatingPointLiteral
            //
            case 299: {
                setResult(
                    new NumericConstantAST2(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 300:  NumericConstantAST ::= DoubleLiteral
            //
            case 300: {
                setResult(
                    new NumericConstantAST3(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 301:  KeywordAsIdentifier ::= namespace
            //
            case 301: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 302:  KeywordAsIdentifier ::= import
            //
            case 302: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 303:  KeywordAsIdentifier ::= machine
            //
            case 303: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 304:  KeywordAsIdentifier ::= pattern
            //
            case 304: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 305:  KeywordAsIdentifier ::= neg
            //
            case 305: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 306:  KeywordAsIdentifier ::= find
            //
            case 306: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 307:  KeywordAsIdentifier ::= gtrule
            //
            case 307: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 308:  KeywordAsIdentifier ::= precondition
            //
            case 308: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 309:  KeywordAsIdentifier ::= postcondition
            //
            case 309: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 310:  KeywordAsIdentifier ::= action
            //
            case 310: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 311:  KeywordAsIdentifier ::= apply
            //
            case 311: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 312:  KeywordAsIdentifier ::= xor
            //
            case 312: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 313:  KeywordAsIdentifier ::= asmfunction
            //
            case 313: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 314:  KeywordAsIdentifier ::= rule
            //
            case 314: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 315:  KeywordAsIdentifier ::= skip
            //
            case 315: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 316:  KeywordAsIdentifier ::= fail
            //
            case 316: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 317:  KeywordAsIdentifier ::= call
            //
            case 317: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 318:  KeywordAsIdentifier ::= update
            //
            case 318: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 319:  KeywordAsIdentifier ::= print
            //
            case 319: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 320:  KeywordAsIdentifier ::= println
            //
            case 320: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 321:  KeywordAsIdentifier ::= log
            //
            case 321: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 322:  KeywordAsIdentifier ::= debug
            //
            case 322: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 323:  KeywordAsIdentifier ::= info
            //
            case 323: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 324:  KeywordAsIdentifier ::= warning
            //
            case 324: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 325:  KeywordAsIdentifier ::= error
            //
            case 325: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 326:  KeywordAsIdentifier ::= fatal
            //
            case 326: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 327:  KeywordAsIdentifier ::= seq
            //
            case 327: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 328:  KeywordAsIdentifier ::= random
            //
            case 328: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 329:  KeywordAsIdentifier ::= let
            //
            case 329: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 330:  KeywordAsIdentifier ::= in
            //
            case 330: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 331:  KeywordAsIdentifier ::= below
            //
            case 331: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 332:  KeywordAsIdentifier ::= choose
            //
            case 332: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 333:  KeywordAsIdentifier ::= forall
            //
            case 333: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 334:  KeywordAsIdentifier ::= with
            //
            case 334: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 335:  KeywordAsIdentifier ::= do
            //
            case 335: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 336:  KeywordAsIdentifier ::= iterate
            //
            case 336: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 337:  KeywordAsIdentifier ::= if
            //
            case 337: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 338:  KeywordAsIdentifier ::= try
            //
            case 338: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 339:  KeywordAsIdentifier ::= else
            //
            case 339: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 340:  KeywordAsIdentifier ::= new
            //
            case 340: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 341:  KeywordAsIdentifier ::= delete
            //
            case 341: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 342:  KeywordAsIdentifier ::= delete_content
            //
            case 342: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 343:  KeywordAsIdentifier ::= move_content
            //
            case 343: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 344:  KeywordAsIdentifier ::= copy
            //
            case 344: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 345:  KeywordAsIdentifier ::= copy_boundary_edges
            //
            case 345: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 346:  KeywordAsIdentifier ::= skip_boundary_edges
            //
            case 346: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 347:  KeywordAsIdentifier ::= move
            //
            case 347: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 348:  KeywordAsIdentifier ::= rename
            //
            case 348: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 349:  KeywordAsIdentifier ::= setValue
            //
            case 349: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 350:  KeywordAsIdentifier ::= setFrom
            //
            case 350: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 351:  KeywordAsIdentifier ::= setTo
            //
            case 351: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 352:  KeywordAsIdentifier ::= setMultiplicity
            //
            case 352: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 353:  KeywordAsIdentifier ::= setAggregation
            //
            case 353: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 354:  KeywordAsIdentifier ::= setInverse
            //
            case 354: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 355:  KeywordAsIdentifier ::= undef
            //
            case 355: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 356:  KeywordAsIdentifier ::= true
            //
            case 356: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 357:  KeywordAsIdentifier ::= false
            //
            case 357: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 358:  KeywordAsIdentifier ::= one_to_one
            //
            case 358: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 359:  KeywordAsIdentifier ::= one_to_many
            //
            case 359: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 360:  KeywordAsIdentifier ::= many_to_one
            //
            case 360: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 361:  KeywordAsIdentifier ::= many_to_many
            //
            case 361: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 362:  KeywordAsIdentifier ::= ref
            //
            case 362: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 363:  KeywordAsIdentifier ::= fqn
            //
            case 363: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 364:  KeywordAsIdentifier ::= name
            //
            case 364: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 365:  KeywordAsIdentifier ::= value
            //
            case 365: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 366:  KeywordAsIdentifier ::= source
            //
            case 366: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 367:  KeywordAsIdentifier ::= target
            //
            case 367: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 368:  KeywordAsIdentifier ::= multiplicity
            //
            case 368: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 369:  KeywordAsIdentifier ::= aggregation
            //
            case 369: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 370:  KeywordAsIdentifier ::= inverse
            //
            case 370: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 371:  KeywordAsIdentifier ::= toBoolean
            //
            case 371: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 372:  KeywordAsIdentifier ::= toString
            //
            case 372: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 373:  KeywordAsIdentifier ::= toInteger
            //
            case 373: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 374:  KeywordAsIdentifier ::= toDouble
            //
            case 374: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 375:  KeywordAsIdentifier ::= toMultiplicity
            //
            case 375: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 376:  KeywordAsIdentifier ::= out
            //
            case 376: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 377:  KeywordAsIdentifier ::= inout
            //
            case 377: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 378:  KeywordAsIdentifier ::= parallel
            //
            case 378: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 379:  KeywordAsIdentifier ::= changed
            //
            case 379: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 380:  KeywordAsIdentifier ::= when
            //
            case 380: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 381:  KeywordAsIdentifier ::= shareable
            //
            case 381: {
                setResult(
                    new KeywordAsIdentifier(getRhsIToken(1))
                );
                break;
            } 
            //
            // Rule 382:  OptAnnotationsAST ::= $Empty
            //
            case 382: {
                setResult(null);
                break;
            } 
            //
            // Rule 383:  OptAnnotationsAST ::= AnnotationsAST
            //
            case 383:
                break; 
            //
            // Rule 384:  AnnotationsAST ::= AnnotationAST
            //
            case 384:
                break; 
            //
            // Rule 385:  AnnotationsAST ::= AnnotationsAST AnnotationAST
            //
            case 385: {
                setResult(
                    new AnnotationsAST(getLeftIToken(), getRightIToken(),
                                       (IAnnotationsAST)getRhsSym(1),
                                       (IAnnotationAST)getRhsSym(2))
                );
                break;
            } 
            //
            // Rule 386:  AnnotationAST ::= AnnotationNameAST
            //
            case 386:
                break; 
            //
            // Rule 387:  AnnotationAST ::= AnnotationNameAST LPAREN AnnotationBodyAST RPAREN
            //
            case 387: {
                setResult(
                    new AnnotationAST(getLeftIToken(), getRightIToken(),
                                      (AnnotationNameAST)getRhsSym(1),
                                      (IAnnotationBodyAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 388:  AnnotationBodyAST ::= KeyValuePairAST
            //
            case 388:
                break; 
            //
            // Rule 389:  AnnotationBodyAST ::= AnnotationBodyAST COMMA KeyValuePairAST
            //
            case 389: {
                setResult(
                    new AnnotationBodyAST(getLeftIToken(), getRightIToken(),
                                          (IAnnotationBodyAST)getRhsSym(1),
                                          (KeyValuePairAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 390:  KeyValuePairAST ::= TypeNameAST EQUAL TypeNameAST
            //
            case 390: {
                setResult(
                    new KeyValuePairAST(getLeftIToken(), getRightIToken(),
                                        (ITypeNameAST)getRhsSym(1),
                                        (ITypeNameAST)getRhsSym(3))
                );
                break;
            } 
            //
            // Rule 391:  AnnotationNameAST ::= AnnotationLiteral
            //
            case 391: {
                setResult(
                    new AnnotationNameAST(getRhsIToken(1))
                );
                break;
            }    
            default:
                break;
        }
        return;
    }
}

