/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.actions;


import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TreeEditor;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.treeeditor.Plugin;
import org.eclipse.viatra2.treeeditor.ViatraTreeEditor;
import org.eclipse.viatra2.treeeditor.providers.ViatraTreeEventListener;

public class RenameAction extends ViatraTreeEditorSelectionAction {

	public static final String ID = "ViatraTreeEditor.Actions.RenameAction";

	//protected 
	
	public RenameAction(IWorkbenchPart part) {
		super(part);
		setId(ID);
		setText("Rename");
		setToolTipText("Change the model elements' (local) name");
		setImageDescriptor(Plugin.getImageDescriptor("icons/rename.png"));
	}

	public void run() {
//		try 
//		{
			ViatraTreeEditor iVTE = (ViatraTreeEditor)iPart;
			Tree iTreeControl = iVTE.getTreeViewer().getTree();
			TreeItem item = iTreeControl.getSelection()[0];
			if (item==null)	return;
			IModelElement me = (IModelElement) item.getData();
			
			TreeEditor editor = new TreeEditor(iTreeControl);
			editor.horizontalAlignment = SWT.LEFT;
			editor.grabHorizontal = true;
			editor.minimumWidth = 64;
			
			ViatraTreeEventListener list = new ViatraTreeEventListener(me, editor, iVTE);
			Text celleditor = new Text(iTreeControl, SWT.NONE);
			
			celleditor.setText(me.getName());
			celleditor.addFocusListener(list);
			celleditor.addKeyListener(list);
			celleditor.addModifyListener(list);
			celleditor.selectAll();
			celleditor.setFocus();
			editor.setEditor(celleditor,item);
//		}
//		catch (Exception e)
//		{}
	}

	@Override
	public void updateSelf() {
		if (getSelectedObjects().size() == 1 && getSelectedObjects().getFirstElement() instanceof IModelElement)
		{
			String s = ((IModelElement) getSelectedObjects().getFirstElement()).getFullyQualifiedName();
			if (s!=null && s.length()>0)
				setEnabled(true);
			else
				setEnabled(false);
		}
		else
			setEnabled(false);
		
	}

}
