/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.menu;


import java.util.Iterator;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.viatra2.core.IEntity;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.treeeditor.ViatraTreeEditor;
import org.eclipse.viatra2.treeeditor.actions.ViatraTreeEditorSelectionAction;
import org.eclipse.viatra2.treeeditor.providers.ViatraContentProvider.TargetRelationDummy;

public class ViatraTreeEditorMenuListener implements IMenuListener {

	private ViatraTreeEditor iVTE;

	public ViatraTreeEditorMenuListener(ViatraTreeEditor vte) {
		iVTE = vte;
	}

	public void menuAboutToShow(IMenuManager manager) {
		manager.removeAll();

		boolean isEntitySelected = false;
		boolean isElementSelected = false;

		ISelection sel = iVTE.getSite().getSelectionProvider().getSelection();
	
		
		if (sel instanceof StructuredSelection) {
									
			Iterator it = ((StructuredSelection) sel).iterator();
			while (it.hasNext()) {
				Object curr = it.next();
				if (!isElementSelected) {
					if (iVTE.getDeleteAction()!=null)
					{
						manager.add(iVTE.getDeleteAction());
					}
					else
					{
						manager.add(iVTE.getLocalDeleteAction());
					}
					manager.add(iVTE.getRenameAction());
					manager.add(new Separator("new"));
					manager.add(iVTE.getNewRelationAction());
					manager.add(new Separator("misc"));
					manager.add(iVTE.getExportAction());
					manager.add(new Separator());
					isElementSelected = true;
				}
				if (curr instanceof IEntity && !isEntitySelected) {	
					//if (iVTE.getFramework().isRunnable((IModelElement)curr))
					//{
					//		manager.appendToGroup("misc", iVTE.getRunAction());
					//}
					manager.appendToGroup("new", iVTE.getNewEntityAction());
					isEntitySelected = true;
				}
				if (curr instanceof IRelation)
				{
					manager.add(iVTE.getNavigateRelationAction());
				}
				if (curr instanceof TargetRelationDummy)
				{
					manager.add(iVTE.getNavigateRelationSourceAction());
				}
			}
			manager.add(new Separator());
			// add contributed actions
			for (ViatraTreeEditorSelectionAction a : iVTE.getContributedActions())
			{
				manager.add(a);
			}			
		}

		//manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

}
