/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.properties.descriptors;

import java.util.ArrayList;

import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.eclipse.viatra2.core.IModelElement;

/**
 * Property descriptor for a [;,]-delimited string of VPM FQNs.
 * @author Istvan Rath
 *
 */
public class FQNListPropertyDescriptor extends TextPropertyDescriptor
{

	IModelElement iElement;
	
	public FQNListPropertyDescriptor(Object id, String displayName, IModelElement element) {
		super(id, displayName);
		iElement = element;
	}

	@Override
	public CellEditor createPropertyEditor(Composite parent) 
	{
		return new FQNListCellEditor(parent);
	}

	
	
	public class FQNListCellEditor extends TextCellEditor
	{
		
		public FQNListCellEditor(Composite parent) 
		{
			super(parent);
		}
		
		@Override
		protected Control createControl(Composite parent) {
			Control c = super.createControl(parent);
			try
			{
				// set up content assist
				@SuppressWarnings("unused")
				ContentProposalAdapter a = new ContentProposalAdapter(
						c,
						new TextContentAdapter(),
						//new SimpleContentProposalProvider(new String[]{"A","B","C"}),
						new FQNListContentProposalProvider(),
						null,
						//KeyStroke.getInstance("Ctrl+Space"),
						null
						);
			}
			catch (Exception e) { }
			return c;
		}
	}
	
	public class FQNContentProposal implements IContentProposal
	{

		public FQNContentProposal(IModelElement aE, int origPos) 
		{
			e = aE;
			origPosition = origPos;
		}
		
		IModelElement e;
		int origPosition;
		
		public String getContent() {
			return e.getFullyQualifiedName().substring(origPosition);
		}

		public int getCursorPosition() {
			return getContent().length(); // place cursor at the very end
		}

		public String getDescription() {
			return "A fully qualified name whose prefix matches the content.";
		}

		public String getLabel() {
			return e.getFullyQualifiedName();
		}
		
	}
	
	
	public class FQNListContentProposalProvider implements IContentProposalProvider
	{

		public IContentProposal[] getProposals(String contents, int position) 
		{
			// find last delimiter character before position\
			int begin_pos1 = Math.max(0, contents.substring(0,position).lastIndexOf(";"));
			int begin_pos2 = Math.max(0, contents.substring(0,position).lastIndexOf(","));
			int end_pos1 = position+Math.max(0, contents.substring(position).lastIndexOf(";"));
			int end_pos2 = position+Math.max(0, contents.substring(position).lastIndexOf(","));
			int begin = (Math.max(begin_pos1, begin_pos2)>0?Math.max(begin_pos1, begin_pos2)+1:Math.max(begin_pos1, begin_pos2));
			while (Character.isWhitespace(contents.charAt(begin))) begin++; // trim-begin
			String target = contents.substring(
					begin,
					Math.min(end_pos1, end_pos2));
					//.trim();
			
			String validFQN = target.substring(0,Math.max(0, target.lastIndexOf(".")));
			String remainder = target.substring(target.lastIndexOf(".")+1);
			IModelElement validRoot = iElement.getModelSpace().getModelManager().getElementByName(validFQN);
			if (validRoot!=null)
			{
				ArrayList<IContentProposal> r = new ArrayList<IContentProposal>();
				for (IModelElement e : validRoot.getElementsInNamespace())
				{
					if (e.getName().startsWith(remainder))
						//r.add(new FQNContentProposal(e,position - contents.lastIndexOf(validFQN)));
						r.add(new FQNContentProposal(e,position - begin));
				}
				return r.toArray(new IContentProposal[]{});
			}
			return null;
		}
		
	}
	
	
}
