/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.providers;


import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.viatra2.core.ICoreNotificationListener;
import org.eclipse.viatra2.core.IModelElement;
import org.eclipse.viatra2.core.IRelation;
import org.eclipse.viatra2.core.notification.ICoreNotificationObject;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectCreateEntity;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectCreateInstanceOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectCreateRelation;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectCreateSupertypeOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectDeleteEntity;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectDeleteInstanceOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectDeleteRelation;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectDeleteSupertypeOf;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectMoveTo;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetName;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetRelationFrom;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetRelationTo;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetValue;
import org.eclipse.viatra2.core.notification.ICoreNotificationObjectSetViewInfo;
import org.eclipse.viatra2.core.simple.notification.NotificationObjectTransactionBegin;
import org.eclipse.viatra2.core.simple.notification.NotificationObjectUndoableTransactionBegin;

public class ViatraTreeviewNotificationHandler implements ICoreNotificationListener {

	private TreeViewer iTreeViewer;
	private ViatraContentProvider iContentProvider;
	/**
	 * True, if refreshes should be stopped due to a (long) transaction.
	 */
	private boolean inTransaction = false;
	
	private boolean signalDirty = false;
	
	public ViatraTreeviewNotificationHandler(TreeViewer tv, ViatraContentProvider cp) {
		iTreeViewer = tv;
		iContentProvider = cp;
	}

	private void refreshTreeViewer()
	{
		Display.getDefault().asyncExec(new Runnable(){
			public void run() {
				iTreeViewer.refresh();
			}
			
		});
	}
	
	
	public void actionPerformed(final ICoreNotificationObject notification) {
		//System.out.println("Core notification: "+notification.getActionType()+", inTransaction: "+inTransaction);
		
		if (notification.getActionType().equals(ICoreNotificationObject.TA_TRANSACTION_BEGIN) )
	//		||	notification.getActionType().equals(ICoreNotificationObject.TA_SUBTRANSACTION_BEGIN))
		{
				Object info = ((NotificationObjectTransactionBegin)notification).getInfo();
				if (info instanceof Boolean && ((Boolean)info).booleanValue())
				{
					inTransaction = true;
				}
			return;
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.TA_UNDOABLE_TRANSACTION_BEGIN))
		{
			Object info = ((NotificationObjectUndoableTransactionBegin)notification).getInfo();
			if (info instanceof Boolean && ((Boolean)info).booleanValue())
			{
				inTransaction = true;
			}
			return;
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.TA_UNDO_BEGIN))
		{
			//Object info = ((NotificationObjectUndoableTransactionBegin)notification).getInfo();
			//if (info instanceof Boolean && ((Boolean)info).booleanValue())
			//{
				inTransaction = true;
			//}
			return;
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.TA_TRANSACTION_END) ) 
//			||	notification.getActionType().equals(ICoreNotificationObject.TA_SUBTRANSACTION_END))
		{
			inTransaction = false;
			//iTreeViewer.refresh();
			refreshTreeViewer();
			//iContentProvider.setDirty(); // notify editor that the model has changed
			if (signalDirty)
			{
				iContentProvider.setDirty(); // notify editor that the model has changed
				signalDirty = false;
			}
			return;
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.TA_UNDO_END))
		{
			inTransaction = false;
			//iTreeViewer.refresh();
			refreshTreeViewer();
			if (signalDirty)
			{
				iContentProvider.setDirty(); // notify editor that the model has changed
				signalDirty = false;
			}
			return;
		}
		
		//if (inTransaction)
		//	return;
		
		/*
		 * In every other case (other than transaction_begin/end), dignalDirty should be set to true.
		 */
		if (inTransaction)
		{
			signalDirty = true;
			return;
		}
		 
		Display.getDefault().syncExec(new Runnable(){
			public void run() {
				handleNotification(notification);
				
			}
		});
		

	}

	
	private void handleNotification(ICoreNotificationObject notification)
	{
		if (notification.getActionType().equals(ICoreNotificationObject.ACTION_SET_NAME)) {
			ICoreNotificationObjectSetName n = (ICoreNotificationObjectSetName)notification;
			iTreeViewer.update(n.getElement(), null);
			
			// instances display refresh
			for (IModelElement inst_elem : n.getElement().getInstances())
			{
				iTreeViewer.update(inst_elem, null);
			}
			
			// subtypes display refresh
			for (IModelElement st_elem : n.getElement().getSubtypes())
			{
				iTreeViewer.update(st_elem, null);
			}
			
			// target relations display refresh
			for (IRelation trg_rel : n.getElement().getRelationsTo())
			{
				iTreeViewer.update(trg_rel, null);
			}
			
			//iContentProvider.setDirty();
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_SET_VALUE)) {
			ICoreNotificationObjectSetValue n = (ICoreNotificationObjectSetValue)notification;
			iTreeViewer.update(n.getEntity(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_CREATE_INSTANCEOF)) {
			ICoreNotificationObjectCreateInstanceOf n = (ICoreNotificationObjectCreateInstanceOf)notification;
			iTreeViewer.update(n.getInstance(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_DELETE_INSTANCEOF)) {
			ICoreNotificationObjectDeleteInstanceOf n = (ICoreNotificationObjectDeleteInstanceOf)notification;
			iTreeViewer.update(n.getInstance(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_CREATE_SUPERTYPEOF)) {
			ICoreNotificationObjectCreateSupertypeOf n = (ICoreNotificationObjectCreateSupertypeOf)notification;
			iTreeViewer.update(n.getSub(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_DELETE_SUPERTYPEOF)) {
			ICoreNotificationObjectDeleteSupertypeOf n = (ICoreNotificationObjectDeleteSupertypeOf)notification;
			iTreeViewer.update(n.getSub(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_SET_VIEW_INFO)) {
			ICoreNotificationObjectSetViewInfo n = (ICoreNotificationObjectSetViewInfo)notification;
			iTreeViewer.update(n.getElement(), new String[]{"name"});
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_CREATE_ENTITY)) {
			ICoreNotificationObjectCreateEntity n = (ICoreNotificationObjectCreateEntity)notification;
			iTreeViewer.add(n.getContainer(),n.getCreated());
			//iContentProvider.addNotificationHandler(n.getCreated());
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_CREATE_RELATION)) {
			ICoreNotificationObjectCreateRelation n = (ICoreNotificationObjectCreateRelation)notification;
			iTreeViewer.add(n.getFrom(),n.getNewRelation());
			//iContentProvider.addNotificationHandler(n.getNewRelation());
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_MOVE_ELEMENT_TO)) {
			ICoreNotificationObjectMoveTo n = (ICoreNotificationObjectMoveTo)notification;
			iTreeViewer.remove(n.getElement());
			iTreeViewer.add(n.getNewContainer(),n.getElement());
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_DELETE_ENTITY)) {
			ICoreNotificationObjectDeleteEntity n = (ICoreNotificationObjectDeleteEntity)notification;
			iTreeViewer.remove(n.getDeleted());
			//iContentProvider.removeNotificationHandler(n.getDeleted());
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_DELETE_RELATION)) {
			ICoreNotificationObjectDeleteRelation n = (ICoreNotificationObjectDeleteRelation)notification;
			iTreeViewer.remove(n.getDeleted());
			//iContentProvider.removeNotificationHandler(n.getDeleted());
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_DELETE_CONTAINMENT)) {
			// ICoreNotificationObjectDeleteContainment n = (ICoreNotificationObjectDeleteContainment)notification;
			// FIXME what to do for DELETE_CONTAINMENT?
			// save: Do nothing.
			
		}
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_SET_RELATION_FROM)) {
			ICoreNotificationObjectSetRelationFrom n = (ICoreNotificationObjectSetRelationFrom)notification;
			iTreeViewer.remove(n.getRelation());
			iTreeViewer.add(n.getNewFrom(),n.getRelation());
		}
		// ADDED BY XMI:
		else if (notification.getActionType().equals(ICoreNotificationObject.ACTION_SET_RELATION_TO)) {
			ICoreNotificationObjectSetRelationTo n = (ICoreNotificationObjectSetRelationTo)notification;
			iTreeViewer.update(n.getRelation(), new String[]{"name"});
		}
		iContentProvider.setDirty(); // notify editor that the model has changed
	}
	
	public int getListenerCategory() {
		return 0;
	}
	

}
