/*******************************************************************************
 * Copyright (c) 2004-2008 Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Istvan Rath - initial API and implementation
 *******************************************************************************/

 package org.eclipse.viatra2.treeeditor.wizard;

import java.util.Collection;

/**
 * Describes a New Modelspace Wizard model contribution.
 * @author Istv�n R�th
 */
public interface IWizardModelContributor extends Comparable {

	/**
	 * The unique model contribution ID.
	 */
	public String getID();
	
	/**
	 * Contribution IDs required by this model contribution.
	 */
	public Collection<String> getRequiredIDs();
	
	/**
	 * Contribution IDs which exclude this model contribution.
	 */
	public Collection<String> getForbiddenIDs();
	
	/**
	 * Contribution IDs provided by this model multicontribution.
	 * @return
	 */
	public Collection<String> getContainedIDs();
	
	/**
	 * The category ID to which this model contribution belongs.
	 */
	public String getCategoryID();
	
	/**
	 * The name of this model contribution.
	 */
	public String getName();
	
	/**
	 * The description of this model contribution.
	 */
	public String getDescription();
	
	/**
	 * The filename of the VTML file of this model contribution.
	 */
	public String getFileVTML();
	
	/**
	 * The filename of the VPML file of this model contribution.
	 */
	public String getFileVPML();
	
	/**
	 * The Plugin ID of the contributing plugin.
	 */
	public String getPluginID();
}
